#
#---STARTUP.I   Start-up include file.
#
#   This file MUST be included by the first .cml page to be
#   interpreted in a Caucus session.
#
#   Arguments:
#     $inc(1)    filename for current page
#     $inc(2)    additional args (after standard arg(3)-arg(8) for
#                normal invocation of current page)
#
#   Synopsis:
#   =========
#   When a Caucus session is started, the CML interpreter starts
#   with a particular ".cml" page.  The name of this page
#   is part of the URL.
#
#   Originally, Caucus always had to be started at a specific CML
#   page -- in version 3.0, this was Local/start.cml, which in turn
#   ran welcome.cml or center.cml.  Start.cml had the duty of
#   initializing all of the common variables used throughout the
#   Caucus session, as well as deciding if certain special pages
#   (password confirmation, and personal information registration)
#   had to be run first.
#
#   As of version 3.1, this is no longer the case -- Caucus may start at
#   *any* page.  (For example, a session may be started at a page bookmarked
#   from a previous session.)  Nonetheless, this initialization and
#   decision process must STILL happen.
#
#   Usage:
#   ==========
#   Therefore, the initialization process has been moved to startup.i,
#   and every .cml page must have the capability to conditionally
#   include it, if the page is the first one to be interpreted.
#   Since one of the most important common variables is "href", the
#   simplest way to do this is:
#
#       if $empty($(href))
#          include $cml_dir()/startup.i  thisfile.cml  [additional args]
#       end
#
#   If you need to change any aspects of the interface,
#   see the Local/switch.i file.
#
#==============================================================
#
#---DIR    Relative directory (under .../CML) for this interface.
#
#   The value of DIR should get picked up automatically from the
#   function $cml_dir(), which uses the directory name from the
#   URL that invoked the first page.
#
#   If for some reason this is not sufficient, you can change
#   the definition of DIR here.

set dir $user_var ($userid() spdir)
if $empty($(dir))
   set dir    $cml_dir()
end

#==============================================================
#
#---INC    Relative pathname for "Local" directory.

set inc    $(dir)/Local

#==============================================================
#
#---IMG    Relative URL for images in this interface.

set img    http://$host()/~$caucus_id()/GIF41

#==============================================================
#
#---ICON   Relative URL for downloaded images in this interface
#          (see switch.i)

set icon   $(img)

#==============================================================
#
#---SWITCHES: include the default and custom interface switches.

include $(dir)/defaults.i

#---Include the Local "override" values for the switches.
include $(inc)/switch.i

#==============================================================
#
#---HREF   Full URL for pages in this interface.
#
if $equal ($word (1 $opsys()) unix)
   set href    http://$host()/$(sweb)/swebsock/$pid()/$(dir)
end
else
   set href    http://$host()/$(sweb)/swebsock.exe/$pid()/$(dir)
end

#==============================================================
#
#---HELP   Full URL for help files in this interface.
#

set help $(href)/help
set helpdir $(dir)/help

#==============================================================
#
#---Which browser is the user running? 
#   Sets BROWSER_NAME to name and version (e.g. "Netscape 3.03")
#   Sets IS_MACINTOSH to 1 if user runs on a mac
#   Sets one of the following to 1 (rest to 0).
#      IS_NETSCAPE4  Netscape 4 or higher
#      IS_NETSCAPE3  Netscape 3 or higher
#      IS_NETSCAPE2  Netscape 2 or unknown browser
#      IS_EXPLORER3  Internet Explorer 3.0 or higher
#      IS_EXPLORER4  Internet Explorer 4.0 or higher
#
#   Sets USE_BUTTON to 1 if only buttons (and not <A HREF="#">'s) may have 
#   onClick events (netscape 2 and ie 3).
#
set browser_name (Unknown)
set is_netscape4 0
set is_netscape3 0
set is_netscape2 0
set is_explorer4 0
set is_explorer3 0

set is_macintosh $greater ($str_index (Macintosh $http_user_agent()) 0)

set browser      $str_index (MSIE     $http_user_agent())
if $greater ($(browser) -1)
   set version      $str_sub   ($plus ($(browser) 5) 1 $http_user_agent())
   set is_explorer3 $greater   ($(version) 2)
   set is_explorer4 $greater   ($(version) 3)
   set browser_name $str_sub   ($(browser) 15 $http_user_agent()) ;
   set semicolon    $str_index (; $(browser_name))
   set browser_name $str_sub   (0 $(semicolon) $(browser_name))
end

else
   set browser  $str_index (Mozilla/ $http_user_agent())
   if $greater ($(browser) -1)
      set version $str_sub ($plus ($(browser) 8) 1 $http_user_agent())
      set is_netscape3 $greater ($(version) 2)
      set is_netscape4 $greater ($(version) 3)
      set browser_name Netscape \
                       $str_sub ($plus($(browser) 8) 4 $http_user_agent())
   end
end

set is_netscape2 $not ($or ($(is_explorer3) $(is_explorer4) $(is_netscape3)))
set use_button $or ($(is_netscape2) \
                    $and ($(is_explorer3) $not ($(is_explorer4))) )

#==============================================================
#
#---Miscellaneous strings used throughout the interface:

#---IS31: For code that knows the difference, state that this is version 3.1
set is31 1

#---PAGERESPS: # of responses per page of item
set pageresps $word (1 $user_var ($userid() pageresps) $(default_pageresps))

#---CONTEXT: # of responses that appear in a page *before* a selected response
set context   $word (1 $user_var ($userid() context)   $(default_context))

#---LMARGIN: size of left margin (pixels or %) in item & response display
set lmargin   $word (1 $user_var ($userid() lmargin)   $(default_lmargin))

#---RMARGIN: size of right margin (pixels or %) in item & response display
set rmargin   $word (1 $user_var ($userid() rmargin)   $(default_rmargin))

#---COLS:  width of textbox, in characters
set cols $word (1 $user_var ($userid() boxwide) $(default_cols))

#---ROWS:  height of textbox, in characters
set rows $word (1 $user_var ($userid() boxhigh) $(default_rows))

#---TARGET_WINDOW  Code inside an <A HREF> to pop up a target window
set target_window onClick="win = window.open ('', 'fromcau', \
    'resizeable=1,toolbar=1,scrollbars=1,menubar=1,width=500,height=380');" \
    TARGET="fromcau"

#---CML Interface version number
set i_version  i16

include $(dir)/helpstrings.i


#==============================================================


#---Downloadable icons are no longer allowed (but keep the
#   code in case we change our minds someday!)
set badicon
#if $(downloadable_icons)
#   if $not_empty ( $user_var  ($userid() icons))
#      set icon file:/$user_var($userid() icons)
#      set badicon <A  HREF="$(href)/badicons.cml">\
#                  <IMG SRC="$(icon)/clearpix.gif"  \
#                       ALT="Fix Broken Icons!" BORDER=0></A>
#   end
#end

#---Pre-load the cached conference list with "personal" and "popular"
#   conferences, and the name of conference $arg(3), if supplied.
#   The "___" entry keeps the list from being empty, i.e. from
#   implying "all conferences".
set  l_confs $file($(inc)/l_confs.i)
set  m_confs $user_var($userid() my_confs)
if $not ($or ($equal ($arg(3) x) $empty ($arg(3)) $equal ($arg(3) 0)))
   set prevconf $co_name ($arg(3))
end
eval $cl_list ( $(l_confs) $(m_confs) $(prevconf) ___ )

#---Initialize the itemlist counter.
set ilist       0
set ilistmax  200

#---Kick the "unique # counter" a few times.
eval $unique() $unique()

#---Check to make sure that the web server supplied an actual
#   authorized userid.
if $empty ($http_remote_user())
   "Location: $(href)/nouserid.cml
   "
   quit
end

#---Prepare to start up the interface.
#
#   Create a "thread" of the following pages, in order:
#      (0) Caucus Center
#      (1) current page, with additional args from $inc(2)
#      (2) personal info registration page (if needed -- new user only)
#      (3) password confirmation page (if needed)
#
#   Then do a "Location:" to the last page of the thread.  As the
#   interface unwinds the thread, we will get back to the desired
#   page -- and its caller will always appear to be the Caucus Center.


set arg(2) $page_save (1 center.cml 0+x+x+x+x+x+x # $(center_name))

set fragment $if ( $equal ($inc(1) viewitem.cml) #here #)
set page1    $page_save (1 $inc(1) $arg(2)+$arg(3)+$arg(4)+$arg(5)+$arg(6)+\
                           $arg(7)+$arg(8)$inc(2) $(fragment) ? )

if $not ($my_exist())
   set page1 $page_save (1 perinfo.cml $(page1)+x # Personal Information)
end

if $and ($(confirm_password) $pw_can_verify())
   set page1 $page_save (1 passchk.cml $(page1)+x # Password Confirmation)
end

#---OK, start unwinding the thread!
"%location($(href)/$page_return ($(page1) # ))
return
#--------------------------------------------------------------
#                        END OF STARTUP.I
#--------------------------------------------------------------
