#
#---INDV_COP.I   Handle results of COPY from indvresp.cml.
#
# Input:
#   $form(sconf)     name of conference (to copy/move to)
#   $form(itemchk)   new item checkbox
#   $form(anonly)    anonymous copy checkbox
#   checklist        list of responses
#   icnum            number of current conference
#   iinum            number of current item
#   rpage            name of page to return to
#     
# If the copy fails, indvresp.cml is reloaded with one of the
# following error codes:
#   1 no access to conference
#   2 no item supplied
#   3 no such item
#   4 adding failed (frozen?)
#   5 cannot create new item
#   6 copying not allowed
#-----------------------------------------------------------------------

set  new_cname $lower ($replace (32 _ $form(sconf)))
eval $cl_list($(new_cname))
set  to_inum   -1
set  to_cnum   $word  (1 $cl_list($(new_cname)))

if $equal($(rpage) indvresp.cml)
  set rstring indvresp.cml?$unique()+$(icnum)+$(iinum)+\$(rval)
end
else
  set rstring $(rpage)?$arg(1)+$arg(2)+$arg(3)+$arg(4)+\$(rval)+\
              $arg(6)+$arg(7)+arg(8)#Error
end

#---If not an organizer (in to_cnum), and copying not allowed, fail.
set allowcopy $word (1 $conf_var ($(to_cnum) allowcopy) $(allow_copy))
if $and ($less ($cl_access($(to_cnum)) 3) $not ($(allowcopy)))
   set rval 6
   "%location($(href)/$reval($(rstring)))
   "
   quit
end

set iname     $upper1 ($replace (_ 32 $cl_name($(icnum))))

set rfirst $word (1 $(checklist))
set anon   $and  ($form(anonly) $gt_equal($cl_access($(to_cnum)) 3))

#---Copy to a new item?  Create the item, first.
if $form(itemchk)
   if $equal ($(rfirst) 0)
      set cir $(icnum) $(iinum) 0

      #---Anonymous copy/move just gets original conf name.
      if $(anon)
         set copied $word (1 $re_copied ($(cir)))
         if $empty ($(copied))
            set copied $cl_name($(icnum))
         end
      end
      #---"Signed" copy/move gets conf name, copier, and original date/time.
      else
         set copied $quote($re_copied ($(cir)))
         if $empty ($(copied))
            set copied  $quote ($cl_name($(icnum)) $(iinum) 0 $re_time($(cir)))
         end
      end

      set ok  $add_item ($(to_cnum) $re_prop ($(cir)) $re_owner($(cir))     \
                $quote ($re_author ($(cir))) $quote ($re_title ($(cir)))   \
                $(copied)  $re_text($(cir)) )
      set rrest $rest (2 $(checklist))
   end

   #---If copying just responses to a new item, make up some boilerplate
   #   item text & title.
   else
      set ok  $add_item ($(to_cnum) 0 $userid() $quote($my_name()) \
                  $quote(Discussion from $(iname) item $(iinum)) 0 \
                    The first $sizeof($(checklist)) responses  \
                    have been copied from $(iname) item $(iinum).)
      set rrest $(checklist)
   end
   if $equal ($(ok) 0)
      set rval 5
      "%location($(href)/$reval($(rstring)))
      "
      quit
   end
   set to_inum $(ok)
   set to_rnum 0
end

#---Copying *just* responses?  Figure out which ones, where.
else
   set items   $it_parse ($(to_cnum) $form(itemno))
   set to_inum $word (2 $(items))
   if $empty ($(to_inum))
       set rval 3
      "%location($(href)/$reval($(rstring)))
      "
      quit
   end
   set rrest $(checklist)
   set to_rnum $plus (1 $it_resps ($(to_cnum) $(to_inum)))
end

#---Copy the selected responses over.
for r in $(rrest)
   set cir $(icnum) $(iinum) $(r)

   #---Anonymous copy/move just gets original conf name.
   if $(anon)
      set copied $word (1 $re_copied ($(cir)))
      if $empty ($(copied))
         set copied $cl_name($(icnum))
      end
   end
   #---"Signed" copy/move gets conf name, copier, and original date/time.
   else
      set copied $quote($re_copied ($(cir)))
      if $empty ($(copied))
         set copied  $quote ($cl_name($(icnum)) $(iinum) $(r) $re_time($(cir)))
      end
   end

   set ok  $add_resp ($(to_cnum) $re_prop ($(cir)) $(to_inum) \
             $re_owner($(cir)) $quote ($re_author ($(cir))) \
             $(copied) $re_text($(cir)))
   if $equal ($(ok) 0)
      set rval 4
      "%location($(href)/$reval($(rstring)))
      "
      quit
   end
end
#------------------------------------------------------------------
