/*** CHECK_LICENSE.  Check the validity of a Caucus license. 
/
/    valid = check_license (license);
/
/    Parameters:
/       int     valid;                (is license valid?)
/       struct  license_t *license;   (license block, from Initapi object)
/
/    Purpose:
/       (API service function).
/
/       Is a particular Caucus license (as recorded in a license
/       structure) valid?  If not, why not?
/
/    How it works:
/       Checks to see if 'license' has expired.  If not, checks to
/       see if this system is in the list of allowed system id's.
/
/    Returns: LICE_OK       (0)   License is OK.
/             LICE_EXPIRED  (1)   License has expired.
/             LICE_BADCLOCK (2)   Somebody set the clock back!
/             LICE_BADSYSID (3)   Bad system id.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions: a_init()
/
/    Called by: UI
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  api/checklic.c
/ */

/*** Copyright (C) 1993 Camber-Roth.   All rights reserved. */

/*: CR  1/19/93 13:18 New function. */
/*: CR  1/26/93 15:51 Decrypt license info with PLAIN. */
/*: CR  2/19/93 14:23 Add "#include done.h", NULLSTR. */
/*: JV  3/16/93 10:02 Add handicap.h and done.h. */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "done.h"
#include "api.h"
#include "license.h"
#include "done.h"

#undef   NULLSTR
#define  NULLSTR(x)   (!(x[0]))

#define  BETA_EXPIRE  L(875382902)  /* 9/27/97 */

FUNCTION  check_license (struct license_t *license)
{
   int    ok, i, success;
   Chix   sid;
   int4   systime();
   char  *strplain();

   sid     = chxalloc (L(20), THIN, "copyprot sid");
   success = 0;

   /*** If the list of allowable sys id's is not empty, make sure that
   /    the current system id is in the list. */
   if (NOT NULLSTR (license->sysid[0])) {
      systemid (sid);
      for (i=ok=0;   NOT NULLSTR(license->sysid[i]);   ++i) {
         if (chxeq (sid, CQ(strplain(license->sysid[i]))))  ok = 1;
      }

      if (NOT ok)  DONE (LICE_BADSYSID);
   }


   /*** If this license doesn't expire, we're OK.  Otherwise, make sure
   /    it hasn't expired, and that the clock hasn't been artificially
   /    set back. */
/* if (systime() > BETA_EXPIRE)               DONE (LICE_EXPIRED); */
   if (NOT PLAIN(license->expires))           DONE (LICE_OK);
   if (systime() > PLAIN(license->expires))   DONE (LICE_EXPIRED);
   if (NOT sysclkchk (0))                     DONE (LICE_BADCLOCK);

   DONE (LICE_OK);

 done:
   chxfree (sid);
   return  (success);
}
