/*** CHG_SETTING.   Change a particular SET option.
/
/    ok = chg_setting (optstr, which, t_value, n_value);
/
/    Parameters:
/       int     ok;       (success?)
/       Chix    optstr;   (string of options, from a_get_person)
/       int     which;    (number of selected option; 1 is 1st)
/       Chix    t_value;  (new value, if option has text value)
/       int     n_value;  (new value, if option has numeric value)
/
/    Purpose:
/       Replace a selected option from a user's option string (typically
/       taken from Person->setopts, as returned by a_get_person) with
/       a particular string or numeric value.
/
/    How it works: the table lists the WHICH number, the name, and value
/    of each option.
/       1  debug         number (0=off)
/       2  screensize    number (of lines)
/       3  screenwidth   number (of single-width characters)
/       4  format        number (0=off, 1=on)
/       5  editor        text   (name of editor)
/       6  tab           number (tab turns into this many blanks)
/       7  page          number (ascii value of desired new page character)
/       8  print         number (0=>default printcode, 1=>user edited printcode)
/       9  verbose       number (0=>off, 1=>on)
/      10  dictionary    number (of dictionary to use)
/      11  eot           text   (of EOT string; ""=>2return, $endfile$=>1eof)
/      12  expiration    number (0=>off, 1=>on)
/      13  text_entry    number (0=>terminal, 1=>editor, 2=>wordwrap)
/      14  receipt_size  number (of lines in return receipt)
/      15  mytext        number (0=>later, 1=>now, 2=>never)
/      16  startmenu     text   (menu to start on conference entry; or "")
/      17  import_mail   number (0=>off, 1=>import, 2=>copy)
/      18  export_mail   number (0=>off, 1=>export, 2=>copy.  Not used.)
/
/    Returns: 1 on success.
/             0 on bad arguments.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by: UI
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  api/getset.c
/ */

/*** Copyright (C) 1992 Camber-Roth.   All rights reserved. */

/*: CR 12/13/92 11:37 New function. */
/*: CR  1/13/93 14:55 2nd arg of chxalter is int4. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"

FUNCTION  chg_setting (Chix optstr, int which, Chix t_value, int n_value)
{
   static Chix save_defaults = nullchix;
   Chix   word, temp;
   char   str[40];
   int4   pos;
   int    i;

   if (optstr == nullchix)            return (0);
   if (which  <  1   ||  which > 18)  return (0);

   word = chxalloc (L(16), THIN, "chg_set word");

   /*** Pluck out the string value of the WHICH'th option. */
   pos = chxtoken (word, nullchix, which, optstr);

   /*** If OPTSTR does not have that option, fill in *all* the missing
   /    options from the dictionary. */
   if (EMPTYCHX (word)) {

      /*** Find the first "missing" option I.   Get options I through 18
      /    from the dictionary into WORD. */
      for (i=1;   i<=18;   ++i)
         if (chxtoken (word, nullchix, i, optstr) < 0)  break;
      temp = chxalloc (L(80), THIN, "chg_set tempchx");
      mdstr    (temp, "sys_Tset0", &save_defaults);
      chxtoken (nullchix, word, i, temp);
      chxfree  (temp);

      /*** Append the "missing" options to the end of OPTSTR.  Now find
      /    option WHICH again. */
      chxcat   (optstr, CQ(" "));
      chxcat   (optstr, word);
      pos = chxtoken (word, nullchix, which, optstr);
   }

   switch (which) {

      case ( 4):  /* FORMAT */
         n_value = 1 - n_value;

      case ( 1):  /* DEBUG */
      case ( 2):  /* SCREENSIZE */
      case ( 3):  /* SCREENWIDTH */
      case ( 6):  /* TAB */
      case ( 7):  /* PAGE */
      case ( 8):  /* PRINT */
      case ( 9):  /* VERBOSE */
      case (10):  /* DICTIONARY */
      case (12):  /* EXPIRATION */
      case (13):  /* TEXT_ENTRY */
      case (14):  /* RECEIPT_SIZE */
      case (15):  /* MYTEXT */
      case (17):  /* IMPORT_MAIL */
      case (18):  /* EXPORT_MAIL */
         sprintf  (str, "%d", n_value);
         chxalter (optstr, pos, word, CQ(str));
         break;

      case ( 5):  /* EDITOR */
         chxalter (optstr, pos, word, t_value);
         break;

      case (11):  /* EOT */
         if      (EMPTYCHX (t_value))                temp = CQ("2return");
         else if (chxeq (t_value, CQ("$endfile$")))  temp = CQ("1eof");
         else                                        temp = t_value;
         break;

      case (16):  /* STARTMENU */
         temp = (EMPTYCHX (t_value) ? CQ("off") : t_value);
         chxalter (optstr, pos, word, temp);
         break;
   }

   chxfree (word);
   return  (1);
}
