/*** CEDEDCOM.  Command handler for Caucus EDitor.
/
/    CEDEDCOM prompts for CAUCUS EDitor commands from the user
/    and interprets them.  CED contains the pointers to the
/    the file currently being edited.
/
/    If TRUNCATE is true, any text added with the Insert command
/    will be truncated (with a warning) to 77 columns. 
/
/    Editing commands are:
/      STOP                   leave editor
/      ADD     a              insert after line 'a'
/      DELETE  a  b           delete lines a-b
/      SEARCH  a  b           search for a string in a-b
/      CHANGE  a  b           search for and change a string in a-b
/      SHOW    a  b           print lines a-b with line numbers
/      VIEW    a  b           print lines a-b w/o  line numbers  */

/*** Copyright (C) 1984-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: AM  2/07/89 10:24 Jcaucus changes*/
/*: CR  4/05/89 11:32 Put 'nodwtosw' back in. */
/*: CW  5/18/89 15:20 Add XW unit codes. */
/*: JV 11/12/90 10:55 Replaced global mtu[] with dynamic string. */
/*: CR  7/01/91 23:37 Rearrange include files. */
/*: CR  7/24/91 21:37 Make Lint happier. */
/*: CR 10/10/91 20:44 sysfree tmptext before returning! */
/*: JX  3/28/92 10:15 Rewrite using memory instead of file. Chixify. */
/*: CX  4/10/92 17:07 Finish CHANGE, minor chx fixes (chxformat, etc). */
/*: JX  5/16/92 16:01 Minor chx fixes (chxformat, chxindex, etc). */
/*: CR 10/13/92 14:09 Clear tmptext in cednotfnd. */
/*: CR 12/10/92 14:59 chxalter arg is int4. */
/*: CR 12/16/92 16:04 DELETE code: only free non-null chx. */
/*: JV  5/02/93 16:02 Clear tmptext in cededcom.c. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"
#include "ced.h"

extern Chix  ss_colon;
extern union null_t  null;

FUNCTION  cededcom (ced, truncate)
   Ced *ced;
   int  truncate;
{
   static Chix edittable, s_search, s_new, s_old;
   Chix  str, com, strl0, strl1, old, new, tmptext;
   int   j, lowest, code, success;
   int4  i, l0, l1;

   ENTRY ("cededcom", "");

   str = chxalloc (L(200), THIN, "cededc str");
   com = chxalloc ( L(20), THIN, "cededc com");
   old = chxalloc ( L(60), THIN, "cededc old");
   new = chxalloc ( L(60), THIN, "cededc new");
   strl0   = chxalloc ( L(20), THIN, "cededc strl0");
   strl1   = chxalloc ( L(20), THIN, "cededc strl1");
   tmptext = chxalloc (L(100), THIN, "tmptext");

   mdtable ("ced_table", &edittable);

   while (1) {

      /*** Prompt for Caucus editor command. */
      code = prompter (ss_colon, edittable, 2, "ced_Hcom", 40, str, nullchix);
      if (code == -4)  FAIL;

      chxtoken  (com, nullchix, 1, str);
      jixreduce (com);
      chxtoken  (strl0, nullchix, 2, str);
      chxtoken  (strl1, nullchix, 3, str);
      if (NOT chxnum (strl0, &l0))  l0 = ced->mark+1;
      if (NOT chxnum (strl1, &l1))  l1 = l0;

      /*** Bound the values of the first & last line numbers on the editing
      /    command, and supply defaults if not given. */
      lowest = (code==4 ? -1 : 0);
      l0 = min (ced->size-1, max (lowest, (int) l0-1));
      l1 = min (ced->size-1, max (lowest, (int) l1-1));

      /*** STOP */
      if (code == 0)   SUCCEED  

      /*** DELETE */
      else if (code == 1) {
         for (i=l0, j=l1+1;   j<ced->size;   ++i, ++j) {
            if (ced->line[i] != nullchix)  chxfree (ced->line[i]);
            ced->line[i] = ced->line[j];
            ced->line[j] = nullchix;   /* Just in case! */
         }
         ced->size = i;
         ced->mark = min ((int) l0, ced->size-1);
      }

      /*** LINES */
      else if (code == 5)  {
         if (l0 < 0)  continue;
         sysbrkclear();
         for (i=l0;   i<=l1;   ++i) {
            chxclear   (tmptext);
            chxformat  (tmptext, CQ("%4d "), i+1, L(0), nullchix, nullchix);
            unit_write (XWTX, tmptext);
            unit_write (XWTX, ced->line[i]);
            unit_write (XWTX, CQ("\n"));
            if (sysbrktest())  {sysbrkclear();   break;}
         }
         ced->mark = l1;
      }

      /*** VIEW */
      else if (code == 6)   {
         if (l0 < 0)  continue;
         sysbrkclear();
         for (i=l0;   i<=l1;   ++i) {
            unit_write (XWTX, ced->line[i]);
            unit_write (XWTX, CQ("\n"));
            if (sysbrktest())  {sysbrkclear();   break;}
         }
         ced->mark = l1;
      }

      /*** SEARCH */
      else if (code == 3)  {
         mdstr (tmptext, "ced_Psearch", &s_search);
         if (prompter (tmptext, nullchix, 0, "ced_Hsearch",
                       60, old, nullchix) != -1) {
            continue;
         }
         if (EMPTYCHX (strl0))  l0 = ced->mark + 1;
         if (EMPTYCHX (strl1))  l1 = ced->size - 1;
         for (i=l0;   i<=l1;   ++i) {
            if (chxindex (ced->line[i], L(0), old) >= 0)  break;
         }
         if (i <= l1) {
            chxclear  (tmptext);
            chxformat (tmptext, CQ("%4d "), L(i+1), L(0), nullchix, nullchix);
            unit_write (XWTX, tmptext);
            unit_write (XWTX, ced->line[i]);
            unit_write (XWTX, CQ("\n"));
            ced->mark = i;
         }
         else cednotfnd (old, l0, l1, tmptext);
      }

      /*** CHANGE */
      else if (code == 2)  {
         mdstr (tmptext, "ced_Pold", &s_old);
         if (prompter (tmptext, nullchix, 0, "ced_Hold",
                       60, old, nullchix) != -1) {
            continue;
         }
         mdstr (tmptext, "ced_Pnew", &s_new);
         if (prompter (tmptext, nullchix, 0, "ced_Hnew",
                       60, new, nullchix) <= -3) 
            continue;
         if (EMPTYCHX (strl0))  l0 = ced->mark + 1;
         if (EMPTYCHX (strl1))  l1 = ced->size - 1;
         for (i=l0;   i<=l1;   ++i) {
            if (chxindex (ced->line[i], L(0), old) >= 0) {
               /*** Max line length is 200 characters in ALL cases. */
               if (chxlen (ced->line[i]) - chxlen (old) + chxlen (new) >= 200)
                    mdwrite  (XWER, "ced_Ttooint4", null.md);
               else chxalter (ced->line[i], L(0), old, new);
               break;
            }
         }
         if (i <= l1) {
            chxclear  (tmptext);
            chxformat (tmptext, CQ("%4d "), L(i+1), L(0), nullchix, nullchix);
            unit_write (XWTX, tmptext);
            unit_write (XWTX, ced->line[i]);
            unit_write (XWTX, CQ("\n"));
            ced->mark = i;
         }
         else cednotfnd (old, l0, l1, tmptext);
      }

      /*** INSERT */
      else if (code == 4)  {
         ced->mark = l0;
         cedintext (ced, truncate);
      }
   }

done:
   chxfree (str);
   chxfree (com);
   chxfree (old);
   chxfree (new);
   chxfree (strl0);
   chxfree (strl1);
   chxfree (tmptext);
   RETURN (success);
}

FUNCTION  cednotfnd (str, l0, l1, tmptext)
   Chix   str, tmptext;
   int4   l0, l1;
{
   Chix format;

   format = chxalloc (L(100), THIN, "format");

   mdstr      (format, "ced_Fnofind", null.md);
   chxclear   (tmptext);
   chxformat  (tmptext, format, l0+1, l1+1, str, nullchix);
   unit_write (XWER, tmptext);
   chxfree (format);
}
