
/*** CEDEDIT.  Edit the contents of FILE.
/
/    CEDEDIT allows the user to edit the contents of FILE.  If TRUNCATE
/    is true, CEDEDIT will truncate (with a warning) any line entered
/    that is longer than 77 characters (unless it is entered via
/    input redirection from a file).
/
/    CEDEDIT returns 1, unless FILE or the CED editor file could
/    not be opened, in which case it immediately returns 0.  */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: CW  5/18/89 15:20 Add XW unit codes. */
/*: CR  9/29/89 17:03 Allowed CEDEDIT to edit an empty file. */
/*: CR 11/19/89 17:26 Mark block.type as "bad". */
/*: CR  4/08/91 19:18 Use new confnum argument to unit_name(). */
/*: UL 12/11/91 20:12 Make unit_name args 3-6 int4.*/
/*: JX  3/28/92 09:43 Rewrite to use memory instead of disk. Chixify. */
/*: JX  5/16/92 16:07 Fix Chix. */
/*: CX  6/29/92 14:41 Initialize, and also deallocate, all of CED! */
/*: CR 10/08/92 22:46 Move <stdio.h> around. */
/*: CL 12/11/92 15:51 Long sysmem arg. */
/*: CK  9/20/93 16:43 Complain if file too big for CED. */
/*: CR  5/06/95 15:43 Allow editing of new (create it) file! */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "ced.h"

extern union  null_t         null;

FUNCTION  cededit (file, truncate)
   int    file;
   Flag   truncate;
{
   Ced  *ced;
   int   i, success, ok;
   char *sysmem();

   ENTRY ("cededit", "");

   mdwrite (XWTX, "ced_Tusing", null.md);

   /*** Allocate CED buffer, and initialize it. */
   ced = (Ced *) sysmem (L(sizeof (Ced)));
   if (ced == (Ced *) NULL) {
      buglist ("cededit: not enough memory.\n");
      mdwrite (XWER, "ced_Enomem", null.md);
      RETURN (0);
   }
   ced->size =  0;
   ced->mark = -1;
   for (i=0;   i<MAXCED;   ++i)  ced->line[i] = nullchix;


   /*** Read the contents of FILE into the text buffer. */
   ok = 1;
   if (unit_view (file)) {
      ok = cedinfile (ced, file);
      unit_close (file);
   }
   if (NOT ok)  { mdwrite (XWER, "ced_Etoobig", null.md);   FAIL; }
   if (NOT unit_make (file))                                FAIL;

   /*** Process editing commands until the user is done. */
   cededcom  (ced, truncate);

   /*** Write the contents of the editing file back out to FILE. */
   for (i=0;   i<ced->size;   ++i) {
      unit_write (file, ced->line[i]);
      unit_write (file, CQ("\n"));
   }
   unit_close (file);
   SUCCEED;

done:
   /*** Deallocate all of the CED structure. */
   for (i=0;   i<MAXCED;   ++i) {
      if (ced->line[i] == nullchix)  break;
      chxfree (ced->line[i]);
   }
   sysfree ((char *) ced);

   RETURN  (success);
}
