/*** EDITFILE.   Edit a FILE using the supplied EDITOR.
/
/    Edits FILE with EDITOR.  TRUNCATE is supplied because the
/    Caucus editor understands the truncation of lines longer than
/    77 characters.
/
/    Returns 1 if editing was successful and the resulting file was
/    saved.  Returns 0 if the file was not saved, or if EDITOR could
/    not be found or executed.   If there was a system-defined
/    editor list, it is used. */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: CW  5/18/89 15:20 Add XW unit codes. */
/*: CR  9/11/89 13:33 Replace mac_expand() with mdalloc() and mac_subs(). */
/*: CR 10/30/89 12:22 Fix improper use of mac_subs(). */
/*: CR  1/26/90 14:26 Check success status of cededit(). */
/*: CR  3/22/90 12:41 New success code: 2 good, 1 ok but no change, 0 fail. */
/*: CR  6/28/90 15:24 Replace all "####"s with "$2". */
/*: JV  7/22/90 17:20 Change decl of TRUNCATE to match caucus.h. */
/*: JV 11/13/90 15:25 Replace global mtu[] with dynamic string. */
/*: CR 10/10/91 20:45 Sysfree tmptext before returning! */
/*: JX  3/28/92 11:05 Chixify. */
/*: CX  4/10/92 16:29 Fix minor chix errs (sysfree, chxformat, etc). */
/*: JX  5/16/92 16:16 Fix Chix. */
/*: JX  9/30/92 15:50 Fixify. */
/*: CR 10/09/92 16:58 Allocate mactext. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/10/92 17:06 Long ascofchx args. */
/*: CP  8/06/93 15:38 Add 'master' arg to mac_subs() call. */

#include "caucus.h"
#include "chixuse.h"
#include <stdio.h>

extern struct master_template  master;
extern struct flag_template    flag;
extern union  null_t           null;
extern Chix   ss_dotnl, ss_caucus;

FUNCTION  editfile (file, useeditor, truncate)
   int    file;
   Chix   useeditor;
   Flag   truncate;
{
   char   macnamestr[40];
   static Chix editnames, editptrs;
   Chix   format, tmptext, expansion, mactext, editor, macname;
   short  ed_success, found;
   Chix   mac_subs();
   
   ENTRY ("editfile", "");

   editor    = chxalloc ( L(60), THIN, "editfile editor");
   format    = chxalloc (L(100), THIN, "editf format");
   macname   = chxalloc ( L(20), THIN, "editfile macname");
   mactext   = chxalloc ( L(20), THIN, "editfile mactext");
   tmptext   = chxalloc ( L(40), THIN, "editfile tmptext");

   ed_success = 0;

   /*** The CAUCUS editor can always be used. */
   chxtoken  (editor, nullchix, 1, useeditor);
   jixreduce (editor);
   if (chxeq (editor, ss_caucus))  ed_success = cededit (file, truncate) + 1;

   /*** If there is a system-manager defined editor list, use that, and
   /    perform macro expansion on the editor name.  If the editor was
   /    not in the list, tell the user which ones are. */
   else if (flag.editlist) {
      mdtable ("editnames", &editnames);
      found = tablematch (editnames, editor);

      /*** Is the selected editor in the list of defined edit macros? */
      if (found >= 0) {

         /*** Yes, pull out the name MACNAME of the macro. */
         mdtable  ("editptrs", &editptrs);
         chxtoken (macname, nullchix, found+1, editptrs);
         ascofchx (macnamestr, macname, L(0), L(40));

         /*** Attempt to expand it.  Keep the "$2" from being expanded by
         /    a trick with "####", 'cause SYSEDITOR wants to expand that
         /    part all by itself. */
         chxcat   (editor, CQ(" ####"));
         if (mdstr (mactext, macnamestr, null.md)) {
            expansion = mac_subs (mactext, editor, &master);
            while (chxalter (expansion, L(0), CQ("####"), chxquick("$2", 1)) ) ;
            ed_success = syseditor (file, expansion);
            chxfree  (expansion);
         }

         /*** Complain if we couldn't expand the macro. */
         else {
            mdstr      (format, "mai_Ebadexp", null.md);
            chxformat  (tmptext, format, L(0), L(0), editor, nullchix);
            unit_write (XWER, tmptext);
            ed_success = 0;
         }
      }

      /*** Complain that this editor is not allowed, and tell the user
      /    which ones are. */
      else {
         list_editors (editnames);
         ed_success = 0;
      }
   }

   else if ( (ed_success = syseditor (file, editor)) > 0)  ;

   else {
      mdwrite    (XWER, "ced_Tnoed", null.md);
      unit_write (XWER, editor);
      unit_write (XWER, ss_dotnl);
      ed_success = 0;
   }

   chxfree (format);
   chxfree (tmptext);
   chxfree (editor);
   chxfree (macname);
   chxfree (mactext);

   RETURN  (ed_success == 2);
}
