/*** TEXTENTER.  Read plaintext from the user, and ENTER it into FILE.
/
/    TEXTENTER reads text from the user, and appends it to FILE.  If
/    FILE does not exist, it is created.  If TRUNCATE is true, overint4
/    input lines are trimmed to 76 characters, and a warning is printed.
/
/    If the user has chosen to enter text via the editor, TEXTENTER
/    invokes editfile() directly, rather than prompting for the
/    text a line at a time.
/
/    If MAXCHARS and MAXLINES are not 0, TEXTENTER tries to limit the
/    amount of text entered to that number of characters and lines,
/    respectively.  There is no guarantee that it will succeed.
/
/    if CONTEXT is true, it means that there is a context, i.e. text
/    already on the screen that the user would like to (continue to)
/    see, if possible, while entering the new response.
/
/    TEXTENTER assumes that FILE has been locked, and will be unlocked
/    after it returns.
/
/    TEXTENTER returns 1 normally, or 0 if (a) the user pressed
/    <CANCEL> and confirmed that the text should be cancelled,
/    or (b) if FILE could not be opened. */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: CR  4/03/89 12:40 Use syspluck() to get redirection file name. */
/*: JV  9/15/89 18:36 Add _beg_entertext and _end_entertext AFM's. */
/*: CW  5/18/89 15:20 Add XW unit codes. */
/*: CW  6/12/89 15:18 Add handling for SET TEXT_ENTRY EDITOR. */
/*: CR 10/17/89 15:17 Replace 2nd arg of FIRE_ARM with "". */
/*: JV  1/30/90 16:41 Expand userreg.texten=2 means use wordwrap. */
/*: JV  1/30/90 22:01 Move firing of AFM before RETURNs. */
/*: JV  3/05/90 16:17 If wordwrap fails, return (0). */
/*: CR  3/05/90 18:57 Add context argument, wordwrap -> systextinput(). */
/*: CR  3/19/90 16:17 Default EDITOR is thisuser.editor. */
/*: CR  4/16/90 16:06 Split off input redirection into input_redirect(). */
/*: CR  4/24/90 20:08 Remove unused 'lock' variable. */
/*: CR  6/05/90 22:51 Modify text entry method based on flag.fullscreen. */
/*: CR  6/28/90 13:09 Modify text entry method based on flag.infile. */
/*: JV  7/22/90 17:18 Change decl of TRUNCATE to match caucus.h. */
/*: CR  7/24/91 21:39 Make Lint happier. */
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: JX  3/28/92 13:10 Chixify. */
/*: JX  5/16/92 16:57 Fix Chix. */
/*: CX  6/04/92 17:38 Replace systextinput() with wordwrap(), sysvmedit(). */
/*: CX  6/05/92 13:14 Replace flag.infile with u_keybd. */
/*: CR 10/15/92 12:42 Use new version of syspluck(). */
/*: TP 10/19/92 13:52 Use input_control() to determine source of input. */
/*: CR 10/21/92 11:39 Replace input_redirect() with copy_redirect(). */
/*: CR 12/10/92 15:17 syspluck 2nd arg is int4. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CP  2/07/94 14:22 Return failure if file does not exist. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "chixuse.h"
#include "done.h"
#include "ced.h"

extern Userreg                thisuser;
extern struct screen_template screen;
extern struct flag_template   flag;
extern union  null_t          null;
extern Chix   ss_inred;
extern Chix   afm_name;

/*ARGSUSED*/

FUNCTION  textenter (file, truncate, maxchars, maxlines, context)
   int    file, maxchars, maxlines, context;
   char   truncate;
{
   Chix   str, editor, redfile, newconf;
   int    length, code, method, success;
   short  redirect;

   ENTRY ("textenter", "");

   str     = chxalloc (L(200), THIN ,"texte str");
   editor  = chxalloc (L(80),  THIN ,"texte editor");
   newconf = chxalloc (L(40),  THIN ,"texte newconf");
   redfile = chxalloc (L(100), THIN ,"texte redfile");

   /* First fire off the initial AFM. */
   fire_afm ("_beg_entertext", nullchix, newconf);
   
   chxcpy (editor, thisuser->editor);
   length = (truncate ? 78 : 200);

   /*** If Fullscreen is OFF, methods 2 & 3 become regular line entry. */
   method = thisuser->texten;
   if (method > 1  &&  NOT flag.fullscreen)       method = 0;

   /*** If all input is coming from a file ("-i" command line option),
   /    methods 2 & 3 become regular line entry. */
   if (method > 1  &&  input_control (IC_ISUNIT | IC_DATA, nullchix) != XKEY)
       method = 0;

   /*** The user has several choices about the METHOD of entering text:
   /      0 (terminal) => just type in a line at a time at the terminal.
   /      1 (edit)     => use the default text editor.
   /      2 (wordwrap) => use automatic word-wrap entry. (not vm/cms)
   /      3 (window)   => use text-entry window. (vm/cms only)  */


   if      (method==1) editfile  (file, thisuser->editor, truncate);

   else if (method==2) {
      if (NOT wordwrap  (file, screen.width-5))  FAIL;
   }
   else if (method==3) {
      if (NOT sysvmedit (file, context))         FAIL;
   }

   else {

      if (NOT unit_append (file))  FAIL;
   
      while (1) {
   
         /*** Get the next line (or command) from the user. */
         code = textinput (str, length);
         if (code == -1)  { unit_close (file);   FAIL; }
         if (code ==  0)    break;
         redirect = (chxindex (str, L(0), ss_inred) == 0);
   
         /*** If the user wants to edit... */
         if (iseditreq (str, nullchix, thisuser->editor, editor)) {
            unit_close (file);
            editfile   (file, editor, truncate);
            if (NOT unit_append (file) )  FAIL;
            mdwrite (XWTX, "ced_Tcontype", null.md);
         }
   
         /*** Handle legal input redirection. */
         else if (redirect) {
            if (syspluck (str, L(0), redfile))  copy_redirect (file, redfile);
         }
   
         else  {
            unit_write (file, str);
            unit_write (file, CQ("\n"));
         }
            
      }
      unit_close (file);
   }

   /*** Come here to examine the text for reformatting/truncation,
   /    regardless of how the text was entered. */
   while (1) {
      switch (textform (file, truncate, editor)) {
         case (-1):  fire_afm ("_end_entertext", nullchix, newconf); FAIL;
         case ( 0):  editfile (file, editor, truncate);   break;
         case ( 1):  fire_afm ("_end_entertext", nullchix, newconf); SUCCEED;
      }
   }

done:

   chxfree (str);
   chxfree (editor);
   chxfree (newconf);
   chxfree (redfile);
   if (success)  success = unit_check (file);
   RETURN  (success);
}
