/*** TEXTREFORM.  Reformat text.
/
/    TEXTREFORM reads lines of text from the unit IN, reformats them,
/    and writes them out to the unit OT.
/
/    Reformatting means packing the lines together to fill each line
/    out to 76 characters, or as close as possible without breaking
/    a word in two.  A line read from IN that starts with whitespace
/    starts a new line.  A blank line is echoed as is, and the next
/    line from IN starts a new line. */

/*: AA  7/29/88 17:32 Source code master starting point */
/*: AM  2/07/89 10:24 Jcaucus changes*/
/*: JV  8/29/91 14:49 Lintify. */
/*: JX  3/28/92 14:00 Chixify. */
/*: JX  5/16/92 17:05 Fixify. */
/*: JV 10/16/92 16:05 Fix chxcatsub bug, free buf & temp, static blanks. */
/*: CL 12/11/92 11:12 Long Chxcatsub args. */
/*: CL 12/11/92 13:46 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

#define  FLUSH(b)  if (chxvalue (b, L(0)))  \
            {unit_write (ot, b);  unit_write (ot, CQ("\n")); chxclear (b); }

FUNCTION  textreform (in, ot, str)
   int    in, ot;
   Chix   str;
{
   Chix   buf, temp;
   int    i;

   ENTRY ("textreform", "");

   buf  = chxalloc (L(300), THIN, "textr buf");
   temp = chxalloc (L(100), THIN, "textr temp");

   /*** Read a line at a time from the input file, reformat and write. */
   while (unit_read (in, str, L(0))) {
      chxtrim (str);
      sysclean(str);
      fixtabs (str);

      /*** A blank line flushes BUF and writes a blank line.
      /    A line starting with whitespace flushes BUF and starts a new line.
      /    Anything else is cat'ed on to the end of BUF. */
      if      (EMPTYCHX   (str))     { FLUSH (buf); unit_write (ot, CQ("\n")); }
      else if (whitespace (chxvalue (str, L(0))) ) {
              FLUSH (buf); chxcpy (buf,  str); }
      else {
         if (NOT EMPTYCHX (buf))  chxcat (buf, CQ(" "));
         chxcat (buf, str);
      }

      /*** While BUF is longer than 75 characters, chop off one line's
      /    length from the front and write it out.  Do the chopping in
      /    the middle of whitespace. */
      while (chxlen (buf) > 75) {
         for (i =74;  i>35;   --i)   if (whitespace (chxvalue (buf, L(i)))) break;
         if  (i==35)  i = 74;
         ++i;
         chxcatsub (temp, buf, L(0), L(i));
         unit_write (ot, temp);
         unit_write (ot, CQ("\n"));
         chxclear  (temp);
         chxcatsub (temp, buf, L(i), ALLCHARS);
         chxcpy    (buf, temp);
         chxclear  (temp);
        
      }
   }
   FLUSH (buf);
   chxfree (buf);
   chxfree (temp);
   RETURN (1);
}

FUNCTION whitespace(c)
   int4 c;
{
   Chix blanks;
   int  j;
   int4 b;

   blanks = chxalloc (L(20), THIN, "whitespace blanks");
   chxspecial (SC_GET, SC_BLANK, blanks);
   for (j=0; (b = chxvalue (blanks, L(j))) != L(0);   ++j) {
      if (c == b)   { chxfree (blanks); return (1); }
   }
   chxfree (blanks);
   return (0);
}
