/*** CHXABR.  Check if one chix is an abbreviation of another.
/
/    i = chxabr (a, b);
/
/    Parameters:
/       int   i;
/       Chix  a, b;
/
/    Purpose:
/       Check if one chix is an abbreviation of another.
/       An empty Chix is not a valid abbreviation.
/
/    How it works:
/       Returns 1 if one of the chixen is an abbreviation of the other.
/
/    Returns: 1 if A is an abbreviation of B
/             2 if B is an abbreviation of A
/             0 if neither is an abbrevation of the other,
/             or if either is empty or not a Chix.
/
/    Error Conditions:  invalid chix (also calls chxerror to handle errors)
/  
/    Side effects:      none
/
/    Related functions:
/
/    Called by:  Application.
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  chx/chxabr.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: JX  6/04/92 15:36 New function. */
/*: CL 12/10/92 15:22 Long chxindex args. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  chxabr (a, b)
   Chix   a, b;
{
   ENTRY ("chxabr", "");

   if (NOT chxcheck (a, "chxtype"))  RETURN (0);
   if (NOT chxcheck (b, "chxtype"))  RETURN (0);
   if (EMPTYCHX (a) || EMPTYCHX (b)) RETURN (0);

   if (chxindex (a, L(0), b) == 0) RETURN (2);
   if (chxindex (b, L(0), a) == 0) RETURN (1);
   
   RETURN (0);
}

