/*** CHX_INIT.  Set up internal text structures, types, and methods.
/
/    chx_init (type)
/
/  Parameters: 
/       int type;     // Type of character set we're using.
/
/  Purpose:
/     CHX_init defines and sets up various data structures that the
/     chx functions need.  These should be things that need to exist
/     before *any* strings are processed.
/
/  How it works:
/     TYPE must be one of:  (defined in TEXT.H).
/        0: ASCII
/        1: JIX
/        2: ISO8859
/
/  Returns: 1.
/
/  Error Conditions: none.
/
/  Side Effects:
/     Sets default collating order.
/     Defines which characters are "blanks" (word separators).
/
/  Related to:
/
/  Called by:  main functions.
/
/  Home: chx/chxinit.c
*/

/*: JX 07/13/92 10:47 Create function. */
/*: JX 10/09/92 17:42 Fixify. */
/*: CR 12/11/92 18:22 Replace ALLCHARS with LASTVALUE. */
/*: CR 12/18/92 13:47 Make LASTVALUE larger than any 16 bit value. */
/*: JV  9/08/93 10:22 Add ENTRY. */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "text.h"

#define  LASTVALUE  L(100000)

FUNCTION chx_init (type)
int type;
{
   Chix blanks;
   char *bugtell();

   ENTRY ("chx_init", "");

   blanks = chxalloc (L(8), THIN, "chxinit blanks");

   /*** Set initial collation rules before any arguments. */
   chxcollate (CLEAR_CT, L(0), L(0), L(0), L(0));
   chxcollate (ADDRULE_CT, L('!'), L('!'), L(0), L(0));
   chxcollate (ADDRULE_CT, L('~'), L('~'), LASTVALUE, L(0));

   /*** Define word separators. */
   switch (type) {
      case JIX:
         chxcatval  (blanks, THIN, L(32));
         chxcatval  (blanks, WIDE, L(41377));    /* Kanji DW space. */
         chxcatval  (blanks, THIN, L(10));
         chxcatval  (blanks, THIN, L( 9));
         chxspecial (SC_SET, SC_BLANK, blanks);
         chxfree    (blanks);
         break;
      case ASCII:
      case ISO8859:
         chxcatval  (blanks, THIN, L(32));
         chxcatval  (blanks, THIN, L(10));
         chxcatval  (blanks, THIN, L( 9));
         chxspecial (SC_SET, SC_BLANK, blanks);
         break;
      default:
         buglist (bugtell ("CHX_INIT:", 0, L(0), "Bad language type.\n"));
         break;
   }

   RETURN (1);
}
