/*** CHXNEXTWORD.  Get "next" word out of text.
/
/    where = chxnextword (word, text, pos);
/
/    Parameters:
/       int4  where;  (starting position of found word, -1 if none)
/       Chix  word;   (put next word here)
/       Chix  text;   (text being scanned)
/       int4 *pos;    (search for next word from here; updated)
/
/    Purpose:
/       Extracts the "next" word (blank, tab, or newline-separated token)
/       from chix TEXT. 
/
/    How it works:
/       Scans from *pos, and updates *pos as the word is passed over, so
/       that chxnextword() can be used in a loop.
/
/    Returns: -1 on failure
/             starting position of this word
/
/    Related functions: chxspecial()
/
/    Called by:  application
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  chx/chxnextw.c
/ */

/*** Copyright (C) 1996 Camber-Roth.  All rights reserved. */

/*: CX  1/18/96 23:36 New function. */

#include <stdio.h>
#include "chixdef.h"
#include "done.h"

FUNCTION  int4 chxnextword (word, text, pos)
   Chix   word, text;
   int4  *pos;
{
   static int4 *blanklist = NULL;
   int          j, letter, text_type, space;
   int4         ptr, start_pos, c;
   char        *sysmem();

   ENTRY ("chxnextw", "");

   /*** Input validation. */
   if (NOT chxcheck (word, "chxnextw"))  RETURN (-1);
   if (NOT chxcheck (text, "chxnextw"))  RETURN (-1);

   /*** On first call, get the list of "blank" characters.  If it's 
   /    empty, use the standard ascii blank, ' '. */
   if (blanklist == NULL) {
      chxspecial (SC_GET, SC_BLANK, word);
      if (chxvalue (word, L(0)) == L(0))  chxcatval (word, THIN, L(' '));
      chxcatval (word, THIN, L('\n'));
      chxcatval (word, THIN, L('\t'));
      blanklist = (int4 *) sysmem (sizeof(int4) * (chxlen (word) + 2));
      for (j=0;   (blanklist[j] = chxvalue (word, L(j))) != L(0);   ++j)  ;
   }


   /*** Starting at POS, skip any "spaces" in TEXT... */
   for (ptr=*pos;   (c = chxvalue (text, ptr)) != L(0);   ++ptr) {
      for (space=j=0;   blanklist[j];   ++j) {
         if (c == blanklist[j])  { space = 1;   break; }
      }
      if (NOT space)  break;
   }

  *pos = start_pos = ptr;
   if (c == L(0))  RETURN (-1);

   /*** The word starts with this character... */
   text_type = chxtype (text);
   chxclear  (word);
   chxcatval (word, text_type, c);

   /*** And keep accumulating until we get to a space (or end of string). */
   for (++ptr;   (c = chxvalue (text, ptr)) != L(0);   ++ptr) {

      for (space=j=0;   blanklist[j];   ++j) {
         if (c == blanklist[j])  { space = 1;   break; }
      }
      if (space)  break;
      chxcatval (word, text_type, c);
   }

  *pos = ptr;
   RETURN (start_pos);
}
