/*** CHXREVDEX.  Reverse search: find chix Y in chix X, starting at POS.
/
/    n = chxrevdex (x, pos, y);
/
/    Parameters:
/       int4  n;     (character position in X where Y starts)
/       Chix  x;     (search in this chix...)
/       int4  pos;   (...starting at this position...)
/       Chix  y;     (...for this chix.)
/
/    Purpose:
/       Search *backwards* through chix X, starting at position POS, for
/       the "first" (last?) occurence of chix Y.  Return the number of 
/       the character position in X where Y starts.  (Note that, as usual,
/       Y may be found in X, even if they have different types, so int4 
/       as their corresponding characters match after promotion.)
/
/    How it works:
/       If POS > length of X, search starting from the end of X.
/       Starting at POS, for each character in X, see if that character begins
/       a match with Y.  Call chxscan() to check for the match.
/
/    Returns: N, number of character position in X where Y starts.
/       The first character of X is number 0.
/
/       Returns -1 if:  Y is not found in X,
/                       Y or X are empty.
/                       Y or X are not chix.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  chx/chxrevdex.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CX  5/26/92 13:41 New function. */
/*: CR 12/11/92 13:26 POS, return value now int4. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  int4 chxrevdex (x, pos, y)
   Chix   x, y;
   int4   pos;
{
   int4   x_end, y_end;
   int4   i;
   char  *p;

   ENTRY ("chxrevdex", "");

   /*** Input validation. */
   if (NOT chxcheck (x, "chxrevdex"))   RETURN (-1);
   if (NOT chxcheck (y, "chxrevdex"))   RETURN (-1);
   if (chxvalue (y, L(0)) == L(0))          RETURN (-1);
   if (x->actlen < 0)  chxlen (x);
   if (pos == ALLCHARS  ||
       pos >= x->actlen)                pos = x->actlen - 1;

   /*** Starting at POS and working backwards, examine each character in X.
   /    Does it start a match with Y? */
   for (i=pos, p = x->p + pos * x->type;    i>=0;    --i, p -= x->type) {

      chxscan (p, x->type, y->p, y->type, &x_end, &y_end);
      if (y_end == L(0))  RETURN (i);
   }

   RETURN (-1);
}
