
/*** CHXVALUE.  Return value of a selected character in a chix.
/
/    v = chxvalue (x, n);
/
/    Parameters:
/       int4  v;   (value of N'th character of X)
/       Chix  x;   
/       int4  n;
/
/    Purpose:
/       Return the value of the N'th character of chix X as a int4 int.
/       The first character of a chix is character number 0.
/
/       Frequently used to determine if a chix is "empty", i.e. if
/       the first character contains all 0 bytes.
/
/    How it works:
/
/    Returns: value of N'th character, or -1 on error.
/   
/    Error Conditions:  X not a chix.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application and some chx...() functions.
/
/    Operating system dependencies: none
/
/    Known bugs:
/       Limitation: requires that a character in a chix be no more than 
/       sizeof(int4) bytes, typically 4.
/
/    Home:  chx/chxvalue.c
/ */

/*** Copyright (C) 1990 Camber-Roth.  All rights reserved. */

/*: CX 12/10/90 23:36 New function. */
/*: CR 12/11/92 13:33 Make last arg int4. */
/*: CR 12/18/92 13:59 Speed up function by testing x->actlen. */

#include <stdio.h>
#include "chixdef.h"

FUNCTION  int4 chxvalue (x, n)
   Chix   x;
   int4   n;
{
   int4   v;
   int    i;
   char  *p;

   ENTRY ("chxvalue", "");

   /*** Input validation. */
   if (NOT chxcheck (x, "chxvalue"))  RETURN (-1);
   if (0 >  x->actlen)                chxlen (x);
   if (n >= x->actlen)                RETURN ( L(0));
   
   p = x->p + x->type * n;
   v = *p++ & 0xFF;
   for (i=1;   i<x->type;   ++i)  { v = v << 8;   v += (*p++ & 0xFF); }

   RETURN (v);
}
