/*** ADDITEM.  Handle the user's creation of a new item. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: JV  9/02/88 14:14 Cleaned up locking and unlocking XITX.*/
/*: AM  2/07/89 10:26 Jcaucus changes*/
/*: CR  4/28/89 12:59 Check return from createitem(). */
/*: CW  5/17/89 16:21 Use XW unitcodes in place of XITT.*/
/*: CR  9/05/89 21:44 Add code to handle MYTEXT settings. */
/*: CR  9/15/89 16:58 Check thisuser->read_only. */
/*: CR  3/04/90 22:44 Add context argument to textenter() call. */
/*: CR  7/14/90 11:38 Use thisuser->is_organizer. */
/*: CR  8/12/90 20:21 Cast textenter arg to make Lint happy. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/09/91 13:51 Add new confnum argument to updatetext() call. */
/*: CR  6/24/91 17:43 Use new chg_ditem() function to do real work. */
/*: CR  7/02/91 17:38 Define strtoken(). */
/*: CR  7/08/91 12:16 Add 3rd arg to chixfile(). */
/*: CR  7/28/91 15:48 chxalloc() first argument is int4. */
/*: CR  7/30/91 13:51 Replace license check with is_distributed(). */
/*: CX 11/11/91 22:54 Chixify. */
/*: JV 10/31/91 14:41 Add dest_chn arg to chgxitem() call. */
/*: CR 11/15/91 13:16 Check return from chg_ditem(). */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: CR  1/03/92 14:26 Check error return from chixfile(). */
/*: JX  5/19/92 21:22 Fix unit_lock call. */
/*: CX  6/05/92 16:06 XKEY --> u_keybd. */
/*: JX  8/03/92 11:36 CHN's are chix. */
/*: CI 10/06/92 12:52 Chix Integration. */
/*: TP 10/19/92 15:21 Remove u_keybd. */
/*: CR 10/22/92 19:02 Only display res_Tenitem if input is session. */
/*: CR  1/13/93 12:48 Add chxwrap() after chixfile(). */
/*: JV  4/05/93 17:12 Add attach arg to chg_ditem(). */
/*: CP  8/10/93  0:03 Replace arg 'in' with 'red'. */
/*: CK 10/13/93 17:59 Check # of items against license.itemmax. */
/*: CR  5/06/95 16:43 Use limit_10k to limit # lines of text. */
/*: CR  6/16/95 16:12 Add text properties. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "xaction.h"
#include "redirect.h"

extern struct master_template  master;
extern struct license_t        license;
extern Userreg                 thisuser;
extern struct partic_t         thisconf;
extern union  null_t           null;
extern short                   confnum;
extern Chix                    confname;

FUNCTION  additem (red)
   Redirect *red;
{
   Chix   title, newstr, chn, date, time, ctext, format;
   int    newitem, error1, error2, success, u_data, u_sess, in, ok;
   int4   systime();

   ENTRY ("additem", "");

   apply_redirect (red);

   if (thisuser->read_only) {
      mdwrite (XWER, "ent_Ereadonly", null.md);
      RETURN  (0);
   }

   if (NOT thisuser->is_organizer  &&  master.noenter) {
      mdwrite (XWER, "ent_Tnoent", null.md);
      RETURN  (0);
   }

   if (master.items >= MAXITEMS - 5) {
      mdwrite (XWER, "ent_Tfull", null.md);
      RETURN  (0);
   }

   if (PLAIN(license.itemmax) > 0  &&  master.items >= PLAIN(license.itemmax)) {
      mdwrite (XWER, "lic_Eitemmax", null.md);
      RETURN  (0);
   }

   /*** If we're reading the text of the item from the terminal, 
   /    create an empty text file, and enter the item text into it. */
   in = red->unit;
   if (in == XKEY) {
      in = XITX;
      unit_lock  (XITX, WRITE, L(0), L(0), L(0), L(0), nullchix);
      unit_kill  (XITX);
      u_data = input_control (IC_ISUNIT | IC_DATA, nullchix);
      u_sess = input_control (IC_ISUNIT | IC_SESS, nullchix);
      if (u_data == u_sess) {
         unit_write (XWTX, NEWCPAGE);
         mdwrite (XWTX, "res_Tenitem", null.md);
      }
      ok = textenter (XITX, (int) thisuser->truncate, 0, 0, 0);
      unit_unlk (XITX);
      if (NOT ok) {
         unit_write (XWTX, NEWCPAGE);
         mdwrite    (XWTX, "ent_Tencancel", null.md);
         RETURN     (0);
      }
   }

   title   = chxalloc (L(90), THIN, "additem titlestr");
   date    = chxalloc (L(90), THIN, "additem date");
   time    = chxalloc (L(90), THIN, "additem time");
   chn     = chxalloc (L(90), THIN, "additem chn");
   newstr  = chxalloc (L(90), THIN, "additem newstr");
   format  = chxalloc (L(90), THIN, "additem format");
   success = 0;

   /*** Get the title (last chance to cancel item entry). */
   if (NOT entertitle (title))  FAIL;

   /*** Load the text of the item into the chix CTEXT. */
   ctext = chxalloc (L(500), THIN, "additem ctext");
   if (in == XITX)  unit_lock (XITX, WRITE, L(0), L(0), L(0), L(0), nullchix);
   chixfile (in, ctext, "");
   chxwrap  (ctext, 77);
   if (limit_10k (ctext))  mdwrite (XWER, "ced_Elines", null.md);
   if (in == XITX)  unit_unlk (XITX);

   /*** Get the date & time, and actually add the item. */
   sysdaytime (date, time, 0, systime());
   newitem = -1;
   if (NOT chg_ditem (XT_NEW, confnum, -1, &newitem,
                 nullchix, thisuser->name, thisuser->id, ctext,
                 title, date, time, (Attachment) NULL, 0, nullchix, &error1)) {
      mdwrite (XWTX, "ent_Tencancel", null.md);
      chxfree (ctext);
      FAIL;
   }
   logfeature (CQ("itemadd"), confname, L(newitem), L(0), L(1), L(chxlen(ctext)));

   /*** If distributed, write the add-item transaction. */
   if (is_distributed (confnum)) {
      getmyhost (chn);
      chgxitem  (XT_NEW, confnum, newitem, chn, thisuser->name, thisuser->id,
                         title, ctext, date, time, nullchix, 0, &error2);
   }
   chxfree (ctext);

   /*** Add this item to the list of Uncategorized Items. */
   chxformat (newstr, CQ("%d"), (int4) newitem, L(0), nullchix, nullchix);
   subuncat  (0, newstr, -1);

   /*** Tell the user we created this item. */
   mdstr      (format, "ent_Fwasenter", null.md);
   chxclear   (newstr);
   chxformat  (newstr, format, (int4) newitem, L(0), nullchix, nullchix);
   unit_write (XWTX, NEWCPAGE);
   unit_write (XWTX, newstr);

   /*** If the user does not want the new item to appear as NEW,
   /    mark it as SEEN in the user's participation record.  This
   /    may mean some new items get marked UNSEEN--if so, warn
   /    the user about this fact. */
   if (thisuser->mytext == NEVER) {
      if (newitem > thisconf.items + 1) {
         mdwrite (XWTX, "res_Tinever", null.md);
         chxclear  (newstr);
         chxformat (newstr, CQ("%6d"), (int4) thisconf.items+1, L(0),
                                              nullchix, nullchix);
         if (newitem > thisconf.items + 2)
            chxformat (newstr, CQ("-%d\n"), (int4) newitem-1, L(0),
                                              nullchix, nullchix);
         unit_write (XWTX, newstr);
      }
      thisconf.items = newitem;
      thisconf.responses[newitem] = 0;
      sysuserid  (newstr);
      unit_lock  (XUPA, WRITE, L(confnum), L(0), L(0), L(0), newstr);
      store_user (&thisconf);
      unit_unlk  (XUPA);
   }
   SUCCEED;

done:
   chxfree (title);
   chxfree (date);
   chxfree (time);
   chxfree (chn);
   chxfree (newstr);
   chxfree (format);
   RETURN  (success);
}
