
/*** ADDSUBJ.   Make a new subject. 
/
/    ADDSUBJ gets the name of the subject from IN (if not the terminal),
/    or plucks it off the COMMAND line (if there), or prompts the user.
/
/    MINE means a MYSUBJECT, otherwise a regular SUBJECT. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: CW  5/17/89 16:21 Use XW unitcodes in place of XITT.*/
/*: CR  7/14/90 11:38 Use thisuser->is_organizer. */
/*: CR  7/17/91 17:52 Change to use chg_dsubject(). */
/*: CR  7/23/91 23:56 Add addxosubj() call. */
/*: CR  7/30/91 13:51 Replace license check with is_distributed(). */
/*: CX 11/12/91 14:11 Chixify. */
/*: JV 10/31/91 15:00 Add dest_chn arg to addxosubject() call. */
/*: CR 11/07/91 15:13 Add confname arg to chg_dsubject() call. */
/*: JV 11/13/91 22:01 Add confname arg to *3rd* chg_dsubject() call. */
/*: JX  2/25/92 11:30 Fix declaration of confname. */
/*: JX  5/26/92 19:02 Fixify. */
/*: CX  6/05/92 16:20 XKEY --> u_keybd. */
/*: JX  8/03/92 11:37 CHN's are chix. */
/*: CR 10/11/92 21:54 Remove CQ from (md)prompter() call. */
/*: TP 10/19/92 15:24 Remove u_keybd. */
/*: CP  8/10/93  0:15 Replace arg 'in' with 'red'. */
/*: CK 10/11/93 15:45 Add addsubj_complain(). */
/*: CR  7/05/94 16:45 Add handling for DSUBPUNC. */

#include <stdio.h>
#include "caucus.h"
#include "xaction.h"
#include "redirect.h"
#include "derr.h"

extern struct master_template  master;
extern Userreg                 thisuser;
extern union  null_t           null;
extern short                   confnum;
extern Chix                    confname;
extern Chix                    ss_dotnl;

FUNCTION  addsubj (red, command, mine)
   Redirect *red;
   int       mine;
   Chix      command;
{
   Chix   name, owner;
   int    error1, error2, success;

   ENTRY ("addsubj", "");

   owner = chxalloc (L(20), THIN, "addsubj owner");
   name = chxalloc (L(20), THIN, "addsubj name");

   apply_redirect (red);

   if (mine)  sysuserid (owner);

   if (NOT mine  &&  NOT thisuser->is_organizer) {
      mdwrite (XWER, "cus_Tonlyorg", null.md);
      FAIL;
   }

   /*** If ADD SUBJECT <FILE, read each line from FILE and make each
   /    one a subject. */
   if (red->unit != XKEY) {
      if (NOT unit_view (red->unit))  FAIL;
      unit_write (XWTX, NEWCPAGE);
      while (unit_read (red->unit, name, L(0))) {
         chxstrip   (name);
         mdwrite    (XWTX, "add_Tmaksub", null.md);
         unit_write (XWTX, name);
         unit_write (XWTX, ss_dotnl);
         if (chg_dsubject (XT_NEW, confnum, confname, owner, name,
                            nullchix, &error1)) {
            if (EMPTYCHX(owner)  &&  is_distributed (confnum))
               addxosubject (XT_NEW, confnum, name, nullchix, nullchix,
                              &error2);
         }
         else addsubj_complain (name, error1);
      }
      unit_close (red->unit);
      
      SUCCEED;
   }


   /*** If ADD SUBJECT, prompt user for NAME of subject to create. */
   if (chxtoken (null.chx, name, 3, command) < 0) {
      if (mdprompter ("add_Pwhatsub", null.chx, 0, "add_Hwhatsub", 
                       80, name, null.chx) < -1)  FAIL;
      chxstrip  (name);
      if (chg_dsubject (XT_NEW, confnum, confname, owner, name, 
                        nullchix, &error1)) {
         if (EMPTYCHX(owner)  &&  is_distributed (confnum))
            addxosubject (XT_NEW, confnum, name, nullchix, nullchix, &error2);
      }
      else addsubj_complain (name, error1);
      SUCCEED;
   }


   /*** If ADD SUBJECT NAME, use NAME directly as the name of the subject. */
   chxstrip  (name);
   if (chg_dsubject (XT_NEW, confnum, confname, owner, name, 
                     nullchix, &error1)) {
      if (EMPTYCHX(owner)  &&  is_distributed (confnum))
         addxosubject (XT_NEW, confnum, name, nullchix, nullchix, &error2);
   }
   else addsubj_complain (name, error1);
   SUCCEED;

done:
   chxfree (name);
   chxfree (owner);
   RETURN  (success);
}


FUNCTION  addsubj_complain (name, error)
   Chix   name;
   int    error;
{
   Chix  format, temp;

   if (error == DSUBPUNC) {
      mdwrite (XWER, "add_Fsubpunc", null.md);
      return (1);
   }

   if (error!=DOEXIST  &&  error!=DSUBSET)  return (1);

   format = chxalloc (L(80), THIN, "addsub_c format");
   temp   = chxalloc (L(80), THIN, "addsub_c temp");

   mdstr (format, "add_Fsubconflict", null.md);
   chxformat  (temp, format, L(0), L(0), name, nullchix);
   unit_write (XWER, temp);

   chxfree (temp);
   chxfree (format);
   return (1);
}
