/*** DESCSUBJ.   Describe a subject.
/
/    DESCSUBJ is used to describe a SUBJECT or PSUBJECT named by
/    the user on the command line.  If no subject is named, all subjects
/    (or psubjects) are listed.
/
/    MINE means PSUBJECT if true, and SUBJECT otherwise. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: AM  2/07/89 10:26 Jcaucus changes*/
/*: JV  8/23/89 13:26 Chg'd header comment, added err mess on no match. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR  3/29/90 21:57 Check for failure return from matchnames(). */
/*: CR  3/29/91 17:27 Use AND/OR code for matchnames() call. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: JV  5/15/91 17:08 Add new loadnpart() arg. */
/*: CR  8/02/91 14:00 Add punctuation argument to matchnames() call. */
/*: CR 10/29/91 11:45 Add confnum arg to chixGetline() call. */
/*: CR 11/07/91 14:18 Add confnum arg to matchnames() call. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  5/13/92 12:03 Chixify, Xlate PFILEBUF, text, line to Textbuf. */
/*: CR 10/06/92 15:28 Put leading space in LIST SUBJ output. */
/*: CR 10/11/92 21:56 Remove CQ from nlnames() call. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CR  2/08/93 14:36 chxfree cnf! */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */
/*: CP  8/10/93 13:37 Use new Redirect object in handling redirection. */

#include <stdio.h>
#include "caucus.h"
#include "redirect.h"

extern union  null_t        null;
extern Chix                 confname;
extern Chix                 ss_all, ss_dotnl;
extern short                confnum;

FUNCTION  descsubj (red, command, mine)
   Redirect *red;
   int       mine;
   Chix      command;
{
   struct   namlist_t *namdir, *this, *names, *matches, *nlnode(), *nlnames();
   int      success;
   int4     subnum;
   short    none, i, ok;
   short    uad, uaf, usd, usf, utd, utf;
   Chix     owner, format, cnf, mtu, str, s, line, chixGetline();
   Textbuf  tbuf, make_textbuf();

   ENTRY ("descsubj", "");

   s      = chxalloc (L(80), THIN, "descusbj s");
   cnf    = chxalloc (L(80), THIN, "descusbj cnf");
   mtu    = chxalloc (L(80), THIN, "descusbj mtu");
   str    = chxalloc (L(100), THIN, "descusbj str");
   owner  = chxalloc (L(40), THIN, "descsubj owner");
   format = chxalloc (L(40), THIN, "descsubj format");

   sysuserid (owner);
   apply_redirect (red);

   if (mine) {
      uad = XUOD;   uaf = XUON;          usd = XUND;   usf = XUNN;
      utd = XUSD;   utf = XUSF;
   }
   else {
      uad = XCOD;   uaf = XCON;          usd = XCND;   usf = XCNN;
      utd = XCSD;   utf = XCSF;
   }
   none = 1;

   /*** Get the name of the subject from the command line, or else
   /    prompt the user for it. */
   if (chxtoken (str, s, 3, command) != -1)  chxcpy (str, s);
   else if (mdprompter ("des_Pwsub", nullchix, 0, "des_Hwsub", 80, str, ss_all)
                < -1)  FAIL;
   chxstrip    (str);
   jixreduce   (str);
   chxsimplify (str);

   /*** If ALL was selected, display all subjects... */
   if (chxeq (str, ss_all)) {

      /*** Load the alphabetical subject list namepart directory. */
      if (NOT unit_lock (uad, READ, L(confnum), L(0), L(0), L(0), owner))  FAIL;
      namdir = nlnode(4);
      ok = loadnmdir (uad, namdir);
      unit_unlk (uad);
      if (NOT ok)  { nlfree (namdir);  FAIL; }
      tbuf = make_textbuf (TB_RESPS, TB_LINES, "descsubj tbuf");

      /*** Load each namepart file in turn, and display contents. */
      for (this=namdir->next;   this != null.nl;   this = this->next) {
         if (this->num <= 0)  continue;
         if (NOT unit_lock (uaf, READ, L(confnum), L(this->num), L(0), L(0), owner))
          break;
         if (NOT loadnpart (uaf, this->num, tbuf))
                                                  { unit_unlk (uaf);  break; }
         unit_unlk (uaf);
   
         for (i=0;   i<tbuf->l_used;   ++i) {
            chxtoken (str, nullchix, 1, tbuf->line[i]);
            while (chxalter (str, L(0), chxquick("#", 0), chxquick(" ", 1))) ;
            chxtoken (cnf, nullchix, 1, str);
            if (NOT chxeq (cnf, confname))  continue;
            chxtoken (cnf, s, 2, str);
            unit_write (red->unit, CQ(" "));
            unit_write (red->unit, s);
            unit_write (red->unit, CQ("\n"));
         }
      }
      nlfree (namdir);
      free_textbuf (tbuf);
      none = 0;
   }

   /*** User supplied some name(s) of subject(s) to be described. */
   else {
      names = nlnames (str, null.chx);
      ok = matchnames (usd, usf, confnum, owner, names, AND, &matches,
          NOPUNCT);
      nlfree (names);
      if (NOT ok)  goto done;
      if (nlsize (matches) == 0) {
         if (mine) mdstr (format, "des_Enopsbjmch", null.md);
         else      mdstr (format, "des_Enosbjmch",  null.md);
         chxformat (mtu, format, L(0), L(0), str, nullchix);
         chxconc (mtu, mtu, ss_dotnl);
         unit_write (red->unit, mtu);
         none = 0;
         goto done;
      }
   
      /*** For each matching subject, display the full name. */
      for (this=matches->next;   this!=null.nl;   this=this->next) {
         /*** Find the subject number. */
         chxtoken (str, nullchix, 2, this->str);
         chxnum   (str, &subnum);

         /*** See if that subject is in this conference. */
         if ( (line = chixGetline (utd, utf, confnum, owner, 0, (int) subnum, 0,
                            (Textbuf) NULL)) == nullchix) continue;
         chxtoken (cnf, nullchix, 2, line);
         if (NOT chxeq (cnf, confname))  continue;

         /*** Display the full name of the subject. */
         unit_write (red->unit, CQ(" "));
         unit_write (red->unit, chixGetline (utd, utf, confnum, owner, 0, (int) subnum,
                                   1, (Textbuf) NULL));
         unit_write (red->unit, CQ("\n"));
      }
      nlfree (matches);
      none = 0;
   }
   success= none==0;
   
done:
   if (none)  mdwrite (XWER, "des_Enosub", null.md);
   chxfree (s);
   chxfree (mtu);
   chxfree (str);
   chxfree (owner);
   chxfree (format);
   chxfree (cnf);
 
   RETURN (success);
}
