/*** ITEMPERSON.  Display info from SHOW ITEMS range PERSON name.
/
/    Display, on unit OUT, how much person PERSNAME has seen
/    of item I.  */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: CR 10/07/89 14:34 Pull code out of showitem() and make new function. */
/*: CR 10/10/89 12:24 Re-use THISCONF, and reload it at the end. */
/*: CR  6/26/90 10:38 Display responses/name/userid header properly. */
/*: CR  2/20/91  1:36 Change showitperson() to use person_format().  */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/25/91  0:45 Remove unnecessary decl's. */
/*: CR  5/28/91 18:40 XUPA must be locked with confnum as arg. */
/*: JV  5/15/91 17:10 Add new loadnpart arg. */
/*: CR  7/16/91 17:08 Add HOST argument to person_format() calls. */
/*: CR  7/30/91 18:14 Add 3rd arg to loadreg(). */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  5/13/92 12:19 Xlate PFILEBUF,line,text->Textbuf; userreg_t->Userreg. */
/*: CR  2/08/93 14:40 chxfree format in 2nd function. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"

extern Userreg            thisuser;
extern struct partic_t    thisconf;
extern union  null_t      null;
extern Chix               ss_all, ss_dotnl;
extern short              confnum;

FUNCTION  item_person (out, persname, i)
   int    out, i;
   Chix   persname;
{
   struct   namlist_t *wholist, *this, *namdir, *nlnode(), *nluserid();
   short    ok, j;
   short    reload;         /* Need to reload this user's THISCONF? */
   int      found, success;
   Chix     id, mtu, format;
   Textbuf  tbuf, make_textbuf();

   ENTRY ("item_person", "");

   id  = chxalloc (L(40), THIN, "itempers id");
   mtu = chxalloc (L(200), THIN, "itempers mtu");
   format = chxalloc (L(40), THIN, "itempers format");

   /*** If SHOW ITEMS PERSONS, list the people that have seen this 
   /    item. Precede each name with the number of the highest 
   /    numbered response they have seen. '0' means they just read
   /    the item text. 'F' means that they FORGOT this item.
   /    Print out each name and id. */

   /*** If not the exact word ALL, use PERSNAME as name of user and show
   /    the persons that match it. */
   found = reload = 0;
   if (NOT chxeq (ss_all, persname)) {
      wholist = nluserid (persname, 0);
      sysbrkclear();
      for (this=wholist->next;   this!=null.nl;   this = this->next) {
         if (sysbrktest())      SUCCEED;
         if (showitperson (out, this->str, i, &found))  reload = 1;
      } 

      if (NOT reload) {
         mdstr   (format, "sho_Enobody", null.md);
         chxformat (mtu, format, L(0), L(0), persname, nullchix);
         chxconc (mtu, mtu, ss_dotnl);
         unit_write (XWER, mtu);
      }
      nlfree (wholist);
   }

   /*** Otherwise (ALL case), list everyone in this conference. */
   else {
      if (NOT unit_lock (XCMD, READ, L(confnum), L(0), L(0), L(0), nullchix))  FAIL;
      namdir = nlnode(4);
      ok = loadnmdir (XCMD, namdir);
      unit_unlk (XCMD);
      if (NOT ok)     { nlfree (namdir);            FAIL; }
   
      /*** Load each namepart file in turn, and display name & id. */
      sysbrkclear();
      tbuf = make_textbuf (TB_RESPS, TB_LINES, "itempers tbuf");
      for (this=namdir->next;   this != null.nl;   this = this->next) {
         if (sysbrktest())                                         break;
         if (this->num <= 0)                                       continue;
         if (NOT unit_lock (XCMN, READ, L(confnum), L(this->num), L(0), L(0), nullchix))
            continue;

         if (NOT loadnpart (XCMN, this->num, tbuf))
                                               { unit_unlk (XCMN);  continue; }
         unit_unlk (XCMN);
   
         for (j=0;   j<tbuf->l_used;   ++j) {
            if (sysbrktest())    break;
            if (chxtoken (id,  nullchix, 2, tbuf->line[j]) == -1) 
               continue;
            if (showitperson (out, id, i, &found))  reload = 1;
         }
      }
      nlfree (namdir);
      free_textbuf (tbuf);
   }    

   /*** If RELOAD is true, we over-wrote the current user's in-memory
   /    participation record in THISCONF.  Load it back in. */
   if (reload) {
      thisconf.items = 0;
      if (unit_lock (XUPA, READ, L(confnum), L(0), L(0), L(0), thisuser->id)) {
         load_user (&thisconf);
         unit_unlk (XUPA);
      }
   }
   SUCCEED;

done:
   chxfree (id);
   chxfree (mtu);
   chxfree (format);

   RETURN (success);
}

/*** SHOWITPERS displays a line of text, telling how many responses
/    this PERSON has seen of ITEM.  PERSON is a ptr to a userid.
/    Returns 1 on success, 0 on failure (no-one of that id).
/    If this PERSON has seen this ITEM at all, increments FOUND.
/ */

FUNCTION showitperson (out, person, item, found)
   int    out, item;
   Chix   person;
   int   *found;
{
   static Chix    s_item;
   Userreg  persreg, make_userreg();
   int      r, success;
   Chix     s, person_format();
   Chix     seen, format;
   
   ENTRY ("showitpers", "");

   seen = chxalloc (L(80), THIN, "showitpers seen");
   format = chxalloc (L(80), THIN, "showitpers format");

   /*** Load the registration and participation info for this person. */
   persreg = make_userreg ();
   persreg->briefs = persreg->print = null.nl;
   if (NOT loadreg (person, persreg, NULLSITE))              FAIL;

   if (NOT unit_lock (XUPA, READ, L(confnum), L(0), L(0), L(0), person)) RETURN (0);

   if (NOT load_user (&thisconf))   { unit_unlk (XUPA);      FAIL; }
   unit_unlk (XUPA);

   /*** Decide what to do based on highest response seen.
   /    -1 means item never seen.  -2 means forgotten.  Any other
   /    number is # of responses seen. */
   r = thisconf.responses[item];
   if (r == -1)  SUCCEED;

   if (r == -2)  chxofascii (seen, "F");
   else          chxformat  (seen, CQ("%d"), L(r), L(0), nullchix, nullchix);

   mdstr (format, "sho_Fitpers", &s_item);
   s = person_format (format, persreg->name, persreg->id, nullchix, seen,
                      null.nl);

   if (*found == 0)  mdwrite (out, "sho_Titperhead", null.md);
   ++(*found);
   if (s != nullchix)  unit_write (out, s);
   chxfree (s);
   SUCCEED;

done:
   chxfree (seen);
   chxfree (format);
   free_userreg (persreg);
   RETURN (success);
}
