/*** OK_SHOW_ATTACHMENT.  Display the attachment to this response?
/
/    ok = ok_show_attachment (cnum, inum, rnum, red, attach, noattach,
/                             showattach);
/
/    Parameters:
/       int       ok;         (Display the attachment?)
/       int       cnum;       (conference number)
/       int       inum;       (item number)
/       int       rnum;       (response number)
/       Redirect *red;        (redirection/output information)
/       int       attach;     ('attach' modifier specified?)
/       int       noattach;   ('noattach' modifier specified?)
/       int       showattach; (value of SET SHOWATTACH)
/
/    Purpose:
/       Determine if the user really wants to display the attachment
/       to this response.
/
/    How it works:
/       Logical "and" of:
/          NOATTACH specified?
/          SHOWATTACH default
/          Does response exist?
/          Does attachment exist?
/          Can user show attachment on this terminal?
/          Ask user (if necessary)
/
/       RED may be NULL.
/
/    Returns:  1   (yes)
/              0   (user requested no attachments)
/             -1   (no response)
/             -2   (no attachment)
/             -3   (can't display attachment on this terminal)
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  showitem(), ....?
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  cmd1/okshowat.c
/ */

/*: CP  8/19/93 14:34 New function. */
/*: CP  8/24/93 23:54 Allow RED to be NULL. */

#include <stdio.h>
#include "caucus.h"
#include "api.h"
#include "redirect.h"

extern Chix yesnotable;

FUNCTION  ok_show_attachment (cnum, inum, rnum, red, attach, noattach,
                                    showattach)
   int       cnum, inum, rnum, attach, noattach, showattach;
   Redirect *red;
{
   Resp      respobj;
   int       success, show;
   Chix      answer, def;
   Attach_Mapping map;

   ENTRY ("ok_show_attach", "");

   /*** Did the user explicitly request no attachments? */
   if (noattach)                            RETURN (0);
   if (NOT attach  &&  showattach == 0)     RETURN (0);   /* SHOWATTACH OFF */

   /*** See if the response exists. */
   respobj = a_mak_resp ("ok_show_attach");
   respobj->cnum = cnum;
   respobj->inum = inum;
   respobj->rnum = rnum;
   if (a_get_resp (respobj, P_TITLE, A_WAIT) != A_OK)    DONE(-1);
   
   /*** See if the attachment exists. */
   if (respobj->attach == (Attachment) NULL  ||
       respobj->attach->size <= 0)                       DONE(-2);

   /*** If the user specified redirection, s/he always wants to
   /    "display" it (by copying it to a file). */
   if (red != (Redirect *) NULL  &&  red->unit == XIRE)  SUCCEED;

   /*** Can this terminal type display the attachment? */
   map = ok_to_display_attach (respobj->attach);
   if (map == (Attach_Mapping) NULL)                     DONE(-3);

   /*** If SET SHOWATTACH ASKME, ask the user about this attachment. */
   if (NOT attach  &&  showattach == 2) {
      answer = chxalloc (L(20), THIN, "ok_to_ answer");
      def    = chxalloc (L(20), THIN, "ok_to_ def");  /* default = NO */
      chxtoken (def, nullchix, 1, yesnotable);
      show = mdprompter ("res_Pshowat", yesnotable, 2, "res_Hshowat", 10,
                              answer, def);
      chxfree  (answer);
      chxfree  (def);
      if (show <= -3)  sysbrkset();
      if (show <   1)  DONE(0);
   }

   SUCCEED;

done:
   a_fre_resp (respobj);
   RETURN     (success);
}
