/*** SHOW_ATTACH.  Show an attachment.
/
/    ok = show_attachment (cnum, inum, rnum, red, nowait);
/
/    Parameters:
/       int       ok;       (Success?)
/       int       cnum;     (conference number)
/       int       inum;     (item number)
/       int       rnum;     (response number)
/       int       nowait;   ('nowait'   modifier specified?)
/       Redirect *red;      (redirection/output information)
/
/    Purpose:
/       "Show" an attachment to a response.
/
/    How it works:
/       Tries to get the attachment to response CNUM/INUM/RNUM.
/       If RED specifies an output redirection, the attachment is
/       copied to that file.  Otherwise, we try to display it on
/       the user's terminal.
/
/    Returns:
/        1      success
/       -1      no such response
/       -2      no attachment on this response
/       -3      this terminal cannot display this attachment
/
/    Side effects:
/
/    Related functions:
/
/    Called by:  showitem(), ....?
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  cmd1/showatta.c
/ */

/*: CP  8/19/93 14:34 New function. */
/*: CP  8/24/93 23:59 Allow RED to be NULL. */
/*: CK  9/03/93 14:54 Sysprv off then on, when showing attach to file. */

#include <stdio.h>
#include "caucus.h"
#include "api.h"
#include "redirect.h"

FUNCTION  show_attachment (cnum, inum, rnum, red, nowait)
   int       cnum, inum, rnum, nowait;
   Redirect *red;
{
   Resp      respobj;
   Chix      command;
   char      comstr[512];
   int       success;
   Attach_Mapping map;

   ENTRY ("show_attach", "");

   respobj = a_mak_resp ("show_attach");
   command = chxalloc   (L(100), THIN, "show_atta command");
   
   /*** Get the attachment. */
   respobj->cnum = cnum;
   respobj->inum = inum;
   respobj->rnum = rnum;
   if (a_get_resp (respobj, P_TITLE, A_WAIT) != A_OK)      DONE(-1);
   
   if (respobj->attach == (Attachment) NULL  ||
       respobj->attach->size <= 0)                         DONE(-2);

   /*** If user specified redirection, just copy the attachment to the
   /    requested file name. */
   if (red != (Redirect *) NULL  &&  red->unit == XIRE) {
      if (a_get_resp (respobj, P_ATTACH, A_WAIT) != A_OK)  DONE(-2);
      sysprv(0);
      sysrename (ascquick(respobj->attach->name), red->fname);
      sysprv(1);
      SUCCEED;
   }

   /*** Otherwise, see if this terminal type can display it. */
   map = ok_to_display_attach (respobj->attach);
   if (map == (Attach_Mapping) NULL)                       DONE(-3);

   /*** Actually get the attachment, and display the temp file it created. */
   if (a_get_resp (respobj, P_ATTACH, A_WAIT) != A_OK)  DONE(-2);
   chxcpy (command, map->command);
   while  (chxalter (command, L(0), CQ("$1"), respobj->attach->name)) ;
   ascofchx (comstr, command, L(0), L(512));
   syscmd   (comstr, (nowait ? SYS_NOWAIT : SYS_WAIT));
   SUCCEED;

done:
   a_fre_resp (respobj);
   chxfree    (command);

   RETURN (success);
}
