/*** SHOWITEM.   SHOW items and responses from a Caucus command line.
/
/    ok = showitem (red, command);
/
/    Parameters:
/       int       ok;       (Success?)
/       Redirect *red;      (redirection/output information)
/       Chix      command;  (command line that shows item or resps)
/
/    Purpose:
/       Parse and execute a command that SHOWs items or responses.
/
/    How it works:
/       Version 2.2 added "SHOW ITEM range PERSONS name"
/       which displays, for each item in "range", how much has
/       been seen by person "name".  "name" may be ALL, that is,
/       everyone in the conference.
/
/    Returns: 1 if at least one item or response was displayed
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects: updates thisconf global.
/
/    Related functions:
/
/    Called by:  interpret()
/
/    Operating system dependencies: none
/
/    Known bugs:      none
/
/    Home:  cmd1/showitem.c
/ */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: JV  8/23/89 13:36 Chg's error mess on no valid items. */
/*: JV  9/18/89 16:15 Add AFMs _beg_showitem and _end_showitem. */
/*: JV  9/21/89 14:23 Begin adding "SHOW ITEM n PERSON name" stuff. */
/*: JV 10/03/89 16:45 Finish adding "SInPn" stuff. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR  9/05/89 15:06 Handle LIFO modifier. */
/*: CR 10/10/89 11:22 mdprompter() fails if return < -1 (not 0). */
/*: CR 10/10/89 15:16 Use pluck_person() to parse PERSON keyword. */
/*: CR 10/11/89 15:54 Update SEEN after showtext(). */
/*: CR 10/11/89 16:11 Revert to "no items selected" if parserange fails. */
/*: CR 10/17/89 15:17 Replace 2nd arg of FIRE_ARM with "". */
/*: CR 10/30/89 12:05 Remove char *t; */
/*: CR 12/01/89 10:37 Add AFM _mid_showitem. */
/*: CR  8/12/90 19:13 Cast show_resp() arg to make Lint happy. */
/*: JV  7/22/90 17:54 Cast mod.lifo to int to match def in function. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: CR  1/20/92 16:47 Add cnum arg to load_header() call. */
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: JX  5/27/92 11:25 Chixify. */
/*: CR  4/24/92 14:15 Set NOMORE only if there were responses to be seen! */
/*: CR  9/09/92 14:02 Add logfeature() stuff. */
/*: CI 10/06/92 15:32 Chix Integration. */
/*: CR  2/08/93 14:43 chxfree format & mtu! */
/*: JV  4/03/93 11:25 Add attach arg to load_header(). */
/*: JV  6/08/93 17:32 Don't need header or title. */
/*: CN  5/15/93 14:05 Replace parse_range() with new_parse(). */
/*: CT  5/28/93 13:47 Replace SELECT with Rlist. */
/*: CP  6/23/93 23:44 Use new form of new_parse(). */
/*: CP  6/24/93 13:21 Use all information in Rlists; new header comments. */
/*: CK  7/28/93 17:10 Check 'first' against -1. */
/*: CP  8/04/93 16:44 Skip item if first > last. */
/*: CP  8/06/93 12:03 Use r_bound() to limit first/last. */
/*: CP  8/09/93  0:44 Add attachment stuff. */
/*: CP  8/10/93 12:01 Define ESCAPE to get out of rlist loop. */
/*: CP  8/19/93 16:22 Use ok_show_attach(), show_attachment(). */
/*: CK  8/20/93 15:45 Remove debugging. */
/*: CP  8/24/93 15:36 Pass showattach to showresp(), showtext(), resploop(). */
/*: CP  8/27/93 16:46 Handle BRIEF condition during item display. */
/*: CP  9/09/93 16:51 Replace new_parse() with parse_or_ask([0x). */
/*: CK  9/15/93 14:04 Add 'mod1st' arg to parse_or_ask(). */
/*: CK  9/19/93  0:07 Add 'onlyattach' arg to show_resp() call. */
/*: CK 10/07/93 17:05 Use &thisitem in parse_or_ask() call. */
/*: CK 10/15/93 13:00 If output redirected, don't show attachments. */
/*: CP 11/11/93 13:37 Add 'resp1' arg to show_format() call. */
/*: CP 11/18/93 15:35 Add cnum, inum args to r_bound() call. */
/*: CP 12/03/93 20:35 NOMORE: compare LAST (not master.resp) against SEEN. */
/*: CP 12/10/93 21:43 If SEEN reaches LAST, set it to FINAL. */
/*: CR  5/24/94 16:36 Check nomore & sysbrktest() if M_BRIEF. */

#include <stdio.h>
#include "caucus.h"
#include "modifier.h"
#include "api.h"
#include "redirect.h"
#define   RLNULL   ( (Rlist) NULL )
#define   NC       nullchix
#define   ESCAPE   goto escape

extern struct master_template  master;
extern struct partic_t         thisconf;
extern struct this_template    thisitem;
extern Userreg                 thisuser;
extern union  null_t           null;
extern short                   confnum;
extern Chix                    confname;

FUNCTION  showitem (red, command)
   Redirect *red;
   Chix      command;
{
   struct namlist_t *nluserid(), *nlnode();
   Chix   str, title, newconf, persname, headstr, instance, format, attach;
   int    seen, high, back, resp, item, success, showattach, onlyattach;
   int    shown, itemshown, interrupt, last, first, r_first, ok, final;
   short  i, everchanged, nowchanged;
   short  askuser, nomore=0, persons;
   char   modifier[20];
   Rlist  rptr, result;

   ENTRY ("showitem", "");

   instance = chxalloc ( L(60), THIN, "showitem instance");
   str      = chxalloc (L(160), THIN, "showitem str");
   title    = chxalloc (L(160), THIN, "showitem title");
   format   = chxalloc ( L(40), THIN, "showitem format");
   attach   = chxalloc ( L(60), THIN, "showitem attach");
   newconf  = chxalloc ( L(40), THIN, "showitem newconf");
   headstr  = chxalloc (L(160), THIN, "showitem headstr");
   persname = chxalloc ( L(80), THIN, "showitem persname");
   result   = RLNULL;


   /*** See if PERSONS modifier was found, and if so pluck out the
   /    name(s) of the specified person. */
   persons = pluck_person (command, persname);

   chxtoken (str, instance, 2, command);
   if (NOT parse_or_ask (instance, &result, modifier, attach, confnum, 
                                   &thisitem, 1))  FAIL;

   /*** If the user specifically requested to show an attachment... */
   if (modifier[M_ATTACH]  &&  NOT modifier[M_BRIEF]) {

      /*** Use the first item:resp instance given. */
      if (NOT one_response (&item, &resp, result, &master)) {
         mdwrite (XWER, "res_Tnoitem", null.md);
         FAIL;
      }

      ok = ok_show_attachment (confnum, item, resp, red, 1, 0,
                                 thisuser->showattach);
      if (ok == 1)
         ok = show_attachment (confnum, item, resp, red, modifier[M_NOWAIT]);

      if (ok == -1  ||  ok == -2)  mdwrite (XWER, "res_Enoattach",  null.md);
      if (ok == -3)                mdwrite (XWER, "res_Enoattshow", null.md);
      SUCCEED;
   }

   /*** If the PERSON modifier was found... */
   if (persons) {
      /*** Check if PERSNAMES is empty.  If so, prompt the user for
      /    the name of the PERSON. */
      if (EMPTYCHX(persname)  &&
          mdprompter ("sho_Pwho", NC, 0, "sho_Hwho", 80, persname, NC) < -1)
             FAIL;
  
      /*** PERSNAM will contain name(s) when done. */
      chxsimplify (persname);
      chxstrip    (persname);
      modifier[M_BRIEF] = 1;      /* set up to print BRIEF item listing.  */
   }
  
   mdstr (format, (modifier[M_BRIEF] ? "res_Fshowhead8" : "res_Fshowhead7"),
                   null.md);

   if (NOT modifier[M_ATTACH])  apply_redirect (red);

   /*** DISPLAY all the items & responses selected, or until the
   /    user wants NOMORE. */
   itemshown   = 0;
   everchanged = 0;
   showattach  = thisuser->showattach;
   if (modifier[M_NOATTACH])  showattach = 0;
   if (red->unit != XWTX)     showattach = 0;
   onlyattach = (modifier[M_BRIEF]  &&  modifier[M_ATTACH]);
   if (sysbrktest())  ESCAPE;
   for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
      for (i=rptr->i0;   i<=rptr->i1;   ++i) {
         if (i == 0)  continue;

         /*** Definitions:
         /    FIRST is the first response (besides the item proper) to
         /       be DISPLAYED. 
         /    LAST is the last response that should be displayed.
         /    HIGH is the high water mark for what this user has effectively
         /       seen on this item.
         /    BACK tells us how many responses back from HIGH were made by this
         /       user (and therefore have not actually been seen yet.)  */
         if (NOT r_bound (&first, &last, &final, rptr, confnum, i, 
                           master.responses[i]))                  continue;

         r_first = (first==0 ? 1 : first);
         high = thisconf.responses[i];
         back = thisconf.backresp [i];

         /*** Display item text, or (if only responses), item header. */
         seen      = -1;
         interrupt =  0;
         if (first == 0  &&  NOT persons) {
            /*** First fire off the initial AFM, then show the item. */
            fire_afm ("_beg_showitem", null.chx, newconf);
            unit_write (red->unit, NEWCPAGE);

            if (modifier[M_BRIEF]) {
               show_format    (red->unit, format, i, 0, 0, final, 0, title, 
                                          headstr);
               show_responses (red->unit, confnum, i, 0, 0, 0, 1, 0, &shown, 
                                          onlyattach);
            }
            else {
               interrupt = NOT showtext (red->unit, i, final, showattach);
               itemshown = 1;
               fire_afm ("_mid_showitem", null.chx, newconf);
               if (NOT interrupt) seen = 0;  /* Saw resp 0, i.e. the item. */
            }
         }
   
         else {  /* If we're not showing the item text, show the header. */
            if (NOT load_header (confnum, i, headstr, title, 
                                    (Attachment) NULL))         ESCAPE;
            if (persons)    unit_write (red->unit, CQ("\n"));
            show_format (red->unit, format, i, 0, r_first, final, 0, title, 
                                    headstr);
         }
   
         /*** If SHOW ITEM range PERSONS name, handle the special case,
         /    and loop back to the next item. */
         if (persons) {
            item_person (red->unit, persname, i);
            if (sysbrktest())   ESCAPE;
            continue;
         }
   
         /*** Otherwise, continue deciding exactly which items/responses
         /    are to be displayed. */
   
   
         /*** Go on to display responses (if we haven't already been
         /    interrupted). */
         resp = seen;
         if (NOT interrupt) {
            resp = show_responses (red->unit, confnum, i, r_first, last, 
                     modifier[M_LIFO], modifier[M_BRIEF], showattach, &shown,
                                          onlyattach);
            /*** Included deleted responses as "seen". */
            if (resp >= last  ||  (r_first > last  &&  r_first <= final))
               resp = final;
         }

         /*** Adjust HIGH and BACK based on what was *really* seen. */
         if (resp > seen)           seen = resp;
         if (seen > high)        {  back = 0;            high = seen; }
         if (seen > high - back)    back = high - seen;
   
         /*** Should we ask the user if s/he wants to enter a response? */
         askuser = (red->unit==XWTX  &&  NOT modifier[M_PASS]);
   
         /*** Did the user say No More? (STOP or ^C at RFP prompt, or ^C
         /    during SHOW > or SHOW PASS.)   Applies only if there were
         /    responses to be seen! */
         if (askuser)  nomore = NOT resploop (i, &seen, &high, &back, 
                                                 showattach);
         else          nomore = (last > seen  &&  master.responses[i] > 0);
   
         /*** Don't update "seen" records if BRIEF modifier given. */
         if (modifier[M_BRIEF]) {
            if (nomore  ||  sysbrktest())  ESCAPE;
            continue;
         }

         /*** Adjust permanent record of what this user has seen based
         /    on SEEN, HIGH, and BACK. */
         nowchanged = 0;
         if (thisconf.responses[i] == -2)  { 
            high = seen = -2;
            nowchanged = 1;
         }
         if (high > thisconf.responses[i]) {
            thisconf.responses[i] = high;
            thisconf.backresp [i] = back;
            nowchanged = 1;
         }
         if (seen > thisconf.responses[i]) {
            thisconf.responses[i] = seen;
            thisconf.backresp [i] = 0;
            nowchanged = 1;
         }
         if (seen > thisconf.responses[i] - thisconf.backresp[i]) {
            thisconf.backresp [i] = high - seen;
            nowchanged = 1;
         }
   
         if (nowchanged) {
            everchanged = 1;
            if (thisconf.items < i)  thisconf.items = i;
         }
   
         /*** Log number of responses read. */
         if (shown + itemshown > 0)
            logfeature (CQ("respread"), confname, L(i), L(0), 
                         L(shown + itemshown), L(0));
   
         /*** After successfully showing the item, fire the closing AFM. */
         fire_afm ("_end_showitem", null.chx, newconf);
     
         if (nomore  ||  sysbrktest())  ESCAPE;
      }
   }
escape:

   /*** If the permanent record of what the user had seen has changed,
   /    rewrite it. */
   if (everchanged) {
      sysuserid  (str);
      unit_lock  (XUPA, WRITE, L(confnum), L(0), L(0), L(0), str);
      store_user (&thisconf);
      unit_unlk  (XUPA);
   }
   SUCCEED;

done:
   a_fre_rlist (result);
   chxfree (instance);
   chxfree (str);
   chxfree (title);
   chxfree (newconf);
   chxfree (attach);
   chxfree (headstr);
   chxfree (persname);
   chxfree (format);

   RETURN  (success);
}
