/*** SHOWMESS.   SHOW a selected set of messages according to COMLINE.
/
/    COMLINE contains a command line of the form:
/
/       SHOW MESSAGES list mods
/
/    where "list" is a list of message numbers, including the words
/       THIS or LAST;
/
/    and   "mods" is optionally the words NEW, PASS, or NOMARK.
/ 
/    SHOWMESS displays the selected messages on unit OUT, and if
/    OUT is the user's terminal, prompts the user for the dispostion
/    of the messages by calling MESS_DISP. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: CR  6/30/89 16:38 Remove unitclose/lock(out), done already in show(). */
/*: JV  8/23/89 13:48 Chg'd error mess on invalid item number. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR  8/31/89 15:45 Add code for return-receipt. */
/*: CR 10/08/89 11:43 Simplify/change: mes_T0selected --> mes_F0selected. */
/*: CR 11/14/89 12:54 Undo previous change; mes_F0selected doesn't work.  */
/*: CR  8/12/90 20:27 Cast messparse arg to make Lint happy. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  8/07/91 14:25 Add 'confnum' arg to syspdir() call. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  5/27/92 14:32 Chixify. */
/*: CR  8/25/92 18:10 Add "NOMARK" modifier processing. */
/*: CR  9/08/92 17:31 Log # of new messages read via logfeature().  */
/*: CI 10/06/92 15:36 Chix Integration. */
/*: CP  8/09/93 16:30 Replace arg 'out' with 'red'. */
/*: CK  9/19/93 12:03 Add 'old' arg to messparse() call. */
/*: CR  2/10/95 16:53 Tighten up use of unit_lock() return value. */

#include <stdio.h>
#include "caucus.h"
#include "redirect.h"
#define  I(x)  ( (int) x )

extern struct message_t      mess;
extern struct pdir_t         pdir;
extern union  null_t         null;

FUNCTION  showmess (red, comline)
   Redirect *red;
   Chix      comline;
{
   static Chix reptab[] =
      { nullchix, nullchix, nullchix, nullchix, nullchix, nullchix };
   static Chix  mesmodtab;
   Chix   id, all, last, str, args;
   Flag   ftab[10];
   int    messread, success;
   short  m, n;

   ENTRY ("showmess", "");

   id   = chxalloc (L(20), THIN, "showmess id");
   all  = chxalloc (L(20), THIN, "showmess all");
   str  = chxalloc (L(40), THIN, "showmess str");
   args = chxalloc (L(80), THIN, "showmess args");
   last = chxalloc (L(30), THIN, "showmess last");

   apply_redirect (red);

   /*** Load the current message directory. */
   sysuserid (id);
   if (NOT unit_lock (XUMD, READ, L(0), L(0), L(0), L(0), id))  FAIL;
   mess.highest = loadmdir (0, mess.num);
   unit_unlk (XUMD);

   if (mess.highest >= MAXMESS)   mdwrite (XWER, "mes_Tbinfull", null.md);

   sysbrkclear();

   /*** If we have any messages at all, parse the message range, and
   /    go look at them with MESS_VIEW! */
   if (mess.highest > 0) {

      chxformat (all, CQ(" 1-%d "), L(mess.highest), L(0), nullchix, nullchix);
      chxformat (last, CQ(" %d "),  L(mess.highest), L(0), nullchix, nullchix);
      reptab[0] = all;
      reptab[1] = last;

      if ( (chxtoken (str, args, 3, comline)) == -1)  chxofascii (args, " ");
      mdtable    ("mes_Ashowmod", &mesmodtab);
      parsewords (args, mesmodtab, reptab, ftab);

      if (messparse (args, I(ftab[2]), I(ftab[5]), mess.highest, mess.num))  {
         unit_write (red->unit,  NEWCPAGE);

         /*** Examine all of the messages selected to be SEEn. */
         messread = 0;
         for (m=1;   m<=mess.highest;   ++m)  if (mess.num[m] & MSG_SEE) {
  
            /*** Control loop for message disposition.  After seeing message
            /    M, the user may ask to reread it, reply to it, delete it,
            /    or even look at another message out of sequence. */
            for (n=m;   n>0;   n = messdisp (n, mess.highest, mess.num)) {
               if (NOT mess_view (red->unit, n))     break;
               if (mess.num[n] & MSG_RRR)      messreceipt (n);
               if (mess.num[n] & MSG_NEW)    ++messread;

               mess.num[n] &= (~ MSG_RRR);
               mess.num[n] &= (~ MSG_SEE);
               if (NOT ((mess.num[n] & MSG_MLN) || ftab[4]) )
                  mess.num[n] &= (~ MSG_NEW);
               if (red->unit != XWTX  ||  ftab[3])      break;
            }
  
            if (n < 0) break;
         }

         /*** Log number of new messages read. */
         if (messread > 0)
            logfeature (CQ("mailread"), nullchix, L(0), L(0), L(messread), L(0));

         /*** Reload the message directory, integrating it with any
         /    changes we just made. */
         if (NOT unit_lock (XUMD, WRITE, L(0), L(0), L(0), L(0), id))  FAIL;
         mess.highest = loadmdir (mess.highest, mess.num);
         syspdir (&pdir, XUMF, 0, 0, id);

         mess.highest   = messdel (&pdir, mess.highest, mess.num);
         mess.high_seen = min (mess.highest, mess.high_seen);
         storemdir (mess.highest, mess.num);
         unit_unlk (XUMD);
      }

      else {
/*--Maybe someday...
         strsimplify (args);
         mdstr   (format, "mes_F0selected", null.md);
         sprintf (mtu, format, args);
         unit_write (XWER, mtu);
*/
         mdwrite (XWER, "mes_T0selected", null.md);
      }
   }

   else       mdwrite (XWER, "mes_Thave0",     null.md);

   SUCCEED;

done:
   chxfree (id);
   chxfree (all);
   chxfree (str);
   chxfree (args);
   chxfree (last);

   RETURN  (success);
}
