/*** SHOWPERSON.   CLI: SHOW information about a person. 
/
/    SHOWPERSON displays specific information about a particular
/    person, or ALL the members of this conference.  It is called
/    by two commands: DESCRIBE PERSON and SHOW PERSON.
/
/    COMMAND is the original invoking command.  The output is written
/    on unit OUT.  SHOWPERSON displays everything about the selected
/    person if SHOW is true, otherwise it just does a brief display.
/
/    SHOWPERSON handles the commands:
/
/      DESCRIBE  PERSON   ALL             (all members)
/      DESCRIBE  PERSON   name
/      DESCRIBE  PERSON  "name"
/      SHOW      PERSON   ALL             (all members)
/      SHOW      PERSON   name
/      SHOW      PERSON  "name"
/      SHOW      PERSON  "name"  item_range
/
/    The last case displays how much of the items in 'item_range' the
/    selected person(s) have seen.    */

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:38 Source code master starting point*/
/*: CR  3/01/89 15:44 Test for <CANCEL> while display matching people. */
/*: JV  8/23/89 12:07 Added error message if noone of that name. */
/*: JV  9/06/89 12:51 Added numbering of people in display. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR 10/31/89 18:06 Do not use return from person_is to set FOUND. */
/*: CR  1/26/90 21:59 For "SHOW PERS ITEM", confirm user is a member! */
/*: CR  2/03/90 17:29 Remove unused "perout". */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/25/91  0:58 Remove duplicate unit_lock() confnum arg. */
/*: JV  5/15/91 17:12 Add new loadnpart arg. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  5/13/91 12:49 Chixify, xlate to Textbuf. */
/*: JX  8/10/92 10:00 Add SHOW/LIST PERSON {ORGANIZ|INCL|READON|EXCL}. */
/*: CR 11/18/92 12:17 Use tablefind() (not tablematch) with permodtab. */
/*: CL 12/11/92 15:52 Long sysmem arg. */
/*: CR  1/13/93 13:47 Use make_part(). */
/*: CR  2/08/93 14:09 chxfree word & format, too! */
/*: CN  5/15/93 15:20 Replace parse_range with new_parse(). */
/*: CT  5/28/93 13:47 Replace SELECT with Rlist. */
/*: CP  6/23/93 23:46 Use new form of new_parse(). */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */
/*: CP  8/09/93 11:41 Add 'attach' arg to new_parse() call. */
/*: CP  8/09/93 16:34 Replace arg 'out' with 'red'. */
/*: CP  8/12/93 17:32 Correct order of args in new_parse() call. */
/*: CP  9/09/93 17:19 Add 'clearmod' arg to new_parse() call. */
/*: CK  9/15/93 13:35 Add 'mod1st' arg to new_parse() call. */
/*: CK 10/07/93 17:13 Add ERRCHK, ERRPOS args to new_parse() call. */

#include <stdio.h>
#include "caucus.h"
#include "api.h"
#include "modifier.h"
#include "redirect.h"
#define   RLNULL   ( (Rlist) NULL )

extern union  null_t          null;
extern struct this_template   thisitem;
extern Chix                   ss_all, ss_dotnl;
extern short                  confnum;

FUNCTION  showperson (red, command, show)
   Redirect *red;
   int       show;
   Chix      command;
{
   static  Chix permodtab = nullchix;
   struct  namlist_t *wholist, *this, *nluserid();
   struct  namlist_t *namdir,  *nlnode();
   struct  partic_t *thatuser, user;
   int     success, choice, priv, errpos;
   short   i, ok, seen, found=0, whosize, j=0;
   Chix    id, word, str, s, format, mtu, keep[2];
   Textbuf tbuf, make_textbuf();
   char    modifier[MODIFIERS+5];
   Rlist   result;

   ENTRY ("showperson", "");

   s       = chxalloc (L(80), THIN, "showp s");
   id      = chxalloc (L(20), THIN, "showp id");
   str     = chxalloc (L(80), THIN, "showp str");
   mtu     = chxalloc (L(200), THIN, "showp mtu");
   word    = chxalloc (L(20), THIN, "showp word");
   keep[0] = chxalloc (L(80), THIN, "showp keep0");
   keep[1] = chxalloc (L(80), THIN, "showp keep1");
   format  = chxalloc (L(80), THIN, "showp format");
   result  = RLNULL;

   apply_redirect (red);

   tbuf = make_textbuf(TB_RESPS, TB_LINES, "showpers tbuf");
   make_partic (&user, MAXITEMS);

   if (NOT mdtable ("sho_Apermod", &permodtab)) FAIL;

   /*** Get name of person from command line, or prompter user for it. */
   if (chxtoken (str, s, 3, command) != -1)   chxcpy (str, s);
   else if (mdprompter ("sho_Pwho", nullchix, 0, "sho_Hwho", 80, str, nullchix)
              < -1)  FAIL;
   chxsimplify (str);

   /*** Does command say "tell me about the items this person has seen?" */
   seen = 0;
   if (chxquote (str, 0, keep)) {
      if ( chxtoken (word, s, 2, str) != -1) {
         seen = new_parse (s, &result, modifier, nullchix, confnum,
                              &thisitem, 1, 1, 1, &errpos);
         if (NOT seen  &&  errpos >= 0) {
            report_parse_err (s, errpos);
            FAIL;
         }
      }
      chxquote  (str, 1, keep);
      chxstrip  (str);
   }

   /*** Possible cases:
   /    1) SHOW PER ALL [item...]
   /    2) SHOW PER ["]NAME["] [item...]
   /    3) SHOW PER {privilege} [item...]   (implicit ALL) */
   choice = tablefind (permodtab, str);

   /*** Case 2: Must be a user name, show the people that match. */
   if (choice == -1) {
      thatuser =  (seen & show ? &user : null.par);

      /*** Find userid(s) that match this name and examine each of them. */
      wholist = nluserid (str, 0);
      whosize = nlsize (wholist);
      sysbrkclear();
      for (this=wholist->next;   this!=null.nl;   this = this->next) {
         ++found;
         if (sysbrktest())  break;

         /*** If more than one person, number them */
         if (whosize > 1)  person_is (red->unit, this->str, show, thatuser, ++j);
         else              person_is (red->unit, this->str, show, thatuser, 0);

         /*** Display list of items & responses seen if requested, and
         /    the person is a member. */
         if (seen && show) {
            if (thatuser->items >= 0)  showseen (red->unit, result, thatuser);
            else                       mdwrite  (red->unit, "sho_Enotmem", null.md);
         }
      }

      /*** If nobody at all matched, complain. */
      if (NOT found) {
         mdstr   (format, "sho_Enobody", null.md);
         chxformat (mtu, format, L(0), L(0), str, nullchix);
         chxconc (mtu, mtu, ss_dotnl);
         unit_write (red->unit, mtu);
      }
      nlfree (wholist);
   }

   /*** Case 1: describe everyone in this conference.
   /    Case 3: describe all people who have the specified privilege. */
   else {
      if (NOT unit_lock (XCMD, READ, L(confnum), L(0), L(0), L(0), nullchix)) FAIL;
      namdir = nlnode(4);
      ok = loadnmdir (XCMD, namdir);
      unit_unlk (XCMD);
      if (NOT ok)     { nlfree (namdir);           FAIL; }
   
      /*** Load each namepart file in turn, and display contents. */
      sysbrkclear();
      for (this=namdir->next;   this != null.nl;   this = this->next) {
         if (sysbrktest())    break;
         if (this->num <= 0)  continue;
         if (NOT unit_lock (XCMN, READ, L(confnum), L(this->num), L(0), L(0),
                            nullchix))
            continue;
         if (NOT loadnpart (XCMN, this->num, tbuf))
                                                { unit_unlk (XCMN); continue; }
         unit_unlk (XCMN);
   
         for (i=0;   i<tbuf->l_used;   ++i) {
            if (sysbrktest())    break;
            if (chxtoken (id,  nullchix, 2, tbuf->line[i]) == -1) continue;
            if (choice == 0) {                             /* ALL */
               person_is (red->unit, id, show, null.par, ++j);
               continue;
            }

            priv = allowed_on (id, null.nl, confnum);
            if (priv == 3 && choice == 1 ||           /* Organizer */
                priv == 2 && choice == 2 ||           /* Include   */
                priv == 1 && choice == 3 ||           /* Readonly  */
                priv == 0 && choice == 4)             /* Exclude   */
               person_is (red->unit, id, show, null.par, ++j);
         }
      }
      nlfree (namdir);
   }
   SUCCEED;

done:
   free_textbuf (tbuf);
   a_fre_rlist  (result);
   chxfree (s);
   chxfree (id);
   chxfree (str);
   chxfree (mtu);
   chxfree (word);
   chxfree (keep[0]);
   chxfree (keep[1]);
   chxfree (format);
   free_partic (&user);
   RETURN (success);
}
