/*** SHOWSUBJ.   SHOW a subject in detail.
/
/    SHOWSUBJ is used to show a SUBJECT or PSUBJECT named by
/    the user on the command line.  If no subject is named, all subjects
/    (or psubjects) are listed.  The display includes the list of
/    items attached to the subject.
/
/    MINE means PSUBJECT if true, and SUBJECT otherwise. */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:39 Source code master starting point*/
/*: JV  8/23/89 13:32 Chg'd header comment, added err mess if no match. */
/*: CW  5/17/89 16:22 Use XW unitcodes in place of XITT.*/
/*: CR  3/29/90 21:43 Check OK for failure from matchnames(). */
/*: CR  4/20/90 18:29 Expand sizes of str[] and word[]. */
/*: CR  3/29/91 17:27 Use AND/OR code for matchnames() call. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: JV  5/15/91 17:14 Add new loadnpart arg. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: CR  8/02/91 14:00 Add punctuation argument to matchnames() call. */
/*: CR 10/29/91 11:45 Add confnum arg to chixGetline() call. */
/*: CR 11/07/91 14:18 Add confnum arg to matchnames() call. */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  5/13/92 15:13 Chixify. *I think it's finished*/
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CR  2/08/93 14:47 chxfree mtu! */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */
/*: CP  8/09/93 16:32 Replace arg 'out' with 'red'. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"
#include "textbuf.h"
#include "redirect.h"

extern struct master_template  master;
extern union  null_t           null;
extern Chix                    confname;
extern Chix                    ss_all, ss_dotnl;
extern short                   confnum;

FUNCTION  showsubj (red, command, mine)
   Redirect *red;
   int       mine;
   Chix      command;
{
   struct namlist_t *namdir, *this, *names, *matches, *nlnode(), *nlnames();
   int4   subnum;
   short  none, i, ok, j;
   short  uad, uaf, usd, usf, utd, utf;
   short *list, subs;
   Chix   s, str, owner, word, format, mtu, line, chixGetline();
   Textbuf tbuf, make_textbuf();

   ENTRY ("showsubj", "");

   s     = chxalloc (L(16), THIN, "showsubj s");
   str   = chxalloc (L(100), THIN, "showsubj str");
   mtu   = chxalloc (L(200), THIN, "showsubj mtu");
   word  = chxalloc (L(160), THIN, "showsubj word");
   owner = chxalloc (L(40), THIN, "showsubj owner");
   format= chxalloc (L(200), THIN, "showsubj format");
   tbuf  = make_textbuf(TB_RESPS, TB_LINES, "showsubj tbuf");

   apply_redirect (red);

   sysuserid (owner);
   if (mine) {
      uad = XUOD;   uaf = XUON;          usd = XUND;   usf = XUNN;
      utd = XUSD;   utf = XUSF;
   }
   else {
      uad = XCOD;   uaf = XCON;          usd = XCND;   usf = XCNN;
      utd = XCSD;   utf = XCSF;
   }
   none = 1;

   /*** Get the subject name off the command line, or prompt for it. */
   if (chxtoken (str, s, 3, command) != -1)  chxcpy (str, s);
   else if (mdprompter ("sho_Pwsub", nullchix, 0, "sho_Hwsub", 80, str, ss_all)
                < -1)  {none = 1; goto done;}
   chxstrip    (str);
   jixreduce   (str);
   chxsimplify (str);

   /*** If ALL was selected, display all subjects... */
   if (chxeq (str, ss_all)) {

      /*** Load the alphabetical subject list namepart directory. */
      if (NOT unit_lock (uad, READ, L(confnum), L(0), L(0), L(0), owner))  goto
          done;
      namdir = nlnode(4);
      ok = loadnmdir (uad, namdir);
      unit_unlk (uad);
      if (NOT ok)  { nlfree (namdir);   goto done; }

      /*** Load each namepart file in turn, and display contents. */
      sysbrkclear();
      list = master.responses;
      for (this=namdir->next;   this != null.nl;   this = this->next) {
         if (sysbrktest())    break;
         if (this->num <= 0)  continue;
         if (NOT unit_lock (uaf, READ, L(confnum), L(this->num), L(0), L(0), owner))
          break;
         if (NOT loadnpart (uaf, this->num, tbuf))
                                                  { unit_unlk (uaf); 
          break; }
         unit_unlk (uaf);
   
         for (i=subs=0;   i<tbuf->l_used;   ++i) {
            chxcpy   (str, tbuf->line[i]);
            chxalter (str, L(0), chxquick("#", 5), chxquick (" ", 6));
            chxtoken (word, nullchix, 1, str);
            if (NOT chxeq (word, confname))  continue;

            chxtoken (word, nullchix, 3, str);
            chxnum   (word, &subnum);
            list[subs++] = subnum;
         }

         /*** For each subject that we pulled out of the Alphabetized list,
         /    display the subject name and list of attached items. */
         for (i=0;   i<subs;   ++i) {
            if (sysbrktest())    break;
            for (j=1; (line = chixGetline (utd, utf, confnum, owner, 0, list[i], j,
                                    (Textbuf) NULL)) != nullchix;
                 ++j) {
               unit_write (red->unit, line);
               unit_write (red->unit, CQ("\n"));
            }
         }
      }
      nlfree (namdir);
      srvldmas (1, &master, confnum);
      none = 0;
   }

   /*** User meant to display a particular subject or subjects. */
   else {
      names = nlnames (str, nullchix);
      ok = matchnames (usd, usf, confnum, owner, names, AND, &matches,
          NOPUNCT);
      nlfree (names);
      if (NOT ok)  goto done;
      if (nlsize (matches) == 0) {
         if (mine)  mdstr (format, "des_Enopsbjmch", null.md);
         else       mdstr (format, "des_Enosbjmch", null.md);
         chxformat (mtu, format, L(0), L(0), str, nullchix);
         chxconc   (mtu, mtu, ss_dotnl);
         unit_write (XWER, mtu);
         none = 0;
         goto done;
      } 
   
      /*** For each matching subject, display the full name. */
      for (this=matches->next;   this!=null.nl;   this=this->next) {
         /*** Find the subject number. */
         chxtoken (str, nullchix, 2, this->str);
         chxnum   (str, &subnum);

         /*** See if that subject is in this conference. */
         if ( (line = chixGetline (utd, utf, confnum, owner, 0, (int) subnum, 0, 
                            (Textbuf) NULL)) == nullchix)  continue;
         chxtoken (word, nullchix, 2, line);
         if (NOT chxeq (word, confname))  continue;

         /*** Display the subject name and list of attached items. */
         for (i=1; (line = chixGetline (utd, utf, confnum, owner, 0, (int) subnum,
                                    i, (Textbuf) NULL)) != nullchix;
              ++i) {
            unit_write (red->unit, line);
            unit_write (red->unit, CQ("\n"));
         }
         if (i == 2)  unit_write (red->unit, CQ("\n"));
      }
      nlfree (matches);
      none = 0;
   }
   
done:
   if (none)  mdwrite (XWER, "des_Enosub", null.md);
 
   free_textbuf (tbuf);
   chxfree (s);
   chxfree (str);
   chxfree (word);
   chxfree (owner);
   chxfree (format);
   chxfree (mtu);
   RETURN (none == 0);
}
