/*** ASSIGN.  Assign a value to a memory variable or a 
/             Site-Dependent Registration Information variable.
/
/    Usage syntax for the Caucus ASSIGN command:
/
/       ASSIGN  Vn  [string...]
/    or
/       ASSIGN  Un  [string...]
/
/    "Un" must be the name of one of the SDRI variables U0 thru U9.
/    "Vn" must be the name of one of the memory variables V0 through V9.
/    The rest of the command line ("string...") is placed into the selected
/    memory variable.  (Only the first 80 characters of SDRI variables or 
/    100 characters of memory variables are used; the rest is truncated.)
/    If no string is supplied, ASSIGN sets Un or Vn to the empty string.
*/

/*** Copyright (C) 1989 Camber-Roth.  All rights reserved. */

/*: CR  9/14/89 12:48 New function. */
/*: JV  8/07/90  8:55 Fix code so it checks for 'V' in 'Vn'.*/
/*: JV  8/07/90  9:15 Add Site-Dependent Reg Info.*/
/*: JX  2/22/92 10:30 Chixify. */
/*: JX  5/26/92 19:36 Fixify. */
/*: CX  8/06/92 11:42 REST must be 3rd arg on, not 2nd; trim blanks. */
/*: CR 10/11/92 13:04 Test return value from chxnum. */
/*: CL 12/11/92 11:13 Long Chxcatsub args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include "caucus.h"
#include <stdio.h>
#include "chixuse.h"
#include "done.h"

#define U 0
#define V 1

extern union  null_t null;
extern Chix   memvar[];
extern Chix   sitevar[];

FUNCTION  assign (command)
   Chix   command;
{
   int    vartype, success;
   int4   vnum;
   Chix   var, varnum, rest;

   ENTRY ("assign", "");

   var     = chxalloc (L(4),  THIN, "assign var");
   rest    = chxalloc (L(40), THIN, "assign rest");
   varnum  = chxalloc (L(4),  THIN, "assign varnum");

   /*** Check arguments: must be "ASSIGN Vn [string...]". */
   if (chxtoken (var, rest, 2, command) == -1) {
      mdwrite (XWER, "ass_Enovar", null.md);
      FAIL;
   }

   jixreduce (var);

   /*** Check variable name: U or V. */
   if      (chxvalue (var, L(0)) == 'u') vartype = U;
   else if (chxvalue (var, L(0)) == 'v') vartype = V;
   else {
      mdwrite (XWER, "ass_Enovar", null.md);
      FAIL;
   }

   /*** Parse memory variable number; must be in [0,9]. */
   chxcatsub (varnum, var, L(1), L(1));
   if (NOT chxnum (varnum, &vnum)  ||  vnum < 0  ||  vnum > 9) {
      mdwrite (XWER, "ass_Enovar", null.md);
      FAIL;
   }

   /*** Make sure that the memory for this variable has been allocated. */
   if (vartype == V && memvar[vnum] == null.chx) {
      if ((memvar[vnum] = chxalloc (L(102), THIN, "assign memvar")) == null.chx){
         mdwrite (XWER, "ass_Enomem", null.md);
         FAIL;
      }
   }

   chxtoken (nullchix, rest, 3, command);
   chxtrim  (rest);
   if (vartype == U) {
      chxclear  (sitevar[vnum]);
      chxcatsub (sitevar[vnum], rest, L(0), L(100));
   } else {
      chxclear  (memvar[vnum]);
      chxcatsub (memvar[vnum],  rest, L(0), L(100));
   }
   SUCCEED;

done:
   chxfree (var);
   chxfree (varnum);
   chxfree (rest);
   RETURN (success);
}
