/*** CHECK.  Check for new material in all conferences.  
/
/    new = check (comline);
/  
/    Parameters:
/       Chix comline;     (the command line containing a CHECK command)
/  
/    Purpose:
/       Users need to know which conferences to visit.  This command
/       tells them where to go...
/  
/    How it works:
/  
/    Returns:
/      1 if there was any new stuff in confs.
/      0 if there wasn't
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by: main/interpre.c
/  
/    Home: cmd2/check.c
/  */

/*** Copyright (C) 1988-1992 Camber-Roth.  All rights reserved. */

/*: CR  3/21/90 11:43 New function, taken from old allconfs(). */
/*: CR  6/21/90 12:37 Add LCNF arg to unit_wipe() call. */
/*: CR  7/17/90 13:48 Rearrange unit_wipe()s. */
/*: CR  9/14/90 11:42 Parse for output redirection; change arg to comline. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  5/01/91 13:07 Do not unit_wipe() or srvldmas() on confnum <= 0. */
/*: JV  5/15/91 17:38 Add new loadnpart arg. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: CR  7/02/91  0:00 Include "unitwipe.h". */
/*: CR  8/07/91 14:25 Add 'confnum' arg to syspdir() call. */
/*: JV 11/01/91 14:55 Don't check True Confnames. */
/*: JV 12/09/91 14:06 Don't check True Confnames (fix). */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: CR 12/31/91 11:37 Expand all char arrays. */
/*: CR  1/06/92 14:55 Load the message directory on each call. */
/*: JX  2/22/92 11:00 Chixify. Add header. */
/*: JX  5/19/92 19:48 Fix Chix. */
/*: CR 10/12/92 14:35 Correct conf num parsing. */
/*: CR 10/13/92  9:44 Fix chxformat, display of newmess. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */
/*: CP  8/10/93 13:37 Use new Redirect object in handling redirection. */
/*: CP  8/20/93 14:22 Use parse_rout(), not parse_rin(). */

#include <stdio.h>
#include "caucus.h"
#include "unitwipe.h"
#include "chixuse.h"
#include "redirect.h"
#include "done.h"

extern struct master_template  master;
extern struct message_t        mess;
extern union  null_t           null;
extern struct pdir_t           pdir;
extern short                   confnum;
extern Chix                    confname;

FUNCTION  check (comline)
   Chix  comline;
{
   struct namlist_t *namdir, *this, *nlnode();
   int      newmess, i, j, ok, new, savenum;
   int4     pos;
   int4     num, excl, true;
   Chix     id, savename, code, message, format;
   Redirect red;
   Textbuf  tbuf, make_textbuf();

   ENTRY ("check", "");

   id       = chxalloc (L(80), THIN, "id");
   code     = chxalloc (L(80), THIN, "code");
   format   = chxalloc (L(20), THIN, "format");
   message  = chxalloc (L(20), THIN, "message");
   savename = chxalloc (L(80), THIN, "savename");

   tbuf = make_textbuf (TB_RESPS, TB_LINES, "check tbuf");

   parse_rout     (&red, comline);
   apply_redirect (&red);
   unit_write (red.unit, NEWCPAGE);
   sysuserid  (id);

   /*** Count the number of new messages. */
   if (unit_lock (XUMD, READ, L(0), L(0), L(0), L(0), id)) {
      mess.highest   = loadmdir (0, mess.num);
      mess.high_seen = mess.highest;
      unit_unlk (XUMD);
   }
   for (i=1, newmess=0;   i<=mess.highest;   ++i)
      if (mess.num[i] & MSG_NEW)  ++newmess;

   /*** Tell the user about them. */
   mdstr   (format, "chk_Fmess", null.md);
   chxformat  (message, format, L(newmess), L(0), null.chx, null.chx);
   unit_write (red.unit, message);

   /*** Save the current conference number and name. */
   chxcpy (savename, confname);
   savenum = confnum;

   /*** Get a list of all conference numbers this user belongs to. */
   sysuserid (id);
   syspdir (&pdir, XUPA, 0, 0, id);
   new = 0;
   if (pdir.parts == 0)  goto done;

   /*** Load the directory of conference names. */
   if (NOT unit_lock (XSCD, READ, L(0), L(0), L(0), L(0), nullchix))  goto done;
   namdir = nlnode(4);
   ok = loadnmdir (XSCD, namdir);
   unit_unlk (XSCD);
   if (NOT ok)     { nlfree (namdir);            goto done; }
   
   /*** Load a partfile full of conference names. */
   if (confnum > 0)  unit_wipe(LCNF);
   for (this=namdir->next;   this != null.nl;   this = this->next) {
      if (sysbrktest())    break;
      if (this->num <= 0)  continue;
      if (NOT unit_lock (XSCN, READ, L(0), L(this->num), L(0), L(0), nullchix))
          continue;
      if (NOT loadnpart (XSCN, this->num, tbuf)) 
          { unit_unlk (XSCN);  continue; }
      unit_unlk (XSCN);
   
      /*** Check each conference by number: is this user a member?
      /    Only check Local Confnames. Pre-CaucusLink data will have
      /    a zero at CODE position 6. */
      for (i=0;   i<tbuf->l_used;   ++i) {
         if (sysbrktest())    break;
         chxtoken (confname, nullchix, 1, tbuf->line[i]);
         chxtoken (code,     nullchix, 2, tbuf->line[i]);
         excl = chxvalue (confname, L(0));
         if (EMPTYCHX (code) || excl == L('!')) continue;

         /*** Only check Local Confnames, but allow old syntax (6=='\0') */
         true = chxvalue (code, L(6));
         if (true == L('T'))    continue;

         /*** Extract the conf number from "o#002#T" format. */
         pos = 2;
         num = chxint4 (code, &pos);

         /*** If so, report on this conference. */
         for (j=0;   j<pdir.parts;   ++j)   if (num == pdir.resp[j]) {
            confcheck (red.unit, (int)num, &new);
            unit_wipe (LCNF);
            break;
         }
      }
   }
   nlfree (namdir);

   /*** Restore the current conference number and name. */
   chxcpy (confname, savename);
   confnum = savenum;
   if (confnum > 0)  srvldmas (1, &master, confnum);

done:
   if (red.locked) {
      unit_close (red.unit);
      unit_unlk  (red.unit);
   }

   free_textbuf (tbuf);
   chxfree (id);
   chxfree (code);
   chxfree (format);
   chxfree (message);
   chxfree (savename);

   RETURN  (new);
}
