/*** CONFCHECK.   Report on new items & responses in conference NUM. 
/
/   confcheck (out, num, anynew)
/
/  Parameters:
/     int    out;        (unit to write text)
/     int    num;        (conf number to report on)
/     int   *anynew;     (were there any new?)
/
/  Purpose:
/     CONFCHECK displays the number of new items and responses in a conf.
/
/  How it works:
/
/  Returns:
/     1 on 
/     0 on
/
/  Error Conditions:
/
/  Side Effects:
/
/  Related to:
/
/  Called by: check()
/
/  Home: cmd2/confchec.c
/
/ */

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:57 Source code master starting point */
/*: CR 10/09/89 11:35 Use XW unit codes. */
/*: CR  3/21/90 11:55 Add out argument, move to cmd2. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: JV 10/31/91 15:09 Declare sprintf */
/*: UL 12/11/91 18:42 Make unit_lock args 3-6 int4.*/
/*: JX  2/22/92 14:00 Chixify and add header. */
/*: JX  5/26/92 19:40 Fixify. */
/*: JX  7/23/92 17:40 Chxalloc thisconf->lastin. */
/*: CR 10/12/92 13:59 Modify comments. */
/*: JV 11/17/92 18:34 Allocate thisconf off heap, not stack. */
/*: CR 11/27/92 13:31 Declare sysmem(). */
/*: CL 12/11/92 15:52 Long sysmem arg. */
/*: CR  1/13/93 13:49 Use make_partic(), free_partic(). */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"

extern struct master_template master;
extern union  null_t          null;
extern short  confnum;
extern Chix   confname;

FUNCTION  confcheck (out, num, anynew)
   int    out, num, *anynew;
{
   int    success;
   struct partic_t thisconf;
   Chix   userid, format, mess;
   short  i, ok;
   int4   totalresp, newitems, newresp;
   char  *sysmem();

   ENTRY ("confcheck", "");

   userid = chxalloc (L(100), THIN, "confch userid");
   format = chxalloc (L(100), THIN, "confch format");
   mess   = chxalloc (L(100), THIN, "confch mess");
   make_partic (&thisconf, MAXITEMS);

   confnum = num;

   if (NOT srvldmas (1, &master, confnum))  FAIL;

   /*** Load this user's participation file for this conference. */
   sysuserid (userid);
   if (NOT unit_lock (XUPA, READ, L(num), L(0), L(0), L(0), userid))  FAIL;
   ok = load_user (&thisconf);
   unit_unlk (XUPA);
   if (NOT ok)   FAIL;

   for (newitems=0, i=thisconf.items+1;   i<=master.items;   ++i)
      if (master.responses[i] >= 0)  ++newitems;

   for (totalresp=newresp=0,  i=1;    i<=thisconf.items;     ++i)
      if (thisconf.responses[i] >= 0  && 
          thisconf.responses[i] <  master.responses[i]) {
         ++newresp;
         totalresp += (master.responses[i] - thisconf.responses[i]);
      }

   if (newitems > 0  ||  newresp > 0) {
      if (NOT *anynew) {
         mdwrite (out, "chk_Theader", null.md);
         *anynew = 1;
      }
      mdstr     (format, "chk_Fheader1", null.md);
      chxformat (mess, format, newitems, newresp, nullchix, nullchix);
      mdstr     (format, "chk_Fheader2", null.md);
      chxformat (mess, format, totalresp, L(0), confname, nullchix);
      unit_write (out, mess);
   }
   SUCCEED;

done:
   chxfree (userid);
   chxfree (format);
   chxfree (mess);
   free_partic (&thisconf);

   RETURN (success);
}
