/*** CONSEARCH.  Does the user wish to CONTINUE SEARCHING?
/
/    consearch (i, r, ask);
/
/    Parameters:
/       int   i    (item that had the hit)
/             r    (resp that had the hit)
/           *ask   (should we continue bugging the user)
/
/    Purpose:
/       Prompt the user.
/
/    How it works:
/
/    Returns:
/      Reply:                     Effect:
/        STOP, NO, and <CANCEL>   returns 0.
/        YES and OK               returns 1.
/        ALL                      returns 1, sets ASK to 0.
/
/        SHOW                prints the entire text of the item or response,
/                              and asks the question again.
/        NEXT                returns 2. (Skip rest of responses in this item)
/
/    Error Conditions:
/
/    Side Effects:
/
/    Related to:
/
/    Called by: searchit.c
/
/    Home: cmd2/consearch.c
/ */

/*** Copyright (C) 1987-1992 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:39 Source code master starting point */
/*: CR  9/06/89 10:35 Add 5th arg to show_resp.*/
/*: JX  2/22/92 14:30 Chixify and add header. */
/*: JX  5/26/92 19:43 Fixify. */
/*: CR  9/09/92 14:03 Add SHOWN arg to show_resp() call. */
/*: CI 10/06/92 16:49 Chix Integration. */
/*: JV  4/23/93 17:01 Add NEXT. */
/*: CP  7/01/93 16:04 Add 'brief' arg to show_resp() call. */
/*: CP  8/24/93 15:38 Add 'attach' arg to show_resp(), showtext() calls. */
/*: CK  9/19/93  0:07 Add 'onlyattach' arg to show_resp() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"

extern short confnum;

FUNCTION  consearch (i, r, ask)
   int    i, r, *ask;
{
   Chix   answer, yes, findtab;
   int    choice, success, shown;

   ENTRY ("consearch", "");

   yes     = chxalloc (L(16), THIN, "yes");
   answer  = chxalloc (L(40), THIN, "answer");
   findtab = chxalloc (L(40), THIN, "findtab");

   mdtable  ("fnd_Aconsrc", &findtab);
   chxtoken (yes, nullchix, 3, findtab);

   while (1) {
      choice = mdprompter ("fnd_Pconsrc", findtab, 2, "fnd_Hconsrc",
                                     10, answer, yes);
      if (choice <= 1)  FAIL;
      if (choice == 4)  *ask = 0;
      if (choice <= 4)  SUCCEED;
      if (choice == 6)  DONE (2);
      if (choice == 5) {
         unit_write (XWTX, NEWCPAGE);
         if (r==0)  showtext       (XWTX, i, 0, 0);
         else       show_responses (XWTX, confnum, i, r, r, 0, 0, 0, &shown, 0);
      }
   }

done:
   chxfree (findtab);
   chxfree (answer);
   chxfree (yes);
   RETURN  (success);
}
