/*** DO_CMI.  Handle "_cmi" command processing.
/
/    ok = do_cmi (thisuser);
/
/    Parameters:
/       int      ok;         (success?)
/       Userreg  thisuser;   (current user definition)
/
/    Purpose:
/       Process the user's outstanding "system" mail, and import
/       it as appropriate into the user's Caucus mailbox.
/  
/    How it works:
/       Check to make sure there's room for NEWMESS new messages.
/       If so, run the appropriate cmi macro with the name of a temp file.
/  
/    Returns: 1 on success
/             0 on failure
/             0 if no importing can or should be done
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Called by:  interpret()
/  
/    Home: cmd2/docmi.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CP  8/01/93 23:08 New function. */
/*: CR  3/16/95 18:11 Only do _cmi if there's room for NEWMESS new msgs. */

#include <stdio.h>
#include "caucus.h"

#define  NEWMESS    50   /* Must match #s in cmi_import.a, cmi_copyin.a! */

extern struct license_t       license;
extern struct message_t       mess;

FUNCTION  do_cmi (thisuser)
   Userreg  thisuser;
{
   char   cmi_macro[512], fname[512];

   ENTRY ("do_cmi", "");

   /*** If user wants, IMPORT mail from outside mail system. */
   if (thisuser->import_mail == 0)                           RETURN (0);
   if (NOT (PLAIN(license.function_mask) & FM_CMI))          RETURN (0);

   /*** Make sure there is room for at least NEWMESS messages... */
   unit_lock (XUMD, READ, L(0), L(0), L(0), L(0), thisuser->id);
   mess.highest   = loadmdir (0, mess.num);
   unit_unlk (XUMD);
   if (mess.highest > MAXMESS - NEWMESS - 10)                RETURN (0);

   if (NOT unit_lock (XITY, WRITE, L(0), L(0), L(0), L(0), CQ("")))  RETURN (0);

   unit_name (fname, XITY, L(0), L(0), L(0), L(0), CQ(""));

   /*** Choose Import (and delete), vs. Import (and copy) mail. */
   if (thisuser->import_mail == 1)   strcpy (cmi_macro, "_import_mail ");
   if (thisuser->import_mail == 2)   strcpy (cmi_macro, "_copyin_mail ");

   /*** Build the rest of the relevant cmi macro, and execute it. */
   strcat   (cmi_macro, fname);
   fire_afm (cmi_macro, nullchix, nullchix);

   sysparmail (XITY);         /* Parse mail and send to self. */
   unit_kill  (XITY);
   unit_unlk  (XITY);

   RETURN (1);
}
