/*** IS_MATCH.  Is there a match in STR for a list of WORDS?
/
/    ok = is_match (str, words, wordsneeded);
/
/    Parameters:
/       int       ok;            (match found?)
/       Chix      str;           (string to be searched)
/       Namelist  words;         (namelist of words to search for)
/       int      *wordsneeded;   (negative # of words needed to make a match)
/
/    Purpose:
/       Low-level search engine.  Searches STR for matches with words
/       in namelist WORDS.
/
/    How it works:
/       Is_match() examines each entry in namelist WORDS to see if
/       it is found in STR.  Entries in WORDS can be:
/          (a) single words; found if an initial substring of a word in STR
/          (b) wildcard pattern: "abc*def".  Found if "abc" appears as
/              the initial substring of a word in STR, followed *anywhere*
/              in STR by "def".
/
/       When an entry in WORDS is found, it is marked (in the num field)
/       of the namelist entry) as "found", and WORDSNEEDED is incremented.
/       (Entries already marked as found are not searched for.)
/
/    Returns: 1 if WORDSNEEDED reaches 0.
/             0 otherwise.
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies: None.
/
/    Known bugs:
/
/    Home:  cmd2/ismatch.c
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: CR 11/03/88 14:10 Change name on copyright. */
/*: CR 10/28/89 11:42 Add "sliding search" for JAPAN. */
/*: JX  2/25/92 14:30 Chixify. Needs to be finished. */
/*: DE  3/26/92 16:35 Patched chxindex w/ 0, */
/*: JX  5/13/92 15:39 Removed unnecessary POS. */
/*: CR  8/03/92 17:53 Handle matching "some*thing". */
/*: CI 10/06/92 17:41 Chix Integration. */
/*: CR 10/12/92 12:34 Fix 'chxindex (str, i, fore) == i'. */
/*: CR 12/10/92 15:29 chxindex 2nd arg is int4. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern union null_t null;

FUNCTION  is_match (str, words, wordsneeded)
   struct namlist_t *words;
   Chix   str;
   int   *wordsneeded;
{
   struct namlist_t *p;
   Chix   blanks, fore, aft;
   int    success;
   short  between;
   int4   b, c, i, j;

   ENTRY ("is_match", "");

   blanks = chxalloc (L(10), THIN, "ismatch blanks");
   fore   = chxalloc (L(10), THIN, "ismatch fore");
   aft    = chxalloc (L(10), THIN, "ismatch aft");

   chxspecial (SC_GET, SC_BLANK, blanks);

   /*** The controlling loop examines STR one word at a time. */
   between = 1;
   for (i=0;   c=chxvalue (str, L(i));    i++) {

      /*** Is C a blank?  If so, set BETWEEN, and go on. */
      for (j=0;   (b = chxvalue (blanks, L(j))) != L(0);   ++j) 
          if (c == b)  break;
      if (c == b)  { between = 1;   continue; }

      if (NOT between)              continue;

      /*** Here at the start of a word in STR.  Consider each element P of
      /    WORDS, and see if it is an initial substring of this word. */
      between = 0;
      for (p=words->next;   p!=null.nl;   p = p->next)   if (p->num) {

         chxbreak (p->str, fore, aft, CQ("*"));

         if (EMPTYCHX(fore)    ||  chxindex (str, i, fore) == i) {
            if (EMPTYCHX(aft)  ||  chxindex (str, i, aft)  >= 0) {
               p->num = 0;
               if (++*wordsneeded == 0)  SUCCEED;
            }
         }
      }
   }

   FAIL;

done:
   chxfree (blanks);
   chxfree (fore);
   chxfree (aft);
   RETURN  (success);
}
