/*** MENU_COMMAND.  Execute a command from a menu, and process RETURNs.
/
/    menu_command (command, cname, next, base, menu_level, action)
/
/  Parameters:
/     Chix  command          (
/             cname          (
/              next          (
/              base          (
/     int   *action          (
/        menu_level          (
/
/  Purpose:
/    MENU_COMMAND executes the command stream COMMAND by calling
/    interpret().  If a RETURN was encountered, MENU_COMMAND 
/    processes the RETURN command, and decides how far back in the
/    menu stack control must return.
/
/  How it works:
/    BASE is the base name of the macro, and MENU_LEVEL is the menu
/    stack depth.
/
/  Returns:
/    MENU_COMMAND returns 1 if a regular command stream was executed,
/    and 0 if a RETURN (or JOIN or STOP) was found.  In the latter
/    case, it sets one of the following ACTION codes for the caller:
/
/       0: interpret() saw a JOIN or STOP command.  The name of the next
/          conference is returned in NEXT.
/
/       1: a RETURN was encountered that pops back one (and only one) level;
/          or, an invalid menu name was given on the command line.
/
/       2: a RETURN was encountered that may pop back multiple levels.
/
/  Error Conditions:
/
/  Side Effects:
/
/  Related to:
/
/  Called by: 
/
/  Home:  cmd2/menucomm.c
/ */

/*** Copyright (C) 1989-1992 Camber-Roth.  All rights reserved. */

/*: CR  9/08/89 15:53 New function. */
/*: JV 10/31/91 15:08 Declare sprintf */
/*: JX  2/25/92 14:31 Chixify, add header. */
/*: DE  5/26/92 16:34 Change stray "" to null.chx */
/*: CR 10/11/92 12:55 Only treat LEVEL as a number if is_num is true. */
/*: CR 10/28/92 16:46 Clear next before chxformat. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern union null_t null;

FUNCTION  menu_command (command, cname, next, base, menu_level, action)
   Chix  command, cname, next, base;
   int   *action, menu_level;
{
   int4   level;
   int    is_num, is_match;

   ENTRY ("menu_command", "");

   while (1) {
      /*** Interpret that macro text, and get back the ACTION code.
      /    1 means continue looping in the caller's menu.
      /    0 means JOIN or STOP, so we pop all the way back.
      /    2 means RETURN, and we must process its argument, and
      /      decide if we've returned far enough. */
     *action = interpret (command, cname, next, null.chx);
   
      if (*action == 1)  RETURN (1);
   
      if (*action == 0)  RETURN (0);
   
      /*** Only remaining case is *action == 2. */
      if (EMPTYCHX (next))  chxcpy (next, CQ("-1"));
      is_num   = chxnum (next, &level);
      is_match = chxeq  (next, base);

      /*** If "RETURN name" matches this menu, or "RETURN N" matches
      /    this level, or N is 0, we've returned as far as we need to,
      /    and should continue to INTERPRET() this command stream. */
      if (is_match  || 
         (is_num    &&  (level == menu_level  ||  level == 0)))  continue; 

      /*** Otherwise return one more level.  ("RETURN -n" must be
      /    incremented first.) */
      if (is_num  &&  level <  0)  {
         chxclear  (next);
         chxformat (next, CQ("%d"), ++level, L(0), null.chx, null.chx);
      }
      if (is_num  &&  level == 0) *action = 1;
      RETURN (0);
   }
}
