/*** SEARCHINTRO.   Search the intro text of a person.
/
/    found = searchintro (user, forlist, needed);
/
/    Parameters:
/       int               found;    (success?)
/       Userreg           user;     (user's registration info)
/       struct namlist_t *forlist;  (list of words to search for)
/       int               needed;   (match needs negative # of words)
/
/    Purpose:
/       Searchintro() is a low-level part of the chain of functions
/       that implement the SEARCH PERSON command.  It searches the
/       intro text of a person for the desired words, and returns
/       "hit" or "miss".
/
/    How it works:
/       Assumes user registration info is in USER, namelist of words
/       to search for is in FORLIST.  NEEDED is (negative) number of
/       words needed from FORLIST to make a "hit".
/
/    Returns: 1 on "hit", 0 on "miss".
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  searchperson()
/
/    Operating system dependencies: None.
/
/    Known bugs:
/
/    Home:  cmd2/searchin.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  8/02/92 20:14 New function. */
/*: CI 10/07/92 17:53 Chix Integration. */

#include <stdio.h>
#include "caucus.h"

extern union  null_t        null;

FUNCTION   searchintro (user, forlist, needed)
   int     needed;
   Userreg user;
   struct  namlist_t *forlist;
{
   struct  namlist_t *p, *intro;
   Chix    textline;
   int     success;

   ENTRY ("searchintro", "");

   textline = chxalloc (L(100), THIN, "searchintro textline");

   /*** Initialize counters, including "not found" marker for each word. */
   for (p=forlist->next;   p!=null.nl;   p = p->next)  p->num = 1;

   for (intro=user->briefs->next;   intro!=null.nl;   intro = intro->next) {
      chxcpy    (textline, intro->str);
      jixreduce (textline);
      if (is_match (textline, forlist, &needed))  SUCCEED;
   }

   FAIL;

done:
   chxfree (textline);
   RETURN  (success);
}
