
/*** SEARCHITEM.  Search items & responses for WORDS.
/
/    ok = searchitem (red, range, words, needed, saidlist, full);
/
/    Parameters:
/      int       ok;        (success?)
/      Redirect *red;       (redirection information: where to show results)
/      Chix      range;     (range of items to search)
/      Namelist  words;     (words to search for)
/      int       needed;    (how many words are needed for a match)
/      Namelist  saidlist;  (who might have said these words?)
/      int       full;      (display full text of match?)
/ 
/    Purpose:
/       Search selected items and responses for WORDS.
/ 
/    How it works:
/       If RANGE is empty, prompt the user for a RANGE.
/
/       Call parse_or_ask() to get the Rlist of selected items & responses.
/
/       For each selected response, call lookthrough() to do the
/       actual searching and display of hits.
/ 
/    Returns: 1 on success,
/             0 on failure.
/ 
/    Error Conditions:
/ 
/    Side Effects:
/ 
/    Related to:
/ 
/    Called by:
/ 
/    Home: cmd2/searchit.c
/ */

/*: AA  7/29/88 17:40 Source code master starting point */
/*: JV  8/23/89 16:54 Chg'd err mess for invalid item range. */
/*: JV  9/15/89 18:22 Added _beg_searchitem and _end_searchitem AFM's */
/*: CR 10/09/89 11:30 Use XW unit codes. */
/*: CR 10/11/89 16:11 Revert to "no items selected" if parserange fails. */
/*: CR 10/17/89 15:17 Replace 2nd arg of FIRE_ARM with "". */
/*: JX  2/25/92 16:00 Chixify, add header. */
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */
/*: DE  5/26/92 16:02 Fix chixification */
/*: CR  8/02/92 18:02 Add SAIDLIST argument. */
/*: CR  8/23/92 21:56 Add FULL display argument. */
/*: CI 10/07/92 15:22 Chix Integration. */
/*: CR 10/12/92 12:46 Report success properly. */
/*: CR  1/25/93 12:20 Always call parse_range() on range! */
/*: JV  4/23/93 16:55 Add NEXT to jump directly to next item. */
/*: CN  5/15/93 15:20 Replace parse_range with new_parse(). */
/*: CT  5/28/93 13:47 Replace SELECT with Rlist. */
/*: CP  6/25/93 14:04 Add new args to new_parse() call. */
/*: CP  7/01/93 20:39 Remove old 'doitem' arg, use Rlist properly. */
/*: CP  8/06/93 15:18 Use r_bound() to limit first/last responses. */
/*: CP  8/09/93 11:41 Add 'attach' arg to new_parse() call. */
/*: CP  8/30/93 15:23 Replace 'out' arg with 'red'. */
/*: CP  9/10/93  9:51 Replace new_parse() with parse_or_ask(). */
/*: CK  9/15/93 14:04 Add 'mod1st' arg to parse_or_ask(). */
/*: CK 10/07/93 17:05 Use &thisitem in parse_or_ask() call. */
/*: CP 11/18/93 15:35 Add cnum, inum args to r_bound() call. */
/*: CP 12/10/93 21:31 Add 'final' arg to r_bound() call. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "api.h"
#include "modifier.h"
#include "redirect.h"
#define   RLNULL   ( (Rlist) NULL )

extern struct master_template master;
extern struct this_template   thisitem;
extern struct fresh_template  fresh;
extern struct partic_t        thisconf;
extern short                  confnum;
extern union  null_t          null;
extern Chix                   ss_all, ss_dotnl;
extern Chix                   afm_name;

FUNCTION  searchitem (red, range, words, needed, saidlist, full)
   int       full;
   Redirect *red;
   Chix      range;
   struct    namlist_t *words, *saidlist;
   int       needed;
{
   int    ask, first, last, success, r, i, cont, final;
   char   modifier[MODIFIERS+5];
   Chix   newconf;
   Rlist  rptr, result;

   ENTRY ("searchitem", "");

   /* First fire off initial AFM. */
   newconf = chxalloc (L(20), THIN, "newconf");
   fire_afm ("_beg_searchitem", null.chx, newconf);
   success = 0;
   result  = RLNULL;
  
   if (NOT parse_or_ask (range, &result, modifier, nullchix, confnum, 
                                &thisitem, 1))  FAIL;

   /*** Proceed to search the items chosen by the user. */
   ask     = (red->unit == XWTX);
   for (rptr=result->next;   rptr!=RLNULL;   rptr=rptr->next) {
      for (i=rptr->i0;   i<=rptr->i1;   ++i) {
         if (i == 0)                                           continue;
         if (NOT r_bound (&first, &last, &final, rptr, confnum, i,
                                        master.responses[i]))  continue;
         if (sysbrktest())                                     break;
  
         /*** Search the selected responses. */
         for (r=first;   r<=last;   ++r) {
            if (sysbrktest())  break;
  
            if (rightauthor (i, r, saidlist)  &&
                lookthrough (red->unit, i, r, words, needed, full)) {
               success = 1;
               if (ask) {
                  cont = consearch (i, r, &ask);
                  if (cont == 0)  SUCCEED;  /* That was the match. */
                  if (cont == 2)  break;    /* Skip rest of this item. */
               }
            }
         }
      }
   }

   if (NOT success)  mdwrite (XWTX, "fnd_Tfailed", null.md);

done:
   /* Finally fire closing AFM. */
   fire_afm    ("_end_searchitem", null.chx, newconf);
   a_fre_rlist (result);
   chxfree  (newconf);
   RETURN   (success);
}
