
/*** CONF_ENTRY.  Find the MISC/confs001 entry for a unique conference.
/
/   ok = conf_entry (name, result)
/
/   Parameters:
/      Chix   name;     (conference name)
/      Chix   result;   (resulting entry from confs001 file)
/
/    Purpose:
/
/    How it works:
/       CONF_ENTRY takes NAME, a fully specified conference name,
/       and looks for a unique match in the system XSCD/XSCN conference 
/       name tables.
/   
/    Returns:
/        0   failure
/        n   success
/
/    Error Conditions:
/  
/    Known bugs:      none
/
/    Home:  conf/confentry.c
/ */

/*** Copyright (C) 1997 Screen Porch.  All rights reserved. */

/*: CR  9/16/97 15:08 New function, based on cnumber(). */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern union null_t null;

FUNCTION  int conf_entry (Chix name, Chix result)
{
   struct namlist_t *list, *owners, *this, *nlnode();
   Chix   fullname;
   int    success, ok, size;

   ENTRY ("conf_entry", "");

   fullname = chxalloc (L(80), THIN, "fullname");

   /*** Get namelist of conferences matching NAME. */
   list = nlnode (1);
   nladd (list, name, 0);
   ok = matchnames (XSCD, XSCN, 0, null.chx, list, AND, &owners, NOPUNCT);
   nlfree (list);
   if (ok != 1)  RETURN (0);

   /*** If no matches, return and complain. */
   if ( (size = nlsize (owners)) == 0) {
      nlfree (owners);
      RETURN (0);
   }

   /*** Scan the list of matches and find the one exact match. */
   success = 0;
   for (this=owners->next;   this!=null.nl;   this = this->next) {
      chxtoken  (fullname, null.chx, 1, this->str);
      if (chxeq (fullname, name)) {
         chxcpy (result, this->str);
         success = 1;
         break;
      }
   }

   chxfree (fullname);
   nlfree  (owners);
   RETURN  (success);
}
