
/*** MV_CONF.  Rename a conference.
/
/    ok = mv_conf (name1, name2, lib)
/
/    Parameters:
/       Chix  name1;          Original full name of the conference.
/       Chix  name2;          New full name of the conference.
/       char *lib;            root path of caucus (web) file library.
/
/    Purpose:
/
/    How it works:
/       If LIB is non-null, rename the lib directory for this
/       conference as well.
/
/    Returns: 0 name1 doesn't exist
/            -1 name2 does exist (rename fails)
/             1 success
/
/    Error Conditions:
/ 
/    Known bugs:
/
/    Home:  conf/mvconf.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  9/16/97 15:32 New function, based on makeconf(). */

#include <stdio.h>
#include "unitcode.h"
#include "handicap.h"
#include "derr.h"
#include "conf.h"
#include "chixuse.h"
#include "namelist.h"
#include "null.h"
#include "done.h"

extern union null_t     null;

FUNCTION  mv_conf (Chix name1, Chix name2, char *lib)
{
   Chix     result1, result2;
   int      success, cnum;
   char     temp[200], dir1[256], dir2[256];
   char     *ptr;
   Namelist add, del, nlnode();

   ENTRY ("mv_conf", "");

   result1 = chxalloc (L(80), THIN, "mv_conf result1");
   result2 = chxalloc (L(80), THIN, "mv_conf result2");

   if (NOT conf_entry (name1, result1))  DONE( 0);
   if (    conf_entry (name2, result2))  DONE(-1);

   chxcpy   (result2, result1);
   chxalter (result2, L(0), name1, name2);
   
   add = nlnode(1);
   del = nlnode(1);
   nladd (del, result1, 0);
   nladd (add, result2, 0);

   modnames (XSCD, XSCN, nullchix, 0, L(0), add, del, NOPUNCT);
   nlfree   (add); 
   nlfree   (del);

   ascofchx (temp, result1, L(0), L(200));
   for (ptr=temp;   *ptr && *ptr!='#';   ++ptr) ;
   sscanf (ptr+1, "%d", &cnum);
   
   /*** Update the TrueConf file. */
   unit_lock  (XCTC, WRITE, L(cnum), L(0), L(0), L(0), nullchix);
   unit_make  (XCTC);
   unit_write (XCTC, name2);   unit_write (XCTC, CQ("\n"));
   unit_close (XCTC);  
   unit_unlk  (XCTC);

   /*** Clear the trueconf cache. */
   get_trueconf (-1, nullchix, nullchix, nullchix);

   /*** Move the (web) conference library. */
   if (lib != NULL  &&  lib[0]) {
      sprintf   (dir1, "%s/%s", lib, ascquick(name1));
      sprintf   (dir2, "%s/%s", lib, ascquick(name2));
      if (syscheck (dir1))  sysrename (dir1, dir2);
   }
   DONE (1);

done:
   chxfree (result1);
   chxfree (result2);
   RETURN  (success);
}
