
/*** OKCONF.  Is this a legal conference name?
/
/    ok = okconf (name);
/  
/    Parameters:
/       int   ok;     (ok name?)
/       Chix  name;   (name of new conference)
/  
/    Purpose:
/      Is this an OK name for a conference?
/  
/    How it works:
/  
/    Returns: 1 if conference name is ok
/             0 if no name
/            -1 name too int4
/            -2 does not begin with a letter
/            -3 characters outside of [a-z][0-9]_.
/            -4 reserved words: STOP, HELP, LIST
/  
/    Error Conditions:
/  
/    Side Effects:
/  
/    Related to:
/  
/    Home: conf/okconf.c
*/

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  9/17/97 17:02 New function, based on old oknewconf */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "chixuse.h"

#define  MAX_CONFNAME  80

extern union null_t null;

FUNCTION  int okconf (Chix name)
{
   int    good, v, len, i;
   static Chix  jointab, helptab;

   good = 1;

   /*** Is the name too int4, empty, or not start with a letter? */
   if (EMPTYCHX (name))                           return (0);
   len = chxlen (name);
   if (len > MAX_CONFNAME)                        return (-1);
   if (NOT cf_lower((char) chxvalue (name, L(0))))  return (-2);

   /*** Allow only letters, numbers, ".", and _'s. */
   for (i=0;   i<len;   i++) {
      v = chxvalue (name, L(i));
      if (cf_lower (v))  continue;
      if (cf_digit (v))  continue;
      if (v == '_')      continue;
      if (v == '.')      continue;
      good = 0;
      break;
   }

   if (NOT good)                                  return (-3);

   /*** Do not allow reserved words like LIST, STOP, HELP. */
   mdtable ("mai_Ajoin",    &jointab);
   mdtable ("gen_Aprmhelp", &helptab);
   if (tablefind  (jointab, name) >= 0  ||
       tablematch (helptab, name) >= 0)           return (-4);


   return (1);
}
