/*** RMCONF.  Remove a Caucus conference.
/
/   int ok = rmconf (confnum)
/
/   Parameters:
/      int  confnum;   (Local Conference Number of conference)
/
/   Purpose:
/      This is the analogue of MAKECONF().  RMCONF() removes all traces
/      of a conference, including all of the conference DB, the Users'
/      Participation Files, and the conference entry(ies) in XSCN.
/
/   How it Works:
/
/   Returns:
/      1 on success
/      0 on failure
/
/   Error Conditions:
/      Couldn't lock or load member directory.
/
/   Related Functions: make_conf
/
/   Called by: cremove, reqreport
/
/   Home: conf/rmconf.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: JV  1/01/92 11:08 Source code master starting point */
/*: CR  9/25/92 17:12 If get_trueconf fails, use confname. */
/*: DE  5/26/92 12:45 Fixed inconsistant args */
/*: CI 10/07/92 11:10 Integration Clean-up. */
/*: CI 10/07/92 17:51 Chix Integration. */
/*: CP  5/06/92 12:41 Add L(0) as 'item' arg to modnames() call. */
/*: CP  8/01/93 14:32 Add 'name' arg to make_textbuf() call. */
/*: JV  9/26/93 13:13 Add ENTRY. */
/*: JZ 10/01/93 11:27 Remove file attachment directory. */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"
#include "done.h"
#include "textbuf.h"

extern Chix                    confname;
extern union  null_t           null;

FUNCTION  rmconf (int confnum)
{
   char   attdir[120];
   struct namlist_t *namdir, *this, *nlnode();
   short   i, ok, success;
   Chix    id, tname, lname, orig_host;
   Textbuf tbuf, make_textbuf();

   ENTRY ("rmconf", "");

   id          = chxalloc (L(40),  THIN, "id");
   tname       = chxalloc (L(100), THIN, "tname");
   lname       = chxalloc (L(100), THIN, "lname");
   orig_host   = chxalloc (L(100), THIN, "orig_host");
   tbuf = make_textbuf (TB_RESPS, TB_LINES, "rmconf tbuf");

   /*** Get the TrueConf name, if it exists. */
   if (NOT get_trueconf (confnum, lname, tname, orig_host)) {
      chxcpy (lname, confname); 
   }
   if (NOT EMPTYCHX(tname)) {
      chxcat (tname, CQ("@"));
      chxcat (tname, orig_host);
   }

   /*** Go through the list of members of this conference.  For each
   /    one, delete the participation file. */
   if (NOT unit_lock (XCMD, READ, L(confnum), L(0), L(0), L(0), nullchix)) 
       FAIL;
   namdir = nlnode(4);
   ok = loadnmdir (XCMD, namdir);
   unit_unlk (XCMD);
   if (NOT ok)     { nlfree (namdir);      FAIL; }

   /*** Load each namepart file in turn, and delete XUPA file for
   /    each user. */
   for (this=namdir->next;   this != null.nl;   this = this->next) {
      if (this->num <= 0)  continue;
      if (NOT unit_lock (XCMN, READ, L(confnum), L(this->num),
          L(0), L(0), nullchix))
       continue;
      if (NOT loadnpart (XCMN, this->num, tbuf))
         { unit_unlk (XCMN); continue; }
      unit_unlk (XCMN);

      for (i=0;   i < tbuf->l_used;   ++i) {
         if (chxtoken (id, nullchix, 2, tbuf->line[i]) == -1)
       continue;
         if (unit_lock (XUPA, WRITE, L(confnum), L(0), L(0), L(0), id)) {
             unit_kill (XUPA);
             unit_unlk (XUPA);
         }
      }
   }
   nlfree (namdir);

   /*** Remove the conference name from the master list of conferences. */
   cremove_name (lname, confnum);
   if (NOT EMPTYCHX(tname)) cremove_name (tname, confnum); 

   /*** Finally, delete the conference and attachment directories. */
   sysrmconf  (confnum);
   unit_name  (attdir, XACD, L(confnum), L(0), L(0), L(0), nullchix);
   sysrmdir   (attdir);

   /*** Clear the trueconf cache. */
   get_trueconf (-1, nullchix, nullchix, nullchix);

   SUCCEED;

done:
   free_textbuf (tbuf);
   chxfree (id);
   chxfree (tname);
   chxfree (lname);
   chxfree (orig_host);

   RETURN (success);
}


FUNCTION  cremove_name (Chix name, int confnum)
{
   struct namlist_t *what, *match, *add, *del, *p, *nlnode();
   int    success;
   Chix   conf;
   char   temp[100];

   ENTRY ("cremove_name", "");
   conf      = chxalloc (L(40), THIN, "crem conf");

   /*** Find a list of matches for NAME. */
   what = nlnode(1);
   nladd  (what, name, 0);
   matchnames (XSCD, XSCN, 0, nullchix, what, AND, &match, NOPUNCT);
   nlfree (what);

   /*** Find the one that EXACTLY matches NAME. */
   for (p=match->next;   p!=null.nl;   p=p->next) {
      chxtoken (conf, nullchix, 1, p->str);
      if (chxeq (conf, name))  break;
   }
   if (p==null.nl) { nlfree (match);  FAIL; }
 
   /*** Delete that matching entry from XSCN. */
   add = nlnode(1);
   sprintf (temp, "!del o#%d", confnum);
   nladd (add, CQ(temp), 0);
   del = nlnode(1);
   nladd (del, p->str, 0);
   modnames (XSCD, XSCN, nullchix, 0, L(0), add, del, NOPUNCT);
   nlfree (match);
   nlfree (del);
   nlfree (add);
   SUCCEED;

done:
   chxfree (conf);
   RETURN (success);
}
