
/*** SWEB.H.   Include file for SWEB programs. */
 
/*** Copyright (C) 1995 Screen Porch LLC.  All rights reserved.  */

/*: CR  2/27/95 22:25 New file. */

#include "systype.h"
#include "entryex.h"
#include "object.h"

#ifndef  SWEB
#define  SWEB  1

/*** Define common "readability" macros. */
#define  FUNCTION
#define  NOT               !
#define  INFINITY          1000000

#ifndef  L
#define  L(x)              ( (int4) x )
#endif

#ifndef  NULLSTR
#define  NULLSTR(x)        (!(x[0]))
#endif

/*** Server port number for swebd: used for NT only.   Should eventually
/    become a configurable number in swebd.conf. */
#define  SERVER_PORT  8023

/*** Define codes for logger() calls. */
#include "logger.h"

/*** Define pipe buffer size, and read and write size. */
#define  PIPEBUF   8192
#define  PIPEWRITE 4096
#define  PIPEREAD  8192

#if LNX12
#undef   PIPEWRITE
#define  PIPEWRITE  512
#endif

/*** Define "hose" types & info. */
#if UNIX | NUT40
typedef int     Hose;
#define BADHOSE  (-1)
#endif
#if WIN32
#include <windows.h>
#include <iostream.h>
typedef HANDLE  Hose;
#define BADHOSE  ((HANDLE) NULL)
#define USE_OVERLAP    0
#endif

/*** Max # of attempts by swebsock to connect to swebd. */
#define  MAX_ATTEMPT   3

/*** Define "socket message" stuff. */
#define  SOCKMSGMAX 4096 - 4 - 4 - 1
struct sockmsg_t {
   int   len;
   int   eof;
   char  buf[SOCKMSGMAX+1];
};

/*** For use in "func_xxx()" calls; must use arg RESULT. */
#define  BAD(x)            chxcat (result, CQ("<I>"x"</I>"))

/*** Name of swebfail file. */
#define  SWEBFAIL          "swebfail.html"

/*** Max size of CML file. */
#define  MAXCML     2000

/*** Minimum required amount of free disk space */
#define  MIN_DISK_SPACE  7000    /* KB */
#define  MIN_DISK_FILES   500
#define  LICE_NODISK       10

/*** Bit masks for item/response property values. */
#define  M_TEXTTYPE   0x000F
#define  M_BITS       0xFFF0
#define  M_ISORG      0x0010

/*** Bit masks for manager features. */
#define  MGR_SET      0x001    /* set list of managers! */
#define  MGR_CRCONF   0x002    /* create conference */
#define  MGR_RMCONF   0x004    /* remove conference */
#define  MGR_BEORG    0x008    /* become organizer */
#define  MGR_IS       0x010    /* is a manager */
#define  MGR_CHGPASS  0x020    /* can change passwords */
#define  MGR_MKID     0x040    /* can add ids */
#define  MGR_RMID     0x080    /* can remove ids */
#define  MGR_RESERV   0x100    /* 001 -> 100 reserved */

/*** Max number of scripts to auto-run from swebd on start-up. */
#define  MAX_SCRIPTS     20

typedef  struct sweb_conf_t * SWconf;
struct sweb_conf_t {

   /*** This information is in the initial, primary server daemon. */
   char  hostname   [200];      /* Name of this host.  */
   char  filename   [200];      /* Configuration file. */
   char  caucus_name[ 20];      /* Caucus userid (string). */
   int   caucus_id;             /* Run as effective uid caucus_uid. */
   int   real_id;               /* Run as "real" id real_id. */
   int   caucus_group;          /* Run as "effective" group. */
   int   real_group;            /* Run as "real" group. */
   char  caucus_path[200];      /* pathname of caucus directory. */
   char  cml_path   [200];      /* pathname of CML directory. */
   char  pw_prog    [200];      /* password-changing program. */
   char  pw_path    [256];      /* password file, if passprog is "internal". */
   int   pw_can_add;            /* passprog can add uid/pw? */
   int   pw_can_change;         /* passprog can change uid/pw? */
   int   pw_can_delete;         /* passprog can delete uid/pw? */
   int   pw_can_verify;         /* passprog can verify uid/pw? */
   int   pw_lock;               /* passfile should be locked? */
   int   diskform;              /* Disk_Format code (character set) */
   int   browserform;           /* Browser_Format code (character set) */
   int   check_client_ip;       /* Test client ip for consistency? */
   int   timeout;               /* sub-server timeout after no requests. */
   FILE *verbose;               /* debugging output? */
   char  caucuslib  [200];      /* Caucus library directory. */
   char  httplib    [200];      /* URL of Caucus library directory. */
   char  adminmail  [200];      /* E-mail of administrator */
   int   pagetime;              /* Timeout for swebsock. */
   char *scripts[MAX_SCRIPTS];  /* Scripts to run on swebd start-up. */
   char  url_end [256];         /* Chars that mark the end of a URL. */
   char  url_trim[256];         /* Chars that should be trimmed from URL end. */

   /*** The rest is specific to each instance of the subserver. */
   char  remote     [100];      /* remote client's userid. */
   char  ip_addr    [100];      /* remote client's ip address */
   char  cmlname    [200];      /* cml file name and dir. */
   int   secure;                /* security code to identify this subserver. */
   int   output;                /* redirected output file descriptor. */
   int   new_win_x;             /* new_win width */
   int   new_win_y;             /* new_win height */
   int   reload;                /* is current page a reload of imm prev page? */
   int   cache_timeout;         /* some caches expire after this much time. */
};


/*** A CML file, as read in by loadcml(). */
typedef  struct cmlfile_t * CML_File;
struct cmlfile_t {
   char  filename[200];         /* original filename. */
   int   cdex;                  /* index to current line in lines[], lnums[] */
   Chix  errtext;               /* accumulated error text */
   Chix  lines[MAXCML];         /* Chix text of each line. */
   short lnums[MAXCML];         /* original line numbers */
};


/*** HTTP cgi environment variables. */
struct http_t {
   char  user_agent[100];
   char  referer   [300];
};


/*** Function prototypes. */
FUNCTION  int   cml_globals (void);
FUNCTION  void  clean_newlines (Chix x);
FUNCTION  int   fix_query   (char *query);
FUNCTION  int   touch_cl    (int4 cnum);
FUNCTION  int   func_cl     (Chix result, char *what, Chix arg);
FUNCTION  int   func_my     (Chix result, char *what, Chix arg);
FUNCTION  int   func_peo    (Chix result, char *what, Chix arg);
FUNCTION  int   func_re     (Chix result, char *what, Chix arg, int reload);

FUNCTION  int   stralter    (char *a, char *b, char *c);

#endif
