
/*** MDSTR.   Get a text string from the Master Dictionary.
/
/    ok = mdstr (str, name, save);
/
/    Parameters:
/       int     ok;      (success?)
/       Chix    str;     (place to put result)
/       char   *name;    (name of string to fetch)
/       Chix   *save;    (place to save memory-resident strings)
/
/    Purpose:
/       Look in the current dictionary for a string called NAME.  Put
/       it in STR.
/
/    How it works:
/       If the string is marked memory-resident in the Master Dictionary,
/       and SAVE is not NULL, and *SAVE is NULL, then:
/         (a) allocate a chix.
/         (b) set SAVE to point to that chix.
/         (c) copy the string into that chix.
/
/       This also means that on future calls to MDSTR, if *SAVE is already
/       set to point to the memory-resident copy of the string, then it
/       is copied directly to STR w/o going to the Master Dictionary.
/
/       (For memory-resident strings, MDSTR also remembers which dictionary
/       the string came from.  If the user switches dictionaries, MDSTR will
/       take note and load the string from the new dictionary.)
/
/    Returns: 1 on success, 0 if no string NAME in the dictionary.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  inout/mdstr.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:41 Source code master starting point*/
/*: CR 11/10/88 14:21 Complain if we can't find string. */
/*: CR 11/22/88 12:24 Modify header comments. */
/*: CR  9/12/89 16:42 Declare char *bugtell(). */
/*: CR 10/22/89 19:38 Use 2 bytes to hold dictionary number for string. */
/*: CR  8/12/90 19:18 Cast sysmem() call. */
/*: JV  7/22/90 18:03 Cast arg to sysmem to shut lint up. */
/*: CX 10/20/91 11:43 Chixify, redo header comments. */
/*: CX  5/09/92 20:53 Change to use sysmdstr(). */
/*: CL 12/11/92 15:52 Long sysmem arg. */

#include <stdio.h>
#include "caucus.h"
#include "diction.h"

extern union  null_t    null;
extern Userreg          thisuser;

FUNCTION  mdstr (str, name, save)
   Chix   str;
   char  *name;
   Chix  *save;
{
   int    status;
   char   temp[100];
   char  *bugtell();

   ENTRY ("mdstr", "");

   if (save != null.md  &&  *save != null.chx) {
      if ((*save)->dic == thisuser->dict) { chxcpy (str, *save);   RETURN (1); }
      chxfree (*save);
      *save = null.chx;
   }

   if (NOT sysmdstr (name, thisuser->dict, str, &status)) {
      bugtell   ("MDSTR: Can't find string: ", 0, L(0), name);
      sprintf (temp, "(no string for '%s')", name);
      chxofascii (str, temp);
      RETURN (0);
   }

   str->dic = thisuser->dict;

   if ( (status & MDRES)  &&  (save != null.md)) {
      *save = chxalloc (L(2), THIN, "mdstr *save");
      chxcpy (*save, str);
   }

   RETURN (1);
}
