
/*** FIRE_AFM.   Execute an auto-fire macro.
/
/    code = fire_afm (name, cname, cnext);
/
/    Parameters:
/       int    code;        (returned execution status of macro)
/       char  *name;        (name of auto-fire macro)
/       chix   cname;       (name of current conference)
/       chix   cnext;       (name of "cnext" conference)
/
/    Purpose:
/       Fire_afm() is simply a convenient way to call interpret() in
/       order to execute a particular auto-fire macro.
/
/    How it works:
/       Fire_afm() calls interpret() to do the actual work.  It
/       returns the following values for CODE, and sets CNEXT
/       accordingly:
/
/         CODE   REASON                  NEXTCONF
/         -----  -------------------     ---------------------
/          1     ran out of commands     ""
/          1     executed "_break"       ""
/          0     saw JOIN or STOP        new conf name or STOP
/          2     saw RETURN              argument to RETURN
/
/       Note: CNAME and CNEXT may be nullchix.
/
/    Returns: (see above)
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:  interpret()
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  main/fireafm.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CX  3/17/92 14:49 New function (to replace FIRE_AFM macro). */
/*: CX  3/17/92 15:44 Replace FIRE_AFM with fire_afm(). */

#include <stdio.h>
#include "caucus.h"

FUNCTION  fire_afm (name, cname, cnext)
   char  *name;
   Chix   cname, cnext;
{
   Chix   afm, empty, cn, nx;
   int    code;

   ENTRY ("fire_afm", "");

   empty = chxalloc ( L(0), THIN, "fire_afm empty");
   afm   = chxalloc (L(40), THIN, "fire_afm afm");
   cn    = (cname!=nullchix ? cname : chxalloc (L(0), THIN, "fire_afm cn"));
   nx    = (cnext!=nullchix ? cnext : chxalloc (L(0), THIN, "fire_afm nx"));

   chxofascii (afm, name);
   code = interpret (afm, cn, nx, empty);

   chxfree (afm);
   chxfree (empty);
   if (cname==nullchix)  chxfree (cn);
   if (cnext==nullchix)  chxfree (nx);

   RETURN  (code);
}
