/*** INTERPRET.  Interpret user commands from AND NOW?.
/
/    ok = interpret (comstream, cname, nextconf, comline);
/
/    Parameters:
/       int    ok;          (Success?)
/       Chix   comstream;   (commands to be executed)
/       Chix   cname;       (name of current conference)
/       Chix   nextconf;    (returned: next conference to go to)
/       Chix   comline;     ("original" comline; source of $1, $2 macro args)
/
/    Purpose:
/       Interpret (execute) Caucus user commands in COMSTREAM.
/
/    How it works:
/       COMSTREAM may contain none, one, or more commands (separated
/       by semicolons).  Interpret() modifies COMSTREAM, so the caller
/       should expect it to be trashed.
/
/       CNAME must be the name of the current conference.
/
/       COMLINE is the "original" command line that invoked this
/       command stream.  It is used only when the "original" command
/       was a macro, to supply the $1, $2, etc. arguments.  All other
/       times it is the empty string.   COMLINE may be a nullchix.
/
/       Interpret() calls the appropriate CLI functions to execute the
/       respective commands.  It sets the value of NEXTCONF to either
/       the desired new conference, or the value of "_return", as shown
/       in the "Returns:" section below.
/
/    Returns:   Why                   NEXTCONF set to:
/       1       ran out of commands     ""
/       1       executed "_break"       ""
/       0       saw JOIN or STOP        new conf name or STOP
/       2       saw RETURN              argument to RETURN
/
/    Error Conditions: 
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  Caucus functions
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  main/interpre.c
/ */

/*** Copyright (C) 1985, 1987 Camber-Roth.  All rights reserved.  */

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR 11/11/88 21:21 Change showlicense() call. */
/*: AM  2/07/89 10:29 Jcaucus changes*/
/*: CR  7/14/89 13:15 Always unlock XICF if it was locked. */
/*: JV  9/14/89 13:41 Split apart a command line. */
/*: CR  8/07/89 10:39 Unlock XICF properly!*/
/*: CW  5/18/89 14:53 Add XW unit codes. */
/*: CW  6/07/89 14:10 Change echo() --> doecho(). */
/*: CR  9/07/89 13:16 Always sysfree(mac) after macro expansion. */
/*: CR  9/08/89 13:24 Add MENU handling code. */
/*: CR  9/25/89 10:59 Add handling for 4th argument. */
/*: CR 10/17/89 15:07 Always clear NEXTCONF. */
/*: CR 11/07/89 16:25 Hack to get _listconf to work. */
/*: CR  1/25/90 12:53 Add EDIT command, invokes editcom(). */
/*: CR  3/09/90 14:13 Add _welcome, _load_status, _show_status. */
/*: CR  3/21/90 12:00 Add _list_status, _check.  Remove nowleaving(). */
/*: CR  4/18/90 21:59 Modify esc_semicolon to reduce conflict with 8bit. */
/*: CR  5/01/90 15:06 "<file" at AND NOW only if in <file, in macro, or -r.*/
/*: CR  7/17/90 14:06 Listconfs() now saves/restores confnum itself. */
/*: CR  9/14/90 11:44 Check() is called with command line, not unit #. */
/*: CJ 11/22/90 16:47 _listconf, _show_status, _list_stat now handle ">". */
/*: CR  2/17/91 22:10 VERSION command clears screen first. */
/*: JV  2/06/91 17:23 Add docs. */
/*: CR  4/06/91 17:06 Integrate recent changes. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  6/25/91 15:27 Add 'confnum' arg to srvldmas() calls. */
/*: CR 11/08/91 12:41 Add _break command. */
/*: UL 12/11/91 18:43 Make unit_lock args 3-6 int4.*/
/*: CR  1/03/92 15:04 Allow redirection on welcome(). */
/*: DE  3/20/92 17:25 Chixified */
/*: DE  3/26/92 16:48 Patched chxindex w/ 0,*/
/*: CX  4/10/92 12:50 Changed command separation to use next_comm(). */
/*: DE  4/10/92 13:57 converted sysfree -> chxfree */
/*: JX  5/26/92 17:42 Removed mdalloc() call. */
/*: CX  6/05/92 11:56 Replace com_unit with u_com, u_keybd. */
/*: JX  6/15/92 16:24 Fix chixification. */
/*: CR  4/18/92 16:34 Add processing of "_restrict" prefix. */
/*: CR  7/11/92 13:33 _list_status with arg calls status_format(). */
/*: CR  8/06/92 15:37 Make "<" at AND NOW revert input to keyboard. */
/*: CR  9/09/92 17:10 Add "_log" handling.  */
/*: CI 10/05/92 15:33 Chix Integration. */
/*: CR 10/11/92 22:07 Add comments. */
/*: TP 10/19/92 16:19 Use input_control() to redirect command input. */
/*: TP 10/20/92 13:27 Replace redirection code with input_redirect(). */
/*: JV 10/15/92 10:90 Chxfree MAC. */
/*: CR 10/22/92 17:11 Simplify freeing of newcomm. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/11/92 11:17 Long Chxcatsub args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: JV  3/18/93 12:35 Add WAIT arg to syscmd(). */
/*: CK  7/29/93 16:57 Reload command tables after each command. */
/*: CP  8/01/93 23:23 Add cases for "enter", "_cmi". */
/*: CP  8/06/93 15:29 Add 'master' arg to mac_substitute() call. */
/*: CP  8/10/93 13:37 Use new Redirect object in handling redirection. */
/*: CP  9/10/93 11:02 New header comments. */
/*: JZ 10/05/93 11:19 Add code for "enter". */
/*: CP 11/21/93 18:46 Debugging display of command before/after expansion. */
/*: CP 11/29/93 15:15 Add "_callclose". */

#include <stdio.h>
#include "caucus.h"
#include "sys.h"
#include "inputcon.h"
#include "redirect.h"

#define  DEBUG(x,y)  { UWA(x);   UWC(y);   UWA("'\n"); }
#define  UWA(x)       unit_write (XWTX, CQ(x))
#define  UWC(x)       unit_write (XWTX, x)

extern struct message_t       mess;
extern Userreg                thisuser;
extern struct partic_t        thisconf;
extern struct master_template master;
extern struct screen_template used;
extern struct fresh_template  fresh;
extern struct flag_template   flag;
extern union  null_t          null;
extern short                  confnum;
extern Chix                   ss_inred, ss_dotnl;
extern int                    debug;

FUNCTION  interpret (comstream, cname, nextconf, comline)
   Chix   comstream, cname, nextconf, comline;
{
   Chix     command, keyword, macword, str, tcom, newstr, format, datared;
   Chix     mac, newcomm, mtu;
   int      leave, action, result, success, error;
   char     temp[256];
   short    found;
   static   Chix  comtable, quitable, mactable, exptable, s_datared;
   Redirect red;
   Chix     mac_subs();

   ENTRY ("interpret", "");

   mac      = chxalloc (L(60),  THIN, "interpre mac");
   mtu      = chxalloc (L(60),  THIN, "interpre mtu");
   str      = chxalloc (L(60),  THIN, "interpre str");
   newstr   = chxalloc (L(80),  THIN, "interpre newstr");
   format   = chxalloc (L(80),  THIN, "interpre format");
   command  = chxalloc (L(204), THIN, "interpre command");
   keyword  = chxalloc (L(50),  THIN, "interpre keyword");
   macword  = chxalloc (L(50),  THIN, "interpre macword");
   datared  = chxalloc ( L(8),  THIN, "interpre datared");
   success  = 0;

   chxclear(nextconf);

   /*** Main command stream loop.  Pluck out, one at a time, commands
   /    separated by ";", and interpret them. */
   newcomm = nullchix;
   while (next_command (command, comstream)) {
      if (newcomm != nullchix)  chxfree (newcomm);
  
      if (disk_failure (0, &error, temp)) {
         printf ("\n\nDisk-write failure %d on %s.\n", error, temp);
         printf ("Recommend you should QUIT immediately.\n");
      }

      /*** Load command tables: regular, stop, and macros.  Do this each
      /    time through next_command(), in case a command suddenly changes
      /    the dictionary number. */
      mdtable  ("commands",   &comtable);
      mdtable  ("stopcoms",   &quitable);
      mdtable  ("macronames", &mactable);
      mdtable  ("macroptrs",  &exptable);
      mdstr    (datared, "mai_Kdatared", &s_datared);

      if (debug & DEB_COMM) DEBUG ("Orig: '", command);

      if ( (newcomm = mac_subs (command, comline, &master)) == nullchix) {
         /*** Something bad happened. */
         continue;
      }
      if (debug & DEB_COMM) DEBUG ("Expa: '", newcomm);

      /*** Look for "_restrict" prefix (and remove it), and set
      /    restrict_user appropriately. */
      chxtoken (keyword, nullchix, 1, newcomm);
      if (flag.restrict_user = (tablematch (comtable, keyword) == 26)) {
         chxalter (newcomm, L(0), keyword, CQ(""));
         while (chxvalue (newcomm, L(0)) == ' ')
            chxalter (newcomm, L(0), chxquick(" ",0), chxquick("",1));
      }

      /*** Command escaping? */
      if (chxvalue(newcomm,0L)=='!') {
         if (flag.escape  ||  (flag.in_macro && NOT flag.restrict_user)) {
            syscmd  (ascquick(tcom=CHXSUB(newcomm,1L)), SYS_WAIT);
            chxfree (tcom);
         }
         else     mdwrite (XWER, "mai_Enocommand", null.md);
         continue;
      }

      /*** Process data or command input redirection at AND NOW? */
      if (input_redirect (newcomm, datared,  IC_DATA, XIAL)  ||
          input_redirect (newcomm, ss_inred, IC_COMM, XICF))   continue;

      /*** Pull out the first two words from the command, and see if the
      /    first word matches anything in the standard command table. */
      while (chxalter (newcomm, L(0), chxquick("\t",0), chxquick(" ",1))) ;
      chxsimplify (newcomm);
      if (EMPTYCHX (newcomm))    continue;
      chxcat   (newcomm,CQ(" "));
      chxclear (str);
      chxcatsub (str, newcomm, L(0), L(40));
      chxtoken (keyword, nullchix, 1, str);
      jixreduce (keyword);

      /*** Any of the synonymns for STOP in QUITTABLE become "JOIN STOP". */
      if (tablefind (quitable, keyword) >= 0) {
         mdstr    (newcomm, "mai_Kjoinstop", null.md);
         chxtoken (keyword, nullchix, 1, newcomm);
      }

      found = tablematch (comtable, keyword);

      /*** Command not recognized.  Check for macro expansion. */
      if (found == -1  &&  (found = tablematch (mactable, keyword)) >= 0) {
         chxtoken   (macword, nullchix, found+1, exptable);

         if (mdstr (mac, ascquick (macword), null.md)) {
            while (chxalter (mac, L(0), chxquick("\n",0), chxquick(";",1))) ;
            ++flag.in_macro;
            result = interpret (mac, cname, nextconf, newcomm);
            --flag.in_macro;
            if (result != 1)     DONE(result);
         }
         else {
            mdstr      (format, "mai_Ebadexp", null.md);
            chxformat  (mtu, format, L(0), L(0), keyword, nullchix);
            unit_write (XWER, mtu);
         }
         continue;
      }

      if (found == -1) {
         mdstr     (format, "mai_Fbadcom", null.md);
         chxclear  (newstr);
         chxformat (newstr, format, L(0), L(0), keyword, nullchix);
         unit_write (XWER, newstr);
      }

      /*** Ambiguous command. */
      else if (found == -2)   ambiguous (keyword);

      /*** Known commands. */
      else {
         chxclear(nextconf);
         leave = 0;

         switch (found) {
            case ( 0):   doecho    (newcomm);             break;
            case ( 1):   help      (newcomm, "h_help");   break;
            case ( 2):   add       (newcomm);             break;
            case ( 3):   del       (newcomm, &leave);     break;
            case ( 4):   show      (newcomm);             break;
            case ( 5):   describe  (newcomm);             break;
            case ( 6):   change    (newcomm);             break;
            case ( 7):   forget    (newcomm);             break;
            case ( 8):   search    (newcomm);             break;
            case ( 9):   chxconc   (newcomm, CQ("x "), newcomm);
                         add       (newcomm);             break;
            case (10):   status    (newcomm);             break;
            case (11):   customize (newcomm);             break;
            case (12):   set       (newcomm);             break;
            case (13):   chxtoken (nextconf, nullchix, 2, newcomm);
                               /* JOIN!! */
                         leave = 1;                       break;
            case (14):   freeze    (newcomm);             break;

            case (15):
               unit_write  (XWTX, NEWCPAGE);
               showlicense (1);
               break;

            case (16):   /*** MENU ***/
               action = domenu (newcomm, cname, nextconf);
               if (action == 0)     FAIL;
               if (action == 2)     DONE(2);
               break;

            case (17):   /*** RETURN from menu ***/
               chxtoken (nextconf, nullchix, 2, newcomm);
               DONE(2);

            case (18):   assign    (newcomm);             break;

            case (19):   /*** Hack to get _listconf. */
               parse_rout     (&red, newcomm);
               apply_redirect (&red);
               listconfs  (red.unit, 0);
               if (red.locked) { unit_close(red.unit);  unit_unlk(red.unit); }
               break;

            case (20):   editcom   (newcomm);             break;

            case (21):   /*** _Welcome: Display the organizer's welcome text.*/
               parse_rout     (&red, newcomm);
               apply_redirect (&red);
               unit_write (red.unit, NEWCPAGE);
               welcome    (red.unit);
               if (red.locked) { unit_close(red.unit);  unit_unlk(red.unit); }
               break;


            case (22):   /* Load status. */
               /*** Load masteres, then message directory. */
               srvldmas (1, &master, confnum);
               touch_master();
               if (unit_lock (XUMD, READ, L(0), L(0), L(0), L(0), thisuser->id)) {
                  mess.highest   = loadmdir (0, mess.num);
                  mess.high_seen = mess.highest;
                  unit_unlk (XUMD);
               }
               break;

            case (23):   /* List status. */
               parse_rout     (&red, newcomm);
               apply_redirect (&red);
               if (chxtoken (newstr, nullchix, 2, newcomm) == -1)
                     sysstatus     (red.unit);
               else  status_format (red.unit, newstr);
               if (red.locked) { unit_close(red.unit);  unit_unlk(red.unit); }
               break;

            case (24):   /* Show status. */
               parse_rout     (&red, newcomm);
               apply_redirect (&red);
               show_new   (red.unit, &thisconf);
               if (red.locked) { unit_close(red.unit);  unit_unlk(red.unit); }
               break;

            case (25):   /* Check all conferences. */
               check (newcomm);
               break;

            case (27):   /* "_break", act as though command stream done. */
               chxclear(comstream);
               break;

            case (28):   /* "_log", invoke logfeature() logging function. */
               logger_cmd (newcomm);
               break;

            case (29):   /* "enter", same as JOIN for now. */
               chxtoken (nextconf, nullchix, 2, newcomm);
               leave = 1; 
               break;

            case (30):   /* "_cmi", invoke Caucus mail import. */
               do_cmi (thisuser);
               break;

            case (31):   /* "_callclose", close menu input redirection. */
               if (input_control (IC_ISUNIT | IC_MENU, nullchix) == XIMF)
                   input_control (IC_CLOSE  | IC_MENU, nullchix);
         }

         if (leave)    FAIL;
      }
   }
   SUCCEED;

 done:
  
   if (newcomm != nullchix)  chxfree (newcomm);
   chxfree (mac);
   chxfree (mtu);
   chxfree (str);
   chxfree (newstr);
   chxfree (format);
   chxfree (command);
   chxfree (keyword);
   chxfree (macword);
   chxfree (datared);

   RETURN ( success );
}
