
/*** Main program for CAUCUS!    (CV2)
/
/    Invoked as:
/      caucus_x  caucus_path  [ conf_name | - [ server_name ]] [options]
/
/    Where:
/      caucus_path   is the path name of the caucus directory
/      conf_name     is the (optional) initial conference
/      -             is the (optional) indicator of "no initial conference"
/      server_name   is the name of the server, for client/server only
/      options       are "-x" unix-style command line options.
/                    (see argchk() for details.)
/
/    Notes:
/      1. "conf_name" or "-" is required for client/server caucus models.
/ */

/*** Copyright (C) 1985, 1987, 1993 Camber-Roth.  All rights reserved.  */

char copyright[] =
"Copyright (C) 1996 Screen Porch Inc.  All rights reserved.";

char *trade[] = {
"This program belongs to Screen Porch Inc.  It is considered a trade secret",
"and is not to be divulged or used by parties who have not received",
"written authorization from the owner."};

/*: AA  7/29/88 17:47 Source code master starting point */
/*: CR 09/27/88 22:49 Move argchk() to just before syspcinit(). */
/*: CR 11/11/88 21:21 Move sysexit() to top, change showlicense() call. */
/*: CR  1/09/89 14:13 Remove {}'s from copyright notice. */
/*: CR  3/21/89 15:00 Add servername, and more comments. */
/*: AM  2/07/89 10:29 Jcaucus changes*/
/*: CR  3/29/89 14:27 Remove extra entry_test() call. */
/*: CR  4/05/89 13:29 Put back 'debug'. */
/*: JV  8/04/89 13:48 Added optchk() call. */
/*: JV  8/13/89 15:19 Added flagstart. */
/*: CW  5/18/89 14:53 Add XW unit codes. */
/*: CR 10/30/89 11:59 Remove userid[]. */
/*: JV  2/06/90 14:52 If globals() fails, bail out (no dicti). */
/*: CR  3/26/90 18:15 Set flagstart properly depending on args. */
/*: CR  4/12/90 16:31 Move sysglobals() after optchk(). */
/*: CR  5/18/90 15:30 Parse "caucus_x path -y master.opt" properly. */
/*: CR  6/21/90 12:26 Add LUSR arg to unit_wipe() call. */
/*: CR  9/25/90 15:42 Add SYSPUT debugging statements. */
/*: CJ 11/22/90 20:48 Add terminal state stuff. */
/*: CJ 11/22/90 22:41 Add new systurn() call. */
/*: CR 12/23/90 22:23 Move SYSINIT back to the top! */
/*: CR  2/01/91 16:15 Use new syshnginit() function. */
/*: CR  5/02/91 18:36 Remove initsort(). */
/*: CR  7/02/91  0:00 Include "unitwipe.h", use new globals funcs. */
/*: CR  7/29/91 23:45 Comment out ENTRY.  Unneeded in main. */
/*: DE  3/23/92 13:08 Chixified */
/*: JX  5/26/92 17:43 Remove kanji. */
/*: CX  6/05/92 16:51 Allocate confid here.   Remove mdglobals(). */
/*: JX  6/30/92 10:55 Initialize collation sequence. */
/*: JX  7/07/92 14:49 Define word separators. */
/*: JX  7/13/92 10:47 Add chx_init(). */
/*: CR  8/25/92  4:37 Initialize elapsed() time indicator. */
/*: CR  9/04/92 17:59 Add logfeature() call. */
/*: CI 10/05/92 12:49 Chix Integration. */
/*: CR 10/09/92 17:13 Comment out 'blanks' usage.  (No chxfree!) */
/*: JV 10/15/92 11:37 Free global tbuf at end to ease debugging. */
/*: CR 10/30/92 14:15 arg/optchk systurns() -> main; FILEOUTPUT stuff. */
/*: CR 11/10/92 11:09 Move close on IC_SESS from join(). */
/*: CR 11/20/92 13:46 Display copyright according to license.function_mask. */
/*: CR  1/26/93 15:49 Use PLAIN and strplain() to decrypt license info. */
/*: JV  6/08/93 20:04 Call chkprv(), a_init(), a_exit(). */
/*: CP  6/30/93 23:33 Add API initialization. */
/*: CP  7/08/93 16:21 Integrate 2.5 changes. */
/*: JJ  7/22/93 10:39 Add parse_attach_map(). */
/*: CK  7/28/93 14:45 Move no args checking sooner in code. */
/*: CK  7/28/93 15:49 Add -h "no check priv" command line argument. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */
/*: CP  8/11/93 13:10 Change att_ui_type to TERM_ASCII_UNIX. */
/*: CP  8/18/93 17:46 Remove att_ui_type. */
/*: CB  1/18/94 18:45 Remove free_textbuf(tbuf) call; it's in a_exit() ! */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "unitwipe.h"
#include "text.h"
#include "units.h"
#include "inputcon.h"
#include "api.h"

extern struct screen_template  used, screen;
extern struct termstate_t      term_original, term_in_caucus;
extern struct flag_template    flag;
extern struct namlist_t       *sysmgrprint;
extern struct license_t        license;
extern Chix                    confname;
extern Chix                    servername;
extern int                     debug;
extern union  null_t           null;
extern Userreg                 thisuser;
extern Textbuf                 tbuf;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   int4   starttime, systime();
   int    i;
   short  flagstart = 1;   /* start of command line switches. */
   int    checkpriv = 1;   /* Should we call chkprv() to check privileges? */
   char   logdir[256];
   
   Userreg make_userreg();
   Initapi initapi;

   sys_nt();

   /*** Save "original" terminal state. */
   systurn (&term_original,  GET_TERM, 0);
   systurn (&term_in_caucus, GET_TERM, 0);

   /*** Check for -d (debug) and -h (don't check privs, for ups debugging)
   /    immediately, since we need to know about them before the regular
   /    argument and option checking. */
   for (i=1;   i<argc;   ++i) {
      if (streq (argv[i], "-d"))  debug     = 1;
      if (streq (argv[i], "-h"))  checkpriv = 0;
   }
   if (debug) sysputstr ("Main 1\n");

   screen.lines = 23;    used.lines = 0;
   screen.width = 80;    used.width = 0;

/* mallocmap(); */
   confname = chxalloc (L(32), THIN, "main confname");

   /*** If no args, display license information and exit. */
   if (argc <  2  ||  (NOT checkpriv  &&  argc < 3)) {
      thisuser = make_userreg(); /* Kludge so that THISUSER has a dict. */
      showlicense(0);
      exit (0);
   }

   strcpy (logdir, argv[1]);
   strcat (logdir, "/SOCKET");
   logger_init (logdir, argv[0], "cau");

   /*** Initialize the API, getting the conference id string from the 
   /    first argument. */
   if (debug) sysputstr ("Main 1.1: before api initialization.\n");
   initapi = a_mak_init("main initapi");
   chxcpy (initapi->confid, CQ(argv[1]));
   initapi->terminal = 1;
   if (debug) sysputstr ("Main 1.1a: before api initialization.\n");
   a_init (initapi); 

   if (debug) sysputstr ("Main 1.2: before parse_attach_map.\n");
   if (parse_attach_map (CQ("attach_map")) < 0) {
      printf ("Couldn't parse attach_map.\n");
      return (1);
   }


   if (checkpriv  &&  chkprv (argv[0], 0)) exit (0);

   /*** Get the initial conference name, if there is one, from the 2nd arg.
   /    Also pick up server name, if it is supplied. */
   if (debug) sysputstr ("Main 2: before server name check\n");
   if (argc >= 3  &&  *argv[2] != '-') {
      chxcpy (confname,  CQ(argv[2]));
      flagstart = 3;
   }
   if (argc >= 4  &&  streq (argv[2], "-")) {
      chxcpy (servername, CQ(argv[3]));
      flagstart = 4;
   }

   /*** Initialize break-catching and any remaining system-dependent
   /    stuff.   Process command-line flags.   Verify expiration date
   /    and cpu identity. */
   if (debug) sysputstr ("Main 3: before argchk\n");
   argchk  (argc, argv, flagstart);
   if (debug) sysputstr ("Main 4: after argchk\n");
   utilglobals ();
   optchk ();

   /*** Set up initial terminal state, based on argchk/optchk options. */
   if (NOT flag.echo)  systurn (&term_in_caucus, CAUCUS_ECHO + OS_ECHO, OFF);
   if (flag.eightbit)  systurn (&term_in_caucus, EIGHT_BIT, ON);

   /*** Use Text Type JIX internally. */
   if (flag.japan) chx_init (JIX);
   else            chx_init (ASCII);

   sysexit (-1);
   if (debug) sysputstr ("Main 5: after sysexit\n");

   unitglobals();
   if (debug) sysputstr ("Main 6: after unitglobals\n");
   if (NOT mainglobals()) sysexit (1);
   if (debug) sysputstr ("Main 7: after mainglobals\n");
   sysbrkinit ();
   syshnginit (1);
   elapsed();
   if (debug) sysputstr ("Main 8: after sysbrkinit\n");

   if (NOT copyprotect()) {
      showlicense(1);
      sysexit(1);
   }
   starttime = systime();
   if (debug) sysputstr ("Main 9: after copyprotect\n");
   unit_write (XWTX, NEWCPAGE);
   if (PLAIN(license.function_mask) & FM_COPYR) sysversion (XWTX, 1);
 
   sysglobals();
   xfpglobals ();
   if (NOT syspcinit())    manager_text (flag.entrfile);

   /*** Tell the user how to get help, and start the main loop that asks
   /    which conference the user wants to join. */
   if (EMPTYCHX (confname))   mdwrite (XWTX, "mai_Ttypehelp", null.md);

   loadprint  (&sysmgrprint, XSPC, 1);

   /*** A PC may handle many different users during one execution
   /    of the program.  A normal system will handle just one user,
   /    and then exit. */
   if      (flag.baud || flag.loop)   {  while (loginshell())   caucus(); }
   else if (flag.login)               {  if    (loginshell())   caucus(); }
   else                                                         caucus();

   logfeature (CQ("caucus"), nullchix, L(0), starttime, L(0), L(0));

   /*** If FILEOUTPUT option was specified, close the output file. */
   if (flag.outfile != nullchix) {
      unit_control (XITT, UNIT_SETSUB);
      unit_close   (XIOF);
      unit_unlk    (XIOF);
   }
   unit_wipe (LUSR);

   input_control (IC_CLOSE | IC_SESS, nullchix);

   /*** Close up API, clean up after global chxallocations to ease 
   /    debugging. */
/* mallocmap(); */
   a_exit();
   a_fre_init (initapi);
   sysexit    (0);
   return    (0);
}
