
/*** NEXT_COMMAND.  Pluck the next command out of a command line.
/
/    more = next_command (command, line);
/
/    Parameters:
/       int    more;        (was another command was found?)
/       Chix   command;     (returned command)
/       Chix   line;        (original command line)
/
/    Purpose:
/       Line may contain multiple Caucus commands separated by ";"s.
/       Next_command() removes the first such command from LINE, and
/       places it in COMMAND.  (It also removes all leading blanks
/       or tabs.)
/
/    How it works:
/
/    Returns: 1 if a command was found
/             0 otherwise
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  interpret()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  main/nextcomm.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  4/08/92 11:30 New function. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/11/92 11:17 Long Chxcatsub args. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */

#include <stdio.h>
#include "caucus.h"
#include "derr.h"

FUNCTION  next_command (command, line)
   Chix   command, line;
{
   int4   this, semi, skip;
   Chix   temp, c1, c2;

   ENTRY ("next_command", "");

   chxclear (command);

   if (EMPTYCHX(line))  RETURN (0);

   /*** Find the end of the first ; terminated command in LINE. */
   for (semi=0;   this = chxvalue (line, L(semi));   ++semi) {
      if (this == ';'  &&  (semi==0  ||  chxvalue (line, L(semi-1)) != '\\') )
         break;
   }

   /*** Copy the first such command into COMMAND, skipping over leading 
   /    blanks or tabs, and replace all "\;" with just ";". */
   for (skip=0;   (1);   ++skip) {
      this = chxvalue (line, L(skip));
      if (this != ' '  &&  this != '\t')  break;
   }
   chxcatsub (command, line, skip, semi - skip);
   c1 = chxquick ("\\;", 1);
   c2 = chxquick (  ";", 2);
   while (chxalter (command, L(0), c1, c2)) ;
   
   /*** Put the remaining commands into LINE. */
   temp = chxalloc (L(80), THIN, "next_comm this");
   chxcatsub (temp, line, semi+1, ALLCHARS);
   chxcpy    (line, temp);
   chxfree   (temp);

   RETURN (1);
}
