/*** ADD_CMIMESS.  Send a Caucus message to a system e-mail address.
/
/    ok = add_cmimess (recip, author, authorid, messtype, subject, newtext,
/                      prevtext);
/
/    Parameters:
/       int    ok;          (Success?)
/       Chix   recip;       (recipient's mail address)
/       Chix   author;      (full Caucus name of sender)
/       Chix   authorid;    (sender's userid)
/       int    messtype;    (message type: MREPLY, MSEND, etc.)
/       Chix   subject;     (message subject string)
/       Chix   newtext;     ("new" message text)
/       Chix   prevtext;    (text of any previous messages, as in replies)
/
/    Purpose:
/       Prepare and send a Caucus message via the "system mailer" to an
/       e-mail address external to Caucus.
/
/    How it works:
/       Messtype must be MSEND, MFORWARD, or MREPLY, from mess.h.
/
/    Returns: 1 on success.
/             0 on any error condition, see below.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  mess_send(), add_caumess().
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  mess/addcmime.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  2/25/93 18:59 New function, extracted from mess_send(). */
/*: JV  6/23/93 10:05 Remove call to sysmstrans, get dict strings instead. */
/*: CR 12/20/96 16:33 Handle optional '=' at start of recip. */

#include <stdio.h>
#include "caucus.h"
#include "inputcon.h"
#include "derr.h"
#include "units.h"
#include "diction.h"

#define ASCSIZE L(300)

extern union null_t null;

FUNCTION  add_cmimess (recip, author, authorid, messtype, subject, 
                       newtext, prevtext)
   Chix   recip, author, authorid, subject, newtext, prevtext;
   int    messtype;
{
   char fname[200], asc_comm[ASCSIZE];
   static char *messkind[] =
      { "MESSAGE from", "FORWARDED by", "REPLY from", "MESSAGE from"};
   Chix   temp, preface, authorname, date, time, to_id, messtext,
          format1, format2, sendcmd;
   char  *logmesstype;
   int    success, lang;
   int4   size, pos, equals;
   int4   systime();

   ENTRY ("add_cmimess", "");

   temp       = chxalloc (  L(40), THIN, "add_cmimess temp");
   date       = chxalloc (  L(40), THIN, "add_cmimess date");
   time       = chxalloc (  L(40), THIN, "add_cmimess time");
   to_id      = chxalloc (  L(40), THIN, "add_cmimess to_id");
   format1    = chxalloc (  L(80), THIN, "add_cmimess format1");
   format2    = chxalloc (  L(80), THIN, "add_cmimess format2");
   preface    = chxalloc (  L(80), THIN, "add_cmimess preface");
   sendcmd    = chxalloc (  L(80), THIN, "add_cmimess sendcmd");
   messtext   = chxalloc (L(2000), THIN, "add_cmimess messtext");
   authorname = chxalloc (  L(60), THIN, "add_cmimess authorname");

   success = 0;
   if (messtype > 3)  messtype = 3;
   sysdaytime  (date, time, 0, systime());

   removeid  (authorname, author, authorid, temp);
   if (NOT sysgetnode (temp)) chxclear(temp);

   chxformat (preface, CQ("\n<<%-12s>> %-35s"), L(0), L(0), 
                       chxquick (messkind[messtype], 1), authorname);
   chxformat (preface, CQ("  %s %s\n"), L(0), L(0), date, time);
   chxformat (preface, CQ("                 %s@%s\n"), L(0), L(0), 
                       authorid, temp);

   /*** To build the actual message text: first, grab the 2nd through
   /    N'th words of the first line of the "previous" message text. */
   if (NOT EMPTYCHX (prevtext)) {
      pos = 0;
      chxnextline (temp, prevtext, &pos);
      chxtoken (nullchix, messtext, 2, temp);
      chxcat   (messtext, CQ("\n"));
   
      /*** Then add the remainder of the "previous" message text. */
      chxcatsub (messtext, prevtext, pos, ALLCHARS);

      /*** Put in a separator between prev and new text. */
      if (NOT EMPTYCHX(newtext))  chxcat (messtext, CQ("- - - - -\n"));
   }

   /*** Now add the preface, and finally the new message text. */
   chxcat (messtext, preface);
   chxcat (messtext, newtext);
   size  = chxlen (messtext);

   /*** Get the actual e-mail address from RECIP. */
   equals = (chxvalue (recip, L(0)) == '=' ? L(1) : L(0));
   chxcatsub (to_id, recip, equals, ALLCHARS);

   /*** Put the TEXT into the XITX temp file.  Get the full pathname
   /    of that file in FNAME. */
   unit_name (fname,  XITX, L(0), L(0), L(0), L(0), nullchix);
   if (NOT unit_lock (XITX, WRITE, L(0), L(0), L(0), L(0), nullchix))  FAIL;
   if (NOT unit_make (XITX))            { unit_unlk (XITX);    FAIL; }
   unit_write (XITX, messtext);
   unit_close (XITX);

   mdstr (format1, "mes_Fcmisend1", null.md);
   mdstr (format2, "mes_Fcmisend2", null.md);
   sysescquote (subject);

   /* Format: "cmdname subject to_id fname" */
   chxformat (sendcmd, format1, L(0), L(0), subject, to_id);
   chxformat (sendcmd, format2, L(0), L(0), CQ(fname), nullchix);

   /* Change from jix to relevant I/O format. */
   lang = unit_control (XIRE, UNIT_GET);
   switch (lang) {
      case (LANG_NORM):
      case (LANG_ISO):  ascofchx  (asc_comm, sendcmd, L(0), ASCSIZE);  break;
      case (LANG_EUC):  eucofjix  (asc_comm, sendcmd, L(0), ASCSIZE);  break;
      case (LANG_SJIS): sjisofjix (asc_comm, sendcmd, L(0), ASCSIZE);  break;
      default:                     asc_comm[0] = '\0';               break;
   }

   if (syscmd (asc_comm, SYS_WAIT) > 0) {
      logmesstype = (messtype==MREPLY ? "mailrepl" : "mailsent");
      logfeature (CQ(logmesstype), to_id, L(0), L(0), L(0), size);
      success = 1;
   }

   unit_kill (XITX);
   unit_unlk (XITX);

done:
   chxfree (to_id);
   chxfree (temp);
   chxfree (date);
   chxfree (time);
   chxfree (format1);
   chxfree (format2);
   chxfree (preface);
   chxfree (sendcmd);
   chxfree (messtext);
   chxfree (authorname);

   RETURN  (success);
}
