
/*** GETMSUBJ.   Get the subject of an existing message.
/
/    getmsubj (id, messnum, subject);
/
/    Parameters:
/       Chix   id;          (userid of owner of message)
/       int    messnum      (message number)
/       Chix   subject;     (place to put subject field)
/
/    Purpose:
/       Fetch the subject field out of an existing message, not including
/       the "SUBJECT:" preface.
/
/    How it works:
/       Getmsubj() reads the subject line of message MESSNUM beint4ing
/       to user ID.  It removes the "SUBJECT:" preface, and returns the
/       remaining text in SUBJECT.
/
/       If the message does not exist, or an error occurs, SUBJECT is
/       set to the empty string.
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  mess_send(), messreceipt()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  mess/getmsubj.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */

/*: CR  7/09/91 11:38 New function. */
/*: CR  7/30/91  0:00 Add missing ENTRY! */
/*: CR  1/10/92 22:47 Hard-code SUBJECT field. */
/*: JX  5/26/92 15:20 Chixify. */
/*: CL 12/10/92 15:22 Long chxindex args. */
/*: CL 12/11/92 11:17 Long Chxcatsub args. */

#include <stdio.h>
#include "caucus.h"

FUNCTION  getmsubj (id, messnum, subject)
   Chix   id, subject;
   int    messnum;
{
   Chix   s, getmline();
   int4   sbj;

   ENTRY ("getmsubj", "");

   chxclear (subject);
   if ( (s = getmline (id, messnum, 2, 1)) == nullchix)  RETURN (1);

   if ( (sbj = chxindex (s, L(0), CQ("     SUBJECT: "))) >= 0)
      chxcatsub (subject, s, sbj+14, ALLCHARS);

   RETURN (1);
}
