
/*** MESSHEADER.  Translate message header according to current dictionary.
/
/    new = messheader (str);
/
/    Parameters:
/       Chix   new;         (returned ptr to translated header)
/       Chix   str;         (original header)
/
/    Purpose:
/       Certain key words in message headers (MESSAGE, FORWARD, REPLY)
/       are always stored in english in the message partfiles.  When
/       a message is being displayed, those keywords must be translated
/       into the current dictionary's equivalent keywords.
/
/       Messheader() performs that function.
/
/    How it works:
/       Messheader() copies STR into an internal static buffer.  It
/       then looks for any of the standard keywords (MESSAGE, FORWARD,
/       REPLY, and SUBJECT) and replaces them with the equivalent keyword
/       from string mes_Atype.  Finally, it returns a pointer to that
/       internal buffer.
/
/    Returns: pointer to translated string.
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  messview(), messlist()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  mess/messhead.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  1/10/92 23:12 New function. */
/*: CR  1/20/92 17:53 Declare strtoken(). */
/*: DE  3/26/92 17:45 Chixified */
/*: DE  4/21/92 15:40 Put in args missing from chxalter */
/*: JX  6/23/92 16:46 Allocate RETURN_PTR. */
/*: CR 10/08/92 13:41 Remove "from" parts, SUBJECT. */
/*: CL 12/10/92 14:52 Long chxalter args. */
/*: CL 12/10/92 15:22 Long chxindex args. */

#include <stdio.h>
#include "caucus.h"

FUNCTION  Chix messheader (str)
   Chix   str;
{
   static Chix s_mestype;
   static Chix line=nullchix, return_ptr=nullchix;
   static Chix hdr[] = {nullchix, nullchix, nullchix};
   Chix   type, replace;
   int    i;
   int4   pos;

   ENTRY ("messheader", "");

   type     = chxalloc (L(100), THIN, "messhead type");
   replace  = chxalloc (L(80),  THIN, "messhead replace");

   if (line == nullchix) {
      line       = chxalloc (L(200), THIN, "messhead line");
      return_ptr = chxalloc (L(200), THIN, "messhead line");
   }

   if (hdr[0] == nullchix) {
      hdr[0]  = chxalloc (L(15), THIN, "messhead hdr[0]");
      hdr[1]  = chxalloc (L(15), THIN, "messhead hdr[1]");
      hdr[2]  = chxalloc (L(15), THIN, "messhead hdr[2]");
      chxcpy (hdr[0], CQ("MESSAGE"));
      chxcpy (hdr[1], CQ("FORWARDED"));
      chxcpy (hdr[2], CQ("REPLY"));
    }

   /*** Find the message header keyword. */
   for (i=0;   i<3;   ++i)  if ( (pos = chxindex (str, L(0), hdr[i])) >= 0)  break;
   if (pos < 0)  {  return_ptr = str;  goto done;  }

   /*** Get the matching keyword from the dictionary. */
   mdstr    (type, "mes_Atype", &s_mestype);
   chxtoken (replace, nullchix, i+1, type);
   while (chxalter (replace, L(0), chxquick("_",0), chxquick(" ",1))) ;

   /*** Replace hard-coded message keyword with its dictionary equivalent. */
   chxcpy   (line, str);
   chxalter (line, pos, hdr[i], replace);
   chxcpy   (return_ptr, line );

 done:

   chxfree ( type );
   chxfree ( replace );

   RETURN   ( return_ptr );
}
