
/*** MESSPARSE.   Parse a message-range and flag chosen messages with MSG_SEE.
/
/    ok = messparse (range, new, old, highest, flags);
/
/    Parameters:
/       int    ok;        (any messages selected?)
/       Chix   range;     (specific message range, from command line)
/       int    new;       (select only new messages?)
/       int    old;       (select only old messages?)
/       int    highest;   (number of highest message)
/       short  flags[];   (return map of selected messages here)
/  
/    Purpose:
/       Parse a set of messages, and return a "map" of the messages selected.
/
/    How it works:
/       Parse the messages specifically listed in the string RANGE.  Reduce
/       the result to just new messages (if NEW arg is true) or just old
/       messages (if the OLD arg is true).
/
/       For each message M that should be seen, set bit MSG_SEE in FLAGS[M].
/       If it should not be seen, clear bit MSG_SEE.
/
/       Note that it is up to the caller to supply initial values in FLAGS;
/       messparse() neither clears nor sets the FLAGS values except as
/       specified above.
/       
/    Returns: 1 if any messages were selected
/             0 otherwise
/
/    Error Conditions: 
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  show_mess(), desc_mess(), del_mess()
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  mess/messpars.c
/ */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:48 Source code master starting point */
/*: JV  7/22/90 17:46 Change def of NEW to match all invocations. */
/*: DE  3/27/92 12:20 Chixified */
/*: DE  4/21/92 15:20 Put in args missing from chxvalue */
/*: JX  5/27/92 16:52 Cast return from chxvalue. */
/*: JX  6/23/92 15:49 Rewrite parsing loop. */
/*: CL 12/11/92 13:47 Long Chxvalue N. */
/*: CN  5/15/93 21:02 Make 'flags' short. */
/*: CK  9/19/93 11:39 Add 'old' arg, new header comments. */

#include <stdio.h>
#include "caucus.h"

FUNCTION  messparse (range, new, old, highest, flags)
   Chix   range;
   Flag   new, old;
   short  flags[];
   int    highest;
{
   int    i, m1, m2, is_valid, select;
   int4   value, pos;

   ENTRY ("messparse", "");

   /*** Parse numeric parts of RANGE. */

   is_valid = 0;
   for (pos=0;   value = chxvalue (range, pos);   ) {
      if (cf_digit ((int) value)) {
         m1 = (int) chxint4 (range, &pos);
         if (m1>0  &&  m1<=highest)  { flags[m1] |= MSG_SEE;   is_valid = 1; }
      }

      else if (value == '-') {
         ++pos;
         m1 = max (m1, 1);
         m2 = min ( (int) chxint4 (range, &pos), highest);
         for (i=m1;   i<=m2;   ++i)  { flags[i]  |= MSG_SEE;   is_valid = 1; }
      }

      else if (value  == ' ' ||
               value  == ',') ++pos;

      else                 break;
   }

   /*** Apply NEW or OLD modifier. */
   if (new || old) {

      /*** If no messages had been selected, assume ALL. */
      if (NOT is_valid)
         for (i=1;   i<=highest;   ++i)   flags[i] |= MSG_SEE;

      /*** Reduce by NEW or OLD modifier. */
      is_valid = 0;
      for (i=1;   i<=highest;   ++i)  if (flags[i] & MSG_SEE) {
         select = flags[i] & MSG_NEW;
         if (old)  select = NOT select;
         if (select)  is_valid  = 1;
         else         flags[i] &= (~MSG_SEE);
      }
   }

   RETURN (is_valid);
}
