/*** MESSRECEIPT.   Send a return-receipt message to author of message N. */

/*** Copyright (C) 1989 Camber-Roth.  All rights reserved. */

/*: CR  8/31/89 15:27 New function. */
/*: CR  1/31/91 18:31 Use person_format() to format user's name. */
/*: CR  2/08/91 17:06 Force author and type of message to 35 & 12 char max. */
/*: CR  4/08/91 18:49 Add new unit_lock argument. */
/*: CR  4/26/91 14:37 Expand buffers to_id, str. */
/*: CR  7/09/91 14:07 Rewrite to use add_dmess(). */
/*: CR  7/24/91 21:47 Remove unused variables. */
/*: CR  7/28/91 15:48 chxalloc() first argument is int4. */
/*: CR 12/11/91 23:55 Allow for RR message across the net. */
/*: CR 12/12/91 12:50 addxmess() call has 0 "original parts". */
/*: JX  5/15/92 11:36 Add Chix carvmark. */
/*: CR 12/16/92 13:50 Correct append of subject; remove newstr. */
/*: CR  2/27/93 14:15 Replace add_dmess() with add_caumess(). */
/*: JV  4/05/93 18:17 Add attach arg to add_dmess(). */

#include <stdio.h>
#include "caucus.h"
#include "chixuse.h"

extern Userreg           thisuser;
extern union  null_t     null;

FUNCTION  messreceipt (messnum)
   int   messnum;
{
   static  Chix retrecsubj;
   int4    rrsize, parts;
   int     error, success;
   int4    systime();
   Chix    prevtext, newtext, header, format;
   Chix    to_id, str, subject, date, time, chn, thishost;
   Chix    getmline();

   ENTRY ("messreceipt", "");

   to_id      = chxalloc (L(200), THIN, "messrece to_id");
   str        = chxalloc (L(200), THIN, "messrece str");
   subject    = chxalloc (L(100), THIN, "messrece subject");
   date       = chxalloc (L(40),  THIN, "messrece date");
   time       = chxalloc (L(40),  THIN, "messrece time");
   chn        = chxalloc (L(80),  THIN, "messrece chn");
   thishost   = chxalloc (L(200), THIN, "messrece thishost");
   newtext    = chxalloc (L(200), THIN, "messreceipt newtext");
   prevtext   = chxalloc (L(200), THIN, "messreceipt prevtext");
   format     = chxalloc (L(200), THIN, "messreceipt format");
   success    = 0;

   /*** From the original message, pluck out the id of the return-
   /    receipt requestor, and the requested receipt size. */
   if ( (header = getmline (thisuser->id, messnum, 0, 1)) == nullchix)  FAIL;
   chxtoken (str, nullchix, 2, header);
   chxbreak (str, to_id, chn, CQ("@"));
   chxtoken (str, nullchix, 3, header);
   chxnum   (str, &parts);
   chxtoken (str, nullchix, 4, header);
   rrsize = 0;
   chxnum   (str,   &rrsize);

   /*** Pluck out the old subject field. */
   getmsubj (thisuser->id, messnum, subject);

   /*** Build the text of the return receipt message out of the dictionary
   /    string mes_Thedrec, and the extracted subject from the original
   /    message. */
   mdstr     (newtext, "mes_Thedrec", null.md);
   chxcat    (newtext, subject);
   chxcatval (newtext, THIN, L('\n'));

   /*** To this, add receipt-size (RRSIZE) lines from the original message. */
   if (rrsize > 0) {
      mdstr     (format, "mes_Frectext", null.md);
      chxformat (newtext, format, rrsize, L(0), nullchix, nullchix );
      
      parts = (int4) prevmess (messnum, thisuser->id, 0, (int) rrsize, 
                               prevtext);
      chxcat   (newtext, prevtext);
      chxclear (prevtext);
   }

   /*** Get the remaining info for add_dmess(): a fixed subject field,
   /    and the current date and time. */
   mdstr    (subject, "mes_Tretrec", &retrecsubj);
   sysdaytime  (date, time, 0, systime());

   /*** There are 3 possible cases for how to send the return receipt:
   /    (a) local user who "SET EXPORT"s mail to their system mailbox.
   /    (b) local user, regular Caucus mail.
   /    (c) remote CaucusLink user. */
   getmyhost (thishost);
   if (EMPTYCHX(chn)  ||  chxeq (chn, thishost)) {
      if (add_caumess (thisuser->name, thisuser->id, nullchix, subject, MSEND, 
                       date, time, prevtext, 0, -1, to_id, newtext,
                       (Attachment) NULL, &error))
         mdwrite   (XWTX, "mes_Trecsent", null.md);
   }
   else {
      if (addxmessage (thisuser->name, thisuser->id, subject, MSEND,
                       prevtext, 0, -1, 0, nullchix, to_id, chn, 
                       newtext, date, time, &error))
         mdwrite   (XWTX, "mes_Trecsent", null.md);
   }

   SUCCEED;

 done:
      
   chxfree (str);
   chxfree (chn);
   chxfree (date);
   chxfree (time);
   chxfree (to_id);
   chxfree (format);
   chxfree (subject);
   chxfree (thishost);
   chxfree (prevtext);
   chxfree (newtext);

   RETURN  (success);
}
