
/*** BUF_WRITE.   Write to a hose, but with internal buffering.
/    BUF_CLOSE.   Close a hose,   but flush internal buffering.
/
/    ok = buf_write (hose, buf, len);
/    buf_close (hose);
/   
/    Parameters:
/       int      ok;       (successful write?)
/       Hose     hose;       (hose to write or close)
/       char    *buf;      (data to be written)
/       int      len;      (length of data in BUF)
/
/    Purpose:
/
/    How it works:
/
/    Returns: 1 on success, 0 on error (such as broken hose).
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  sweb/bufwrite.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  4/24/97 22:47 New function, based on old mywrite. */
/*: CR  6/12/97 14:56 Rename, use hoses. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include <string.h>

static  struct sockmsg_t sockmsg = {0, 0};
extern  FILE  *sublog;

FUNCTION  buf_write (
   Hose   hose, 
   char  *buf,
   int    len)
{
   int    pos, space, data, move, sent, ackgot;
   char   temp;
   extern int errno;
   ENTRY ("buf_write", "");

   if (len == 0)                                           RETURN (1);

   for (pos=0;   pos < len;   pos+=move) {
      space = SOCKMSGMAX - sockmsg.len;
      data  = len - pos;
      move = (data > space ? space : data);
      memcpy (sockmsg.buf + sockmsg.len, buf+pos, move);
      sockmsg.len += move;
      if (sockmsg.len == SOCKMSGMAX) {

#if UNIX
         sent   = send (hose, (char *) &sockmsg, sizeof(sockmsg), 0);
#endif

#if NUT40 | WIN32
         sent   = hose_write (hose, &sockmsg, sizeof(sockmsg));
#endif

         /*** Simulate slow page. */
/*       logger (1, LOG_FILE, "(sockmsg) ");  */
/*       sleep (10);  */

#if LNX12
         ackgot = recv (hose, &temp, 1, 0);
#endif
/*       fprintf (stderr, "sent=%d, errno=%d, ackgot=%d\n", 
                           sent, errno, ackgot);   */
         sockmsg.len = 0;
      }
   }

   RETURN (1);
}


FUNCTION  void buf_close (Hose hose)
{
   int    ackgot, sent;
   extern int     errno;
   char   temp;

   sockmsg.eof = 1;

#if UNIX
   sent   = send  (hose, (char *) &sockmsg, sizeof(sockmsg), 0);
   ackgot = recv  (hose, &temp, 1, 0);
   close (hose);
#endif

#if NUT40 | WIN32
   sent   = hose_write (hose, &sockmsg, sizeof(sockmsg));
   ackgot = hose_read  (hose, &temp, 1, 0);
   hose_close (hose);
#endif

/* fprintf (stderr, "EOF: sent=%d, errno=%d, ackgot=%d\n", sent, errno, ackgot);  */

   sockmsg.len = 0;
   sockmsg.eof = 0;
   return;
}
