
/*** CREATE_CONF.   Create a new conference: $create_conf ()
/
/    create_conf (result, arg);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       Chix                arg;     (function argument)
/
/    Purpose:
/       Implement CML function $create_conf (name priorg [2nd orgs] ustr)
/
/    How it works:
/       Evaluates to: new conference number
/                     0   if no permission
/                    -1   if conf exists already
/                    -2   no primary org id supplied
/                    -3   database error
/                    -4   bad name
/
/    Returns: 
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  createco.c
/ */

/*** Copyright (C) 1997 Camber-Roth.  All rights reserved. */

/*: CR  9/04/97 17:36 New function. */

#include <stdio.h>
#include "unitcode.h"
#include "done.h"
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

extern int   cml_debug;
extern Clist g_cllist, g_clp;

FUNCTION  create_conf (Chix result, Chix arg)
{
   Clist  node;
   int    cnum, success, op, error;
   char   temp[100];
   Chix   name, word, priorg, orgs;
   int4   func_mgr();
   ENTRY ("create_conf", "");

   name   = chxalloc (L(40), THIN, "create_co name");
   word   = chxalloc (L(40), THIN, "create_co word");
   priorg = chxalloc (L(40), THIN, "create_co priorg");
   orgs   = chxalloc (L(80), THIN, "create_co orgs");

   /*** Make sure this user has the manager bit for creating confs. */
   sysuserid (word);
   if (NOT (func_mgr (name, "mgr_list", word) & MGR_CRCONF))  DONE(0);
   
   /*** If this conference already exists, return -1. */
   chxtoken   (name, nullchix, 1, arg);
   jixreduce  (name);
   if (okconf (name) <= 0)  DONE(-4);
   func_cl (word, "cl_list", name);
   cnum = 0;
   for (node=g_cllist->next;   node!=(Clist) NULL;   node=node->next) {
      if (chxeq (name, node->lname)) {
         cnum = node->cnum;
         break;
      }
   }
   if (cnum > 0)  DONE(-1);

   /*** Get the primary org. */
   chxtoken (priorg, nullchix, 2, arg);
   if (chxvalue (priorg, L(0)) == 0)  DONE(-2);

   /*** Assemble the list of ":organizer"s for writing. */
   chxcpy (orgs, CQ("\n:organizer\n"));
   for (op=2;   chxtoken (word, nullchix, op, arg) >= 0;   ++op) {
      if (chxvalue (word, L(0)) == ':')  break;
      chxcat    (orgs, word);
      chxcatval (orgs, THIN, L('\n'));
   }

   /*** Actually write the text that will become the userlist file. */
   if (unit_lock (XITX, WRITE, L(0), L(0), L(0), L(0), nullchix)) {
      if (unit_make (XITX)) {

         /*** Write a supplied (or default) preface, for ":include", etc. */
         if (chxtoken (nullchix, word, op, arg) >= 0) 
              unit_write (XITX, word);
         else unit_write (XITX, CQ(":include\n:readonly\n"));
         unit_write (XITX, orgs);
         unit_close (XITX);
      }
      unit_unlk (XITX);
   }

   success = make_conf (name, priorg, 0, 0, XITX, &error);
   if (NOT success)  DONE(-3);
   func_cl (word, "cl_reset", name);
   func_cl (word, "cl_list",  name);   /* Put new conf in master Clist. */

done:
   sprintf (temp, "%d", success);
   chxcat  (result, CQ(temp));

   chxfree (name);
   chxfree (word);
   chxfree (priorg);
   chxfree (orgs);
   RETURN  (1);
}
