
/*** FUNC_AD.   Return the resulting value of ad_xxx(), chg_xxx() functions.
/
/    func_ad (result, what, arg, conf);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       char               *what;    (function name)
/       Chix                arg;     (function argument)
/       struct sweb_conf_t *conf;    (server configuration info)
/
/    Purpose:
/       Append value of ad_xxx() function WHAT with arguments ARG to RESULT.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/      Some of the ad_resp and ad_item code could be combined...
/
/    Home:  funcad.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  4/06/95 22:23 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

extern Clist  g_cllist, g_clp;
extern Cstat  g_cstat;
extern Pstat  g_pstat;
extern Vartab g_icache;

extern int   cml_debug;
extern int   g_wrap;
extern int   g_mytext;

FUNCTION  func_ad (
   Chix   result, 
   char  *what, 
   Chix   arg,
   struct sweb_conf_t *conf)
{
   static Resp  resp  = (Resp)  NULL;
   static Chix  author, temp;
   char         tempstr[100];
   int4   cnum, item, rnum, pos, code, prop, zp;
   int    ok, add, is_add_resp, is_add_item;
   ENTRY ("func_ad", "");

   /*** Initialization. */
   if (resp == (Resp) NULL)  {
      resp   = a_mak_resp ("func_ad resp");
      author = chxalloc (L(50), THIN, "func_ad author");
      temp   = chxalloc (L(50), THIN, "func_ad temp");
   }

   /*** $AD_AUTHOR(name).   Prepare psuedonymous author name for next ad. */
   if (streq (what, "ad_author")) {
      chxcpy (author, arg);
      RETURN(1);
   }

   /*** Most ad_xxx() functions take a conference number and a
   /    property number. */
   pos  = L(0);
   cnum = chxint4 (arg, &pos);
   prop = chxint4 (arg, &pos);
   if (pos == L(0))  { BAD("NO CNUM");    RETURN(1); }

   is_add_resp = (streq (what, "ad_resp")  ||  streq (what, "add_resp"));
   is_add_item = (streq (what, "ad_item")  ||  streq (what, "add_item"));

   /*** ADD RESPONSE: $ad_resp (cnum prop inum text) or
   /                  $add_resp(cnum prop inum owner author copied text)  */
   if (is_add_resp) {
      item = chxint4 (arg, &pos);
      if (pos == L(0))  { BAD("NO INUM");    RETURN(1); }

      ok = (touch_cl (cnum)  &&  g_clp->access >= 2);
      if (ok) {
         resp->cnum     = cnum;
         resp->inum     = item;
         resp->textprop = prop;
         resp->bits     = (g_clp->access >= 3 ? M_ISORG : 0);
         chxclear  (resp->copied);
         if (streq (what, "add_resp")) {
            chxtoken  (resp->owner,  nullchix, 4, arg);
            chxtoken  (resp->author, nullchix, 5, arg);
            chxrepval (resp->author, L(0), THIN, L(1), L(' '));
            chxtoken  (temp, nullchix, 6, arg);
            if (chxvalue(temp, L(0)) != '0') {
               sysuserid (resp->copied);
               chxcatval (resp->copied, THIN, L(' '));
               chxrepval (temp, L(0),   THIN, L(1), L(' '));
               chxcat    (resp->copied, temp);
            }
            pos = chxtoken (resp->text->data, nullchix, 7, arg) - 1;
         }
         else {
            sysuserid  (resp->owner);
            chxclear   (resp->author);
            if (NOT EMPTYCHX(author))  chxcpy  (resp->author, author);
            else                       func_my (resp->author, "my_name", arg);
            chxclear   (author);
         }
         chxclear   (resp->text->data);
         col_wrap   (resp->text->data, arg, ++pos, g_wrap);
         ok = a_add_resp (resp, 0, A_WAIT);
/*       fprintf (stderr, "\na_add_resp = %d\n", ok);  */
         ok = (ok == A_OK);

         /*** If adding response succeeded, reload conf status and
         /    rewrite pers status (if changed). */
         if (ok) {
            g_cstat->cnum = 0;
            touch_it (cnum, 0, 0);
            if (mod_add_pstat (g_pstat, resp->inum, resp->rnum, g_mytext))
               a_chg_pstat (g_pstat, P_EVERY, A_WAIT);
         }
      }
      chxcatval (result, THIN, (ok ? '1' : '0'));
   }

   /*** $CHG_RESP(cnum prop inum rnum text).   CHANGE RESPONSE. */
   else if (streq (what, "chg_resp")) {
      item = chxint4 (arg, &pos);
      rnum = chxint4 (arg, &pos);
      if (pos == L(0))  { BAD("NO INUM");    RETURN(1); }

      ok = (touch_cl (cnum)  &&  g_clp->access >= 2);
      if (ok) {
         resp->cnum     = cnum;
         resp->inum     = item;
         resp->rnum     = rnum;
         resp->textprop = prop & M_TEXTTYPE;
         resp->bits     = prop & M_BITS;
         chxclear   (resp->text->data);
         col_wrap   (resp->text->data, arg, ++pos, g_wrap);
         code = a_chg_resp (resp, P_TEXT | P_BITS, A_WAIT);
         ok = (code == A_OK);
      }
      chxcatval (result, THIN, (ok ? '1' : '0'));
      func_re (result, "re_clear", arg, 0);   /* Clear resp cache! */
   }

   /*** $CHG_TITLE(cnum prop inum title).   CHANGE RESPONSE. */
   else if (streq (what, "chg_title")) {
      item = chxint4 (arg, &pos);
      if (pos == L(0))  { BAD("NO INUM");    RETURN(1); }
      ++pos;

      ok = (touch_cl (cnum)  &&  g_clp->access       >= 2 
                             &&  chxvalue (arg, pos) != L(0));
      if (ok) {
         resp->cnum     = cnum;
         resp->inum     = item;
         resp->rnum     = 0;

         chxclear   (resp->title);
         chxcatsub  (resp->title, arg, pos, ALLCHARS);
         code = a_chg_resp (resp, P_TITLE, A_WAIT);
         ok = (code == A_OK);
         item_cache (cnum, item, g_icache, resp);
      }
      chxcatval (result, THIN, (ok ? '1' : '0'));
      func_re (result, "re_clear", arg, 0);   /* Clear resp cache! */
   }

   /*** ADD ITEM: $ad_item (cnum prop title text) or
   /              $add_item(cnum prop owner author title text)  */
   else if (is_add_item) {

      /*** Can this user add an item? */
      ok = touch_cl (cnum);
      chxclear (temp);
      func_co  (temp, "co_add", arg, conf);
      zp  = L(0);
      add = chxint4 (temp, &zp);
      ok  = (ok  &&  g_clp->access + add >= 3);

      if (ok) {
         resp->cnum     = cnum;
         resp->inum     = -1;
         resp->textprop = prop;
         resp->bits     = (g_clp->access >= 3 ? M_ISORG : 0);
         chxclear (resp->copied);

         if (streq (what, "add_item")) {
            chxtoken (resp->owner,  nullchix, 3, arg);
            chxtoken (resp->author, nullchix, 4, arg);
            chxrepval(resp->author, L(0), THIN, L(1), L(' '));
            chxtoken (resp->title, nullchix, 5, arg);
            chxrepval(resp->title, L(0), THIN, L(1), L(' '));
            chxtoken (temp, nullchix, 6, arg);
            if (chxvalue(temp, L(0)) != '0') {
               sysuserid (resp->copied);
               chxcatval (resp->copied, THIN, L(' '));
               chxrepval (temp, L(0),   THIN, L(1), L(' '));
               chxcat    (resp->copied, temp);
            }
            pos = chxtoken (resp->text->data, nullchix, 7, arg);
         }
         else {
            sysuserid  (resp->owner);
            chxclear   (resp->author);
            if (NOT EMPTYCHX(author))  chxcpy  (resp->author, author);
            else                       func_my (resp->author, "my_name", arg);
            chxclear   (author);
            while (chxvalue (arg, pos) == ' ')  ++pos;
            chxnextline (resp->title, arg, &pos);
         }

         chxclear    (resp->text->data);
         col_wrap    (resp->text->data, arg, pos, g_wrap);
         code = a_add_resp (resp, 0, A_WAIT);
         ok = (code == A_OK ? resp->inum : 0);

         /*** If adding item succeeded, reload conf status and
         /    rewrite pers status (if changed). */
         if (ok) {
            g_cstat->cnum = 0;
            touch_it (cnum, 0, 0);
            if (mod_add_pstat (g_pstat, resp->inum, resp->rnum, g_mytext))
               a_chg_pstat (g_pstat, P_EVERY, A_WAIT);
         }
      }
      sprintf    (tempstr, "%d", ok);
      chxofascii (temp, tempstr);
      chxcat     (result, temp);
   }

   else  chxcat (result, CQ("<I>NO FUNC</I>"));

   RETURN(1);
}
