
/*** FUNC_CL.   Return the resulting value of a cl_xxx() function.
/
/    func_cl (result, what, arg);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       char               *what;    (function name)
/       Chix                arg;     (function argument)
/
/    Purpose:
/       Append value of cl_xxx() function WHAT with arguments ARG to RESULT.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  funccl.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  3/28/95 12:00 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"


extern Clist g_cllist, g_clp;
extern FILE *sublog;

FUNCTION  func_cl (Chix result, char *what, Chix arg)
{
   static Chix cname    = nullchix;
   static Chix word, clist;
   static int  first_cl = 1;
   Clist  node, morecl;
   char   temp[200];
   int4   cnum, token, mask;
   int    empty_arg, access;
   Namelist groups, nlnode();
   ENTRY ("func_cl", "");

   if (cname == nullchix) {
      cname = chxalloc (L(40), THIN, "func_cl cname");
      word  = chxalloc (L(40), THIN, "func_cl word");
      clist = chxalloc (L(80), THIN, "func_cl clist");
   }

   /*** $CL_LIST(). */
   if (streq (what, "cl_list")) {

      /*** G_CLLIST is a cache of conference name/number/access information.
      /    Depending on previous calls and the value of ARG, we need to
      /    load different information into this cache. */
      chxcpy         (clist, arg);
      clean_newlines (clist);
      chxtrim        (clist);
      jixreduce      (clist);

      empty_arg = (chxvalue (clist, L(0)) == 0);
      mask = P_EVERY & (~P_MEMBER);
      sysuserid (word);
      if (func_mgr (cname, "mgr_list", word) & (MGR_BEORG | MGR_RMCONF))
         mask = mask | P_OVERRIDE;

      /*** First call, just a_get_clist of ARG. */
      if (first_cl)  {
         chxcpy         (g_cllist->lname, clist);
         a_get_clist    (g_cllist, mask, A_WAIT);
      }

      /*** 2nd+ call, empty g_cllist means we got 'em all already! */
      else if (g_cllist->next != (Clist) NULL  &&  
               chxvalue (g_cllist->lname, L(0)) == 0) ;

      /*** 2nd+ call, empty ARG, go get 'em all! */
      else if (empty_arg) { 
         chxclear       (g_cllist->lname);
         a_get_clist    (g_cllist, mask, A_WAIT);
      }

      /*** 2nd+ call, non-empty g_cllist and ARG; just get new ones. */
      else {

         /*** Build list of conf names in ARG (that are not in g_cllist). */
         morecl = a_mak_clist ("morecl");
         for (token=1;   chxtoken (word, nullchix, token, clist) >= 0;  
            ++token) {
            if (tablepart (g_cllist->lname, word) < 0) {
               chxcat    (morecl->lname, word);
               chxcatval (morecl->lname, THIN, L(' '));
            }
         }

         /*** If anything found, integrate with G_CLLIST. */
         if (chxvalue (morecl->lname, L(0)) != 0) {
            a_get_clist (morecl, mask, A_WAIT);
            merge_clist (g_cllist, morecl);
            chxcatval (g_cllist->lname, THIN, L(' '));
            chxcat    (g_cllist->lname, morecl->lname);
         }
         a_fre_clist (morecl);
      }


      /*** Build the output string of the desired conf numbers. */
      for (node=g_cllist->next;   node!=(Clist) NULL;   node=node->next) {
         if (empty_arg  ||  tablepart (clist, node->lname) >= 0) {
            sprintf (temp, " %d", node->cnum);
            chxcat  (result, CQ(temp));
         }
      }

      first_cl = 0;
      g_clp    = (Clist) NULL;
/*    print_cl(); */
      RETURN(1);
   }

   /*** CL_RESET.  Only used internally to reset the list of "conference
   /    names we have looked for" to "actual conference names we've found"
   /    after a create or rename of a conference. */
   if (streq (what, "cl_reset")) {
      chxclear (g_cllist->lname);
      for (node=g_cllist->next;   node!=(Clist) NULL;   node=node->next) {
         chxcat    (g_cllist->lname, node->lname);
         chxcatval (g_cllist->lname, THIN, L(' '));
      }
      RETURN(1);
   }

   /*** $CL_NUM(name).   Return number of conf 'name' (must be in CL_LIST!) */
   if (streq (what, "cl_num")) {
      chxtoken  (cname, nullchix, 1, arg);
      jixreduce (cname);

      /*** Look for exact match first. */
      for (node=g_cllist->next;   node!=(Clist) NULL;   node=node->next) {
         if (chxeq (cname, node->lname)) {
            sprintf (temp, "%d", node->cnum);
            chxcat  (result, CQ(temp));
            RETURN(1);
         }
      }

      /*** If none, look for abbrev. match. */
      for (node=g_cllist->next;   node!=(Clist) NULL;   node=node->next) {
         if (chxabr (cname, node->lname)) {
            sprintf (temp, "%d", node->cnum);
            chxcat  (result, CQ(temp));
            RETURN(1);
         }
      }

      chxcatval (result, THIN, L('0'));
      RETURN(1);
   }

   /*** All other confxxx() functions take a conference number as an
   /    argument.  "Touch" the global CL stuff to point to the info
   /    for this conference. */
   if      (NOT chxnum (arg, &cnum))   BAD("NO CNUM");

   else if (NOT touch_cl (cnum))       BAD("NO CONF");

   /*** $CL_NAME(). */
   else if (streq (what, "cl_name"))   chxcat (result, g_clp->lname);

   /*** $CL_VISIBLE(). */
   else if (streq (what, "cl_visible"))
      chxcatval (result, THIN, (g_clp->visible ? L('1') : L('0')));

   /*** CL_MEMBER(). */
/*   --------removed this function for now.
   else if (streq (what, "cl_member")) 
      chxcatval (result, THIN, L(g_clp->member ? '1' : '0'));
*/

   /*** CL_ACCESS(). */
   else if (streq (what, "cl_access")) {
      if (chxtoken (word, nullchix, 2, arg) < 0)
         sprintf (temp, "%d", g_clp->access);
      else {
         groups = nlnode (1);
         access = allowed_on (word, groups, cnum);
         nlfree  (groups);
         sprintf (temp, "%d", access);
      }
      chxofascii (result, temp);
   }

   RETURN(1);
}
