
/*** FUNC_CO.   Return the resulting value of a co_xxx() function.
/
/    func_co (result, what, arg, conf);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       char               *what;    (function name)
/       Chix                arg;     (function argument)
/       struct sweb_conf_t *conf;    (server configuration info)
/
/    Purpose:
/       Append value of co_xxx() function WHAT with arguments ARG to RESULT.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  funcco.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  3/28/95 12:00 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

extern int   cml_debug;
extern Clist g_clp;

FUNCTION  func_co (
   Chix   result, 
   char  *what, 
   Chix   arg,
   struct sweb_conf_t *conf)
{
   static Conf  cache = (Conf) NULL;
   static Chix  name, word;
   int4   cnum, atsign, pos;
   int    success, code, token;
   char   temp[512], cname[256];
   ENTRY ("func_co", "");

   /*** Initialization. */
   if (cache == (Conf) NULL) {
      cache = a_mak_conf ("func_co cache");
      word  = chxalloc (L(40), THIN, "func_co word");
      name  = chxalloc (L(40), THIN, "func_co name");
   }

   /*** All co_xxx() functions take a conference number as an
   /    argument.  Decode it, and check the Conf cache. */
   pos  = L(0);
   cnum = chxint4 (arg, &pos);
   if (pos == L(0)  ||  cnum <= 0) {
      BAD("NO CNUM");
      RETURN(1);
   }

   if (cache->cnum != cnum) {
      cache->cnum = cnum;
      code = a_get_conf (cache, P_EVERY, A_WAIT);
      if (code != A_OK) {
         cache->cnum = 0;
         if (cml_debug) {
            sprintf (temp, "func_co: a_get_conf() failed, conf=%d, code=%d\n",
                     cnum, code);
            logger  (0, LOG_FILE, temp);
         }
         RETURN(1);
      }
   }

   /*** $CO_ORG(). */
   if      (streq (what, "co_org")) {
      atsign = chxindex (cache->owner, L(0), CQ("@"));
      if (atsign < 0)  atsign = ALLCHARS;
      chxcatsub (result, cache->owner, L(0), atsign);
   }

   /*** $CO_NAME(). */
   else if (streq (what, "co_name")) {
      chxcat (result, cache->lname);
   }

   /*** $SET_CO_ORG(). */
   else if (streq (what, "set_co_org")) {
      /*** Make sure this user has the manager bit for changing pri org. */
      sysuserid (word);
      if (equal_id (word, cache->owner, nullchix)  || 
          func_mgr (name, "mgr_list", word) & MGR_BEORG) {
         chxtoken   (cache->owner, nullchix, 2, arg);
         success = a_chg_conf (cache, P_OWNER | P_OVERRIDE, A_WAIT);

         /*** If the change succeeded, and *we* are the new primary
         /    organizer, give us full access. */
         if (success == A_OK) {
            touch_it (cnum, 0, 0);
            if (equal_id (word, cache->owner, nullchix)) {
               touch_cl (cnum);
               g_clp->access = 3;
            }
         }
      }
   }

   /*** $CO_INTRO(). */
   else if (streq (what, "co_intro"))  chxcat (result, cache->intro->data);

   /*** $CO_GREET(). */
   else if (streq (what, "co_greet"))  chxcat (result, cache->greet->data);

   /*** $CO_ADD(). */
   else if (streq (what, "co_add"))
      chxcatval (result, THIN, L(cache->add ? '1' : '0'));

   /*** $SET_CO_ADD(). */
   else if (streq (what, "set_co_add")) {
      cache->add = chxint4 (arg, &pos);
      success = (a_chg_conf (cache, P_FADD, A_WAIT) == A_OK);
      chxcatval (result, THIN, L(success ? '1' : '0'));
   }

   /*** $CO_CHANGE(). */
   else if (streq (what, "co_change"))
      chxcatval (result, THIN, L(cache->change ? '1' : '0'));

   /*** $SET_CO_CHANGE(). */
   else if (streq (what, "set_co_change")) {
      cache->change = chxint4 (arg, &pos);
      success = (a_chg_conf (cache, P_FCHANGE, A_WAIT) == A_OK);
      chxcatval (result, THIN, L(success ? '1' : '0'));
   }

   /*** $CO_VISIBLE(). */
   else if (streq (what, "co_visible")) {
      sysuserid (name);
      success = cache->visible;
      chxcatval (result, THIN, L(success ? '1' : '0'));
   }

   /*** $SET_CO_VISIBLE(). */
   else if (streq (what, "set_co_visible")) {
      cache->visible = chxint4 (arg, &pos);
      success = (a_chg_conf (cache, P_FVIS, A_WAIT) == A_OK);
      chxcatval (result, THIN, L(success ? '1' : '0'));
   }

   /*** $CO_USERLIST(). */
   else if (streq (what, "co_userlist")) chxcat (result, cache->userlist->data);


   /*** $SET_CO_USERLIST(). */
   else if (streq (what, "set_co_userlist")) {
      success = 0;
      if (chxvalue (arg, ++pos) != 0) {
         chxclear  (cache->userlist->data);
         chxcatsub (cache->userlist->data, arg, pos, ALLCHARS);
         success = (a_chg_conf (cache, P_USERLIST, A_WAIT) == A_OK); 
      }
      chxcatval (result, THIN, L(success ? '1' : '0'));
   }

   /*** $CO_REMOVE(). */
   else if (streq (what, "co_remove")) {
      success = 0;
      touch_cl (cnum);
      ascofchx (cname, g_clp->lname, L(0), L(256));

      /*** Make sure this user has the manager bit for deleting confs. */
      sysuserid (name);
      if (func_mgr (word, "mgr_list", name) & MGR_RMCONF) {
         success = rmconf (cnum);
      }
      chxcatval (result, THIN, L(success ? '1' : '0'));

      /*** If conference was removed, also remove its upload library. */
      if (success) {
         remove_cl (cnum);
         sprintf   (temp, "%s/%s", conf->caucuslib, cname);
         sysrmdir  (temp);
      }
   }

   /*** $CO_RENAME(cnum newname) */
   else if (streq (what, "co_rename")) {
      touch_cl (cnum);
      success = 0;

      /*** Make sure this user has the manager bit for either 
      /    creating or deleting confs. */
      sysuserid (name);
      if (func_mgr (word, "mgr_list", name) & (MGR_RMCONF | MGR_CRCONF)) {
         chxtoken   (word, nullchix, 2, arg);
         jixreduce  (word);
         if (NOT okconf (word))  success = -1;
         else {
            success = mv_conf (g_clp->lname, word, conf->caucuslib);
            if (success <= 0)  success = success - 2;
         }
      }
      sprintf (temp, "%d", success);
      chxcat  (result, CQ(temp));
      if (success > 0)  {
         chxcpy (g_clp->lname, word);
         func_cl (name, "cl_reset", word);
      }
   }

   /*** $CO_MAKEORG() */
   else if (streq (what, "co_makeorg")) {
      /*** Make sure this user has the manager bit for organizing confs. */
      success = 0;
      sysuserid (name);
      if (func_mgr (word, "mgr_list", name) & MGR_BEORG) {
         chxcat (cache->userlist->data, CQ("\n:organizer\n"));
         chxcat (cache->userlist->data, name);
         success = (a_chg_conf (cache, P_USERLIST | P_OVERRIDE, 
                                A_WAIT) == A_OK); 

         if (success) {
            touch_cl (cnum);
            g_clp->access = 3;  /* Give them org access immediately! */
         }
      }
      chxcatval (result, THIN, L(success ? '1' : '0'));
   }

   RETURN(1);
}
