
/*** FUNC_MGR.   Return the resulting value of a mgr_xxx() function.
/
/    func_mgr (result, what, arg);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       char               *what;    (function name)
/       Chix                arg;     (function argument)
/
/    Purpose:
/       Append value of mgr_xxx() function WHAT with arguments ARG to RESULT.
/
/    How it works:
/
/    Returns: mask value as an integer
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  funcmgr.c
/ */

/*** Copyright (C) 1997 Camber-Roth.  All rights reserved. */

/*: CR  9/03/97 17:51 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

extern int   cml_debug;

FUNCTION  int4 func_mgr (Chix result, char *what, Chix arg)
{
   static Vartab  cache = (Vartab) NULL;
   static Chix    word1, word2, primgr;
   int            mp;
   int4           mask, this;
   char           temp[100];
   int4           mgr_bits();
   ENTRY ("func_mgr", "");

   /*** Initialization. */
   if (cache == (Vartab) NULL) {
      cache = a_mak_vartab ("func_mgr cache");
      cache->id1 = -1;
      a_get_vartab (cache, P_EVERY, A_WAIT);
      word1  = chxalloc (L(40), THIN, "func_mgr word1");
      word2  = chxalloc (L(40), THIN, "func_mgr word2");
      primgr = chxalloc (L(40), THIN, "func_mgr primgr");
      if (cache->name[0] != nullchix)  chxcpy (primgr, cache->name[0]);
      else                          chxofascii (primgr, "caucus");
   }


   if (streq (what, "set_mgr_list")) {
      /*** Ensure that only people who are allowed can change the
      /    manager list! */
      sysuserid (word1);
      if (mgr_bits (word1, cache, word2) & MGR_SET) {
         a_fre_vartab (cache);
         cache = a_mak_vartab ("funcmgr cache");
         cache->id1 = -1;
   
         /*** Make sure the primary manager is always first in the list
         /    and has all capabilities. */
         chxtoken (word1, nullchix, 1, arg);
         if (NOT chxeq (word1, primgr))  vartab_add (cache, primgr, CQ("-1"));

         /*** Write the rest of the list. */
         for (mp=1;   chxtoken (word1, nullchix, mp, arg) >= 0;   ++mp) {
            chxtoken   (word2, nullchix, ++mp, arg);
            vartab_add (cache, word1, word2);
         }
         chxcpy (cache->value[0], CQ("-1"));
         a_chg_vartab (cache, P_EVERY, A_WAIT);
      }
   }


   else if (streq (what, "mgr_list")) {
      chxtoken (word1, nullchix, 1, arg);

      /*** $mgr_list()   (no args) */
      if (chxvalue (word1, L(0)) == 0) {
         for (mp=0;   mp<cache->used;   ++mp) {
            if (chxvalue (cache->name[mp], L(0)) == 0)  continue;
            chxcat  (result, cache->name[mp]);
            chxnum  (cache->value[mp], &this);
            this = this | MGR_IS;
            sprintf (temp, " %d ", this);
            chxcat  (result, CQ(temp));
         }
      }

      /*** $mgr_list(userid) */
      else {
         mask = mgr_bits (word1, cache, word2);
         chxcat    (result, word1);
         chxcatval (result, THIN, L(' '));
         sprintf   (temp, "%d ", mask);
         chxcat    (result, CQ(temp));
      }
   }

   RETURN (mask);
}
   

FUNCTION int4 mgr_bits (Chix who, Vartab list, Chix temp)
{
   int4  mask, last, this;
   int   mp, match;
   ENTRY ("mgr_bits", "");

   for (mask=0, mp=0;   mp<list->used;   ++mp) {
      match = 0;
      last  = chxlen(list->name[mp]) - 1;

      if      (chxvalue (list->name[mp], L(0))   == '*')   match = 1;
      else if (chxvalue (list->name[mp], last) == '*') {
         chxclear   (temp);
         chxcatsub  (temp, list->name[mp], L(0), last);
         match = chxindex (who, L(0), temp) == 0;
      }
      else match = chxeq (list->name[mp], who);

      if (match) {
         chxnum (list->value[mp], &this);
         mask = mask | this | MGR_IS;
      }
   }

   RETURN (mask);
}

