
/*** FUNC_PW.   PassWord (and userid) functions.
/
/    code = func_pw (result, what, arg, allow, conf);
/   
/    Parameters:
/       int      code;    (integer value of return code)
/       Chix     result;  (append resulting value here)
/       char    *what;    (function name)
/       Chix     arg;     (function argument)
/       int      allow;   (allow protected functions)
/       SWconf   conf;    (sweb configuration params)
/
/    Purpose:
/       Evaluate the CML functions:
/          $pw_can_add()
/          $pw_can_change()
/          $pw_can_delete()
/          $pw_can_verify()
/          $pw_add    (id pw [override])
/          $pw_change (id pw)
/          $pw_delete (id)
/          $pw_verify (id pw)
/
/    How it works:
/
/    Returns (for pw_can... functions)
/             1 on success
/             0 if not allowed
/
/    Returns: (for pw_... functions)
/             0 on success
/            -1 on "not allowed" by permission bits or swebd.conf PW_CAN...
/             1 on bad action
/             2 on userid already exists (add a user) 
/             3 on userid does not exist (chg password) 
/             4 on bad password
/             6 on userid too int4 (add)
/             7 on bad chars in userid (add)
/             8 on temp password file busy 
/             9 on can't read  password file 
/            10 on can't write to temp password file 
/            11 on error on close, disk could be full 
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  sweb/funcpw.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR 10/13/97 23:10 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

FUNCTION  int  func_pw (Chix result, char *what, 
                        Chix arg,    int allow, SWconf conf)
{
   int    value, override, code, internal;
   char   temp[256], id[200], pw[200], over[200], self[200];
   Chix   word, ignore;
   ENTRY ("func_pw", "");


   if (strindex (what, "pw_can") == 0) {
      value = 0;
      if      (streq (what, "pw_can_add"))     value = conf->pw_can_add;
      else if (streq (what, "pw_can_change"))  value = conf->pw_can_change;
      else if (streq (what, "pw_can_delete"))  value = conf->pw_can_delete;
      else if (streq (what, "pw_can_verify"))  value = conf->pw_can_verify;
      chxcatval (result, THIN, value ? L('1') : L('0'));
      RETURN (value);
   }

   /*** Parse ID, PW, and OVERRIDE arguments. */
   ascofchx (temp, arg, L(0), L(200));
   strtoken (id,   1, temp);
   strtoken (pw,   2, temp);
   strtoken (over, 3, temp);
   override = (NULLSTR(over) ? 0 : atoi(over));

   /*** Get SELF, the current userid. */
   word   = chxalloc (L(20), THIN, "func_pw word");
   ignore = chxalloc (L(20), THIN, "func_pw ignore");
   sysuserid (word);
   ascofchx  (self, word, L(0), L(200));

   internal = streq (conf->pw_prog, "internal");
   code     = -1;   /* resulting error or success code.  -1 => not allowed. */

   /*** ADD requires OVERRIDE arg or MGR_MKID manager bit. */
   if (streq (what, "pw_add")  &&  conf->pw_can_add) {
      if (override || (func_mgr (ignore, "mgr_list", word) & MGR_MKID) ) {
         code = (internal
              ? pw_actions ("add", conf->pw_path, id, pw, conf->pw_lock) 
              : pw_extern  ("add", conf->pw_path, id, pw, conf->pw_lock,
                                                             conf->pw_prog) );
      }
   }

   /*** CHANGE can be done by self, or with MGR_CHGPASS manager bit. */
   if (streq (what, "pw_change")  && conf->pw_can_change) {
      if (streq (self, id) || 
         (func_mgr (ignore, "mgr_list", word) & MGR_CHGPASS) ) {
         code = (internal
              ? pw_actions ("change", conf->pw_path, id, pw, conf->pw_lock) 
              : pw_extern  ("change", conf->pw_path, id, pw, conf->pw_lock,
                                                             conf->pw_prog) );
      }
   }

   /*** DELETE requires MGR_RMID manager bit. */
   if (streq (what, "pw_delete")  &&  conf->pw_can_delete) {
      if (func_mgr (ignore, "mgr_list", word) & MGR_RMID) {
         code = (internal
              ? pw_actions ("delete", conf->pw_path, id, pw, conf->pw_lock) 
              : pw_extern  ("delete", conf->pw_path, id, pw, conf->pw_lock,
                                                             conf->pw_prog) );
      }
   }

   /*** Anyone can verify... at least, so far. */
   if (streq (what, "pw_verify")  &&  conf->pw_can_verify) {
         code = (internal
              ? pw_actions ("verify", conf->pw_path, id, pw, conf->pw_lock) 
              : pw_extern  ("verify", conf->pw_path, id, pw, conf->pw_lock,
                                                             conf->pw_prog) );
   }

   sprintf (temp, "%d", code);
   chxcat  (result, CQ(temp));

   chxfree (word);
   chxfree (ignore);
   RETURN  (code);
}
