
/*** FUNC_SCAN.  Scan LINE for CML functions, substitute value into RESULT.
/
/    func_scan (result, line, conf, cmlargs, vars, form, incvars, 
/                       macs, cfile, otab);
/   
/    Parameters:
/       Chix      result;   (put resulting text here)
/       Chix      line;     (line of CML text to scan)
/       SWconf    conf;     (server configuration info)
/       char     *cmlargs;  (CML script arguments)
/       Vartab    vars;     (CML variables)
/       Vartab    form;     (form data variables)
/       Chix      incvars;  (text of 'include' variables)
/       Vartab    macs;     (defined macros)
/       CML_File  cfile;    (original cml file for this page)
/       Obtab     otab;     (object table)
/
/    Purpose:
/       Scan the CML text in LINE, evaluate CML functions as they
/       are found, and put the result into RESULT.
/
/    How it works:
/       Scans for strings of the form "$name(value)", and inserts in
/       their place the evaluation of the function.  Recursion is allowed, 
/       i.e. VALUE may also contain function evaluations.
/
/       See func_value() for a list of defined functions.
/
/       The form $(var) evaluates to the value of variable VAR.
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  funcscan.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  3/10/95 17:48 New function. */
/*: CR  3/25/95 13:46 Convert to chix. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

#define  ST_TEXT   0
#define  ST_NAME   1
#define  ST_ARG    2

FUNCTION  func_scan (result, line, conf, cmlargs, vars, form, 
                     incvars, macs, cfile, otab)
   Chix       result, line;
   char      *cmlargs;
   SWconf     conf;
   Vartab     vars, form;
   Chix       incvars;
   Vartab     macs;
   CML_File   cfile;
   Obtab      otab;
{
   static int  recursive_evaluation = 0;
   static int  protect              = 0;
   static Chix dollar = nullchix;
   Chix        name, args, recurs, append;
   int4   pos, value;
   int    type, state, parens;
   char   ascname[200];
   static int counter = 0;
   int        thistime;
   ENTRY ("func_scan", "");

   thistime = ++counter;
   if (dollar == nullchix)  dollar = chxalsub (CQ("$"), L(0), L(4));

   type   = chxtype (line);
   name   = chxalloc ( L(30), type, "func_scan name");
   args   = chxalloc ( L(30), type, "func_scan args");
   recurs = chxalloc (L(100), type, "func_scan recurs");
   append = chxalloc (L(100), type, "func_scan append");

   chxclear (result);
   state = ST_TEXT;
   for (pos=0L;   (value = chxvalue (line, pos));   ++pos) {

      if (state == ST_TEXT) {

         if      (value == '$') {
            chxclear (name);
            state = ST_NAME;
         }

         else {
            /*** A "\" escapes the next character. */
            if (value == '\\'  &&  chxvalue (line, pos+1) == '$') {
               ++pos;
               value = '$';
            }
            chxcatval (result, type, value);
         }
      }

      else if (state == ST_NAME) {
         if (value == '(') {
            chxclear (args);
            parens = 0;
            state  = ST_ARG;
         }
         else if (value == ' ') ;
         else chxcatval (name, type, value);
      }




      /*** In the middle of argument list. */
      else if (state == ST_ARG) {

         /*** ...skip over ()'s inside the argument list. */
         if      (value == '(') {
            ++parens;
            chxcatval (args, type, value);
         }
         else if (value == ')'  &&  parens > 0) {
            --parens;
            chxcatval (args, type, value);
         }

         /*** At the end of the argument list... */
         else if (value == ')') {
            ascofchx  (ascname, name, L(0), L(100));

/*          if (streq (ascname, "reval"))    recursive_evaluation = 1; */
            if (streq (ascname, "protect"))  protect              = 1;

            chxclear   (append);
            if (chxindex  (args, L(0), dollar) >= 0) {
               func_scan  (recurs, args,         conf, cmlargs, vars, form, 
                                                 incvars, macs, cfile, otab);
               func_value (append, name, recurs, conf, cmlargs, vars, form, 
                                   protect, incvars, macs, cfile, otab);
            }
            else  
               func_value (append, name,   args, conf, cmlargs, vars, form,
                                   protect, incvars, macs, cfile, otab);
/*          fix_dollar (append, 1); */



            /*** If recursively expanding the result contains more 
            /    $function()s... */
            if (streq (ascname, "reval"))    recursive_evaluation = 1;
            if (recursive_evaluation && chxindex (append, L(0), dollar) >= 0) {
               func_scan  (recurs, append, conf, cmlargs, vars, form, 
                                                 incvars, macs, cfile, otab);
               chxcpy     (append, recurs);
            }

            chxcat (result, append);
            state = ST_TEXT;
            if (streq (ascname, "reval"))    recursive_evaluation = 0;
            if (streq (ascname, "protect"))  protect              = 0;
         }

         /*** Char in the middle of the arg list. */
         else chxcatval (args, type, value);
      }
   }

   /*** E-Report: if we came out in the middle of a NAME or ARGS, complain! */
   if (state == ST_NAME) {
      char temp[512];
      sprintf (temp, "\n%s:%d No ()'s after apparent function $%s\n",
                      cfile->filename, cfile->lnums[cfile->cdex],
                      ascquick(name));
      chxcat  (cfile->errtext, CQ(temp));
   }
   if (state == ST_ARG) {
      char temp[512];
      sprintf (temp, "\n%s:%d No closing ) after function call $%s(...\n",
                      cfile->filename, cfile->lnums[cfile->cdex],
                      ascquick(name));
      chxcat  (cfile->errtext, CQ(temp));
   }

   chxfree (name  );
   chxfree (args  );
   chxfree (recurs);
   chxfree (append);

   RETURN  (1);
}
