
/*** FUNC_SETPER.   Handle set_per_xxx() functions.
/
/    func_setper (result, what, arg);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       char               *what;    (function name)
/       Chix                arg;     (function argument)
/
/    Purpose:
/       Append value of set_per_xxx() functions WHAT 
/       with arguments ARG to RESULT.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/       Last name handling.
/
/    Home:  funcsetp.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR  6/28/98 22:34 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

#define  FAIL     { success = 0;  goto done; }
#define  SUCCEED  { success = 1;  goto done; }

extern FILE *sublog;
extern Person g_person;
extern int    g_wrap;
extern int    g_mytext;

FUNCTION  func_setper (Chix result, char *what, Chix arg)
{
   static Person  person = (Person) NULL;
   static Chix    ignore;
   static Chix    word;
   Pstat          pstat;
   char           temp[200];
   int            i, code, success;
   int4           cnum;
   ENTRY ("func_setp", "");

   /*** Initialization. */
   if (person == (Person) NULL)  {
      person = a_mak_person ("func_per person");
      word   = chxalloc (L(40), THIN, "func_person word");
      ignore = chxalloc (L(40), THIN, "func_person ignore");
   }

   /*** Only managers with MGR_MKID can use $set_per_xxx() functions. */
   sysuserid (word);
   if (NOT (func_mgr (ignore, "mgr_list", word) & MGR_MKID)) FAIL;

   /*** All set_per_xxx() functions take a userid as an argument. */
   chxtoken (person->owner, nullchix, 1, arg);
   if (chxvalue (person->owner, L(0)) == 0)                  FAIL;
   clean_newlines (person->owner);

   success = 0;

   /*** Trying to create a user? */
   if (streq (what, "set_per_name")) {
      success = 0;
      if (chxtoken (nullchix, person->name, 2, arg) >= 0) {
         clean_newlines (person->name);
         chxclear (g_person->owner);        /* Clear Person cache. */
         for (i=2;   chxtoken (word, nullchix, i, arg) >= 0;   ++i)
            chxcpy (person->lastname, word);
         jixreduce (person->lastname);
         clean_newlines (person->lastname);
         code = a_chg_person (person, P_NAME | P_LASTNM, A_WAIT);
         if (code == A_NOPERSON) {
            code = a_add_person (person, P_NAME | P_LASTNM, A_WAIT);
         }
         success = (code == A_OK);
      }
   }

   else if (streq (what, "set_per_phone")) {
      chxclear (g_person->owner);        /* Clear Person cache. */
      chxtoken (nullchix, person->phone, 2, arg);
      clean_newlines (person->phone);
      success = (a_chg_person (person, P_PHONE, A_WAIT) == A_OK);
   }

   else if (streq (what, "set_per_intro")) {
      chxclear (g_person->owner);        /* Clear Person cache. */
      chxclear (person->intro->data);
      chxtoken (nullchix, word, 2, arg);
      col_wrap (person->intro->data, word, L(0), g_wrap);
      success = (a_chg_person (person, P_TEXT, A_WAIT) == A_OK);
   }

   /*** $set_per_join (userid cnum) */
   else if (streq (what, "set_per_join")) {
      pstat = a_mak_pstat("funcsetp pstat");
      chxtoken (pstat->owner, nullchix, 1, arg);
      chxtoken (word,         nullchix, 2, arg);
      if (chxnum (word, &cnum)) {
         pstat->cnum = cnum;
         code = a_add_pstat (pstat, P_EVERY | P_OVERRIDE, A_WAIT);
         success = (code == A_OK);
      }
      a_fre_pstat (pstat);
   }

done:
   chxcatval (result, THIN, (success ? L('1') : L('0')) );
   RETURN(1);
}
