
/*** FUNC_T2URL.  Detect URL's and turn them into appropriate HTML.
/
/    func_t2url (arg, conf, cmlargs, vars, form, incvars, macs, cfile);
/   
/    Parameters:
/       Chix    arg;     (function argument)
/       SWconf  conf;    (server configuration info)
/       char   *cmlargs; (CML script arguments)
/       Vartab  vars;    (CML variable list)
/       Vartab  form;    (form data variable list)
/       Chix    incvars; (text of 'include' variables)
/       Vartab  macs;    (defined macros)
/       CML_File cfile;  (original cml file)
/
/    Purpose:
/       Output filter: detect URL's in text, and turn them into HTML.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  funct2url.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  8/30/95 17:28 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

#define  PN   6
#define  NEW_WINDOW  1    /* Should hyperlinks open a new window? */

FUNCTION  func_t2url (
        Chix    arg, 
        SWconf  conf, 
        char   *cmlargs,
        Vartab  vars,
        Vartab  form,
        Chix    incvars,
        Vartab  macs,
        CML_File cfile)
{
   static Chix protocol [PN] = 
      {nullchix, nullchix, nullchix, nullchix, nullchix, nullchix};
   static Chix cctp, result, url, newurl;
   int4   cval, start, cv, scan, confnum, pos;
   char   work[200], target[200];
   int    ui, replace, slash, html;
   ENTRY ("func_t2url", "");

   if (protocol[0] == nullchix) {
      protocol[0] = chxalsub (CQ("http:/"),       L(0), L(10));
      protocol[1] = chxalsub (CQ("gopher:/"),     L(0), L(10));
      protocol[2] = chxalsub (CQ("telnet:/"),     L(0), L(10));
      protocol[3] = chxalsub (CQ("ftp:/"),        L(0), L(10));
      protocol[4] = chxalsub (CQ("mailto:"),      L(0), L(10));
      protocol[5] = chxalsub (CQ("https:/"),      L(0), L(10));
      cctp        = chxalsub (CQ("http:/caucus"), L(0), L(20));
      result      = chxalloc (L(100), THIN, "func_t2url result");
      newurl      = chxalloc (L(100), THIN, "func_t2url newurl");
      url         = chxalloc ( L(80), THIN, "func_t2url url");
   }


   /*** Now, look for URL's, so we can automatically convert them. */
   for (scan=0L;   (1);   ) {

      /*** Find start of next URL (if any). */
      for (ui=0;   ui<PN;   ++ui)
         if ( (start = chxindex (arg, scan, protocol[ui])) >= 0)    break;
      if (start < 0)  break;

      /*** Find end of URL, and remove any 'trim' characters from its end. */
      for (cv=start;   cval = chxvalue (arg, cv);   ++cv) {
         if (cval >= 256  ||  conf->url_end[cval])  break;
      }
      while ( (cval = chxvalue(arg, cv-1)), conf->url_trim[cval])  --cv;

      /*** Replace URL with HTML version, making name a link. */
      chxclear  (url);
      chxcatsub (url, arg, start, cv - start);
      replace = 1;

      /*** "http:/caucus" strings get special handling. */
      if (chxindex  (url, L(0), protocol[0]) == 0)
         replace = caucus_url (result, url, conf, cmlargs, 
                                       vars, form, incvars, macs, cfile);

      /*** Regular URLs. */
      if (replace == 1)  chxcpy (result, url);

      if (replace > 0) {
         chxcpy (newurl, CQ("<A HREF=\""));
         chxcat (newurl, result);
         chxcat (newurl, CQ("\""));
#if NEW_WINDOW
         if (replace == 1) {

            /*** Attempt to determine if this likely to be an HTML file.
            /    If so, we'll want to open a new window, and otherwise not.
            /    Heuristics to determine if a URL is an HTML file are:
            /    (a) last slash is last char
            /    (b) last slash is part of "http://"
            /    (c) .htm, .html, or .shtml follows last slash
            /    (d) otherwise, no "." after last slash.  */
            html  = 0;
            slash = chxrevdex (url, L(2000), CQ("/"));
            if (slash == chxlen (url) - 1)  html = 1;
            if (slash > 0  &&  chxvalue (url, slash-1) == '/')    html = 1;
            if (chxrevdex (url, L(2000), CQ(".htm"))   > slash)   html = 1;
            if (chxrevdex (url, L(2000), CQ(".html"))  > slash)   html = 1;
            if (chxrevdex (url, L(2000), CQ(".shtml")) > slash)   html = 1;
            if (NOT html  &&  chxrevdex (url, L(2000), CQ(".")) < slash)
                                                                  html = 1;
            if (html) {
               /*** Old code explicitly used javascript to open a window. 
               sprintf (target, "%swidth=%d,height=%d%s",
                  " onClick=\"win = window.open ('', 'fromcau',"
                  " 'resizable=1,toolbar=1,scrollbars=1,menubar=1,location=1,",
                  conf->new_win_x, conf->new_win_y,
                  "');\"  TARGET=\"fromcau\" ");
               chxcat (newurl, CQ(target));
               */

               /*** New code calls a macro, which could do anything. */
               sprintf (target, "%%target(%d %d)", conf->new_win_x,
                                                   conf->new_win_y);
               chxcat  (newurl, CQ(target));
            }
         }
#endif
         chxcat (newurl, CQ(">"));
         chxcat (newurl, url);
         chxcat (newurl, CQ("</A>"));
   
         chxalter (arg, start, url, newurl);
         scan = start + chxlen(newurl);
      }
      else scan = start + chxlen (url);
   }

   RETURN (1);
}
