
/*** FUNC_VAR.   variable functions.
/
/    func_var (result, what, arg);
/   
/    Parameters:
/       Chix                result;  (append resulting value here)
/       char               *what;    (function name)
/       Chix                arg;     (function argument)
/
/    Purpose:
/       Append value of $user_var(), $conf_var() function WHAT 
/       with arguments ARG to RESULT.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:  should we refresh things automatically after a
/                 certain period of time, in case someone else
/                 put new information in?
/
/    Home:  funcvar.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  9/06/95 12:56 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

#define  SITE_CLEARED  -100

extern int   cml_debug;

FUNCTION  func_var (Chix result, char *what, Chix arg)
{
   static Chix   name = nullchix;
   static Vartab cvar, uvar, ivar, svar;
   int    ok, slot, code, num, empty;
   char   temp[200];
   ENTRY ("func_var", "");

   /*** Initialization. */
   if (name == nullchix) {
      name = chxalloc     (L(40), THIN, "func_var name");
      cvar = a_mak_vartab ("func_var cvar");
      uvar = a_mak_vartab ("func_var uvar");
      ivar = a_mak_vartab ("func_var ivar");
      svar = a_mak_vartab ("func_var svar");
      svar->id1 = SITE_CLEARED;
   }

   /*** $CONF_VAR(cnum vname). */
   if (streq (what, "conf_var")) {
      touch_cv (cvar, arg, 0);
      chxtoken (name, nullchix, 2, arg);
      slot = vartab_find (cvar, name);
      chxcat (result, cvar->value[slot]);
   }

   /*** $CLEAR_CONF_VAR() */
   else if (streq (what, "clear_conf_var")) {
      cvar->id1 = -1;
   }

   /*** $SET_CONF_VAR(cnum vname value). */
   else if (streq (what, "set_conf_var")) {
      touch_cv (cvar, arg, 0);
      chxtoken (name, nullchix, 2, arg);
      slot = vartab_find (cvar, name);
      chxtoken (nullchix, cvar->value[slot], 3, arg);
      ok = (a_chg_vartab (cvar, 0, A_WAIT) == A_OK);
   }

   /*** $LIST_CONF_VAR(cnum vroot) */
   else if (streq (what, "list_conf_var")) {
      touch_cv (cvar, arg, 0);
      empty = (chxtoken (name, nullchix, 2, arg) < 0);
      for (num=0;   num<cvar->used;   ++num) {
         if (empty  ||  chxindex (cvar->name[num], 0, name) == 0) {
            chxcat (result, cvar->name[num]);
            chxcatval (result, THIN, L(' '));
         }
      }
   }

   /*** $ITEM_VAR(cnum inum vname). */
   else if (streq (what, "item_var")) {
      touch_cv (ivar, arg, 1);
      chxtoken (name, nullchix, 3, arg);
      slot = vartab_find (ivar, name);
      chxcat (result, ivar->value[slot]);
   }

   /*** $CLEAR_ITEM_VAR() */
   else if (streq (what, "clear_item_var")) {
      ivar->id1 = -1;
   }

   /*** $SET_ITEM_VAR(cnum inum vname value). */
   else if (streq (what, "set_item_var")) {
      touch_cv (ivar, arg, 1);
      chxtoken (name, nullchix, 3, arg);
      slot = vartab_find (ivar, name);
      chxtoken (nullchix, ivar->value[slot], 4, arg);
      ok = (a_chg_vartab (ivar, 0, A_WAIT) == A_OK);
   }

   /*** $LIST_ITEM_VAR(cnum inum vroot) */
   else if (streq (what, "list_item_var")) {
      touch_cv (ivar, arg, 1);
      empty = (chxtoken (name, nullchix, 3, arg) < 0);
      for (num=0;   num<ivar->used;   ++num) {
         if (empty  ||  chxindex (ivar->name[num], 0, name) == 0) {
            chxcat (result, ivar->name[num]);
            chxcatval (result, THIN, L(' '));
         }
      }
   }


   /*** $SITE_VAR(vname). */
   else if (streq (what, "site_var")) {
      if (svar->id1 == SITE_CLEARED) {
         svar->id1 = -2;
         a_get_vartab (svar, 0, A_WAIT);
      }
      slot = vartab_find (svar, arg);
      chxcat (result, svar->value[slot]);
   }

   /*** $CLEAR_SITE_VAR() */
   else if (streq (what, "clear_site_var")) {
      svar->id1 = SITE_CLEARED;
   }

   /*** $SET_SITE_VAR(vname value). */
   else if (streq (what, "set_site_var")) {
      if (svar->id1 == SITE_CLEARED) {
         svar->id1 = -2;
         a_get_vartab (svar, 0, A_WAIT);
      }
      chxtoken (name, nullchix, 1, arg);
      slot = vartab_find (svar, name);
      chxtoken (nullchix, svar->value[slot], 2, arg);
      ok = (a_chg_vartab (svar, 0, A_WAIT) == A_OK);
   }

   /*** $LIST_SITE_VAR(cnum vroot) */
   else if (streq (what, "list_site_var")) {
      empty = (chxtoken (name, nullchix, 2, arg) < 0);
      for (num=0;   num<svar->used;   ++num) {
         if (empty  ||  chxindex (svar->name[num], 0, name) == 0) {
            chxcat (result, svar->name[num]);
            chxcatval (result, THIN, L(' '));
         }
      }
   }


   /*** $USER_VAR(userid vname). */
   else if (streq (what, "user_var")) {
      if (touch_uv (uvar, arg, name)) {
         chxtoken (name, nullchix, 2, arg);
         slot = vartab_find (uvar, name);
         chxcat (result, uvar->value[slot]);
      }
   }

   /*** $CLEAR_USER_VAR() */
   else if (streq (what, "clear_user_var")) {
      chxclear (uvar->owner);
   }

   /*** $SET_USER_VAR(userid vname value). */
   else if (streq (what, "set_user_var")) {
      if (touch_uv (uvar, arg, name)) {
         chxtoken (name, nullchix, 2, arg);
         slot = vartab_find (uvar, name);
         chxtoken (nullchix, uvar->value[slot], 3, arg);
         code = a_chg_vartab (uvar, 0, A_WAIT);
         ok = (code == A_OK);
      }
   }

   /*** $LIST_USER_VAR(userid vroot) */
   else if (streq (what, "list_user_var")) {
      touch_uv (uvar, arg, name);
      empty = (chxtoken (name, nullchix, 2, arg) < 0);
      for (num=0;   num<uvar->used;   ++num) {
         if (empty  ||  chxindex (uvar->name[num], 0, name) == 0) {
            chxcat (result, uvar->name[num]);
            chxcatval (result, THIN, L(' '));
         }
      }
   }


   RETURN (1);
}


FUNCTION  touch_cv (var, arg, item)
   Vartab var;
   Chix   arg;
   int    item;
{
   int4   pos;
   int    cnum, ok, inum;
   ENTRY ("touch_cv", "");

   ok   = 1;
   pos  = L(0);
   cnum = chxint4 (arg, &pos);
   inum = chxint4 (arg, &pos);
   if (cnum != var->id1  ||  (item && inum!=var->id2)) {
      var->id1 = cnum;
      var->id2 = (item ? inum : 0);
      ok = (a_get_vartab (var, 0, A_WAIT) == A_OK);
   }

   RETURN (ok);
}

FUNCTION  touch_uv (uvar, arg, word)
   Vartab uvar;
   Chix   arg, word;
{
   int    ok;
   ENTRY ("touch_uv", "");

   ok = 1;
   chxtoken (word, nullchix, 1, arg);
   if (NOT chxeq (uvar->owner, word)) {
      chxcpy (uvar->owner, word);
      ok = (a_get_vartab (uvar, 0, A_WAIT) != A_BADARG);
   }

   RETURN (ok);
}
