
/*** INCLUDE_CML.   Handle "include" directive to include another file.
/
/    include_cml (cd, cmlname, vars, form, conf, query, broke, incvars, 
/                     macs, cfile, otab);
/   
/    Parameters:
/       int       cd;      (connection descriptor)
/       Chix      cmlname; (name of file to include)
/       Vartab    vars;    (CML variable list)
/       Vartab    form;    (form data variable list)
/       SWconf    conf;    (server configuration info)
/       char     *query;   (CML arguments from client)
/       int      *broke;   (was a 'break' encountered?)
/       Chix      incvars; (rest of include line)
/       Vartab    macs;    (defined macros)
/       CML_File  cfile;   (original cml file)
/       Obtab     otab;    (object table)
/
/    Purpose:
/       Handle CML "include" directive.  This (recursively) includes
/       another CML file in the one currently being processed.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  include.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  9/05/95 14:02 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include <sys/types.h>
#include "api.h"

extern FILE *sublog;

FUNCTION  include_cml (cd, cmlname, vars, form, conf, query, broke, 
                           incvars, macs, cfile, otab, contd)
   int      cd;
   Chix     cmlname, incvars;
   Vartab   vars, form, macs;
   SWconf   conf;
   char    *query;
   int     *broke, *contd;
   CML_File cfile;
   Obtab    otab;
{
   CML_File newfile, make_cml();
   char     filename[256];
   int      num, size, lnum;
   ENTRY ("include_cml", "");

   newfile = make_cml(cfile->errtext);

   /*** Load the entire included CML file into memory. */
   ascofchx (filename, cmlname, L(0), 255);
   size = loadcml (filename, conf, 0, newfile, vars, form, query, macs, otab);

   if (size > 0) {
      /*** Parse the entire CML file. */
      lnum = parse_cml (cd, newfile, size, 0, conf, vars, form, query, 
                        broke, incvars, macs, otab, contd);
   }
   else {
      char temp[512];
      sprintf (temp, "\n%s:%d Can't open include file '%s'\n",
                      cfile->filename, cfile->lnums[cfile->cdex],
                      filename);
      chxcat  (cfile->errtext, CQ(temp));
   }

   /*** Free the temporary structure... */
   free_cml (newfile);

   RETURN (lnum);
}
