
/*** LOADCML.   Load an entire CML file (and included files) into memory.
/
/    newsize = loadcml (cmlname, conf, size, cfile, vars, form, 
/                                query, macs, otab);
/   
/    Parameters:
/       int        newsize;  (lines used after loading CMLNAME)
/       char      *cmlname;  (name of CML file)
/       SWconf    *conf;     (server configuration info)
/       int        size;     (current lines used in CFILE)
/       CML_File   cfile;    (put text of cml file here)
/       Vartab     vars;     (list of CML variables)
/       Vartab     form;     (forms data)
/       char      *query;    (?)
/       Vartab     macs;     (defined macros)
/       Obtab      otab;     (object table)
/
/    Purpose:
/       Load an entire CML file (and by recursive calls, any $include()
/       files) into memory in CFILE.
/
/    How it works:
/
/    Returns: new size of CFILE.
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  loadcml.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  8/03/95 15:29 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"

extern int   cml_debug, cml_errors;

FUNCTION  loadcml (cmlname, conf, size, cfile, vars, form, query, macs, otab)
   char    *cmlname;
   SWconf   conf;
   int      size;
   CML_File cfile;
   Vartab   vars, form, macs;
   char    *query;
   Obtab    otab;
{
   static Chix lparen = nullchix;
   static Chix rparen = nullchix;
   char   fullcml[250], temp[250];
   int    lnum;
   Chix   word, result;
   FILE  *fp;
   ENTRY ("loadcml", "");

   /*** Initialize static chix used for parsing $include (file). */
   if (lparen == nullchix) {
      lparen = chxalsub (CQ("("), L(0), L(1));
      rparen = chxalsub (CQ(")"), L(0), L(1));
   }

   /*** Build the full name of the CML file, and open it. */
   sprintf (fullcml, "%s/%s", conf->cml_path, cmlname);
   if ( (fp = fopen (fullcml, "r")) == NULL) {
      if (cml_debug) {
/*       fprintf (sublog, "Failed to open: %s\n", fullcml); */
/*       fflush  (sublog); */
      }
/*    if (cml_errors)  fprintf (stderr, "Failed to open: %s\n", fullcml); */
      if (cfile->lines[size] == (Chix) NULL)
          cfile->lines[size] = chxalloc (L(200), THIN, "process load");
      chxclear (cfile->lines[size]);
      RETURN   (size);
   }

   word   = chxalloc (L(50), THIN, "loadcml word");
   result = chxalloc (L(80), THIN, "loadcml result");

   /*** Load the entire CML file into memory. */
   strcpy (cfile->filename, cmlname);
   for (lnum=0;   (1);   ++size) {
      if (cfile->lines[size] == (Chix) NULL)
          cfile->lines[size] = chxalloc (L(200), THIN, "process load");

      if (size > MAXCML - 5) {
         sprintf (temp, "\"<h2>CML File '%s' too Large!</h2>\n", cmlname);
         chxofascii (cfile->lines[size], temp);
         cfile->lnums[size] = lnum;
         ++size;
         break;
      }

      if (NOT readcml (fp, conf, cfile->lines[size], 0, &lnum))  break;
      cfile->lnums[size] = lnum;

      /*** Test for $include(file)... */
      chxtoken (word, nullchix, 1, cfile->lines[size]);
      ascofchx (temp, word, L(0), L(9));
      if (NOT streq (temp, "$include"))  continue;

      /*** Got one.  Recursively load it. */
      if (cml_debug) {
/*       fprintf (sublog, "\n$include(%s)\n", temp); */
/*       fflush  (sublog); */
      }
      /*** Evaluate any functions inside $include(...) */
      chxclear  (result);
      chxclear  (word);
      func_scan (result, cfile->lines[size], conf, query, vars, form, 
                         word, macs, cfile, otab);
      if (chxinside (result, lparen, word, rparen) < 0)  continue;
      ascofchx (temp, word, L(0), ALLCHARS);
      while (stralter (temp, " ", "")) ;
      size = loadcml (temp, conf, size, cfile, vars, form, query, macs, otab);
   }
   fclose  (fp);

   chxfree (word);
   chxfree (result);
   RETURN  (size);
}
