
/*** PROCESS_REQUEST.  Process a SWEB request from a validated client.
/
/    process_request (cd, conf, lice, ip_addr);
/   
/    Parameters:
/       Hose                cd;       (connection descriptor)
/       struct sweb_conf_t *conf;     (server configuration info)
/       int                 lice;     (license code)
/       char               *ip_addr;  (client's ip address)
/
/    Purpose:
/       Read a SWEB request (to parse and evaluate a CML file) from
/       a validated client, and send the results back to the client.
/
/    How it works:
/       If LICE indicates an invalid license, return an appropriate
/       warning page to the client, rather than the page they requested.
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  process.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR  3/09/95 17:33 New function. */
/*: CR  3/25/95 13:13 Convert to chix. */

#include <stdio.h>
#include "systype.h"
#include "chixuse.h"
#include "api.h"
#include "sweb.h"

#if UNIX | NUT40
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <signal.h>
#endif

#define  CMAX  256

static CML_File        cfile;
static Obtab           otab;
static Chix            errtext;

static Vartab          vars, form, macs;
struct        http_t   http;

extern FILE *sublog;
extern int   cml_debug;

static int fail_count = 0;

FUNCTION  process_request (
   Hose                cd, 
   struct sweb_conf_t *conf,
   int                 lice,
   char               *ip_addr )
{
   static   int  first_call = 1;
   static   int4 request1;
   char     query[CMAX], cmlname[CMAX], temp[CMAX];
   int      num, size, broke, contd;
   Chix     empty;
   CML_File make_cml();
   Obtab    make_obtab();
   ENTRY ("process_request", "");

   /*** On first call, initialize the array of CML lines, 
   /    and the forms-data variable list. */
   if (first_call) {
      errtext = chxalloc (L(256), THIN, "process errtext");
      cfile = make_cml(errtext);
      otab  = make_obtab(100);
      vars  = a_mak_vartab ("process vars");
      form  = a_mak_vartab ("process form");
      macs  = a_mak_vartab ("process macs");
      first_call = 0;
      request1   = time(NULL);
   }
   empty   = chxalloc (  L(4), THIN, "process empty");


   /*** Get the name of the CML file to be processed.   Make
   /    sure it exists. */
   hose_read (cd, cmlname,  100, 0);    cmlname[99] = '\0';
   hose_read (cd,  query,   100, 0);    query  [99] = '\0';
   fix_query (query);

   /*** Determine if this is a "reload". */
   conf->reload = reload_check (cmlname, query);

   strcpy  (conf->cmlname, cmlname);
   sprintf (temp, "   page %s?%s... reload=%d", cmlname, query, conf->reload);
   logger  (0, LOG_FILE, temp);
   if (lice == LICE_BADCLOCK)   strcpy (cmlname, "badclock.cml");
   if (lice == LICE_BADSYSID)   strcpy (cmlname, "badsysid.cml");
   if (lice == LICE_NODISK)     strcpy (cmlname, "baddisk.cml");
   if (lice == LICE_EXPIRED  &&  (time(NULL) - request1) > 60)
                                strcpy (cmlname, "expired.cml");

   /*** Get specific CGI environment variables. */
   hose_read (cd, &http.user_agent[0], 100, 0);
   hose_read (cd, &http.referer[0],    200, 0);

   /*** Check for 'spoofing'. */
   if (conf->check_client_ip  &&  NOT streq (ip_addr, conf->ip_addr)) { 
      strcpy (cmlname, "syserr1.cml");
      conf->timeout = 10;
   }

   /*** Read forms data. */
   read_form_data (cd, form, conf);

   /*** Include the globals. */
   cml_globals();

   /*** Load the entire CML file into memory. */
   size = loadcml (cmlname, conf, 0, cfile, vars, form, query, macs, otab);

   /*** If the file does not exist, complain! */
   if (size == 0)  say_no_file (cd, cmlname, conf);

   /*** If it does, parse the entire CML file. */
   else    parse_cml (cd, cfile, size, 0, conf, vars, form, query, &broke, 
                                 empty, macs, otab, &contd);

   logger (0, LOG_FILE, " done");
   chxfree (empty);

   RETURN (1);
}
