
/*** SHAKE_SUBSERVER.  "Shake hands" with subserver, and confirm
/    that we can request a file from it. 
/
/    hose = shake_subserver (sockpath, subpid, subcode, userid);
/   
/    Parameters:
/       Hose   hose;       (returned hose connection to subserver)
/       char  *sockpath,   (path to socket directory)
/       int    subpid,     (pid of desired subserver)
/       int    subcode,    (security code for desired subserver)
/       char  *userid,     (browser user's userid)
/
/    Purpose:
/       "Shake hands" with the desired subserver, i.e. try to connect
/       to it, and see if it thinks we are it's designated user.
/
/    How it works:
/       Opens a socket (Unix) or named pipe (NT) to the subserver 
/       pid # SUBPID, tell it who we are, and see if it likes us. 
/
/    Returns: hose descriptor to read/write to subserver
/             BADHOSE on failure (no subserver, or it doesn't like us)
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  sweb/shakesub.c
/ */

/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: CR 10/27/96 16:13 New function.  Based on old swebcli.c */
/*: CR  4/15/97 15:55 Integrate with NT. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"
#include "sweb.h"

#if UNIX | NUT40
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#endif

#include <time.h>
#include <signal.h>
#include <errno.h>


FUNCTION  Hose shake_subserver (
   char  *sockpath,
   int    subpid,
   int    subcode,
   char  *userid  )
{
   Hose   sockfd, hose_open();
   int    success;
   char   temp[256], remote[256];

   /*** Open a connection to the "hose" the subserver is listening on. */
   sockfd = hose_open (sockpath, subpid, userid);
   if (sockfd == BADHOSE)  return (BADHOSE);

   /*** OK, the subserver's there.  Tell it who we are, and see if it
   /    will serve us. */
   sprintf (temp, "%07d", subcode);
   hose_write   (sockfd, temp,    7);
   hose_write   (sockfd, userid, 25);
   env_var (remote, "REMOTE_ADDR");
   hose_write   (sockfd, remote, 25);

   /*** Nope, it didn't like us.  Could be the user is trying to spoof
   /    swebd; or maybe it's a "stale" URL with a subpid that has been
   /    re-used by someone else. */
   hose_read (sockfd, temp, 3, 0);
   if (strcmp (temp, "NO") == 0)  {
      hose_close (sockfd);
      logger (1, LOG_FILE, "got a NO from the subserver");
      return (BADHOSE);
   }

   return (sockfd);
}
