/*** SWEBA.   Caucus 3.0 stand-alone CML interpreter.
/
/    Invoked as:
/       sweba   conf  cmlfile   [arg1 arg2 ... ]
/
/    Arguments:
/       conf     sweba configuration file
/       cmlfile  CML file to interpret (relative to CML_Path in 'conf')
/       arg1     first argument (as in $arg(1)) to 'cmlfile'.
/       arg2     etc.
/
/    Purpose:
/       Interpret and execute a CML script in "stand-alone" mode,
/       i.e. as a regular program run from the shell (as opposed
/       to being driven from a web browser).
/
/    How it works:
/
/    Exit status: 0 on success, 1 on failure.
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  sweba.c
/ */

/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: CR  8/20/96 13:49 New program. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include "api.h"
#include "license.h"
#include "systype.h"

#if WIN32
#include <windows.h>
#include <iostream.h>
#endif

/*** Define globals needed by other functions (even if not used). */
FILE              *sublog     = NULL;
int                cml_debug  = 0;
int                cml_errors = 1;
struct http_t      http;

extern struct license_t license;

char copyright[] =
"Copyright (C) 1996 Screen Porch LLC.  All rights reserved.";

char *trade[] = {
"This program belongs to Screen Porch LLC.  It is considered a trade secret",
"and is not to be divulged or used by parties who have not received",
"written authorization from the owner."};

int    output_default = 1;


FUNCTION  main (int argc, char *argv[])
{
   static Vartab              vars, form, macs;
   static struct  sweb_conf_t sweb_conf;
   static CML_File            cfile;
   static Obtab               otab;
   Obtab               make_obtab();
   static Chix                errtext;
   Initapi                    api;
   int                        lice_code, num, size, broke, contd;
/* int                        cd = 1; */
   Hose                       cd;
   Chix                       empty;
   char                       cmlname[200], query[200];
   CML_File                   make_cml();
   ENTRY ("sweba main", "");

#if UNIX
   cd = 1;
#endif
#if WIN32
   cd = GetStdHandle (STD_OUTPUT_HANDLE);
#endif

   if (argc < 3) {
      fprintf (stderr, "Usage: sweba conf cmlfile\n");
      exit    (1);
   }

   /*** Load the SWEBD configuration file. */
   strcpy (sweb_conf.filename, argv[1]);
   if (NOT load_conf (&sweb_conf)) {
      fprintf (stderr, "Cannot read configuration file %s.\n", argv[1]);
      exit    (2);
   }

   empty   = chxalloc (  L(4), THIN, "process empty");
   errtext = chxalloc (L(256), THIN, "process errtext");

   /*** Initialize the Caucus API for this user. */
   api = a_mak_init ("sweba init");
   chxcpy (api->confid, CQ(sweb_conf.caucus_path));
   sysuserid (empty);
   strcpy (sweb_conf.remote, ascquick(empty));
   strcpy (api->userid, sweb_conf.remote);
   api->terminal = 0;
   api->diskform = sweb_conf.diskform;
   if (a_init (api) != A_OK) {
      fprintf (stderr, 
                "Can't a_init.  caucus_path='%s'\n", sweb_conf.caucus_path);
      exit (1);
   }
   lice_code = check_license (&license);
   if (lice_code != LICE_OK  &&  lice_code != LICE_EXPIRED)  exit (2);
   syslockfile (sweb_conf.caucus_path, "MISC/lockcml");

   /*** Initialize the array of CML lines, and empty VARS and FORMs sets. */
   cfile = make_cml(errtext);
   otab  = make_obtab (100);
   vars  = a_mak_vartab ("sweba vars");
   form  = a_mak_vartab ("sweba form");
   macs  = a_mak_vartab ("sweba macs");


   strcpy (cmlname, argv[2]);

   /*** Include the globals. */
   cml_globals();

   /*** Load the entire CML file into memory. */
   size = loadcml (cmlname, &sweb_conf, 0, cfile, vars, form, 
                   query, macs, otab);

   /*** Parse the entire CML file. */
   for (query[0]='\0', num=3;   num<argc;   ++num) {
      strcat (query, argv[num]);
      strcat (query, " ");
   }
   chxclear  (empty);
   parse_cml (cd, cfile, size, 0, &sweb_conf, vars, form, 
                  query, &broke, empty, macs, otab, &contd);

   /*** This messes things up in BSDI... */
/* buf_close (cd); */

   a_exit();
   exit (0);
}
