
/*** SWEBS.   Caucus-Web subserver.
/
/    Invoked as:
/       swebs   config   secure
/
/    Arguments:
/       config   full pathname of SWEB configuration file.
/       secure   security code for this subserver.
/
/    Purpose:
/
/    How it works:
/
/    Exit status: 0 on success, 1 on failure.
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  sweb/swebs.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  5/26/97 17:08 New program, derived from 3.1 swebd subserver. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"
#include "sweb.h"
#include "systype.h"
#include "license.h"
#include <sys/types.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <setjmp.h>

#if UNIX
#include <unistd.h>
#endif

int                 cml_debug      = 0;
int                 output_default = 0;
int                 cml_errors     = 0;
struct sweb_conf_t  conf;

static jmp_buf      jmpbuf;

extern struct license_t license;

char copyright[] =
"Copyright (C) 1996 Screen Porch LLC.  All rights reserved.";

char *trade[] = {
"This program belongs to Screen Porch LLC.  It is considered a trade secret",
"and is not to be divulged or used by parties who have not received",
"written authorization from the owner."};


FUNCTION  main (int argc, char *argv[])
{
   int     clientlen, child, caucus, root, run;
   Hose    cd, hose_accept();
   char    temp[1024], remote[32], ip_addr[32], tmpdir[128];
   int     first_service = 1;
   int     subcode, error, pagetime;
   int     lice_code = 0;
   int     count = 0;
   int4    kbytes, files;
   Initapi api;
   void    sub_shutdown();
   void    jumper();
   char   *mytime();

   ENTRY ("swebs main", "");

   sys_nt();
   signal (SIGINT, SIG_IGN);
   logger_init ("", argv[0], "sws");

   if (argc < 3) {
/*    logger (1, LOG_FILE, "Usage: swebs config_file securecode"); */
      display_license();
      exit   (1);
   }

   /*** Load the SWEBD configuration file. */
   strcpy (conf.filename, argv[1]);
   if (NOT load_conf (&conf)) {
      sprintf (temp, "Cannot read configuration file '%s'.", argv[1]);
      logger  (1, LOG_FILE, temp);
      exit    (2);
   }
   logger_init (conf.caucus_path, argv[0], "sws");
   sscanf (argv[2], "%d", &conf.secure);
   conf.secure = conf.secure ^ 0xAA55; 

   /*** Log subserver start-up. */
#if UNIX | NUT40
   sprintf (temp, "\n"
     "    Request   Actual\n"
     "  euid: %3d   %3d\n"
     "  ruid: %3d   %3d\n"
     "  egid: %3d   %3d\n"
     "  rgid: %3d   %3d\n",
     conf.caucus_id,    (int) geteuid(), conf.real_id,      (int) getuid(),
     conf.caucus_group, (int) getegid(), conf.real_group,   (int) getgid());
   logger (0, LOG_FILE, temp);
#endif

   /*** Subserver timeout (or a HANGUP from an external source) should
   /    trigger 'jumper', which will make us exit cleanly. */
#if UNIX | NUT40
   signal (SIGHUP,  jumper);
   signal (SIGALRM, jumper); 
#endif

#if UNIX | NUT40
   /*** Ignore "broken" pipe signal from users too quick to click. */
   signal (SIGPIPE, SIG_IGN);

   /*** Ignore "child died" signal (created by use of $asynch() function). */
   signal (SIGCHLD, SIG_IGN);
#endif


   /*** Check for minimum amount of disk space; if too small,
   /    encode in lice_code. */
   if (sysavail (conf.caucus_path, &kbytes, &files, &error)) {
      if (kbytes < MIN_DISK_SPACE  ||
          (files < MIN_DISK_FILES  &&  files != -1)) lice_code = LICE_NODISK;
   }

   /*** Process each connection and request from the browser client, 
   /    until we get a timeout or HANGUP. */
   if (setjmp (jmpbuf) == 0)  while (1) {

      /*** Wait for a client to request to talk to us, within
      /    conf.timeout seconds. */
      if ( (cd = hose_accept (&conf, conf.timeout)) == BADHOSE)  break;
      
      /*** Get the client's security code, userid, and ip address. */
      hose_read (cd, temp,     7, 0);     temp   [ 7] = '\0';
      hose_read (cd, remote,  25, 0);     remote [25] = '\0';
      hose_read (cd, ip_addr, 25, 0);     ip_addr[25] = '\0';
      sscanf (temp, "%7d", &subcode);
      if (first_service) {
         strcpy (conf.remote,  remote);
         strcpy (conf.ip_addr, ip_addr);
      }

      sprintf (temp, "remote='%s', userid='%s', secure=%07d, subcode=%06d",
                     conf.remote, remote, conf.secure, subcode);
      logger (0, LOG_FILE, temp);

      /*** If this is an illegal attempt to communicate (bad userid
      /    or bad security code), complain, and don't serve anything. */
      if (conf.secure != subcode  ||  NOT streq (remote, conf.remote)) {
         hose_write (cd, "NO", 3);
         hose_close (cd);
         continue;
      }

      if (first_service) {
         first_service = 0;
         sprintf (temp, "connect (%s)...", conf.remote);
         logger (0, LOG_FILE, temp);

         /*** Initialize the Caucus API for this user. */
         api = a_mak_init ("subserver init");
         chxcpy (api->confid, CQ(conf.caucus_path));
         strcpy (api->userid, conf.remote);
         api->terminal = 0;
         api->diskform = conf.diskform;
         if (a_init (api) != A_OK) {
            sprintf (temp, 
                 "Can't a_init.  caucus_path='%s'\n", conf.caucus_path);
            logger (1, LOG_FILE, temp);
            sub_shutdown(-1);
         }
         if (lice_code == 0)  lice_code = check_license (&license);
         syslockfile (conf.caucus_path, "MISC/lockcml");
      }


      /*** This is a valid userid & request, go ahead and process it. */
      hose_write (cd, "OK", 3);
      pagetime = conf.pagetime;
      hose_write (cd, (char *) &pagetime, sizeof(int));
      process_request (cd, &conf, lice_code, ip_addr);

      buf_close (cd);
   }

   logger (0, LOG_FILE, "shutting down!");
   sub_shutdown(23);
}


FUNCTION  void jumper (int value)
{
   longjmp (jmpbuf, 1);
}


/*** Controlled shutdown of subserver. */
FUNCTION  void sub_shutdown (int value)
{
   char   temp[200];
   char  *mytime();

   logger (0, LOG_FILE, "in sub_shutdown");

   /*** Delete any temporary files created by file uploads. */
   form_temp (0);

#if UNIX | NUT40
   sprintf (temp, "%s/SOCKET/sweb%06d", conf.caucus_path, (int) getpid());
   unlink  (temp);
#endif
   a_exit  ();

   /*** Log subserver shut-down. */
   sprintf (temp, "stop:  %s, signal=%d\n", mytime(), value);
   logger  (0, LOG_FILE, temp);

   exit    (0);
}
