
/*** SWEBSOCK.  Send a browser request to a swebd sub-server, including
/    (possibly) an initial request to assign a sub-server.
/
/    Invoked as:
/       swebsock
/
/    Arguments: (supplied as environment variables)
/       PATH_INFO cases:
/          goodpid/code/SP/file.cml...    (send request for file.cml)
/          deadpid/code/SP/file.cml...    (dead subserver)
/          0/0/SP/file.cml                (get me new subserver)
/
/    Purpose:
/
/    How it works:
/
/    Exit status: 0 on success, 1 on failure.
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  swebsock.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR 10/27/96 13:59 New program, merging on old swebcli & swebtalk. */

#include <stdio.h>
#include "chixuse.h"
#include "sweb.h"
#include <sys/types.h>
#include <signal.h>
#include <setjmp.h>

#if UNIX
#include <unistd.h>
#endif

#if UNIX
#include <sys/socket.h>
#include <sys/un.h>
#endif

#if WIN32
#include <fcntl.h>
#include <io.h>
#endif

#define  MAX_STALL  10   /* # of retries for a "stalled" subserver. */

static unsigned char buffer[PIPEBUF+2];
static          char sockpath[200];
static          int  subpid;
static jmp_buf       jmpbuf;

int                  debug = 0;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   struct sockmsg_t   sockmsg;
   Hose   sockfd, shake_subserver();
   int    subcode, dataread, got, amount, attempts;
   int    acksent, bp, pagetime;
   int4   content_length;
   char   remote[100], pid[20], secure[20], content[200];
   char   temp[256], userid[40], pathinfo[300], cml[200], tmpdir[200];
   char   ack;
   char  *socket_path();
   char  *mytime();
   FILE  *fp;
   void   swebt_shutdown (int value);
   void   jumper();

#if WIN32
   /*** Make sure we read forms, etc. input from the web server in
   /    binary mode, else the CONTENT_LENGTH count will be off! */
   _setmode (0, _O_BINARY);
#endif


   /*** ==============================================================
   /    SECTION A: get SOCKPATH and the subserver pid.  */

   /*** Determine the location of SOCKPATH, the directory where the
   /    sockets live. */
   logger_init ("", argv[0], "soc");
   systmpdir (tmpdir, 0);
   strcpy (sockpath, socket_path ("", argv[0]));

   /*** If SOCKPATH is incorrect, log fact in /tmp/talklog and exit. */
   sprintf (temp, "%s/swebfail", sockpath);
   if ( (fp = fopen (temp, "r")) != NULL)  fclose (fp);
   else {
      sprintf (temp, 
         "bad sockpath!\npid=%6d, time=%s, sockpath='%s', argv[0]='%s'\n", 
                       (int) getpid(), mytime(), sockpath, argv[0]);
      logger (1, LOG_FILE, temp);
      exit   (0);
   }


   /*** Verify that swebsock was started. */
   sprintf (temp,  "pid=%6d, time=%s, sockpath='%s'", 
                    (int) getpid(), mytime(), sockpath);
   logger (0, LOG_FILE, temp);

   /*** Get the subserver connection information (and the requested CML
   /    script) out of PATH_INFO. */
   env_var  (userid, "REMOTE_USER");
   if (NOT userid[0])  env_var (userid, "AUTH_USER");
   env_var  (pathinfo, "PATH_INFO");
   if (NULLSTR(pathinfo))  {
      printf ("Content-type: text/plain\n\n");
      sprintf (temp, "No PATH_INFO!\n"
                     "pid=%6d, time=%s, userid=%s No Path Info!\n",
                      (int) getpid(), mytime(), userid);
      logger (1, LOG_FILE | LOG_OUT, temp);
      exit   (1);
   }
   sscanf  (pathinfo, "/%d/%d/%s", &subpid, &subcode, cml);


   /*** ==============================================================
   /    SECTION B: connect to the subserver. */

   /*** If we've got a desired subserver, try to "shake hands" with it. */
   if (subpid > 0) {
      sockfd = shake_subserver (sockpath, subpid, subcode, userid);
      if (sockfd == BADHOSE)  subpid = 0;
   }
      
   /*** If we're looking for a new subserver (or the handshake failed),
   /    request a new subserver and try to "shake hands" with *it*. */
   if (subpid == 0) {
      new_subserver (sockpath, &subpid, &subcode);
      if (subpid > 0)  
         sockfd = shake_subserver (sockpath, subpid, subcode, userid);
   }

   if (subpid < 0) {
      printf ("Content-type: text/plain\n\n");
      sprintf (temp, "Could not start swebs!\n  time=%s, userid=%s",
                      mytime(), userid);
      logger  (1, LOG_FILE | LOG_OUT, temp);
      exit    (1);
   }

   /*** If the handshake failed, give up. */
   if (sockfd == BADHOSE) {
      printf ("Content-type: text/plain\n\n");
      sprintf (temp, "Handshake failed!\n  pid=%6d, time=%s, userid=%s",
                      (int) getpid(), mytime(), userid);
      logger  (1, LOG_FILE | LOG_OUT, temp);
      exit    (1);
   }
      
   /*** Read the "timeout" value for swebsock from swebs! */
   got = hose_read (sockfd, (char *) &pagetime, sizeof(pagetime), 120);

   /*** ==============================================================
   /    SECTION C:  Write data to subserver, and echo results to stdout. */

   /*** Decode the POST'ed content length, if any. */
   env_var (content, "CONTENT_LENGTH");
   if (NULLSTR(content))   content_length = 0;
   else                    sscanf (content, "%d", &content_length);

   /*** Tell the subserver which CML file we want, and the value
   /    of the arguments (if any). */
   cml [99] = '\0';
   hose_write (sockfd, cml,  100);
   env_var (temp, "QUERY_STRING");
   temp[99] = '\0';
   hose_write (sockfd, temp, 100);

   /*** Pass on requested CGI environment variables. */
   env_var (temp, "HTTP_USER_AGENT");
   hose_write (sockfd, temp, 100);
   env_var (temp, "HTTP_REFERER");
   hose_write (sockfd, temp, 200);

   /*** Read any data POST'ed by a form, from stdin, and write it
   /    to the subserver. */
   sprintf (temp, "%07d", content_length);
   hose_write (sockfd, temp, 8);
   for (dataread=0;   dataread < content_length;   dataread += got) {
      amount = content_length - dataread;
      if (amount > PIPEREAD)  amount = PIPEREAD;
#if UNIX | NUT40
      got =  read (0, buffer, amount);
#endif
#if WIN32
      got = _read (0, buffer, amount);
#endif
      /*** If we've run out of data, but CONTENT_LENGTH is not
      /    satisfied, pad with zeroes.   (Shouldn't happen, but
      /    one never knows... */
      if (got <= 0) {
         got = amount;
         for (bp=0;   bp<got;   ++bp)  buffer[bp] = '\0';
      }

      buffer[got] = '\0';
      pipewrite (sockfd, (char *) buffer, got, PIPEWRITE);
   }

   /*** Read subserver's response, and echo it back out to stdout, until
   /    there's no more.  If the subserver appears to "stall", sleep
   /    for a bit and try again up to MAX_STALL times. (That last
   /    sentence no longer makes sense... ???) */
#if UNIX | NUT40
   signal (SIGHUP,  jumper);
   signal (SIGALRM, jumper);
#endif
   if (setjmp (jmpbuf) != 0)  swebt_shutdown(0);
   
   attempts = 0;   
   ack = 0xFF;
   do {
#if UNIX | NUT40
      alarm (pagetime);  
#endif

      got = hose_read (sockfd, (char *) &sockmsg, sizeof(sockmsg), pagetime);

#if LNX12
      acksent = send (sockfd, &ack, 1, 0);
#endif

#if UNIX | NUT40
      alarm (  0);
#endif

      if (got <= 0)  {
         if (++attempts > MAX_STALL)  break;
         syssleep (5);
         continue;
      }

      sockmsg.buf[sockmsg.len] = '\0';
      fputs (sockmsg.buf, stdout);
      fflush (stdout);

/*    { char temp[10000];
        sprintf (temp, "'%s'", sockmsg.buf);
        logger  (1, LOG_FILE, temp);
      } */

/*    syssleep(3);  */   /* Simulate slow client. */

   } while (NOT sockmsg.eof);

   hose_close (sockfd);
   exit  (0);
}


/*** Do a controlled shutdown and cleanup on receipt of signals. */

FUNCTION  void swebt_shutdown (int value)
{
   logger (0, LOG_FILE, "Alarm shutdown");
   exit   (0);
}


FUNCTION  void jumper (int value)
{
   longjmp (jmpbuf, 1);
}
