
/*** SYSFSIZE.   Return the size of a file, in bytes.
/
/    ok = sysfsize (fname, bytes, error);
/
/    Parameters:
/       int    ok;          (success?)
/       char  *fname;       (full pathname of the file)
/       int4  *bytes;       (returned size of file)
/       int   *error;       (error code if failure)
/
/    Purpose:
/       Determine the size of a file.
/
/    How it works:
/
/    Returns: 1 on success, 0 on failure.  Sets ERROR on failure.
/
/    Error Conditions:  (see syserror.h.)
/       SE_NOTIMP     Function not implemented for this system.
/       SE_BADFIL     File FNAME does not exist.
/       SE_NOINFO     Cannot get information for unknown reason.
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  application.
/
/    Operating system dependencies: considerable!
/
/    Known bugs:      none
/
/    Home:  sys/sysfsize.c
/ */

/*** Copyright (C) 1991 Camber-Roth.  All rights reserved. */
/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: CR  8/08/91 12:45 New function. */
/*: NT  4/08/97 16:00 Revamp for Caucus 3.2. */

#include <stdio.h>
#include "handicap.h"
#include "systype.h"
#include "entryex.h"
#include "syserror.h"
#include "int4.h"

#include <sys/types.h>
#include <sys/stat.h>

FUNCTION  sysfsize (fname, bytes, error)
   char  *fname;
   int4  *bytes;
   int   *error;
{
/*--------------------------System V Unix--------------------------------*/
   struct stat  statbuf;

   ENTRY  ("sysfsize", "");

   if (stat (fname, &statbuf))      { *error = SE_BADFIL;    RETURN (0); }
   if (statbuf.st_size < 0)         { *error = SE_NOINFO;    RETURN (0); }
   *error = 0;
   *bytes = statbuf.st_size;

/* printf ("sysfsize: size=%d\n", *bytes); */
   RETURN (1);
}
