
/*** SYSTMPDIR.  Determine the directory for temporary files on this system.
/
/    systmpdir (dir, set);
/
/    Parameters:
/       char   *dir;       (directory name)
/       int     set;       (0=>get, 1=>set)
/
/    Purpose:
/       Get (or set) the full pathname for the directory for
/       temporary files on this system.
/
/    How it works:
/       systmpdir (str, 0)  puts the current temporary directory pathname
/       in STR.  (The default is "/tmp").
/
/       systmpdir (str, 1)  sets the current temporary directory to STR.
/       (This is ignored for Windows/NT.)
/
/    Known bugs:      none
/
/    Home:  sys/systmpdi.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  5/01/97 17:11 New function. */
/*: CR  6/11/98 15:05 Add 2nd arg. */

#include <stdio.h>
#include "handicap.h"
#include "chixuse.h"
#include "systype.h"

#if NUT40 | WIN32
#include <windows.h>
#include <iostream.h>
#endif

FUNCTION  systmpdir (char *dir, int set)
{

#if UNIX
#define  TEMPMAX  256
   static char tempdir[TEMPMAX] = {'/', 't', 'm', 'p', 0};

   if (set  &&  strlen (dir) < TEMPMAX)  strcpy (tempdir, dir);

   if (NOT set)                          strcpy (dir, tempdir);
#endif

#if NUT40 | WIN32
#define BUFSIZE  1000
   DWORD  pathsize;
   char   buf[BUFSIZE];

   if (NOT set) {
      pathsize = GetTempPath (BUFSIZE, buf);
      if (pathsize >= BUFSIZE)  { dir[0] = '\0';   return (1); }
   
      if (buf[pathsize-1] == '\\')  --pathsize;
      buf[pathsize] = '\0';
      strcpy (dir, buf);
   }
#endif

   return (1);
}
