
/*** STRMATCH.  Find word in TABLE that WORD is an abbreviation for.
/
/    STRMATCH searches through TABLE to see if WORD is an abbreviation
/    of one of TABLE's entries.  It returns:
/      -2  if WORD is an abbreviation for 2 or more entries in TABLE
/      -1  if WORD is an abbreviation for NO entries in TABLE
/       n  if WORD is an abbreviation for entry N (0, 1, 2...) in TABLE.
/
/    "...is an abbreviation" means "is an initial substring of".
/
/    A table is a single string, containing a list of tokens separated
/    by blanks.  Each token consists of one or more words, separated by
/    slashes.  For example, here's a simple table with 3 entries:
/
/       "yes  no  help/?"
/
/    Entry 0 is "yes", 1 is "no".  Entry 2 is either "help" or "?".  */

/*** Copyright (C) 1987 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:53 Source code master starting point */
/*: AM  2/07/89 10:33 Jcaucus changes*/
/*: CR  1/27/90 16:12 Fix *&^%! ENTRY/RETURN to be consistent! */
/*: CX  6/07/92 20:43 New function, formerly str tablematch. */

#include <stdio.h>
#include "caucus.h"
 
#define  STARTWORD   1
#define  CHECKWORD   2
#define  NEXTPART    3
#define  NEXTWORD    4

FUNCTION  strmatch (table, word) 
   char  *table, *word;
{ 
   char  *w, *s, *stop;
   short  state, which, res;

   ENTRY ("strmatch", "");

   if (table==NULL  ||  word==NULL  ||
       NULLSTR(table)   ||  NULLSTR(word))         RETURN (-1);

   state  = STARTWORD;
   which  =  0;
   res = -1;
   stop   = table + strlen(table) + 1;

   /*** STRMATCH is implemented as a finite-state machine.  As we march
   /    S character by character through TABLE, we are either:
   /      STARTWORD   looking for start of next token in TABLE
   /      CHECKWORD   checking WORD against word in current token
   /      NEXTPART    looking for next word in current token
   /      NEXTWORD    skipping to end of token.
   /
   /    Processing terminates when S goes BEYOND the null character at
   /    the end of TABLE.  Note that if we get a match (set RES) twice,
   /    then WORD is ambiguous and we return -2. */

   for (s=table;   s != stop;   )  switch (state) {

      case (STARTWORD):
         if (*s != ' ')             { w = word;           state = CHECKWORD; }
         else                                       ++s;
         break;

      case (CHECKWORD):
         if      (*w== 0  &&  res>=0)                     RETURN(-2)
         else if (*w== 0)           { res = which;        state = NEXTWORD; }
         else if (*w==*s)           { ++w;          ++s;                    }
         else                                             state = NEXTPART;
         break;

      case (NEXTPART):
         if      (*s=='/')          { w = word;     ++s;  state = CHECKWORD; }
         else if (*s==' ')          { ++which;            state = STARTWORD; }
         else                                       ++s;
         break;

      case (NEXTWORD):
         if      (*s==' ')          { ++which;            state = STARTWORD; }
         else                                       ++s;
         break;
      }

   RETURN (res);
}
