
/*** CAT2.   Concatenate a file onto the end of a self-extractor program.
/
/    Invoked as:
/       cat2  program  file  result
/
/    Purpose:
/       Concatenate FILE onto the end of PROGRAM, put into RESULT.
/       Assumes PROGRAM is a self-extractor, built on the model
/       of UNCAT.C.
/
/    How it works:
/       Copies PROGRAM to RESULT, appends a 64-byte block of 0xAA,
/       and then copies FILE onto the end of RESULT.
/
/    Exit status:
/
/    Home:  xins/cat2.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/04/98 17:45 1st version. */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#define  MAXBUF 4096

main (int argc, char *argv[])
{
   int  in1, in2, out, got, ct;
   char buf[MAXBUF];

   if (argc <3) {
      printf ("Usage: cat2 program file result\n");
      exit (1);
   }

   in1= _open (argv[1], O_RDONLY | O_BINARY);
   in2= _open (argv[2], O_RDONLY | O_BINARY);
   out= _open (argv[3], O_WRONLY | O_BINARY | O_CREAT, _S_IREAD | _S_IWRITE);
   if (in1 < 0) { printf ("Can't open '%s'.\n", argv[1]);   exit (1); }
   if (in2 < 0) { printf ("Can't open '%s'.\n", argv[2]);   exit (1); }
   if (out < 0) { printf ("Can't open '%s'.\n", argv[3]);   exit (1); }

   while ( (got = _read (in1, buf, MAXBUF)) > 0)   _write (out, buf, got);
   for (ct=0;   ct < 64;   ++ct)  buf[ct] = 0xAA;
   _write (out, buf, 64);
   while ( (got = _read (in2, buf, MAXBUF)) > 0)   _write (out, buf, got);

   _close (in1);
   _close (in2);
   _close (out);
   exit (0);
}
