
/*** FIND_CAUCUS.  Attempt to find previous installation(s) of Caucus.
/
/    find_caucus (result, drives);
/
/    Arguments:
/       char *result;  (put results here)
/       char *drives;  (list of NTFS drive letters, lower-case)
/
/    Purpose:
/       Try find previous installation(s) of Caucus.  Returns
/       a string of pairs of words in RESULT.  The first word in
/       each pair is the userid; the second is the home directory.
/
/    How it works:
/
/    Returns: number of prev installations found
/
/    Known bugs:      none
/
/    Home:  xins/findcaucus.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/30/98 14:28 1st version. */

#include <stdio.h>
#include "ins.h"

#define  MAXTEMP 10000

FUNCTION  find_caucus (char *result, char *drives)
{
   char  temp[MAXTEMP], word[80], home[200], lhome[200], misc[300];
   char *strtoken();
   int   token, success;
   void  logger();
   
   /*** For each user... */
   result[0] = '\0';
   if (NOT nt_users (temp, MAXTEMP))  return (0);
   lhome[1] = 0;
   success  = 0;
   for (token=1;   strtoken (word, token, temp) != NULL;   ++token) {

      /*** Do they have a home directory? */
      if (NOT nt_uinfo (word, "home directory", home))  continue;
      if (home[0] == 0)                                 continue;

      /*** Is it on an NTFS drive? */
      lhome[0] = home[0];
      strlower (lhome);
      if (strindex (drives, lhome) < 0)                 continue;

      /*** Does it contain MISC/dicti000? */
      sprintf (misc, "%s/MISC/dicti000", home);
      if (NOT sysisfile (misc))                         continue;

      strcat (result, word);    strcat (result, " ");
      strcat (result, home);    strcat (result, " ");
      ++success;
   }

   return (success);
}
