
/*** GET_NTFS.   Get NTFS formatted drives and available space.
/
/    get_ntfs (dstr, free);
/
/    Arguments:
/       char *dstr;    (returned string of drive letters of NTFS drives)
/       int  *free;    (returned array of free space in KB)
/
/    Purpose:
/       Find the list of NTFS drives, and get their free space.
/
/    How it works:
/       Starts by clearing DSTR.
/
/       Scans all drives found, determines if it is an NTFS drive,
/       and if so, adds the 1st character of the drive name (lower case)
/       to the end of DSTR.  Puts the amount of free space (in KB) into
/       the equivalent array element in FREE.
/
/    Exit status:
/
/    Known bugs:      none
/
/    Home:  xins/getntfs.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/16/98 14:28 1st version. */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <windows.h>
#include <iostream.h>
#include "ins.h"

FUNCTION get_ntfs (char *dstr, int *free)
{
   int    drives;
   char   drive[4];
   char   volname[MAX_PATH], fsysname[MAX_PATH];
   int    volsernum, maxnamlen, fsysflags, success, letter, bit, dtype;
   int    count;
   long   kbytes, files;
   unsigned int sectors_cluster, bytes_sector, freeclus, clusters;

   dstr[0] = '\0';
   count   =   0;

   drive[1] = ':';
   drive[2] = '\\';
   drive[3] = '\0';
   drives = GetLogicalDrives();
   drives = drives >> 2;   /* skip A and B. */
    
   for (letter='c', bit=2;   bit<26;   ++bit, ++letter)  if (drives & 1) {

      drive[0] = letter;
      dtype = GetDriveType(drive);
      if (dtype != DRIVE_REMOVABLE  &&  dtype != DRIVE_FIXED)  continue;

      success = GetVolumeInformation (drive, volname, MAX_PATH, &volsernum,
         &maxnamlen, &fsysflags, fsysname, MAX_PATH);

      if (NOT success)                      continue;
      if (strcmp (fsysname, "NTFS") != 0)   continue;

      success = GetDiskFreeSpace (drive, &sectors_cluster, &bytes_sector,
                                  &freeclus, &clusters);
      if (NOT success)                      continue;

      kbytes = (sectors_cluster * (bytes_sector/100)) * (freeclus/10);
      free[count  ] = kbytes;
      dstr[count++] = letter;
      dstr[count  ] = '\0';
   }

   return (1);
}
