
/*** GOODKEY.  Is this a "good" activation key?
/
/    ok = goodkey (key, pass, ship, code, expire, users);
/   
/    Parameters:
/       int     ok;      (0 => good; anything else is error)
/       char   *key;     (original 21 digit activation key)
/       char   *pass;    (password part   ppppppp)
/       char   *ship;    (ship part       ssss     e.g. 1998)
/       char   *code;    (code part       ccc      e.g. day + 365)
/       char   *expire;  (expire part     ee       00=>not, nn = # of days)
/       char   *users;   (users part      uuuuu    max # total users)
/  
/    Purpose:
/       Parse activation key into its component parts, and determine
/       if this is a "good" key.
/
/    How it works:
/
/    Returns: 0 on success
/            -1 if activation key is "stale"
/            -2 if password is bad
/
/    Error Conditions:
/  
/    Home:  xins/goodkey.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

#include "ins.h"
#include <time.h>
#define  STALE_DAYS   7

FUNCTION  goodkey (char *key,    char *pass, char *ship, char *code, 
                   char *expire, char *users)
{
   struct tm *tmnow;
   int        codeyear, codeday, totalusers, expires_in, now, year;
   char       result[100], temp[100];

   /*** Get all details about current time. */
   now   = time(NULL);
   tmnow = localtime (&now);
   year  = tmnow->tm_year;
   if (year < 1900)  year += 1900;

   /*** Parse the activation key: pppppppssssccceeuuuuu. */
   strcpy  (pass,   key);      pass  [7] = '\0';
   strcpy  (ship,   key+7);    ship  [4] = '\0';
   strcpy  (code,   key+11);   code  [3] = '\0';
   strcpy  (expire, key+14);   expire[2] = '\0';
   strcpy  (users,  key+16);

   /*** Activation keys for (expiring) trial downloads are 
   /    only good for STALE_DAYS. */
   if (NOT streq (expire, "00")) {
      sscanf (ship, "%d", &codeyear);
      sscanf (code, "%d", &codeday);
      if (codeyear == year - 1) {
         codeyear = year;
         codeday  = codeday - 365;
      }

      if (codeyear != year  ||  tmnow->tm_yday - codeday > STALE_DAYS)  
         return (-1);
   }

   /*** Permute codes to see if this is a valid password. */
   strcpy      (result, "abcdefg");
   lstrpermute (result, ship);
   lstrpermute (result, code);

   sscanf (expire, "%d", &expires_in);
   lnumpermute (result, expires_in / 30);
   lnumpermute (result, expires_in % 30);

   lnumpermute (result, 5);
   sscanf (users, "%d", &totalusers);
   lnumpermute (result, totalusers);

   return (streq (result, pass) ? 0 : -2);
}
