
/*** INS1.   Program to perform phase 1 of a Caucus install on Windows/NT.
/
/    Invoked as:
/       ins1.exe   (typically execute directly from browser after downloading)
/
/    Purpose:
/       Unpackage a Caucus kit, start a web server, and launch a browser
/       that talks to a CGI program under that server to finish the
/       Caucus installation.
/
/    How it works:
/
/    Exit status:
/
/    Known bugs:      none
/
/    Home:  xins/ins1.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/04/98 17:45 1st version. */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <windows.h>
#include <iostream.h>
#include "ins.h"

#define TSIZE   1000
#define MAXBUF  4096
#define NEEDS  20000L  /* Require at least 20 meg free space to install. */

FUNCTION main (int argc, char *argv[])
{
   DWORD    tempsize;
   char     tempdir[TSIZE], path1[TSIZE], path0[TSIZE], startcgi[200];
   char     errtext[TSIZE];
   int      result, last, ok;
   int      in1, out, got, ct, found;
   int      drives, bit;
   char     letter;
   long     pos, find, went;
   unsigned char buf[MAXBUF];
   int      child, rc;
   FILE    *fp;

   /*** Get rid of the console window that may have been opened up. */
   FreeConsole();

/*-Step 1: make temp directory---------------------------------------*/

   /*** Try system default temp directory. */
   tempsize = GetTempPath (TSIZE, tempdir);
   if (tempsize > 0  &&  tempsize < TSIZE) {
      while (stralter (tempdir, "\\", "/")) ;
      last = strlen (tempdir) - 1;
      if (tempdir[last] == '/')  tempdir[last] = '\0';

      result = test_temp (tempdir, NEEDS);
   }
   else result = 0;

   /*** If that fails, try X:\TEMP, for each drive X that exists. */
   if (NOT result) {
      drives = GetLogicalDrives();
      drives = drives >> 2;   /* skip A and B. */
      for (letter='c', bit=2;   bit<26;   ++bit, ++letter)  if (drives & 1) {
         sprintf (tempdir, "%c:/temp", letter);
         result = test_temp (tempdir, NEEDS);
         if (result)  break;
      }
   }
   if (NOT result) {
      sprintf (errtext, "cannot find or make a temporary directory on a\n"
               "   drive with at least %d Meg of free space.\n", NEEDS/1000);
      failure (101, errtext);
   }
   strcat   (tempdir, "/caucus");
   ok = sysmkdir (tempdir, 0700);
   if (NOT ok) {
      sprintf (errtext, " cannot create temporary directory\n '%s'\n", tempdir);
      failure (105, errtext);
   }
  

/*-Step 2: extract the appended file----------------------------------*/
   strcpy (path0, argv[0]);
   if (strindex (path0, ".exe") < 0)  strcat (path0, ".exe");
   strcpy (path1, tempdir);
   strcat (path1, "/xitami.tar");

   in1 = _open (path0,  O_RDONLY | O_BINARY);
   out = _open (path1,  O_WRONLY | O_BINARY | O_CREAT, 
                       _S_IREAD  | _S_IWRITE);
   if (in1 < 0) {
      sprintf (errtext, " cannot open self '%s'.\n", path0);
      failure (108, errtext);
   }
   if (out < 0) {
      sprintf (errtext, " cannot create temp file '%s'.\n", path1);
      failure (108);
   }
   
   for (pos=0;   (1);   pos += got-60) {
      _lseek (in1, pos, 0);
      got = _read (in1, buf, MAXBUF);
      if (got < 64) failure (115, " cannot find 64 0xAA boundary marker.\n"
                                  "  File may be corrupted.\n");

      for (find=0;   find < got - 32;   ++find) {
         for (found=1, ct=0;   ct < 32;   ++ct)
            if (buf[find+ct] != 0xAA)  { found = 0;   break; }
         if (found)  break;
      }

      if (found)  break;
   }
 
   _lseek (in1, pos + find + 64, 0);
   while  ( (got = _read (in1, buf, MAXBUF)) > 0)  _write (out, buf, MAXBUF);
   _close (in1);
   _close (out);

/*-Step 3: unpax the appended file----------------------------------*/
   SetCurrentDirectory (tempdir);
   if (syslaunch ("pax -r -f xitami.tar", 1, 0) <= 0) 
      failure (121, " cannot un pax xitami.tar,\n");


/*-Step 4: start the web server-------------------------------------*/
   sprintf (path1, "%s/xidos32.exe -b 7942", tempdir);
   child = syslaunch (path1, 0, 0);
   if (child <= 0) {
      sprintf (errtext, " cannot launch web server '%s'.\n", path1);
      failure (125, errtext);
   }
   Sleep (4000);

   /*** And launch a browser, pointing it at the first cgi page. */
   sprintf (startcgi, "http://localhost:8022/cgi-bin/ins2.exe?100+%07d+%s",
                       child, tempdir);
   rc = (int) ShellExecute (NULL, NULL, startcgi, "", "", SW_SHOWNORMAL);
   if (rc <= 32) {
      sprintf (errtext, " ShellExecute code %d.  Cannot open \n"
                        "   '%s'\n", rc, startcgi);
      failure (131, errtext);
   }

   exit (0);
}
