
/*** LNUMPERMUTE.  Permute licensing password with a number.
/
/    lnumpermute (result, number);
/
/    Parameters:
/       char   *result;      (result of permutation)
/       int     number;      (permute with this number)
/
/    Purpose:
/       Lnumpermute is used to compute the "licensing password" for
/       the LICE2 and LICEPASS programs.
/
/    How it works:
/       Lnumpermute permutes the string in RESULT, according to the
/       value of NUMBER.
/
/       RESULT will always contain lower-case letters in [a-z].
/
/    Returns: 1
/
/    Error Conditions:
/  
/    Side effects:
/
/    Related functions:
/
/    Called by:  licensing programs.
/
/    Operating system dependencies:
/
/    Known bugs:      none
/
/    Home:  xlice2/lnumperm.c
/ */

/*** Copyright (C) 1993 Camber-Roth.  All rights reserved. */

/*: CR  1/24/93 12:03 1st carvmark */
/*: CK  8/07/93 19:02 Comment out debugging output! */
/*: CK 10/13/93 19:31 Use DEBUG macro to control debugging output. */
/*: CR  9/08/94  0:57 Remove ' from universe of possibilities. */
/*: CR  3/05/98 14:14 Range of RESULT is limited to [a-z]. */

#include <stdio.h>
#include "ins.h"
#define  DEBUG     0

FUNCTION  lnumpermute (result, number)
   char  *result;
   int    number;
{
   int    len, pos, val;

   if (number == 0)  return (1);

   len = strlen (result);
   pos = number % len;
   val = result[pos] - 'a';
   val = val + number;
   val = val % 26;
   result[pos] = val + 'a';

#if DEBUG
   fprintf (stderr, "Permuting by  %20d , new result is '%s'\r\n", 
                     number, result);
#endif
   return (1);
}
