
/*** NT_UINFO.   Get a particular field of info about an NT userid ("account").
/
/    nt_uinfo (userid, info, result);
/ 
/    Parameters:
/       char  *userid;    (userid we're asking about)
/       char  *info;      (desired field)
/       char  *result;    (put resulting info here)
/
/    Purpose:
/       Get a particular kind (or field) of info about a particular
/       NT userid ("account").
/
/    How it works:
/       Runs the command "net user" on USERID, and scans for a
/       line of output that begins with INFO.  Puts the rest of
/       that line into RESULT.  The match with INFO is case-insensitive.
/
/       INFO must match one of the fields below:
/          User name
/          Full Name
/          Comment 
/          User's comment
/          Country code
/          Account active
/          Account expires
/          Password last set
/          Password expires
/          Password changeable
/          Password required
/          User may change password
/          Workstations allowed
/          Logon script
/          User profile
/          Home directory
/          Last logon
/          Logon hours allowed
/          Local Group Memberships
/          Global Group memberships
/
/    Returns: 1 on success
/             0 on failure.
/
/    Error Conditions:
/ 
/    Known bugs:
/
/    Home:  xins/ntuinfo.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/30/98 10:28 New function. */

#include "ins.h"
#include <stdio.h>
#define  MAXTEMP  10000

FUNCTION  nt_uinfo (char *userid, char *info, char *result)
{
   char   temp[MAXTEMP], comline[200], line[200], word[200], match[100];
   char  *cp;
   int    pos;

   /*** Build the "net user USERID" command line.  Since userid may be 
   /    supplied from result of nt_users(), which encodes embedded blanks 
   /    as ^A's, decode it. */
   sprintf (comline, "net user \"%s\"", userid);
   for (cp=comline;   *cp;   ++cp)  if (*cp == 1)  *cp = ' ';

   if (NOT shell (temp, MAXTEMP, comline))  return (0);

   strcpy   (match, info);
   strlower (match);
   for (pos=0;   strnextline (line, temp, &pos);   ) {
      strcpy   (word, line);
      word[28] = '\0';
      strlower (word);
      if (strindex (word, match) == 0) {
         strcpy (result, line+29);
         return (1);
      }
   }

   return (0);
}
