
/*** NT_USERS.   Get list of NT userids ("accounts").
/
/    nt_users (users, umax);
/ 
/    Parameters:
/       char  *users;     (put list of userids here)
/       int    umax;      (max size of USERS)
/
/    Purpose:
/       Get a list of the NT userids ("accounts") on this host.
/
/    How it works:
/       Returns a blank-separated string of userids in USERS,
/       no longer than UMAX characters.
/
/       If a userid normally contains a blank, it will
/       appear in USERS with a 0x01 instead of the blank.
/
/    Returns: 1 on success
/             0 on failure.
/
/    Error Conditions:
/ 
/    Known bugs:
/
/    Home:  xins/ntusers.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/25/98 10:28 New function. */

#include "ins.h"
#include <stdio.h>

#define  MAXTEMP  20000

FUNCTION  nt_users (char *users, int umax)
{
   char   result[MAXTEMP];
   char   line[400], word[100];
   char  *lp;
   int    pos, state, token;
   char  *strtoken();

   users[0] = '\0';
   if (NOT shell (result, MAXTEMP, "net users"))  return (0);

   state = 0;
   for (pos=0;   strnextline (line, result, &pos);   ) {
      if      (strindex (line, "The command completed") == 0)  break;

      else if (strindex (line, "--------------")        == 0)  state = 1;

      else if (state == 1) {
         single_blank (line);
         for (token=1;   strtoken (word, token, line) != NULL;   ++token) {
            if (strlen (users) + strlen(word) + 3 < umax) {
               strcat (users, word);
               strcat (users, " ");
            }
         }
      }
   }

   return (1);
}

  
FUNCTION  single_blank (char *line) 
{
   int    state;
   char  *lp;

   if (NOT line[0])  return(0);

   /*** Turn single blanks into ^A's. */
   for (lp=line+1;   *lp;   ++lp) 
      if (* lp   ==' '  &&  *(lp-1)!=' '  &&  *(lp-1)!=1 
                        &&  *(lp+1)!=' '  &&  *(lp+1)!=0)  *lp = 1;
   return(1);
}
