
/*** PARSE_FORM.   Parse stdin as variables from HTML form input.
/
/    parse_form (vars);
/ 
/    Parameters:
/       struct vars_t *vars;  (variable table to be filled-in)
/
/    Purpose:
/
/    How it works:
/       Runs a state-table parser over stdin, plucking out the name=value&
/       pairs.
/
/    Returns:
/
/    Home:  xins/parseform.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/10/98 20:26 New function, derived from sweb formsing.c */

#include "ins.h"

#define ST_NAME  0
#define ST_VALUE 1

FUNCTION parse_form (struct vars_t *vars)
{
   int   state, slot, cval;
   char *strof();

   vars->used = 0;

   /*** Using a state table, parse stdin into "name=value"
   /    pairs, and store the data in the variable list VARS. */
   state = ST_NAME;
   slot  = 0;
   vars->name[0][0] = vars->value[0][0] = '\0';
   while ( (cval = getchar()) >= 0) {
      if (vars->used == 0)  vars->used = 1;
      if (cval=='\n'  ||  cval=='\r')  continue;

      /*** While accumulating characters in the NAME... */
      if      (state == ST_NAME) {
         if      (cval == '&')  ;  /* chxclear (name); */
         else if (cval == '=')  state = ST_VALUE;
         else                   strcat (vars->name[slot], strof(cval));
      }

      /*** While accumulating characters in the VALUE... */
      else if (state == ST_VALUE) {
         if (cval == '&')  {
            state = ST_NAME;
            slot  = vars->used++;
            vars->name[slot][0] = vars->value[slot][0] = '\0';
            continue;
         }

         /*** Translate +'s and %xx sequences. */
         if (cval == '+')  cval  = ' ';
         if (cval == '%') {
            cval =             unhex (getchar());
            cval = 16 * cval + unhex (getchar());
         }

         if (cval != 0x0D)  strcat (vars->value[slot], strof(cval));
      }
   }

   return (1);
}

FUNCTION  unhex (int x)
{
   if (x >= '0'  &&  x <= '9')  return ( (int) (x - '0')      );
   if (x >= 'a'  &&  x <= 'f')  return ( (int) (x - 'a' + 10) );
   if (x >= 'A'  &&  x <= 'F')  return ( (int) (x - 'A' + 10) );

   return (0);
}

FUNCTION  char *strof (int cval)
{
   static char twobyte[2] = {0, 0};

   twobyte[0] = cval;
   return (twobyte);
}
