
/*** STRNEXTLINE.  Get the "next" line of a string.
/
/    ok = strnextline (a, b, pos);
/
/    Parameters:
/       int   ok;     (another line found?)
/       char *a;      (resulting string)
/       char *b;      (source string)
/       int  *pos;    (starting position in A)
/
/    Purpose:
/       Get the "next" line of a string B, and put it in A.
/
/    How it works:
/       Starting at position POS in B, copy all of the characters up through
/       (but not including) the next newline, into A.  Update POS to the
/       position of the first character in B after said newline.
/
/       If there are no more newlines, copy through the end of B.  Update POS
/       to the position of the 0 character at the end of B.
/
/       A is returned without any trailing \n's or \r's.
/
/    Returns: 2 if another line was found (and it ended in a newline)
/             1 if another line was found.
/             0 if POS was positioned at the end of B.
/             0 on any error.
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  xins/getnextl.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/25/98 14:35 New function, based on stripped-down chxnextline. */

#include <stdio.h>
#include "ins.h"

FUNCTION  strnextline (uchar *a, uchar *b, int *pos)
{
   uchar *ap;
   int    cval, code, len;

   /*** Input validation. */
   if (a == NULL  ||  b == NULL)     return (0);
   if (NOT b[*pos])                  return (0);

   /*** Find the end of the line. */
   for (ap=a;   (1);   ++*pos) {
      cval = *ap++ = b[*pos];
      if (cval ==   0 )  { code = 1;             break; }
      if (cval == '\n')  { code = 2;   ++*pos;   break; }
   }
   *ap = '\0';

   /*** Trim trailing \n or \r's. */
   for (len = strlen(a);   len > 0;   --len) {
      if (a[len-1] == '\n')  continue;
      if (a[len-1] == '\r')  continue;
      break;
   }
   a[len] = '\0';

   return (code);
}
