
/*** SYSLAUNCH.  Launch a program, and return process handle.
/
/    pid = syslaunch (cmdline, wait4, console)
/ 
/    Parameters:
/       int     pid;       (child process id #)
/       char   *cmdline;   (cmdline to execute)
/       int     wait4;     (wait for process to complete?)
/       int     console;   (give child its own console?)
/
/    Purpose:
/
/    How it works:
/
/    Returns: process id number on success
/             -1 if file handles cannot be created
/             -2 if program in cmdline cannot be launched
/
/    Error Conditions:
/ 
/    Known bugs:
/
/    Home:  syslaunch.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR 11/09/98 17:48 New function. */

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <process.h>
#include <windows.h>
#include <iostream.h>
#include "ins.h"

#define  MAXTEMP   1024

FUNCTION  syslaunch (char *cmdline, int wait4, int console)
{
   int    tube[2], got, child_pid, init;
   char   temp[MAXTEMP+2];

   STARTUPINFO          startup;
   PROCESS_INFORMATION  procinfo;
   BOOL                 success;
   HANDLE               rpipe, wpipe, in_bucket, out_bucket;
   SECURITY_ATTRIBUTES  sa;
   char                 program[256];
   char                *prog;

   GetStartupInfo (&startup);
   startup.dwFlags = startup.dwFlags | STARTF_USESTDHANDLES;

   /*** Define a security attribute that allows inheritance. */
   sa.nLength = sizeof (SECURITY_ATTRIBUTES);
   sa.lpSecurityDescriptor = 0;
/* sa.bInheritHandle       = TRUE; */
   sa.bInheritHandle       = FALSE;

   out_bucket = CreateFile ("NUL", GENERIC_WRITE, 0, &sa, OPEN_ALWAYS, 0, 0);
   in_bucket  = CreateFile ("NUL", GENERIC_READ,  0, &sa, OPEN_ALWAYS, 0, 0);
   if (in_bucket == INVALID_HANDLE_VALUE)    return (-1);

   startup.hStdError  = out_bucket;
   startup.hStdInput  = in_bucket;
   startup.hStdOutput = out_bucket;

   /*** Get the program name and command line. */
   strtoken    (program, 1, cmdline);
   prog = 
   prog = (program[1]==':' ? program : NULL);

   /*** Run it as a detached process, inheriting the changed stdout. */
   success = CreateProcess (prog, cmdline,
      0, 0, FALSE,
      (console ? CREATE_NEW_CONSOLE : DETACHED_PROCESS),
      0, /* environment */
      0, /* current directory */
      &startup, 
      &procinfo);

   CloseHandle (in_bucket);
   CloseHandle (out_bucket);

   if (NOT success)  return (-2);
   if (wait4) WaitForSingleObject (procinfo.hProcess, INFINITE);

   return (procinfo.dwProcessId);
}
