
/*** SYSMKDIR.  Make a directory DIR. 
/
/    ok = sysmkdir (name, perms);
/
/    Parameters:
/       int    ok;      (success?)
/       char  *name;    (full path of directory to make)
/       int    perms;   (protection mode, unix std)
/
/    Purpose:
/
/    How it works:
/       The argument PERMS contains the usual Unix 9-bit mask.
/
/       The WIN32 code interprets only the owner bits and the
/       world bits; group bits are ignored.  WIN32 also 
/       gives the owner bits to the "administrators" bits.
/
/    Returns: 1 on success, 0 otherwise
/
/    Known bugs:      none
/
/    Home:  sys1/sysmkdir.c
/ */

/*** Copyright (C) 1984 Camber-Roth.  All rights reserved. */
/*** Copyright (C) 1996 Screen Porch LLC.  All rights reserved. */

/*: AA  7/29/88 17:45 Source code master starting point */
/*: NT  4/08/97 16:00 Revamp for Caucus 3.2. */
/*: CR  6/19/97 21:15 Change meaning of 2nd arg. */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <windows.h>
#include <iostream.h>
#include "ins.h"

FUNCTION  sysmkdir (char *name, int perms)
{
   SECURITY_ATTRIBUTES sa;
   SECURITY_DESCRIPTOR sd;
   BYTE          aclbuf[1024];
   PACL          pacl = (PACL) &aclbuf;
   BYTE          sidbuf[100];
   DWORD         sidbufsize = 100;
   PSID          psid = (PSID) &sidbuf;
   char          dombuf[80];
   DWORD         dombufsize = 80;
   SID_NAME_USE  snu;
   char          username[100];
   DWORD         usernamesize = 100;
   DWORD         mask;
   int           success, res;

   GetUserName (username, &usernamesize);

   InitializeSecurityDescriptor (&sd, SECURITY_DESCRIPTOR_REVISION);
   InitializeAcl (pacl, 1024, ACL_REVISION);

   /*** Owner. */
   LookupAccountName (0, username, psid, &sidbufsize, dombuf, 
      &dombufsize, &snu);
   mask = 0;
   if  ( (perms & 0700) == 0700)  mask  = FILE_ALL_ACCESS;
   else {
      if (perms & 0400)           mask |= FILE_GENERIC_READ;
      if (perms & 0200)           mask |= FILE_GENERIC_WRITE;
      if (perms & 0100)           mask |= FILE_GENERIC_EXECUTE;
   }
   if (mask != 0)  AddAccessAllowedAce (pacl, ACL_REVISION, mask, psid);

   /*** Administrators */
   if (strcmp (username, "administrator") != 0) {
      LookupAccountName (0, "administrators", psid, &sidbufsize, dombuf, 
         &dombufsize, &snu);
      if (mask != 0)  AddAccessAllowedAce (pacl, ACL_REVISION, mask, psid);
   }

   LookupAccountName (0, "everyone", psid, &sidbufsize, dombuf, 
      &dombufsize, &snu);
   mask = 0;
   if  ( (perms & 0007) == 0007)  mask  = FILE_ALL_ACCESS;
   else {
      if (perms & 0004)           mask |= FILE_GENERIC_READ;
      if (perms & 0002)           mask |= FILE_GENERIC_WRITE;
      if (perms & 0001)           mask |= FILE_GENERIC_EXECUTE;
   }
/*
   if (perms & 0007)  mask  = FILE_ALL_ACCESS;
   if (perms & 0004)  mask |= FILE_GENERIC_READ;
   if (perms & 0002)  mask |= FILE_GENERIC_WRITE;
   if (perms & 0001)  mask |= FILE_GENERIC_EXECUTE;
*/
   if (mask != 0)  AddAccessAllowedAce (pacl, ACL_REVISION, mask, psid);

   SetSecurityDescriptorDacl (&sd, TRUE, pacl, FALSE);

   sa.nLength = sizeof(SECURITY_ATTRIBUTES);
   sa.bInheritHandle = FALSE;
   sa.lpSecurityDescriptor = &sd;

   success = CreateDirectory (name, &sa); 

   /*** Succeed if the directory exists!  (Even if create failed because
   /    directory already exists.) */
   res = GetFileAttributes (name);
   if (res < 0  ||  NOT (res & FILE_ATTRIBUTE_DIRECTORY))  return (0);
   return (1);
/* return (success ? 1 : 0); */
}
