
/*** TEST_TEMP.   Test temporary directory for free space.
/
/    ok = test_temp (path, needed);
/
/    Parameters:
/       int    ok;      (1=>success, 0=>failed)
/       char  *path;    (full pathname to temporary directory)
/       long   needed;  (kbytes needed)
/
/    Purpose:
/       Determine if PATH is a "temporary file directory" with
/       NEEDED kbytes of empty space.
/
/    How it works:
/       Checks if PATH exists.  If not, creates it.
/
/    Returns: 1 on success, 0 otherwise
/
/    Home:  xins/testtemp.
/ */

/*: CR 11/04/98 17:45 1st version. */

#include <stdio.h>
#include <windows.h>
#include <iostream.h>
#include "ins.h"

FUNCTION test_temp (char *path, long needed)
{
   BOOL   success;
   unsigned int sectors_cluster, bytes_sector, freeclus, clusters;
   long     kbytes, files;
   char     drive[20];
   int      res, dtype;

   /*** Get the drive root. */
   strncpy (drive, path, 4);
   drive[2] = '\\';
   drive[3] = '\0';

   /*** Make sure it exists and is a real disk (not CDROM or network drive). */
   dtype = GetDriveType(drive);
   if (dtype != DRIVE_REMOVABLE  &&  dtype != DRIVE_FIXED)  return (0);

   res = GetFileAttributes (path);
   if (res < 0  ||  NOT (res & FILE_ATTRIBUTE_DIRECTORY)) {
      if (NOT sysmkdir (path, 0700))  return (0);
   }

   success = GetDiskFreeSpace (drive, &sectors_cluster, &bytes_sector,
                &freeclus, &clusters);
   if (NOT success)  return (0);

   kbytes = (sectors_cluster * (bytes_sector/100)) * (freeclus/10);
   return (kbytes > needed);
}
