
/*** REPAS41.   Create password for REREG41.
/
/    Invoked as:
/       repas41  key
/
/    Arguments:
/       caucus   name of caucus executable file (caucus_x, swebd, swebs, etc.).
/       key      activation key (RRRRRosEEEpppppCCCuuu)
/
/    Purpose:
/       REPAS41 takes a Caucus 4.1 activation key (with a bogus ppppp
/       password field) and generates the proper activation key.
/
/    How it works:
/       Based on rereg41.c
/
/    Exit status:
/
/    Known bugs:
/      Must add system-type codes for platforms that use reverse-endian
/      math.  (See "reverse-endian" comment.)
/
/    Home:  xlice/repas41.c
/ */

/*** Copyright (C) 1999 Screen Porch LLC.  All rights reserved.  */

/*: CR  3/29/99 12:27 New program, based on rereg41.c. */

#include <stdio.h>
#include "lice41.h"
#include "license.h"
#include "systype.h"

#define MAXSYSID 16

#if WIN32
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#endif

FUNCTION  main (argc, argv)
   int   argc;
   char *argv[];
{
   char  result[40];
   int4  mask, expires_in, users, confmax, itemmax, respmax, totalusers;
   int4  searchfile(), systime();
   int   fd, i, j, reverse, long8, weeks, bi;
   char  str[40];
   char  customer[20], os[20], password[20];
   char *strplain(), *strcpy();

#if UNIX | WIN32
   if (argc < 2  ||  strlen(argv[1]) != 21) {
      printf ("0\n");
      exit   (1);
   }
#endif

   /*** Parse activation key, and set standard values of other license
   /    parameters. */
   sscanf (argv[1], "%5s%2s%3d%5s%3d%3d",
                    customer, os, &weeks, password, &confmax, &totalusers);
   expires_in = weeks;
   mask       = 5;
   users      = 0;
   totalusers = totalusers * 100;
   itemmax    = 0;
   respmax    = 0;

   /*** Create password, start with "abcdefg".  Permute by customer # and OS. */
   strcpy      (result, "abcde");
   lstrpermute (result, customer);
   lstrpermute (result, os);
   lnumpermute (result, customer[4]);

   /*** Permute by other license parameters. */
   lnumpermute (result, expires_in / 4);
   lnumpermute (result, expires_in % 4);
   lnumpermute (result, mask);
   lnumpermute (result, confmax);
   lnumpermute (result, totalusers);
   lnumpermute (result, totalusers + 2);

   printf ("%5s%2s%03d%5s%03d%03d\n",
            customer, os, weeks, result, confmax, totalusers/100);
}
