/*** CAUMNT.  Caucus Maintenance program.   CV2.
/
/    CAUMNT combines several of the old Caucus maintenance programs.
/    It is invoked as:
/
/       caumnt_x  caucusid  choice  [options]
/
/    where
/       caucusid      is the caucus id (e.g. "/usr/caucus")
/       choice        is one of:
/           -m [file]    Makes a new conference [write confnum in "file"]
/           -r           Removes an old conference
/           -s           Shell maintenance: password, newuser, mark
/           -k           Kill a user.
/           -p  file     Pre-register users from 'file'.
/           -c           Reset the system clock-check file.
/
/       options       are the optional command line arguments
/                     as used in the Caucus main program.
/
/  */

/*** Copyright (C) 1988 Camber-Roth.  All rights reserved. */

/*: AA  7/29/88 17:58 Source code master starting point */
/*: CR 11/10/88 13:36 Move sysexit() to top. */
/*: CR  3/01/89 15:49 Change call from cstart() to costart(). */
/*: CR  3/21/89 15:46 Make caumnt pick up server name, if there. */
/*: AM  2/07/89 10:43 Jcaucus changes*/
/*: CR  4/05/89 17:15 Integrate Jcaucus changes. */
/*: JV  8/15/89 11:39 Defined servername and confid as global vars. */
/*: CR 10/13/89 13:29 Extern servername and confid. */
/*: JV  3/15/90 10:00 Added call to optchk() for 8-bit stuff. */
/*: CR  7/26/90 23:17 Added sysdirchk() so XITX can exist! */
/*: CR  8/21/90 16:45 Flag.login must always be off, so we use real userid. */
/*: CR 10/07/90  9:45 Add kanji_term(); move sysglobals() call. */
/*: CJ 11/22/90 22:41 Add new systurn() call. */
/*: CR  5/02/91 18:37 Remove initsort(). */
/*: CR  7/02/91 17:52 globals() is now mainglobals(). */
/*: CR  7/24/91 22:19 Make lint happier. */
/*: CR  7/30/91  0:18 Comment out ENTRY, not needed in main. */
/*: JV 11/05/91 17:42 Add argument to costart. */
/*: CR 12/11/91 23:55 Cast NULL in costart() call. */
/*: JV  8/28/92 17:01 Add utilglob, mdglobals. */
/*: CR  8/10/92 17:09 "-m file" writes new conf number to "file". */
/*: DE  6/08/92 13:10 Chixified */
/*: JX  6/08/92 19:13 Add is_registered(), alloc confid, rm servername. */
/*: JX  7/22/92 17:00 Add chx_init(). */
/*: JX  8/06/92 16:05 Add -p for pre-registration. */
/*: CI 10/06/92 17:33 Chix Integration. */
/*: CR 10/13/92 12:36 Fix bad chxformat; remove unused 'outpos'. */
/*: CR 10/22/92 11:38 -m | -p take next arg as file only if not "-". */
/*: JV 12/04/92 13:46 Add sitevar arg to storereg(). */
/*: CK  3/11/93 14:05 Change header comments to reflect comm line parsing. */
/*: CR 11/01/95 16:26 Use new is_registered() return code. */

#include <stdio.h>
#include "caucus.h"
#include "systype.h"
#include "text.h"

extern struct screen_template  used, screen;
extern struct termstate_t      term_in_caucus, term_original;
extern struct flag_template    flag;
extern Chix                    confname, confid, sitevar[];
extern Userreg                 thisuser;
extern union null_t null;

FUNCTION  main (argc, argv)
   int    argc;
   char  *argv[];
{
   Chix   choice, fname, outnum, userid;
   int    optpos, cnum, ok;
   char   logdir[256];

   sys_nt();

/* ENTRY ("caumnt", ""); */

   fname    = chxalloc (L(40), THIN, "caumnt fname");
   choice   = chxalloc (L(40), THIN, "caumnt choice");
   confid   = chxalloc (L(40), THIN, "caumnt confid");
   outnum   = chxalloc (L(16), THIN, "caumnt outnum");
   confname = chxalloc (L(40), THIN, "caumnt confname");
   userid   = chxalloc (L(40), THIN, "caumnt userid");

   screen.lines = 23;    used.lines = 0;
   screen.width = 80;    used.width = 0;

   /*** Get the confid. */
   if (argc <  3)   exit (1);
   chxcpy (confid, CQ(argv[1]));
   optpos = 2;

   strcpy (logdir, argv[1]);
   strcat (logdir, "/SOCKET");
   logger_init (logdir, argv[0], "cau");

   /*** Get CHOICE. */
   chxcpy (choice, CQ(argv[optpos]));

   /*** Get input or output file for -m or -p choices. */
   if (chxeq (choice, CQ("-p")) ||  chxeq (choice, CQ("-m"))) {
      if (argc > optpos+1  &&  *argv[optpos+1] != '-')
         chxofascii (fname, argv[++optpos]);
   }
   ++optpos;

   /*** Process any remaining options flags. */
   systurn (&term_in_caucus, GET_TERM, 0);
   systurn (&term_original,  GET_TERM, 0);
   argchk  (argc, argv, optpos);

   utilglobals();
   optchk ();

   /*** Choose Internal Text Type. */
   if (flag.japan) chx_init (JIX);
   else            chx_init (ASCII);

   /*** Miscellaneous initialization. */
   sysexit (-1);
   unitglobals();
   mainglobals();
   sysbrkinit();
   sysglobals();
   flag.login = 0;  /* Must use real userid! */
   sysuserid (userid);
   sysdirchk (userid);

   /*** As of Chix, sysmdstr() insists that structure THISUSER exists. */
   if ( (ok = is_registered (thisuser)) <= 0) {
      mdwrite (XWTX, (ok==0 ? "mai_Tbadreg" : "mai_Ttoomany"), null.md);
      manager_text (flag.exitfile);
      if (NOT EMPTYCHX(flag.logfile)) log_user (XSLG, 0);
      sysexit (1);
   }
   storereg (thisuser, 0, sitevar);

   /*** Make the conference, and if "-m file" specified, write the
   /    conference number into fname. */
   if (chxeq (choice, CQ("-m"))) {
      cnum = costart (nullchix);
      if (cnum > 0  &&  NOT EMPTYCHX (fname)) {
         if (unit_lock (XIRE, WRITE, L(0), L(0), L(0), L(0), fname)) {
            if (unit_make (XIRE)) {
               chxformat (outnum, CQ("%03d\n"), L(cnum), L(0),
                                  nullchix, nullchix);
               unit_write (XIRE, outnum);
               unit_close (XIRE);
            }
            unit_unlk  (XIRE);
         }
      }
   }

   if (chxeq (choice, CQ("-r")))  cremove();
   if (chxeq (choice, CQ("-s")))  cpasswd();
   if (chxeq (choice, CQ("-k")))  ckill();
   if (chxeq (choice, CQ("-c")))  sysclkchk(1);
   if (chxeq (choice, CQ("-p")))  {screen.lines = 0;  reg_parse_file (fname); }

   sysexit (0);
   return  (0); /* Make Lint happy. */
}
