
/*** MKSWEB.  Make swebd.conf file.
/
/    mksweb (file, hostname, caucus, userid, admin_mail)
/   
/    Parameters:
/       char   *file;       (where to put the result)
/       char   *hostname;   (name of www host)
/       char   *caucus;     (full pathname of caucus directory)
/       char   *userid;     (caucus userid)
/       char   *admin_mail; (administrator e-mail address)
/
/    Purpose:
/       Create a reasonable default swebd.conf configuration file.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  mksweb.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  7/21/97 15:37 New function. */

#include <stdio.h>
#include "chixuse.h"
#include "ntcaucus.h"

FUNCTION  mksweb (char *file,   char *hostname, char *caucus, 
                  char *userid, char *admin_mail)
{
   FILE  *fp;

   fp = fopen (file, "wt");
   fprintf (fp, 
      "#\n"
      "#---SWEBD.CONF      Sweb configuration file.\n"
      "#\n"
      "# This is the Caucus Web server (\"Swebd\") configuration file.\n"
      "# It contains parameters which must be set for your host and server.\n"
      "#\n"
      "# Note!  If you make any changes to this file, you MUST restart your\n"
      "# swebd server for the changes to take effect.\n\n");

   fprintf (fp, 
      "#\n"
      "#==============================================================\n"
      "# Hostname is the DNS name of the host that the server is\n"
      "# running on.  Specify the WWW hostname that you use for\n"
      "# this host.  You should include the port number,\n"
      "# if different from the default, e.g.\n"
      "#\n"
      "#    Hostname   www.xyz.com:8001\n"
      "\n"
      "Hostname     %s\n"
      "\n"
      "#==============================================================\n",
      hostname);

   fprintf (fp, 
      "# Caucus_ID is the userid that owns the Caucus data files.\n"
      "# You MUST set this to the appropriate id for your system.\n"
      "# (You may use the userid name or number.)\n"
      "\n"
      "Caucus_ID    %s\n"
      "\n", userid);

   fprintf (fp, 
      "#==============================================================\n"
      "# Caucus_Path is the full pathname of the directory\n"
      "# where the Caucus programs and data files are installed.\n"
      "# You MUST set this to the proper pathname for your system.\n"
      "\n"
      "Caucus_Path  %s\n"
      "\n", caucus);

   fprintf (fp, 
      "#==============================================================\n"
      "# CML_Path is the full pathname of the directory which\n"
      "# contains the CML scripts.  You MUST change this to the\n"
      "# proper path for your system.\n"
      "\n"
      "CML_Path     %s\\CML\n"
      "\n", caucus);

   fprintf (fp, 
      "#==============================================================\n"
      "# Caucus_Lib is the full Unix pathname of the Caucus \"Library\" area.\n"
      "# Files uploaded by Caucus users are placed in directories\n"
      "# under this library.\n"
      "#\n"
      "# HTTP_Lib is the full URL for the Caucus_Lib directory.  This means\n"
      "# that the Caucus_Lib *must* be somewhere in the web tree of the\n"
      "# server that is running Caucus!\n"
      "\n"
      "Caucus_Lib   %s\\public_html\\LIB\n"
      "HTTP_Lib     http://%s/~%s/LIB\n"
      "\n", caucus, hostname, userid);

   fprintf (fp, 
      "#==============================================================\n"
      "# You can increase security (by reducing the chance of web 'spoofing\"\n"
      "# by turning Check_client_ip ON.  This makes swebd check the client\n"
      "# ip address on every CML page request.  If the ip address changes,\n"
      "# swebd assumes a spoof is being attempted and always outputs the\n"
      "# page syserr1.cml.\n"
      "#\n"
      "# However, clients coming through a firewall may be assigned (what\n"
      "# appears to swebd to be) a dynamic client ip that frequently changes.\n"
      "# This apparently includes users who browse the web through AOL.\n"
      "# If you have users in this situation, turn Check_client_ip OFF.\n"
      "\n"
      "Check_client_ip off\n"
      "\n");
   fprintf (fp, 
      "#==============================================================\n"
      "# When a new Swebd client starts up, it connects to the primary\n"
      "# swebd daemon, which spawns a child which is dedicated to the\n"
      "# client.  The child will exit after Timeout seconds if there\n"
      "# are no requests from the client.\n"
      "\n"
      "Timeout      3600\n"
      "\n");

   fprintf (fp, 
      "#==============================================================\n"
      "# Disk_Format defines the character set in use for the data in\n"
      "# the Caucus data files.  Choices are: ascii, iso8859, euc, and\n"
      "# sjis.  (The last two are the most common japanese language\n"
      "# character sets, and are only supported by the Japanese license\n"
      "# option.)\n"
      "\n"
      "Disk_Format  ascii\n"
      "\n"
      "#==============================================================\n"
      "# Browser_Format defines which character set (same choices as\n"
      "# Disk_Format) the browser is expecting to see.  Sweb automatically\n"
      "# handles conversion between different character sets, to the extent\n"
      "# possible.\n"
      "\n"
      "Browser_Format ascii\n"
      "\n");

   fprintf (fp, 
      "#==============================================================\n"
      "# The PW parameters control how Caucus interacts with the\n"
      "# web server userid & password database.  For a full discussion,\n"
      "# see the Caucus Installation Guide.\n"
      "#\n"
      "# Since Windows/NT web servers vary so wildly in their handling\n"
      "# of userids, the default case for Caucus is NO interaction with\n"
      "# the userid/password database:\n"
      "#    PW_Prog        external\n"
      "#    PW_Lock        off\n"
      "#    PW_Can_Add     off\n"
      "#    PW_Can_Change  off\n"
      "#    PW_Can_Delete  off\n"
      "#    PW_Can_Verify  off\n"
      "#\n"
      "# If your web server supplies a program for manipulating the\n"
      "# userids and passwords, change PW_Prog to be the full pathname\n"
      "# of an appropriate interface program, and set the appropriate\n"
      "# PW_Can masks.\n"
      "\n"
      "PW_Prog        external\n"
      "PW_Lock        off\n"
      "PW_Can_Add     off\n"
      "PW_Can_Change  off\n"
      "PW_Can_Delete  off\n"
      "PW_Can_Verify  off\n"
      "\n"
      "Admin_mail     %s\n", admin_mail);

   fprintf (fp, 
      "#==================End of swebd.conf============================\n");
   fclose (fp);

   return(1);
}
