
/*** NTINS2.   Caucus/NT installation program, 2nd major version
/
/    Invoked as:
/       ntins2  userid homedir servname hostname email adminid 
/               password tarfile path [key]
/
/    Arguments:
/       userid    userid (account) that owns Caucus files
/       homedir   full pathname of home directory of Caucus userid
/       servname  name of caucus service (usually "caucus"!)
/       hostname  WWW hostname of the server
/       email     e-mail address of Caucus administrator
/       adminid   caucus id of primary Caucus manager
/       password  password for 'userid'
/       tarfile   caucus tar kit pathname
/       path      directory path containing original files
/       key       optional activation key [ssssccceeuuuuu]
/
/    Purpose:
/       Do most of dirty work of installing Caucus on NT.
/
/    How it works:
/
/    Exit status: 
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  ntins.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  7/19/97 13:59 New program. */
/*: CR  8/01/97 15:34 new version, meant to be run by an administrator. */
/*: CR  1/19/98 11:40 2nd major version */
/*: CR  3/16/98 16:24 Revised to do w/o tar or pax. */

#include <stdio.h>
#include "chixuse.h"
#include "ntcaucus.h"
#include <windows.h>
#include <iostream.h>
#include <time.h>

static char caucus[256];

FUNCTION main (int argc, char *argv[])
{
   char  *n1(), *n2();
   char   temp[512], hostname[256], admin_mail[256], admin_userid[256];
   char   chomedir[512], netuser[512], actlog[512];
   char   password[20], ship[20], code[20], expire[20], ulimit[20];
   FILE  *fp;
   char  *s;
   char   userid[100];
   long   now, systime();
   int    year, month, day, hour, minute, sec, codeday, sub, codeyear;
   int4   bytes;
   int    j, error, activate, yday;
   LONG   ret;
   HKEY   keyh;
   DWORD  disposition;
   char   keyname[512];
   char   Application[256], AppParameters[256];
   struct tm *tmnow;
   time_t     secs;


   if (argc < 11) {
      printf ("Usage: ntins2 userid homedir servname hostname email "
              "adminid password tarfile path key\n");
      for (j=0;   j<argc;   ++j)  printf ("argv[%d]='%s'\n", j, argv[j]);
      gets (temp);
      exit   (1);
   }
   strcpy (userid, argv[1]);
   strcpy (caucus, argv[2]);

   /*** Create the new userid and its home directory. */
   sprintf (temp, "net user %s /add /expires:never /fullname:\"Caucus\" "
            "/homedir:\"%s\" %s >nul:", userid, caucus, argv[7]);
   system  (temp);
   _mkdir  (caucus);

   /*** Make sure the userid got created; sometimes there is a delay. */
   printf  ("\n=====Section 1:\nCreating '%s' userid... please wait.\n", 
             userid);
   sprintf (netuser, "%s\\netuser.log", caucus);
   _unlink (netuser);
   sprintf (temp, "net user %s >%s\\netuser.log", userid, caucus);
   while (1) {
      system (temp);
      if (sysfsize (netuser, &bytes, &error)) {
         if (bytes > 0)  break;
      }
      system ("cls");
      printf ("\nCreating '%s' userid... please wait.\n", userid);
      syssleep (20);
   }
      
   /*** Set initial permissions on caucus homedir. */
   sprintf (temp, "echo y|cacls %s /g administrators:f >nul:", caucus);
   system  (temp);
   sprintf (temp, "cacls %s /e /t  /g %s:f >nul:", caucus, userid);
   system  (temp);
  
   /*** Stop any previous Caucus service and remove it. */
   printf  ("\n=====Section 2: stop any previous Caucus service\n");
   printf  ("(Ignore any error messages.)\n");
   sprintf (temp, "net stop %s >nul:", argv[3]);
   system  (temp);
   sprintf (temp, "%s\\services\\instsrv %s remove >nul:",
                   caucus, argv[3]);
   system  (temp);

   /*** Move the contents of the distributed V2TARGET directory directly
   /    to the caucus home directory. */
   printf  ("\n=====Section 3: move caucus files to %s.\n", caucus);
   strcpy (chomedir, caucus);
   for (s=chomedir;   *s;   ++s)  if (*s == '\\')  *s = '/';
   sprintf (temp, "%s/V2TARGET", argv[9]);
   for (s=temp;       *s;   ++s)  if (*s == '\\')  *s = '/';
   sysmvdir (temp, chomedir);

   /*** Fix up permissions for everything in fell swoop! */
   printf  ("\n=====Section 4: Changing caucus file permissions.\n");
   if ( (fp = fopen ("permit.bat", "wt")) != NULL) {
      fprintf (fp, "cd %s\n", caucus);
      fprintf (fp, "%c:\n", caucus[0]);
      fprintf (fp, "echo y|cacls *.*  /t    /p administrators:f >nul:\n");
      fprintf (fp, "cacls *.*         /t /e /g %s:f >nul:\n", userid);
      fprintf (fp, "cacls public_html /t /e /g everyone:r >nul:\n");
      fprintf (fp, "cacls sweb        /t /e /g everyone:r >nul:\n");
      fclose  (fp);
      system    ("permit.bat");
   }
   else {
      printf ("\nCannot create permit.bat!  See \n");
      printf ("http://screenporch.com/sp/product/ins_nt/err1.html\n");
      printf ("(Press RETURN to continue.)\n");
      gets   (temp);
   }

   /*** If this is a new installation, set up the directories MISC, USER, 
   /    and C001.  Otherwise, throw them away. */
   printf  ("\n=====Section 5: creating miscellaneous files.\n");
   if (NOT syscheck (n1("MISC"))) {
      sysrename (n1("NEW_MISC"), n2("MISC"));
      sysrename (n1("NEW_USER"), n2("USER001"));
      sysrename (n1("NEW_C001"), n2("C0001"));
      sysmkdir  (n1("TEXT001"),       0777);
      sysmkdir  (n1("TEXT001\\roth"), 0777);
   }
   else {
      syscopy  (n1("NEW_MISC\\confsmsk"), n2("MISC\\confsmsk"));
      sysrmdir (n1("NEW_MISC"));
      sysrmdir (n1("NEW_USER"));
      sysrmdir (n1("NEW_C001"));
   }
      

   /*** All of the macros for sysmac0 have been left out.  This is
   /    probably OK unless the text interface gets any serious use,
   /    which seems unlikely!  See the Unix cinstall script for
   /    more details. */


   /*** Change default sys_Tset0 to use 'terminal' textentry for
   /    text interface, instead of 'wordwrap' (which doesn't work
   /    anyway). */
   if ( (fp = fopen (n1("DIC2\\sysmac0"), "a")) != NULL) {
      fprintf (fp, ":sys_Tset0 table\n");
      fprintf (fp, "off 23 79 off caucus 8 12 default on 0 "
                   ". off terminal 0 later welcome\n");
      fprintf (fp, "off off ascii_unix 2     kermit\n");
      fclose  (fp);
   }

   /*** Compile Caucus dictionary 0. */
   if ( (fp = fopen (n1("compile.bat"), "w")) != NULL) {
      fprintf (fp, "echo off\n");
      fprintf (fp, "cd %s\\DIC2\n", caucus);
      fprintf (fp, "%c:\n", caucus[0]);
      fprintf (fp, "%s\\BIN2\\makemd_x %s\\MISC\\dicti diction0 0 >nul:\n", 
                    caucus, caucus);
      fclose  (fp);
      system    (n1("compile.bat"));
      sysunlink (n1("compile.bat"));
   }

   /*** Set the world-wide-web hostname for this system. */
   strcpy (hostname, argv[4]);
   fp = fopen (n1("www_hostname"), "w");
   fprintf (fp, "%s\n", hostname);
   fclose  (fp);

   /*** Set the administrator information. */
   strcpy (admin_userid, argv[6]);
   strcpy (admin_mail,   argv[5]);

   /*** Create 'managers' file? */
   if (NOT syscheck (n1("MISC/managers"))) {
      if ( (fp = fopen (n1("MISC/managers"), "w")) != NULL) {
         fprintf (fp, "%s\n -1\n", admin_userid);
         fclose  (fp);
      }
   }

   /*** Make a reasonable default swebd.conf file. */
   mksweb (n1("SWEB\\swebd.new"), hostname, caucus, userid, admin_mail);
   if (NOT syscheck (n1("SWEB\\swebd.conf")))
      sysrename (n1("SWEB\\swebd.new"), n2("SWEB\\swebd.conf"));

   mkhtml (n1("public_html\\caucus40.htm"), hostname, userid);
   if (NOT syscheck (n1("public_html\\caucus.htm")))
      sysrename (n1("public_html\\caucus40.htm"),
                 n2("public_html\\caucus.htm"));


   /*** Eventually (at least for some webservers), we'll need to
   /    set up the REG directory... */
   /*
       rm -f REG/swebcli REG/swebt.cgi REG/swebsock
       ln -s $cwd/SWEB/swebcli   REG/swebcli
       ln -s $cwd/SWEB/swebt.cgi REG/swebt.cgi
       ln -s $cwd/SWEB/swebsock  REG/swebsock
       #---REG files.  Save old REG contents, install new ones.
       if test -d CML/REG31; then
          echo " "                                            
                       >>caucus.warn
          echo " - Your old CML/REG31 directory is in CML/REG31$OLD." 
                       >>caucus.warn
          mv    CML/REG31 CML/REG31$OLD
       fi
       cd    CML
       mv    NEW_REG REG31
       cd    REG31
       (echo set chome $cwd; cat register.org) > register.cml
       cd   $cwd
   */

   /*** If there's a previous CML/SP40 directory, rename it
   /    to save the contents.   Then install the new CML
   /    files in CML/SP40. */
   now = systime();
   sysymd (now, &year, &month, &day, &hour, &minute, &sec);
   if (syscheck (n1("CML\\SP40"))) {
      sprintf   (temp, "%s\\CML\\SP40_%02d%02d", caucus, month, day);
      sysrename (n1("CML\\SP40"), temp);
   }
   sysrename (n1("CML\\NEW_SP"), n2("CML\\SP40"));

   /*** Put all of the "error" CML files in the top of the CML
   /    directory. */
   if (NOT syscheck (n1("CML\\expired.cml"))) {
      sysrename (n1("CML\\SP40\\expired.cml"),  n2("CML\\expired.cml"));
      sysrename (n1("CML\\SP40\\badclock.cml"), n2("CML\\badclock.cml"));
      sysrename (n1("CML\\SP40\\badsysid.cml"), n2("CML\\badsysid.cml"));
      sysrename (n1("CML\\SP40\\syserr1.cml"),  n2("CML\\syserr1.cml"));
      sysrename (n1("CML\\SP40\\baddisk.cml"),  n2("CML\\baddisk.cml"));
   }


   /*** Save the old GIF40 directory, install new GIF files. */
   if (syscheck (n1("public_html\\GIF40"))) {
      sprintf (temp, "%s\\public_html\\GIF40_%02d%02d", caucus, month, day);
      sysrename (n1("public_html\\GIF40"), temp);
   }
   sysrename (n1("public_html\\NEW_GIF40"), n2("public_html\\GIF40"));

   /*** Make sure there's an upload directory. */
   if (NOT syscheck (n1("public_html\\LIB"))) 
      sysmkdir (n1("public_html\\LIB"), 0755);

   /*** And SOCKET, ATTACH, ARCHIVE, and GROUPS directories. */
   if (NOT syscheck (n1("SOCKET")))   sysmkdir (n1("SOCKET"),  0777);
/*
   if (NOT syscheck (n1("ATTACH")))   sysmkdir (n1("ATTACH"),  0700);
   if (NOT syscheck (n1("ARCHIVE")))  sysmkdir (n1("ARCHIVE"), 0700);
   if (NOT syscheck (n1("GROUPS")))   sysmkdir (n1("GROUPS"),  0700);
*/
   

   /*** Install SRVANY as the "caucus" service. */
   printf  ("\n=====Section 6: Install Caucus service and registry values.\n");
   sprintf (temp, "%s\\services\\instsrv %s %s\\services\\srvany.exe",
                   caucus, argv[3], caucus);
   system  (temp);


   /*** Create the 'caucus\Parameters' key. */
   sprintf (keyname, "SYSTEM\\CurrentControlSet\\Services\\%s\\Parameters",
            argv[3]);
   ret = RegCreateKeyEx (HKEY_LOCAL_MACHINE, keyname,
                         0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
                         NULL, &keyh, &disposition);
   if (ret != ERROR_SUCCESS) {
      printf ("Can't create key\n  '%s'\n", keyname);
      exit   (1);
   }

   /*** Create string values needed to run swebd, under that key. */
   sprintf (Application,   "%s\\SWEB\\swebd.exe",  caucus);
   sprintf (AppParameters, "%s\\SWEB\\swebd.conf", caucus);
   ret = RegSetValueEx (keyh, "Application",   0, REG_SZ, 
            (CONST BYTE *) Application,   strlen(Application)+1);
   ret = RegSetValueEx (keyh, "AppParameters", 0, REG_SZ, 
            (CONST BYTE *) AppParameters, strlen(AppParameters)+1);

   RegCloseKey (keyh);

   /*** Run the permissions batch file again, just to make sure that
   /    the newly created stuff gets the right permissions. */
   printf  ("\n=====Section 7: Set Caucus file permissions (again).\n");
   system    ("permit.bat");

   /*** But what about TEXT001 and SOCKET? */


   /*** Parse and apply the activation key: pppppppssssccceeuuuuu. */
   printf  ("\n=====Section 8: Apply activation key.\n");
   strcpy  (password, argv[10]);      password[7] = '\0';
   strcpy  (ship,     argv[10]+7);    ship    [4] = '\0';
   strcpy  (code,     argv[10]+11);   code    [3] = '\0';
   strcpy  (expire,   argv[10]+14);   expire  [2] = '\0';
   strcpy  (ulimit,   argv[10]+16);
   sprintf (actlog, "%s/activate.log", caucus);

   activate = 0;
   if (strcmp (expire, "00") == 0)   activate = 1;
   else {
      sscanf (ship, "%d", &codeyear);
      sscanf (code, "%d", &codeday);
      if (codeyear == year - 1) {
         codeyear = year;
         codeday  = codeday - 365;
      }

      secs  = time(NULL);
      tmnow = localtime (&secs);
      yday  = tmnow->tm_yday;

      if (codeyear == year  &&  yday - codeday < 6)  activate = 1;
   }

   if (activate) {
      sprintf (temp, 
         "%s\\BIN2\\lice4 %s\\BIN2\\caucus_x.exe %s %s %s -e %s -f 5 -t %s >%s",
         caucus, caucus, password, ship, code, expire, ulimit, actlog);
      system (temp);

      /*** Did the activation succeed? */
      if ( (fp = fopen (actlog, "r")) != NULL) {
         fgets  (temp, 200, fp);
         fclose (fp);
         activate = (strindex (temp, "Incorrect") < 0);
      }

      if (activate) {
         sprintf (temp, 
         "%s\\BIN2\\lice4 %s\\SWEB\\swebd.exe %s %s %s -e %s -f 5 -t %s >nul:",
            caucus, caucus, password, ship, code, expire, ulimit);
         system (temp);
      
         sprintf (temp, 
         "%s\\BIN2\\lice4 %s\\SWEB\\swebs.exe %s %s %s -e %s -f 5 -t %s >nul:",
            caucus, caucus, password, ship, code, expire, ulimit);
         system (temp);
      
         sprintf (temp, 
          "%s\\BIN2\\lice4 %s\\SWEB\\sweba.exe %s %s %s -e %s -f 5 -t %s >nul:",
            caucus, caucus, password, ship, code, expire, ulimit);
         system (temp);
      }
      else {
         system ("cls");
         printf ("\n====================================================\n"
                   "=                                                  =\n"
                   "=  WARNING!  Your activation key was incorrect.    =\n"
                   "=  Please install Caucus all over again, taking    =\n"
                   "=  care to enter the activation key precisely      =\n"
                   "=  as it was sent to you (including upper and      =\n"
                   "=  lower case.                                     =\n"
                   "=                                                  =\n"
                   "=  If you still have problems, please contact      =\n"
                   "=  Screen Porch for assistance, by sending e-mail  =\n"
                   "=  to support@screenporch.com.                     =\n"
                   "=                                                  =\n"
                   "====================================================\n");
         printf ("\nPress RETURN to continue: \n");
         gets   (temp);
      }
   }

   else {
      system ("cls");
      printf ("\n====================================================\n"
                "=                                                  =\n"
                "=  WARNING!  Your kit cannot be activated.  You    =\n"
                "=  must install it within 5 days of receiving your =\n"
                "=  activation key.                                 =\n"
                "=                                                  =\n"
                "=  You may download a new kit, or contact Screen   =\n"
                "=  Porch for an extension key, by sending e-mail   =\n"
                "=  to sales@screenporch.com.                       =\n"
                "=                                                  =\n"
                "====================================================\n");
      printf ("\nPress RETURN to continue: \n");
      gets   (temp);
   }
                
         

   exit (0);
}


FUNCTION char *n1 (char *name)
{
   static char temp1[256];

   sprintf (temp1, "%s\\%s", caucus, name);
   return  (temp1);
}

FUNCTION char *n2 (char *name)
{
   static char temp2[256];

   sprintf (temp2, "%s\\%s", caucus, name);
   return  (temp2);
}
