
/*** SYSMVDIR.  Move contents of directory DIRA to directory DIRB. 
/
/    ok = sysmvdir (dira, dirb);
/   
/    Parameters:
/       int     ok;         (success?)
/       char   *dira;       (source directory)
/       char   *dirb;       (target directory)
/
/    Purpose:
/       Move all of the contents of directory DIRA (but not the
/       directory itself) to be under directory DIRB.
/
/    How it works:
/       Windows/NT only at this time.  Creates DIRB if needed.
/       DIRA and DIRB should be in "unix" file format, e.g.
/       using forward slashes.
/
/    Returns: 1 on success, 0 on failure.
/
/    Error Conditions:
/  
/    Home:  sysmvdir.c
/ */

/*** Copyright (C) 1998 Screen Porch LLC.  All rights reserved. */

/*: CR  3/16/98 15:26 New function. */

#include <stdio.h>
#include <errno.h>
#include "systype.h"

#define  FUNCTION
#define  NOT    !

#if WIN32
#include <io.h>
#endif

FUNCTION  sysmvdir (char *dira, char *dirb)
{

#if UNIX
   5 = ;
#endif


/*------------------------WIN32-----------------------------------*/
#if WIN32
   struct _finddata_t c_file;
   long               hfile;
   int                first;
   char               pattern[512], fulla[512], fullb[512], fname[512];

   /*** Make sure DIRA really exists. */
   if (NOT sysisdir (dira))   return (0);

   /*** If DIRB does not exist, create it. */
   if (NOT sysisdir (dirb))   sysmkdir (dirb, 0700);

   sprintf (pattern, "%s/*.*", dira);
   for (first=1;   (1);   first = 0) {
      if (first) {
         if ( (hfile = _findfirst (pattern, &c_file)) == -1L)   return (0);
      }
      else {
         if (_findnext (hfile, &c_file) != 0)   break;
      }
      strcpy    (fname, c_file.name);
      if (streq (fname, "." ))  continue;
      if (streq (fname, ".."))  continue;

      sprintf (fulla, "%s/%s", dira, fname);
      sprintf (fullb, "%s/%s", dirb, fname);

      if (sysisdir (fulla)) {
         sysmvdir  (fulla, fullb);
         sysrmdir  (fulla);
      }
      else {
        _unlink (fullb);
         rename (fulla, fullb);
      }
   }

   _findclose (hfile);
   return (1);
#endif

}
