
/*** NTINS.   Caucus/NT installation program.
/
/    Invoked as:
/       ntins  userid  caucus  servname
/
/    Arguments:
/       userid    userid (account) that owns Caucus files
/       caucus    full pathname of home directory of Caucus userid
/       servname  name of caucus service (usually "caucus"!)
/
/    Purpose:
/       Do most of dirty work of installing Caucus on NT.
/
/    How it works:
/
/    Exit status: 
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  ntins.c
/ */

/*** Copyright (C) 1997 Screen Porch LLC.  All rights reserved. */

/*: CR  7/19/97 13:59 New program. */
/*: CR  8/01/97 15:34 2nd version, meant to be run by an administrator. */

#include <stdio.h>
#include "chixuse.h"
#include "ntcaucus.h"
#include <windows.h>
#include <iostream.h>

static char caucus[256];

FUNCTION main (int argc, char *argv[])
{
   char  *n1(), *n2();
   char   temp[256], hostname[256], admin_mail[256], admin_userid[256];
   FILE  *fp;
   char   userid[100];
   long   now, systime();
   int    year, month, day, hour, minute, sec;
   LONG   ret;
   HKEY   keyh;
   DWORD  disposition;
   char   keyname[512];
   char   Application[256], AppParameters[256];

   if (argc < 4) {
      printf ("Usage: ntins userid caucus_dir servname\n");
      exit   (1);
   }
   strcpy (userid, argv[1]);
   strcpy (caucus, argv[2]);

   /*** If this is a new installation, set up the directories MISC, USER, 
   /    and C001.  Otherwise, throw them away. */
   if (NOT syscheck (n1("MISC"))) {
      sysrename (n1("NEW_MISC"), n2("MISC"));
      sysrename (n1("NEW_USER"), n2("USER001"));
      sysrename (n1("NEW_C001"), n2("C0001"));
      sysmkdir  (n1("TEXT001"),       0777);
      sysmkdir  (n1("TEXT001\\roth"), 0777);
   }
   else {
      syscopy  (n1("NEW_MISC\\confsmsk"), n2("MISC\\confsmsk"));
      sysrmdir (n1("NEW_MISC"));
      sysrmdir (n1("NEW_USER"));
      sysrmdir (n1("NEW_C001"));
   }
      

   /*** All of the macros for sysmac0 have been left out.  This is
   /    probably OK unless the text interface gets any serious use,
   /    which seems unlikely!  See the Unix cinstall script for
   /    more details. */


   /*** Change default sys_Tset0 to use 'terminal' textentry for
   /    text interface, instead of 'wordwrap' (which doesn't work
   /    anyway). */
   if ( (fp = fopen (n1("DIC2\\sysmac0"), "a")) != NULL) {
      fprintf (fp, ":sys_Tset0 table\n");
      fprintf (fp, "off 23 79 off caucus 8 12 default on 0 "
                   ". off terminal 0 later welcome\n");
      fprintf (fp, "off off ascii_unix 2     kermit\n");
      fclose  (fp);
   }

   /*** Compile Caucus dictionary 0. */
   if ( (fp = fopen (n1("compile.bat"), "w")) != NULL) {
      fprintf (fp, "cd %s\\DIC2\n", caucus);
      fprintf (fp, "%s\\BIN2\\makemd_x %s\\MISC\\dicti diction0 0\n", 
                    caucus, caucus);
      fprintf (fp, "cd %s\n", caucus);
      fclose  (fp);
      system    (n1("compile.bat"));
      sysunlink (n1("compile.bat"));
   }

   /*** Get the world-wide-web hostname for this system. */
   if (NOT syscheck (n1("www_hostname"))) {
      printf ("What is the WWW hostname for your system?  Include the\n"
              "site name and port number, if needed.\n\n"
              "Examples: www.xyz.com\n"
              "          hal.state_univ.edu:8000\n\n");
      while (1) {
         printf ("WWW hostname? ");   fflush (stdout);
         gets   (hostname);
         stralter (hostname, "\n", "");
         stralter (hostname, "\r", "");
         if (NOT NULLSTR(hostname))  break;
      }
      fp = fopen (n1("www_hostname"), "w");
      fprintf (fp, "%s\n", hostname);
      fclose  (fp);
   }
   else {
      fp = fopen (n1("www_hostname"), "rt");
      fgets  (hostname, 256, fp);
      fclose (fp);
      stralter (hostname, "\n", "");
   }

   /*** Get the administrator information. */
   printf ("\nOne person should be in charge of administering Caucus on\n");
   printf ("your system.  That person is given full Caucus management\n");
   printf ("capabilities, and automatically receives problem notices\n");
   printf ("via e-mail.\n\n");
   admin_userid[0] = '\0';
   admin_mail  [0] = '\0';
   do {
      printf ("Caucus Administrator e-mail: ");
      fflush (stdout);
      gets   (admin_mail);

      if (NOT syscheck (n1("MISC/managers"))) {
         printf ("Caucus Administrator userid (in Caucus): ");
         fflush (stdout);
         gets   (admin_userid);
      }
   } while (admin_mail[0] == '\0');

   /*** Create 'managers' file? */
   if (NOT syscheck (n1("MISC/managers"))) {
      if ( (fp = fopen (n1("MISC/managers"), "w")) != NULL) {
         fprintf (fp, "%s\n -1\n", admin_userid);
         fclose  (fp);
      }
   }

   /*** Make a reasonable default swebd.conf file. */
   mksweb (n1("SWEB\\swebd.new"), hostname, caucus, userid, admin_mail);
   if (NOT syscheck (n1("SWEB\\swebd.conf")))
      sysrename (n1("SWEB\\swebd.new"), n2("SWEB\\swebd.conf"));

   mkhtml (n1("public_html\\caucus40.htm"), hostname, userid);
   if (NOT syscheck (n1("public_html\\caucus.htm")))
      sysrename (n1("public_html\\caucus40.htm"),
                 n2("public_html\\caucus.htm"));


   /*** Eventually (at least for some webservers), we'll need to
   /    set up the REG directory... */
   /*
       rm -f REG/swebcli REG/swebt.cgi REG/swebsock
       ln -s $cwd/SWEB/swebcli   REG/swebcli
       ln -s $cwd/SWEB/swebt.cgi REG/swebt.cgi
       ln -s $cwd/SWEB/swebsock  REG/swebsock
       #---REG files.  Save old REG contents, install new ones.
       if test -d CML/REG31; then
          echo " "                                            
                       >>caucus.warn
          echo " - Your old CML/REG31 directory is in CML/REG31$OLD." 
                       >>caucus.warn
          mv    CML/REG31 CML/REG31$OLD
       fi
       cd    CML
       mv    NEW_REG REG31
       cd    REG31
       (echo set chome $cwd; cat register.org) > register.cml
       cd   $cwd
   */

   /*** If there's a previous CML/SP40 directory, rename it
   /    to save the contents.   Then install the new CML
   /    files in CML/SP40. */
   now = systime();
   sysymd (now, &year, &month, &day, &hour, &minute, &sec);
   if (syscheck (n1("CML\\SP40"))) {
      sprintf   (temp, "%s\\CML\\SP40_%02d%02d", caucus, month, day);
      sysrename (n1("CML\\SP40"), temp);
   }
   sysrename (n1("CML\\NEW_SP"), n2("CML\\SP40"));

   /*** Put all of the "error" CML files in the top of the CML
   /    directory. */
   if (NOT syscheck (n1("CML\\expired.cml"))) {
      sysrename (n1("CML\\SP40\\expired.cml"),  n2("CML\\expired.cml"));
      sysrename (n1("CML\\SP40\\badclock.cml"), n2("CML\\badclock.cml"));
      sysrename (n1("CML\\SP40\\badsysid.cml"), n2("CML\\badsysid.cml"));
      sysrename (n1("CML\\SP40\\syserr1.cml"),  n2("CML\\syserr1.cml"));
      sysrename (n1("CML\\SP40\\baddisk.cml"),  n2("CML\\baddisk.cml"));
   }


   /*** Save the old GIF40 directory, install new GIF files. */
   if (syscheck (n1("public_html\\GIF40"))) {
      sprintf (temp, "%s\\public_html\\GIF40_%02d%02d", caucus, month, day);
      sysrename (n1("public_html\\GIF40"), temp);
   }
   sysrename (n1("public_html\\NEW_GIF40"), n2("public_html\\GIF40"));

/*
   sprintf (temp, "cacls %s\\public_html\\GIF40\\*.* /g Everyone:R /e /t", 
                   caucus);
   system  (temp);
*/

   /*** Make sure there's an upload directory. */
   if (NOT syscheck (n1("public_html\\LIB"))) 
      sysmkdir (n1("public_html\\LIB"), 0755);

   /*** And SOCKET, ATTACH, ARCHIVE, and GROUPS directories. */
   if (NOT syscheck (n1("SOCKET")))   sysmkdir (n1("SOCKET"),  0777);
/*
   if (NOT syscheck (n1("ATTACH")))   sysmkdir (n1("ATTACH"),  0700);
   if (NOT syscheck (n1("ARCHIVE")))  sysmkdir (n1("ARCHIVE"), 0700);
   if (NOT syscheck (n1("GROUPS")))   sysmkdir (n1("GROUPS"),  0700);
*/
   

   /*** Make sure the web servers and service-startup can run
   /    swebd, swebs, and swebsock. */
/*
   sprintf (temp, "cacls %s\\sweb\\*.* /g Everyone:R /e /t", caucus);
   system  (temp);
*/

   /*** Install SRVANY as the "caucus" service. */
   sprintf (temp, "%s\\services\\instsrv caucus %s\\services\\srvany.exe",
                   caucus, caucus);
   system  (temp);


   /*** Create the 'caucus\Parameters' key. */
   sprintf (keyname, "SYSTEM\\CurrentControlSet\\Services\\%s\\Parameters",
            argv[3]);
   ret = RegCreateKeyEx (HKEY_LOCAL_MACHINE, keyname,
                         0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
                         NULL, &keyh, &disposition);
   if (ret != ERROR_SUCCESS) {
      printf ("Can't create key\n  '%s'\n", keyname);
      exit   (1);
   }

   /*** Create string values needed to run swebd, under that key. */
   sprintf (Application,   "%s\\SWEB\\swebd.exe",  caucus);
   sprintf (AppParameters, "%s\\SWEB\\swebd.conf", caucus);
   ret = RegSetValueEx (keyh, "Application",   0, REG_SZ, 
            (CONST BYTE *) Application,   strlen(Application)+1);
   ret = RegSetValueEx (keyh, "AppParameters", 0, REG_SZ, 
            (CONST BYTE *) AppParameters, strlen(AppParameters)+1);

   RegCloseKey (keyh);

   /*** Fix up permissions for everything in fell swoop! */
   fp = fopen ("permit.bat", "wt");
   fprintf (fp, "echo y|cacls *.*  /t    /p administrators:f >nul:\n");
   fprintf (fp, "cacls *.*         /t /e /g %s:f >nul:\n", userid);
   fprintf (fp, "cacls public_html /t /e /g everyone:r >nul:\n");
   fprintf (fp, "cacls sweb        /t /e /g everyone:r >nul:\n");
   fclose  (fp);
   system  ("permit.bat");

   /*** But what about TEXT001 and SOCKET? */

   exit (0);
}

FUNCTION char *n1 (char *name)
{
   static char temp1[256];

   sprintf (temp1, "%s\\%s", caucus, name);
   return  (temp1);
}

FUNCTION char *n2 (char *name)
{
   static char temp2[256];

   sprintf (temp2, "%s\\%s", caucus, name);
   return  (temp2);
}
