
/*** DO_PASSWD.   Password handling.  (Change and Check)
/
/    ok = do_passwd (arg, passprog, change);
/   
/    Parameters:
/       int           ok;        (return code)
/       Chix          arg;       ("id pw1 [pw2]")
/       char *        passprog;  (program to change password)
/       int           change;    (is change ? vs check)
/
/    Purpose:
/       Change or check a userid / password combination.
/       Change the password for user "id", from "oldpw" to "newpw".
/
/    How it works:
/       If CHANGE, changes the password for user "id" to "pw1" (from "pw2").
/
/       Otherwise, checks for validity of "id" "pw1" combination.
/
/    Returns: 0 on success
/             1 if not enough words in arg
/             4 if "pw2" is bad
/             7 if no password changer
/             8 system problem
/
/    Error Conditions:
/  
/    Known bugs:
/
/    Home:  dopasswd.c
/ */

/*** Copyright (C) 1995 Camber-Roth.  All rights reserved. */

/*: CR 11/28/95 16:38 New function. */

#include <stdio.h>
#include "systype.h"
#include "chixuse.h"
#include "sweb.h"

#include <string.h>

FUNCTION  int do_passwd (Chix arg, char *passprog, int change)
{
   int    failcode = 7;

#if UNIX
   Chix   word;
   int    to_child[2], from_child[2], noargs = 0, i, status, got;
   char   what[400], reply[20];

   /*** Was a password-changer program supplied? */
   if (*passprog == 0)  return (7);
   if (change)          change = 1;

   word = chxalloc (L(40), THIN, "do_passwd word");

   /*** Convert ARGS to ascii string with newlines. */
   if (change)  strcpy (what, "c\n");
   else         strcpy (what, "v\n");
   for (i=1;   i<=2+change;   ++i)  {
      if (chxtoken (word, nullchix, i, arg) < 0)  { noargs = i;   break; }
      strcat   (what, ascquick(word));
      strcat   (what, "\n");
   }
   chxfree (word);
   if (noargs)  return (1);

   pipe (  to_child);
   pipe (from_child);

   /*** Parent.  Spawns off a child to execute PASSPROG.
   /    Send the userid, oldpw, and newpw down the pipe. */
   if (fork() > 0) {
      close (  to_child[0]);
      close (from_child[1]);
      write (  to_child[1], what, strlen(what));
      close (  to_child[1]);

      /*** Insist on reading some data from the child. */
      for (i=0;   i<30;   ++i) {
         if ( (got = read (from_child[0], reply, 10)) > 0)  break;
         sleep (3);
      }
    
      wait  (&status);
      close (from_child[0]);
   }

   /*** Child.  Connect stdin to downstream end of TUBE, and execute
   /    PASSPROG in a shell. */
   else {
      dup2     (  to_child[0], 0);
      dup2     (from_child[1], 1);
      close    (  to_child[1]);    close (from_child[0]);
      execl    ("/bin/sh", "sh", "-c", passprog, NULL);
      exit     (1);
   }

   failcode = reply[0] - '0';
   if (failcode > 8)  failcode = 8;
#endif

   return (failcode);
}
