
/*** INCONF.   Handle PICUS commands inside a conference.
/
/    inconf (cnum);
/
/    Parameters:
/       int       cnum;     (current conference num)
/
/    Purpose:
/       Handle PICUS selections inside & relevant to a particular
/       conference.
/
/    How it works:
/
/    Returns:
/
/    Error Conditions:
/ 
/    Side effects:
/
/    Related functions:
/
/    Called by:  picus
/
/    Operating system dependencies:
/
/    Known bugs:
/
/    Home:  zpicus/inconf.c
/ */

/*** Copyright (C) 1992 Camber-Roth.  All rights reserved. */

/*: CR  2/22/93 22:04 New function. */
/*: CK  3/01/93 16:54 Call a_del_pstat() correctly. */
/*: JV  5/28/93 21:15 Add attachments, change text for 15. */
/*: CN  5/18/93 21:18 Add P_ISINCE, P_RSINCE, P_AND testing. */
/*: CP  7/08/93 17:19 Integrate 2.5 changes. */
/*: CP  8/01/93 10:50 Add a_mak...() name arg. */

#include <stdio.h>
#include "chixuse.h"
#include "api.h"
#include "caucus.h"

extern int debug;

FUNCTION  inconf (cnum)
   int    cnum;
{
   static char *what[] = {
      /*  0 */   "*Leave Conf",
      /*  1 */   "*Show Conf Info",
      /*  2 */   "*Customize Org.",
      /*  3 */   "*Cust. Intro",
      /*  4 */   "*Cust. Greeting",
      /*  5 */   "*Cust. Userlist",
      /*  6 */   "*Cust. Change",
      /*  7 */   "*Cust. Add",
      /*  8 */   "*Cust. Subject",
      /*  9 */   "*Cust. Visible",
      /* 10 */   "*Show Person",
      /* 11 */   "*What's New?",
      /* 12 */   "*Show   Resp",
      /* 13 */   "*Forget Item",
      /* 14 */   "*Add    Item",
      /* 15 */   "*Add Response",
      /* 16 */   "*Delete Item",
      /* 17 */   "*Delete Resp",
      /* 18 */   "*Change Item",
      /* 19 */   "*Change Resp",
      /* 20 */   "*Change Name",
      /* 21 */   "*Change Phone",
      /* 22 */   "*Change Intro",
      /* 23 */   "*Resign Conf",
      /* 24 */   "*List Items",
      /* 25 */   "*Show   SET",
      /* 26 */   "*Change SET",
      /* 27 */   "*List Members",
      /* 28 */   "*List People",
      /* 29 */   "*Flip Debug",
      /* 30 */   "*List Subject",
      /* 31 */   "*Add  Subject",
      /* 32 */   "*Del  Subject",
      /* 33 */   "*Chg  Subject",
      /* 34 */   "*List Psubj",
      /* 35 */   "*Add  Psubj",
      /* 36 */   "*Del  Psubj",
      /* 37 */   "*Chg  Psubj",
      /* 38 */   "*List by Title",
      /* 39 */   "*List by Author",
      /* 40 */   "*List by Sub",
      /* 41 */   "*List by Psub",
      /* 42 */   "*Add Attach",
      /* 43 */   "*Show Attach",
      /* 44 */   "*Delete Attach",
      /* 45 */   "*List by Isince",
      /* 46 */   "*List by Rsince",
      /* 47 */   "*AND next List",
                 "", "", "", ""
   };

   char     answer[20], name[80];
   int      choice, done, ok;
   Pstat    pstat;
   int      size, i, and;
   Chix     empty, userid;

   empty  = chxalloc (L(16), THIN, "inconf empty");
   userid = chxalloc (L(16), THIN, "inconf userid");
   sysuserid (userid);
   and    = 0;

   /*** Find the size of the WHAT table when displayed in 3 columns. */
   for (size=0;   *what[size];   ++size);
   size = (size-1) / 4 + 1;

   /*** Command loop. */
   for (done=0;   NOT done;   ) {
  
      /*** Print out the WHAT table in a reasonable form. */
      printf ("\n");
      for (i=0;   i<size;   ++i) {
         printf (" %2d %-15s  %2d %-15s  %2d %-15s  %2d %-15s\n",
                  i, what[i], i+size, what[i+size], i+size+size,
                  what[i+size+size], i+3*size, what[i+3*size]);
      }
  
      answer[0] = answer[1] = '\0';
      printf (": ");
      gets   (answer);
      if (isdigit (answer[0])) sscanf (answer, "%d", &choice);
      else                     choice = -1;

      switch (choice) {
         case ( 0):  done = 1;    break;

         case ( 1):  zshowconf (cnum);    break;

         case ( 2):  zcusporg  (cnum);    break;

         case ( 3):  zcustext  (cnum, P_TEXT);        break;
         case ( 4):  zcustext  (cnum, P_GREET);       break;
         case ( 5):  zcustext  (cnum, P_USERLIST);    break;

         case ( 6):  zcusflag  (cnum, P_FCHANGE);     break;
         case ( 7):  zcusflag  (cnum, P_FADD);        break;
         case ( 8):  zcusflag  (cnum, P_FCHGSUB);     break;
         case ( 9):  zcusflag  (cnum, P_FVIS);        break;

         case (10):  zshowper  ();                    break;
         case (11):  zshownew  (cnum);                break;

         case (12):  zshowresp (cnum);                break;
         case (13):  zforget   (cnum);                break;

         case (14):  zaddresp  (cnum, 0);             break;
         case (15):  zaddresp  (cnum, 1);             break;

         case (16):  zdelresp  (cnum, 0);             break;
         case (17):  zdelresp  (cnum, 1);             break;

         case (18):  zchgresp  (cnum, 0);             break;
         case (19):  zchgresp  (cnum, 1);             break;

         case (20):
         case (21):
         case (22):  zchgper   (choice-20);           break;

         case (23):
            pstat = a_mak_pstat("inconf pstat");
            pstat->cnum = cnum;
            sysuserid (pstat->owner);
            ok = a_del_pstat (pstat, P_EVERY, A_WAIT);
            a_fre_pstat (pstat);
            if (ok != A_OK)  printf ("resign code: %d.\n", ok);
            done = 1;
            break;

         case (24):  zlab      (cnum);                break;

         case (25):  zshowset();                      break;

         case (26):  zchgset();                       break;

         case (27):  zlap      (cnum);                break;

         case (28):  zlap      (0);                   break;

         case (29):  debug = -1 - debug;              break;

         case (30): case (31): case (32): case (33):
         case (34): case (35): case (36): case (37):
            printf ("Subject name? ");
            gets (name);

            if (choice==30)  zlistsubj (cnum, name, empty);
            if (choice==31)  zaddsubj  (cnum, name, empty);
            if (choice==32)  zdelsubj  (cnum, name, empty);
            if (choice==33)  zchgsub   (cnum, name, empty);
            if (choice==34)  zlistsubj (cnum, name, userid);
            if (choice==35)  zaddsubj  (cnum, name, userid);
            if (choice==36)  zdelsubj  (cnum, name, userid);
            if (choice==37)  zchgsub   (cnum, name, userid);
            break;

         case (38):
            printf  ("Title? ");
            gets    (name);
            zlistby (cnum, name, P_TITLE | and, empty);
            and = 0;
            break;

         case (39):
            printf  ("Author? ");
            gets    (name);
            zlistby (cnum, name, P_NAME | and, empty);
            and = 0;
            break;

         case (40):
            printf  ("Subject? ");
            gets    (name);
            zlistby (cnum, name, P_SUBJECT | and, empty);
            and = 0;
            break;

         case (41):
            printf  ("Psubject? ");
            gets    (name);
            zlistby (cnum, name, P_SUBJECT | and, userid);
            and = 0;
            break;

         case (42): zattach (cnum, 0);                    break;

         case (43): zattach (cnum, 1);                    break;

         case (44): zattach (cnum, 2);                    break;


         case (45):
            printf  ("I Since Date? ");
            gets    (name);
            zlistby (cnum, name, P_ISINCE | and, userid);
            and = 0;
            break;

         case (46):
            printf  ("R Since Date? ");
            gets    (name);
            zlistby (cnum, name, P_RSINCE | and, userid);
            and = 0;
            break;

         case (47):
            and = P_AND;
            break;

         default:    break;
      }

      if (NOT done) {
         printf ("\nPause...");
         gets   (answer);
      }
   }

   chxfree (userid);
   chxfree (empty);
   return;
}
